@@ -35,6 +35,11 @@
  * undefined total number of countable operations. */
 #define BG_UNDEFINED_TOTAL (-1)
 
+struct bg_job_t;
+
+/* Type of a function to invoke when the job is done. */
+typedef void (*bg_job_exit_func)(struct bg_job_t *job, void *data);
+
 /* Type of background job. */
 typedef enum
 {
@@ -54,6 +59,8 @@ typedef enum
 	BJF_SUPPLY_INPUT    = 1 << 2, /* Open a pipe for standard input stream. */
 	BJF_CAPTURE_OUT     = 1 << 3, /* Capture output stream(s). */
 	BJF_MERGE_STREAMS   = 1 << 4, /* Merge error stream into output stream. */
+	BJF_KEEP_IN_FG      = 1 << 5, /* Do not detach from terminal session or
+	                                 process group. */
 }
 BgJobFlags;
 
@@ -96,6 +103,9 @@ typedef struct bg_job_t
 	FILE *input;  /* File stream of standard input or NULL. */
 	FILE *output; /* File stream of standard output or NULL. */
 
+	bg_job_exit_func exit_cb; /* Function to invoke when the job exits. */
+	void *exit_cb_arg;        /* Argument to pass to that function. */
+
 	int with_bg_op;                /* Whether bg_op* fields are active. */
 	int on_job_bar;                /* Whether this task was put on a job bar. */
 	pthread_spinlock_t bg_op_lock; /* Lock for accessing bg_op field. */
@@ -125,8 +135,8 @@ typedef void (*bg_task_func)(bg_op_t *bg_op, void *arg);
 /* List of background jobs. */
 extern bg_job_t *bg_jobs;
 
-/* Prepare background unit for the work. */
-void bg_init(void);
+/* Prepare background unit for the work.  Returns zero on success. */
+int bg_init(void);
 
 /* Creates background job running external command.  Returns zero on success,
  * otherwise non-zero is returned.  If *input is not NULL, it's set to input
@@ -150,10 +160,10 @@ int bg_and_wait_for_errors(char cmd[],
 		const struct cancellation_t *cancellation);
 
 /* Runs command in a background and redirects its stdout and stderr streams to
- * file streams which are set.  Input is redirected only if in parameter isn't
- * NULL.  Don't pass pipe for input, it can cause deadlock.  Returns id of
- * background process ((pid_t)0 for non-*nix like systems) or (pid_t)-1 on
- * error. */
+ * file streams which are set.  Input and output are redirected only if the
+ * corresponding parameter isn't NULL.  Don't pass pipe for input, it can cause
+ * deadlock.  Returns id of background process ((pid_t)0 for non-*nix like
+ * systems) or (pid_t)-1 on error. */
 pid_t bg_run_and_capture(char cmd[], int user_sh, FILE *in, FILE **out,
 		FILE **err);
 
@@ -172,6 +182,9 @@ int bg_execute(const char descr[], const char op_descr[], int total,
  * applications whose state is tracked are always ignored by this function. */
 int bg_has_active_jobs(int important_only);
 
+/* Sets exit callback for the job. */
+void bg_job_set_exit_cb(bg_job_t *job, bg_job_exit_func cb, void *arg);
+
 /* Cancels the job.  Returns non-zero if job wasn't cancelled before, but is
  * after this call, otherwise zero is returned. */
 int bg_job_cancel(bg_job_t *job);
@@ -205,8 +218,8 @@ void bg_job_decref(bg_job_t *job);
 
 /* Temporary locks bg_op_t structure to ensure that it's not modified by
  * anyone during reading/updating its fields.  The structure must be part of
- * bg_job_t. */
-void bg_op_lock(bg_op_t *bg_op);
+ * bg_job_t.  Returns non-zero on success. */
+int bg_op_lock(bg_op_t *bg_op);
 
 /* Unlocks bg_op_t structure.  The structure must be part of bg_job_t. */
 void bg_op_unlock(bg_op_t *bg_op);
