@@ -157,6 +157,16 @@ static key_pair_t key_pairs[] = {
 	{ L"<s-tab>",    L"\033"L"[Z"         },
 #endif
 #ifndef __PDCURSES__
+	{ L"<a-0>",      L"\033"L"0"          },
+	{ L"<a-1>",      L"\033"L"1"          },
+	{ L"<a-2>",      L"\033"L"2"          },
+	{ L"<a-3>",      L"\033"L"3"          },
+	{ L"<a-4>",      L"\033"L"4"          },
+	{ L"<a-5>",      L"\033"L"5"          },
+	{ L"<a-6>",      L"\033"L"6"          },
+	{ L"<a-7>",      L"\033"L"7"          },
+	{ L"<a-8>",      L"\033"L"8"          },
+	{ L"<a-9>",      L"\033"L"9"          },
 	{ L"<a-a>",      L"\033"L"a"          },
 	{ L"<a-b>",      L"\033"L"b"          },
 	{ L"<a-c>",      L"\033"L"c"          },
@@ -183,6 +193,16 @@ static key_pair_t key_pairs[] = {
 	{ L"<a-x>",      L"\033"L"x"          },
 	{ L"<a-y>",      L"\033"L"y"          },
 	{ L"<a-z>",      L"\033"L"z"          },
+	{ L"<m-0>",      L"\033"L"0"          },
+	{ L"<m-1>",      L"\033"L"1"          },
+	{ L"<m-2>",      L"\033"L"2"          },
+	{ L"<m-3>",      L"\033"L"3"          },
+	{ L"<m-4>",      L"\033"L"4"          },
+	{ L"<m-5>",      L"\033"L"5"          },
+	{ L"<m-6>",      L"\033"L"6"          },
+	{ L"<m-7>",      L"\033"L"7"          },
+	{ L"<m-8>",      L"\033"L"8"          },
+	{ L"<m-9>",      L"\033"L"9"          },
 	{ L"<m-a>",      L"\033"L"a"          },
 	{ L"<m-b>",      L"\033"L"b"          },
 	{ L"<m-c>",      L"\033"L"c"          },
@@ -313,6 +333,110 @@ static key_pair_t key_pairs[] = {
 	{ L"<c-m-x>",    L"\033"L"\x18"       },
 	{ L"<c-m-y>",    L"\033"L"\x19"       },
 	{ L"<c-m-z>",    L"\033"L"\x1a"       },
+	{ L"<a-s-a>",    L"\033"L"A"          },
+	{ L"<a-s-b>",    L"\033"L"B"          },
+	{ L"<a-s-c>",    L"\033"L"C"          },
+	{ L"<a-s-d>",    L"\033"L"D"          },
+	{ L"<a-s-e>",    L"\033"L"E"          },
+	{ L"<a-s-f>",    L"\033"L"F"          },
+	{ L"<a-s-g>",    L"\033"L"G"          },
+	{ L"<a-s-h>",    L"\033"L"H"          },
+	{ L"<a-s-i>",    L"\033"L"I"          },
+	{ L"<a-s-j>",    L"\033"L"J"          },
+	{ L"<a-s-k>",    L"\033"L"K"          },
+	{ L"<a-s-l>",    L"\033"L"L"          },
+	{ L"<a-s-m>",    L"\033"L"M"          },
+	{ L"<a-s-n>",    L"\033"L"N"          },
+	{ L"<a-s-o>",    L"\033"L"O"          },
+	{ L"<a-s-p>",    L"\033"L"P"          },
+	{ L"<a-s-q>",    L"\033"L"Q"          },
+	{ L"<a-s-r>",    L"\033"L"R"          },
+	{ L"<a-s-s>",    L"\033"L"S"          },
+	{ L"<a-s-t>",    L"\033"L"T"          },
+	{ L"<a-s-u>",    L"\033"L"U"          },
+	{ L"<a-s-v>",    L"\033"L"V"          },
+	{ L"<a-s-w>",    L"\033"L"W"          },
+	{ L"<a-s-x>",    L"\033"L"X"          },
+	{ L"<a-s-y>",    L"\033"L"Y"          },
+	{ L"<a-s-z>",    L"\033"L"Z"          },
+	{ L"<s-a-a>",    L"\033"L"A"          },
+	{ L"<s-a-b>",    L"\033"L"B"          },
+	{ L"<s-a-c>",    L"\033"L"C"          },
+	{ L"<s-a-d>",    L"\033"L"D"          },
+	{ L"<s-a-e>",    L"\033"L"E"          },
+	{ L"<s-a-f>",    L"\033"L"F"          },
+	{ L"<s-a-g>",    L"\033"L"G"          },
+	{ L"<s-a-h>",    L"\033"L"H"          },
+	{ L"<s-a-i>",    L"\033"L"I"          },
+	{ L"<s-a-j>",    L"\033"L"J"          },
+	{ L"<s-a-k>",    L"\033"L"K"          },
+	{ L"<s-a-l>",    L"\033"L"L"          },
+	{ L"<s-a-m>",    L"\033"L"M"          },
+	{ L"<s-a-n>",    L"\033"L"N"          },
+	{ L"<s-a-o>",    L"\033"L"O"          },
+	{ L"<s-a-p>",    L"\033"L"P"          },
+	{ L"<s-a-q>",    L"\033"L"Q"          },
+	{ L"<s-a-r>",    L"\033"L"R"          },
+	{ L"<s-a-s>",    L"\033"L"S"          },
+	{ L"<s-a-t>",    L"\033"L"T"          },
+	{ L"<s-a-u>",    L"\033"L"U"          },
+	{ L"<s-a-v>",    L"\033"L"V"          },
+	{ L"<s-a-w>",    L"\033"L"W"          },
+	{ L"<s-a-x>",    L"\033"L"X"          },
+	{ L"<s-a-y>",    L"\033"L"Y"          },
+	{ L"<s-a-z>",    L"\033"L"Z"          },
+	{ L"<m-s-a>",    L"\033"L"A"          },
+	{ L"<m-s-b>",    L"\033"L"B"          },
+	{ L"<m-s-c>",    L"\033"L"C"          },
+	{ L"<m-s-d>",    L"\033"L"D"          },
+	{ L"<m-s-e>",    L"\033"L"E"          },
+	{ L"<m-s-f>",    L"\033"L"F"          },
+	{ L"<m-s-g>",    L"\033"L"G"          },
+	{ L"<m-s-h>",    L"\033"L"H"          },
+	{ L"<m-s-i>",    L"\033"L"I"          },
+	{ L"<m-s-j>",    L"\033"L"J"          },
+	{ L"<m-s-k>",    L"\033"L"K"          },
+	{ L"<m-s-l>",    L"\033"L"L"          },
+	{ L"<m-s-m>",    L"\033"L"M"          },
+	{ L"<m-s-n>",    L"\033"L"N"          },
+	{ L"<m-s-o>",    L"\033"L"O"          },
+	{ L"<m-s-p>",    L"\033"L"P"          },
+	{ L"<m-s-q>",    L"\033"L"Q"          },
+	{ L"<m-s-r>",    L"\033"L"R"          },
+	{ L"<m-s-s>",    L"\033"L"S"          },
+	{ L"<m-s-t>",    L"\033"L"T"          },
+	{ L"<m-s-u>",    L"\033"L"U"          },
+	{ L"<m-s-v>",    L"\033"L"V"          },
+	{ L"<m-s-w>",    L"\033"L"W"          },
+	{ L"<m-s-x>",    L"\033"L"X"          },
+	{ L"<m-s-y>",    L"\033"L"Y"          },
+	{ L"<m-s-z>",    L"\033"L"Z"          },
+	{ L"<s-m-a>",    L"\033"L"A"          },
+	{ L"<s-m-b>",    L"\033"L"B"          },
+	{ L"<s-m-c>",    L"\033"L"C"          },
+	{ L"<s-m-d>",    L"\033"L"D"          },
+	{ L"<s-m-e>",    L"\033"L"E"          },
+	{ L"<s-m-f>",    L"\033"L"F"          },
+	{ L"<s-m-g>",    L"\033"L"G"          },
+	{ L"<s-m-h>",    L"\033"L"H"          },
+	{ L"<s-m-i>",    L"\033"L"I"          },
+	{ L"<s-m-j>",    L"\033"L"J"          },
+	{ L"<s-m-k>",    L"\033"L"K"          },
+	{ L"<s-m-l>",    L"\033"L"L"          },
+	{ L"<s-m-m>",    L"\033"L"M"          },
+	{ L"<s-m-n>",    L"\033"L"N"          },
+	{ L"<s-m-o>",    L"\033"L"O"          },
+	{ L"<s-m-p>",    L"\033"L"P"          },
+	{ L"<s-m-q>",    L"\033"L"Q"          },
+	{ L"<s-m-r>",    L"\033"L"R"          },
+	{ L"<s-m-s>",    L"\033"L"S"          },
+	{ L"<s-m-t>",    L"\033"L"T"          },
+	{ L"<s-m-u>",    L"\033"L"U"          },
+	{ L"<s-m-v>",    L"\033"L"V"          },
+	{ L"<s-m-w>",    L"\033"L"W"          },
+	{ L"<s-m-x>",    L"\033"L"X"          },
+	{ L"<s-m-y>",    L"\033"L"Y"          },
+	{ L"<s-m-z>",    L"\033"L"Z"          },
 #else
 	{ L"<a-a>",      { K(ALT_A) }         },
 	{ L"<a-b>",      { K(ALT_B) }         },
@@ -380,6 +504,16 @@ static key_pair_t key_pairs[] = {
 	{ L"<insert>",   { K(KEY_IC) }        },
 	{ L"<pageup>",   { K(KEY_PPAGE) }     },
 	{ L"<pagedown>", { K(KEY_NPAGE) }     },
+	{ L"<s-home>",   { K(KEY_SHOME) }     },
+	{ L"<s-end>",    { K(KEY_SEND) }      },
+	{ L"<s-left>",   { K(KEY_SLEFT) }     },
+	{ L"<s-right>",  { K(KEY_SRIGHT) }    },
+	{ L"<s-up>",     { K(KEY_SR) }        },
+	{ L"<s-down>",   { K(KEY_SF) }        },
+	{ L"<s-delete>", { K(KEY_SDC) }       },
+	{ L"<s-insert>", { K(KEY_SIC) }       },
+	{ L"<s-pageup>", { K(KEY_SPREVIOUS) } },
+	{ L"<s-pagedown>", { K(KEY_SNEXT) }   },
 	{ L"<f0>",       { K(KEY_F(0)) }      },
 	{ L"<f1>",       { K(KEY_F(1)) }      },
 	{ L"<f2>",       { K(KEY_F(2)) }      },
@@ -640,21 +774,31 @@ wchar_to_spec(const wchar_t c[], size_t *len, int bs)
 	*len = 1;
 	switch(*c)
 	{
-		case L' ':          strcpy(buf, "<space>");    break;
-		case L'\r':         strcpy(buf, "<cr>");       break;
-		case L'\n':         strcpy(buf, "<c-j>");      break;
-		case L'\177':       strcpy(buf, "<del>");      break;
-		case K(KEY_HOME):   strcpy(buf, "<home>");     break;
-		case K(KEY_END):    strcpy(buf, "<end>");      break;
-		case K(KEY_UP):     strcpy(buf, "<up>");       break;
-		case K(KEY_DOWN):   strcpy(buf, "<down>");     break;
-		case K(KEY_LEFT):   strcpy(buf, "<left>");     break;
-		case K(KEY_RIGHT):  strcpy(buf, "<right>");    break;
-		case K(KEY_DC):     strcpy(buf, "<delete>");   break;
-		case K(KEY_BTAB):   strcpy(buf, "<s-tab>");    break;
-		case K(KEY_PPAGE):  strcpy(buf, "<pageup>");   break;
-		case K(KEY_NPAGE):  strcpy(buf, "<pagedown>"); break;
-		case WC_C_SPACE:    strcpy(buf, "<c-@>");      break;
+		case L' ':             strcpy(buf, "<space>");      break;
+		case L'\r':            strcpy(buf, "<cr>");         break;
+		case L'\n':            strcpy(buf, "<c-j>");        break;
+		case L'\177':          strcpy(buf, "<del>");        break;
+		case K(KEY_HOME):      strcpy(buf, "<home>");       break;
+		case K(KEY_END):       strcpy(buf, "<end>");        break;
+		case K(KEY_UP):        strcpy(buf, "<up>");         break;
+		case K(KEY_DOWN):      strcpy(buf, "<down>");       break;
+		case K(KEY_LEFT):      strcpy(buf, "<left>");       break;
+		case K(KEY_RIGHT):     strcpy(buf, "<right>");      break;
+		case K(KEY_DC):        strcpy(buf, "<delete>");     break;
+		case K(KEY_BTAB):      strcpy(buf, "<s-tab>");      break;
+		case K(KEY_PPAGE):     strcpy(buf, "<pageup>");     break;
+		case K(KEY_NPAGE):     strcpy(buf, "<pagedown>");   break;
+		case K(KEY_SHOME):     strcpy(buf, "<s-home>");     break;
+		case K(KEY_SEND):      strcpy(buf, "<s-end>");      break;
+		case K(KEY_SR):        strcpy(buf, "<s-up>");       break;
+		case K(KEY_SF):        strcpy(buf, "<s-down>");     break;
+		case K(KEY_SLEFT):     strcpy(buf, "<s-left>");     break;
+		case K(KEY_SRIGHT):    strcpy(buf, "<s-right>");    break;
+		case K(KEY_SDC):       strcpy(buf, "<s-delete>");   break;
+		case K(KEY_SIC):       strcpy(buf, "<s-insert>");   break;
+		case K(KEY_SPREVIOUS): strcpy(buf, "<s-pageup>");   break;
+		case K(KEY_SNEXT):     strcpy(buf, "<s-pagedown>"); break;
+		case WC_C_SPACE:       strcpy(buf, "<c-@>");        break;
 
 		case L'\b':
 			if(!bs)
@@ -675,8 +819,16 @@ wchar_to_spec(const wchar_t c[], size_t *len, int bs)
 			}
 			if(c[1] != L'\0' && c[1] != L'\033')
 			{
-				strcpy(buf, "<m-a>");
-				buf[3] += c[1] - L'a';
+				if(isupper(c[1]))
+				{
+					strcpy(buf, "<a-s-a>");
+					buf[5] += c[1] - L'A';
+				}
+				else
+				{
+					strcpy(buf, "<a-a>");
+					buf[3] += c[1] - L'a';
+				}
 				++*len;
 				break;
 			}
