@@ -23,13 +23,15 @@
 #include <assert.h> /* assert() */
 #include <stddef.h> /* NULL size_t */
 #include <stdlib.h> /* free() */
-#include <string.h> /* strcmp() strdup() strpbrk() */
+#include <string.h> /* strcmp() strdup() */
 
 #include "compat/fs_limits.h"
 #include "compat/os.h"
 #include "engine/functions.h"
 #include "engine/text_buffer.h"
 #include "engine/var.h"
+#include "lua/vlua.h"
+#include "modes/cmdline.h"
 #include "ui/cancellation.h"
 #include "ui/tabs.h"
 #include "ui/ui.h"
@@ -43,6 +45,7 @@
 #include "utils/test_helpers.h"
 #include "utils/trie.h"
 #include "utils/utils.h"
+#include "event_loop.h"
 #include "filelist.h"
 #include "macros.h"
 #include "types.h"
@@ -55,17 +58,29 @@ typedef struct
 }
 extcache_t;
 
+/* Data passed to prompt callback in input(). */
+typedef struct
+{
+	int quit;       /* Exit flag for event loop. */
+	char *response; /* Result of the prompt. */
+}
+input_cb_data_t;
+
 static var_t chooseopt_builtin(const call_info_t *call_info);
 static var_t executable_builtin(const call_info_t *call_info);
 static var_t expand_builtin(const call_info_t *call_info);
 static var_t extcached_builtin(const call_info_t *call_info);
 TSTATIC void set_extcached_monitor_type(FileMonType type);
+static var_t filereadable_builtin(const call_info_t *call_info);
 static var_t filetype_builtin(const call_info_t *call_info);
 static int get_fnum(var_t fnum);
 static const char * type_of_link_target(const dir_entry_t *entry);
 static var_t fnameescape_builtin(const call_info_t *call_info);
 static var_t getpanetype_builtin(const call_info_t *call_info);
 static var_t has_builtin(const call_info_t *call_info);
+static var_t input_builtin(const call_info_t *call_info);
+static complete_cmd_func pick_completer(const char type[], int *success);
+static void input_builtin_cb(const char response[], void *arg);
 static var_t layoutis_builtin(const call_info_t *call_info);
 static var_t paneisat_builtin(const call_info_t *call_info);
 static var_t system_builtin(const call_info_t *call_info);
@@ -75,20 +90,22 @@ static var_t execute_cmd(var_t cmd_arg, int interactive, int preserve_stdin);
 
 /* Function descriptions. */
 static const function_t functions[] = {
-	/* Name          Description                    Args   Handler  */
-	{ "chooseopt",   "query choose options",       {1,1}, &chooseopt_builtin },
-	{ "executable",  "check for executable file",  {1,1}, &executable_builtin },
-	{ "expand",      "expand macros in a string",  {1,1}, &expand_builtin },
-	{ "extcached",   "caches result of a command", {3,3}, &extcached_builtin },
-	{ "filetype",    "retrieve type of a file",    {1,2}, &filetype_builtin },
-	{ "fnameescape", "escapes string for a :cmd",  {1,1}, &fnameescape_builtin },
-	{ "getpanetype", "retrieve type of file list", {0,0}, &getpanetype_builtin},
-	{ "has",         "check for specific ability", {1,1}, &has_builtin },
-	{ "layoutis",    "query current layout",       {1,1}, &layoutis_builtin },
-	{ "paneisat",    "query pane location",        {1,1}, &paneisat_builtin },
-	{ "system",      "execute external command",   {1,1}, &system_builtin },
-	{ "tabpagenr",   "number of current/last tab", {0,1}, &tabpagenr_builtin },
-	{ "term",        "run interactive command",    {1,1}, &term_builtin },
+	/* Name           Description                   Args   Handler  */
+	{ "chooseopt",    "query choose options",      {1,1}, &chooseopt_builtin },
+	{ "executable",   "check for executable file", {1,1}, &executable_builtin },
+	{ "expand",       "expand macros in a string", {1,1}, &expand_builtin },
+	{ "extcached",    "caches result of a command",{3,3}, &extcached_builtin },
+	{ "filereadable", "checks for a readable file",{1,1}, &filereadable_builtin },
+	{ "filetype",     "retrieve type of a file",   {1,2}, &filetype_builtin },
+	{ "fnameescape",  "escapes string for a :cmd", {1,1}, &fnameescape_builtin },
+	{ "getpanetype",  "retrieve type of file list",{0,0}, &getpanetype_builtin},
+	{ "has",          "check for specific ability",{1,1}, &has_builtin },
+	{ "input",        "prompt user for input",     {1,3}, &input_builtin },
+	{ "layoutis",     "query current layout",      {1,1}, &layoutis_builtin },
+	{ "paneisat",     "query pane location",       {1,1}, &paneisat_builtin },
+	{ "system",       "execute external command",  {1,1}, &system_builtin },
+	{ "tabpagenr",    "number of current/last tab",{0,1}, &tabpagenr_builtin },
+	{ "term",         "run interactive command",   {1,1}, &term_builtin },
 };
 
 /* Kind of monitor used by the extcached(). */
@@ -147,7 +164,7 @@ executable_builtin(const call_info_t *call_info)
 
 	str_val = var_to_str(call_info->argv[0]);
 
-	if(strpbrk(str_val, PATH_SEPARATORS) != NULL)
+	if(contains_slash(str_val))
 	{
 		exists = executable_exists(str_val);
 	}
@@ -165,19 +182,17 @@ executable_builtin(const call_info_t *call_info)
 static var_t
 expand_builtin(const call_info_t *call_info)
 {
-	var_t result;
-	char *result_str;
-	char *str_val;
-	char *env_expanded_str_val;
+	char *arg = var_to_str(call_info->argv[0]);
 
-	str_val = var_to_str(call_info->argv[0]);
-	env_expanded_str_val = expand_envvars(str_val, 0);
-	result_str = ma_expand(env_expanded_str_val, NULL, NULL, 0);
-	free(env_expanded_str_val);
-	free(str_val);
+	char *env_expanded = expand_envvars(arg,
+			EEF_KEEP_ESCAPES | EEF_DOUBLE_PERCENTS);
+	free(arg);
 
-	result = var_from_str(result_str);
-	free(result_str);
+	char *macro_expanded = ma_expand(env_expanded, NULL, NULL, MER_DISPLAY);
+	free(env_expanded);
+
+	var_t result = var_from_str(macro_expanded);
+	free(macro_expanded);
 
 	return result;
 }
@@ -190,7 +205,7 @@ extcached_builtin(const call_info_t *call_info)
 	static trie_t *cache;
 	if(cache == NULL)
 	{
-		cache = trie_create();
+		cache = trie_create(&free);
 	}
 
 	char *cache_name = var_to_str(call_info->argv[0]);
@@ -280,6 +295,23 @@ set_extcached_monitor_type(FileMonType type)
 	extcached_mon_type = type;
 }
 
+/* Checks whether path is a non-directory entry and its permissions allow
+ * reading.  Returns boolean value describing result of the check. */
+static var_t
+filereadable_builtin(const call_info_t *call_info)
+{
+	int is_readable = 0;
+
+	char *path = var_to_str(call_info->argv[0]);
+	if(path != NULL)
+	{
+		is_readable = (os_access(path, R_OK) == 0 && !is_dir(path));
+		free(path);
+	}
+
+	return var_from_bool(is_readable);
+}
+
 /* Gets string representation of file type.  Returns the string. */
 static var_t
 filetype_builtin(const call_info_t *call_info)
@@ -352,7 +384,7 @@ fnameescape_builtin(const call_info_t *call_info)
 	var_t result;
 
 	char *const str_val = var_to_str(call_info->argv[0]);
-	char *const escaped = shell_like_escape(str_val, 1);
+	char *const escaped = posix_like_escape(str_val, /*type=*/1);
 	free(str_val);
 
 	result = var_from_str(escaped);
@@ -369,23 +401,91 @@ getpanetype_builtin(const call_info_t *call_info)
 		return var_from_str("regular");
 	}
 
-	switch(curr_view->custom.type)
+	return var_from_str(cv_describe(curr_view->custom.type));
+}
+
+/* Asks user for input and returns the result as a string. */
+static var_t
+input_builtin(const call_info_t *call_info)
+{
+	char *prompt = var_to_str(call_info->argv[0]);
+	char *initial = (call_info->argc > 1) ? var_to_str(call_info->argv[1])
+	                                      : strdup("");
+	char *completion = (call_info->argc > 2) ? var_to_str(call_info->argv[2])
+	                                         : strdup("");
+	if(prompt == NULL || initial == NULL || completion == NULL)
 	{
-		case CV_REGULAR:
-			return var_from_str("custom");
-
-		case CV_VERY:
-			return var_from_str("very-custom");
-
-		case CV_CUSTOM_TREE:
-		case CV_TREE:
-			return var_from_str("tree");
-
-		case CV_DIFF:
-		case CV_COMPARE:
-			return var_from_str("compare");
+		goto fail;
 	}
-	return var_from_str("UNKNOWN");
+
+	input_cb_data_t cb_data = { .quit = 0, .response = NULL };
+
+	int correct_completer;
+	complete_cmd_func complete = pick_completer(completion, &correct_completer);
+	if(!correct_completer)
+	{
+		vle_tb_append_linef(vle_err, "Invalid completion type: %s", completion);
+		goto fail;
+	}
+
+	modcline_prompt(prompt, initial, &input_builtin_cb, &cb_data, complete,
+			/*allow_ee=*/1);
+
+	free(prompt);
+	free(initial);
+	free(completion);
+
+	event_loop(&cb_data.quit, /*manage_marking=*/0);
+
+	/* Not returning var_error() on cancellation to allow handling of it by the
+	 * user. */
+	var_t result = var_from_str(cb_data.response == NULL ? "" : cb_data.response);
+
+	free(cb_data.response);
+
+	return result;
+
+fail:
+	free(prompt);
+	free(initial);
+	free(completion);
+	return var_error();
+}
+
+/* Callback invoked after prompt has finished. */
+static void
+input_builtin_cb(const char response[], void *arg)
+{
+	input_cb_data_t *data = arg;
+
+	update_string(&data->response, response);
+	data->quit = 1;
+}
+
+/* Maps completer name onto a function that implements it.  Always sets *success
+ * to indicate whether parsing was successful (this tells apart empty string
+ * from an invalid one).  Returns function pointer or NULL. */
+static complete_cmd_func
+pick_completer(const char type[], int *success)
+{
+	*success = 1;
+
+	if(strcmp(type, "dir") == 0)
+	{
+		return &modcline_complete_dirs;
+	}
+	else if(strcmp(type, "file") == 0)
+	{
+		return &modcline_complete_files;
+	}
+	else if(strcmp(type, "") == 0)
+	{
+		/* No completion. */
+		return NULL;
+	}
+
+	*success = 0;
+	return NULL;
 }
 
 /* Checks current layout configuration.  Returns boolean value that reflects
@@ -427,26 +527,26 @@ layoutis_builtin(const call_info_t *call_info)
 static var_t
 has_builtin(const call_info_t *call_info)
 {
-	var_t result;
+	int result = 0;
 
 	char *const str_val = var_to_str(call_info->argv[0]);
 
 	if(strcmp(str_val, "unix") == 0)
 	{
-		result = var_from_bool(get_env_type() == ET_UNIX);
+		result = (get_env_type() == ET_UNIX);
 	}
 	else if(strcmp(str_val, "win") == 0)
 	{
-		result = var_from_bool(get_env_type() == ET_WIN);
+		result = (get_env_type() == ET_WIN);
 	}
-	else
+	else if(str_val[0] == '#')
 	{
-		result = var_false();
+		result = vlua_handler_present(curr_stats.vlua, str_val);
 	}
 
 	free(str_val);
 
-	return result;
+	return var_from_bool(result);
 }
 
 /* Checks for relative position of current pane.  Returns boolean value that
