@@ -94,12 +94,12 @@ static int try_exe_directory_for_conf(void);
 static int try_home_envvar_for_conf(int force);
 static int try_appdata_for_conf(void);
 static int try_xdg_for_conf(void);
-static void find_data_dir(void);
+static void find_data_dir(char buf[], size_t buf_size);
 static void find_config_file(void);
 static int try_myvifmrc_envvar_for_vifmrc(void);
 static int try_exe_directory_for_vifmrc(void);
 static int try_vifm_vifmrc_for_vifmrc(void);
-static void store_config_paths(void);
+static void store_config_paths(const char data_dir[]);
 static void setup_dirs(void);
 static void copy_help_file(void);
 static void create_scripts_dir(void);
@@ -154,12 +154,15 @@ cfg_init(void)
 	cfg.tab_stop = 8;
 	cfg.ruler_format = strdup("%l/%S ");
 	cfg.status_line = strdup("");
+	cfg.tab_line = strdup("");
 
 	cfg.lines = INT_MIN;
 	cfg.columns = INT_MIN;
 
 	cfg.dot_dirs = DD_NONROOT_PARENT | DD_TREE_LEAFS_PARENT;
 
+	cfg.mouse = 0;
+
 	cfg.filter_inverted_by_default = 1;
 
 	cfg.apropos_prg = strdup("apropos %a");
@@ -170,6 +173,8 @@ cfg_init(void)
 	cfg.delete_prg = strdup("");
 	cfg.media_prg = format_str("%s/" SAMPLE_MEDIAPRG, get_installed_data_dir());
 
+	cfg.nav_open_files = 0;
+
 	cfg.tail_tab_line_paths = 0;
 	cfg.trunc_normal_sb_msgs = 0;
 	cfg.shorten_title_paths = 1;
@@ -180,6 +185,8 @@ cfg_init(void)
 	cfg.cd_path = strdup(env_get_def("CDPATH", DEFAULT_CD_PATH));
 	replace_char(cfg.cd_path, ':', ',');
 
+	cfg.auto_cd = 0;
+
 	cfg.ellipsis_position = 0;
 	cfg.extra_padding = 1;
 	cfg.side_borders_visible = 1;
@@ -187,7 +194,8 @@ cfg_init(void)
 	cfg.flexible_splitter = 1;
 	cfg.display_statusline = 1;
 
-	cfg.border_filler = strdup(" ");
+	cfg.vborder_filler = strdup(" ");
+	cfg.hborder_filler = strdup("");
 
 	cfg.set_title = term_title_restorable();
 
@@ -196,6 +204,7 @@ cfg_init(void)
 	cfg.graphics_delay = 50000;
 	cfg.hard_graphics_clear = 0;
 	cfg.top_tree_stats = 0;
+	cfg.max_tree_depth = 0;
 
 	cfg.timeout_len = 1000;
 	cfg.min_timeout_len = 150;
@@ -221,6 +230,8 @@ cfg_init(void)
 	cfg.name_dec_count = 0;
 
 	cfg.fast_file_cloning = 0;
+	cfg.data_sync = 1;
+
 	cfg.cvoptions = 0;
 
 	cfg.case_override = 0;
@@ -246,12 +257,14 @@ cfg_discover_paths(void)
 {
 	LOG_FUNC_ENTER;
 
+	char data_dir[PATH_MAX + 1];
+
 	find_home_dir();
 	find_config_dir();
-	find_data_dir();
+	find_data_dir(data_dir, sizeof(data_dir));
 	find_config_file();
 
-	store_config_paths();
+	store_config_paths(data_dir);
 
 	setup_dirs();
 }
@@ -394,6 +407,8 @@ try_home_envvar_for_conf(int force)
 	}
 
 	snprintf(vifm, sizeof(vifm), "%s/.vifm", home);
+	system_to_internal_slashes(vifm);
+
 	if(!force && !is_dir(vifm))
 	{
 		return 0;
@@ -450,27 +465,36 @@ try_xdg_for_conf(void)
 	env_set(VIFM_EV, config_dir);
 	free(config_dir);
 
+	/* If XDG is used for configuration directory, create correcponding data
+	 * directory, so it's also used at the same time. */
+	char data_dir[PATH_MAX + 1];
+	find_data_dir(data_dir, sizeof(data_dir));
+	(void)create_path(data_dir, S_IRWXU);
+
 	return 1;
 }
 
-/* Tries to find directory for data files. */
+/* Tries to find XDG directory for storing data files. */
 static void
-find_data_dir(void)
+find_data_dir(char buf[], size_t buf_size)
 {
 	LOG_FUNC_ENTER;
 
+	assert(buf_size >= 4 && "Buffer for find_data_dir() is too small!");
+
 	const char *const data_home = env_get("XDG_DATA_HOME");
 	if(is_null_or_empty(data_home) || !is_path_absolute(data_home))
 	{
-		snprintf(cfg.data_dir, sizeof(cfg.data_dir) - 4, "%s/.local/share/",
-				env_get(HOME_EV));
+		snprintf(buf, buf_size - 4, "%s/.local/share/", env_get(HOME_EV));
 	}
 	else
 	{
-		snprintf(cfg.data_dir, sizeof(cfg.data_dir) - 4, "%s/", data_home);
+		snprintf(buf, buf_size - 4, "%s/", data_home);
 	}
 
-	strcat(cfg.data_dir, "vifm");
+	strcat(buf, "vifm");
+
+	system_to_internal_slashes(buf);
 }
 
 /* Tries to find configuration file. */
@@ -540,10 +564,15 @@ try_vifm_vifmrc_for_vifmrc(void)
 
 /* Writes path configuration file and directories for further usage. */
 static void
-store_config_paths(void)
+store_config_paths(const char data_dir[])
 {
 	LOG_FUNC_ENTER;
 
+	snprintf(cfg.home_dir, sizeof(cfg.home_dir), "%s/", env_get(HOME_EV));
+	copy_str(cfg.config_dir, sizeof(cfg.config_dir), env_get(VIFM_EV));
+	snprintf(cfg.colors_dir, sizeof(cfg.colors_dir), "%s/colors/",
+			cfg.config_dir);
+
 	const char *const trash_dir_fmt =
 #ifndef _WIN32
 			"%%r/.vifm-Trash-%%u,%s/" TRASH ",%%r/.vifm-Trash";
@@ -551,22 +580,29 @@ store_config_paths(void)
 			"%%r/.vifm-Trash,%s/" TRASH;
 #endif
 
-	char *fuse_home;
-	const char *trash_base = path_exists_at(env_get(VIFM_EV), TRASH, DEREF)
-	                       ? cfg.config_dir
-	                       : cfg.data_dir;
-	const char *base = path_exists(cfg.data_dir, DEREF)
-	                 ? cfg.data_dir
-	                 : cfg.config_dir;
+	/* Determine where to store data giving preference to directories that already
+	 * exist. */
+	const char *base;
+	if(path_exists_at(data_dir, TRASH, DEREF))
+	{
+		base = data_dir;
+	}
+	else if(path_exists_at(cfg.config_dir, TRASH, DEREF))
+	{
+		base = cfg.config_dir;
+	}
+	else
+	{
+		base = (is_dir(data_dir) ? data_dir : cfg.config_dir);
+	}
 
-	snprintf(cfg.home_dir, sizeof(cfg.home_dir), "%s/", env_get(HOME_EV));
-	copy_str(cfg.config_dir, sizeof(cfg.config_dir), env_get(VIFM_EV));
-	snprintf(cfg.colors_dir, sizeof(cfg.colors_dir), "%s/colors/",
-			cfg.config_dir);
+	char *trash_base = escape_chars(base, "$");
 	snprintf(cfg.trash_dir, sizeof(cfg.trash_dir), trash_dir_fmt, trash_base);
+	free(trash_base);
+
 	snprintf(cfg.log_file, sizeof(cfg.log_file), "%s/" LOG, base);
 
-	fuse_home = format_str("%s/fuse/", base);
+	char *fuse_home = format_str("%s/fuse/", base);
 	(void)cfg_set_fuse_home(fuse_home);
 	free(fuse_home);
 }
@@ -702,11 +738,18 @@ cfg_load(void)
 	 * views. */
 	curr_stats.global_local_settings = 1;
 
-	if(!vifm_testing())
+	/* Try to load global configuration(s). */
+	int i = 0;
+	while(!vifm_testing())
 	{
-		/* Try to load global configuration. */
+		const char *conf_dir = get_sys_conf_dir(i++);
+		if(conf_dir == NULL)
+		{
+			break;
+		}
+
 		char rc_path[PATH_MAX + 1];
-		snprintf(rc_path, sizeof(rc_path), "%s/%s", get_sys_conf_dir(), VIFMRC);
+		build_path(rc_path, sizeof(rc_path), conf_dir, VIFMRC);
 		(void)cfg_source_file(rc_path);
 	}
 
@@ -756,7 +799,7 @@ source_file_internal(strlist_t lines, const char filename[])
 		return 0;
 	}
 
-	commands_scope_start();
+	cmds_scope_start();
 
 	int encoutered_errors = 0;
 
@@ -788,7 +831,7 @@ source_file_internal(strlist_t lines, const char filename[])
 
 		ui_sb_clear();
 
-		if(exec_commands(line, curr_view, CIT_COMMAND) < 0)
+		if(cmds_dispatch(line, curr_view, CIT_COMMAND) < 0)
 		{
 			show_sourcing_error(filename, line_num);
 			encoutered_errors = 1;
@@ -810,7 +853,7 @@ source_file_internal(strlist_t lines, const char filename[])
 
 	ui_sb_clear();
 
-	if(commands_scope_finish() != 0)
+	if(cmds_scope_finish() != 0)
 	{
 		show_sourcing_error(filename, line_num);
 		encoutered_errors = 1;
