@@ -163,6 +163,10 @@
  *      text = "item1"
  *      ts = 1440801895 # timestamp (optional)
  *  } ]
+ *  exprreg-hist = [ {
+ *      text = "item1"
+ *      ts = 1440801895 # timestamp (optional)
+ *  } ]
  *  active-gtab = 0
  *  use-term-multiplexer = true
  *  color-scheme = "almost-default"
@@ -448,7 +452,7 @@ read_legacy_info_file(const char info_file[])
 			{
 				if((line3 = read_vifminfo_line(fp, line3)) != NULL)
 				{
-					int timestamp = read_optional_number(fp);
+					long long int timestamp = read_optional_number(fp);
 					if(timestamp == -1)
 					{
 						timestamp = time(NULL);
@@ -673,6 +677,7 @@ load_state(JSON_Object *root, int reread)
 	load_dir_stack(root);
 	load_trash(root);
 	load_history(root, "cmd-hist", &curr_stats.cmd_hist);
+	load_history(root, "exprreg-hist", &curr_stats.exprreg_hist);
 	load_history(root, "search-hist", &curr_stats.search_hist);
 	load_history(root, "prompt-hist", &curr_stats.prompt_hist);
 	load_history(root, "lfilt-hist", &curr_stats.filter_hist);
@@ -909,7 +914,8 @@ load_filters(JSON_Object *pane, view_t *view)
 		return;
 	}
 
-	get_bool(filters, "invert", &view->invert);
+	/* Nothing to do if there is no "invert" key. */
+	(void)get_bool(filters, "invert", &view->invert);
 
 	int dot;
 	if(get_bool(filters, "dot", &dot))
@@ -1027,7 +1033,7 @@ load_cmds(JSON_Object *root)
 			char *cmdadd_cmd = format_str("command %s %s", name, cmd);
 			if(cmdadd_cmd != NULL)
 			{
-				exec_commands(cmdadd_cmd, curr_view, CIT_COMMAND);
+				cmds_dispatch(cmdadd_cmd, curr_view, CIT_COMMAND);
 				free(cmdadd_cmd);
 			}
 		}
@@ -1282,7 +1288,7 @@ copy_file(const char src[], const char dst[])
 		.arg3.crs = IO_CRS_REPLACE_FILES,
 	};
 
-	return iop_cp(&args);
+	return (iop_cp(&args) == IO_RES_SUCCEEDED ? 0 : 1);
 }
 
 /* Reads contents of the filename file as a JSON info file and updates it with
@@ -1403,6 +1409,11 @@ serialize_state(int vinfo)
 		store_history(root, "cmd-hist", &curr_stats.cmd_hist);
 	}
 
+	if(vinfo & VINFO_EHISTORY)
+	{
+		store_history(root, "exprreg-hist", &curr_stats.exprreg_hist);
+	}
+
 	if(vinfo & VINFO_SHISTORY)
 	{
 		store_history(root, "search-hist", &curr_stats.search_hist);
@@ -1476,6 +1487,11 @@ merge_states(int vinfo, int session_load, JSON_Object *current,
 		merge_history(session_load, current, admixture, "cmd-hist");
 	}
 
+	if(vinfo & VINFO_EHISTORY)
+	{
+		merge_history(session_load, current, admixture, "exprreg-hist");
+	}
+
 	if(vinfo & VINFO_SHISTORY)
 	{
 		merge_history(session_load, current, admixture, "search-hist");
@@ -1615,13 +1631,13 @@ merge_dhistory(int session_load, JSON_Object *current,
 	if(total > 0)
 	{
 		const char *last_dir = "";
-		get_str(combined[0], "dir", &last_dir);
+		(void)get_str(combined[0], "dir", &last_dir);
 
 		int i, j = 0;
 		for(i = 1; i < total; ++i)
 		{
 			const char *curr_dir = "";
-			get_str(combined[i], "dir", &curr_dir);
+			(void)get_str(combined[i], "dir", &curr_dir);
 			if(stroscmp(last_dir, curr_dir) != 0)
 			{
 				++j;
@@ -1881,7 +1897,7 @@ merge_history(int session_load, JSON_Object *current,
 		return;
 	}
 
-	trie_t *trie = trie_create();
+	trie_t *trie = trie_create(/*free_func=*/NULL);
 	int i, n;
 
 	JSON_Value *combined_value = json_value_init_array();
@@ -1948,7 +1964,7 @@ merge_history_by_order(JSON_Object *current, const JSON_Object *admixture,
 	JSON_Array *updated = json_object_get_array(admixture, node);
 
 	int i, n;
-	trie_t *trie = trie_create();
+	trie_t *trie = trie_create(/*free_func=*/NULL);
 
 	JSON_Value *merged_value = json_value_init_array();
 	JSON_Array *merged = json_array(merged_value);
@@ -2202,6 +2218,7 @@ store_global_options(JSON_Object *root)
 				escape_spaces(cfg.apropos_prg)));
 	append_dstr(options, format_str("%sautochpos", cfg.auto_ch_pos ? "" : "no"));
 	append_dstr(options, format_str("cdpath=%s", cfg.cd_path));
+	append_dstr(options, format_str("%sautocd", cfg.auto_cd ? "" : "no"));
 	append_dstr(options, format_str("%schaselinks", cfg.chase_links ? "" : "no"));
 	append_dstr(options, format_str("columns=%d", cfg.columns));
 	append_dstr(options, format_str("cpoptions=%s",
@@ -2209,11 +2226,9 @@ store_global_options(JSON_Object *root)
 	append_dstr(options, format_str("deleteprg=%s",
 				escape_spaces(cfg.delete_prg)));
 	append_dstr(options, format_str("%sfastrun", cfg.fast_run ? "" : "no"));
-	if(strcmp(cfg.border_filler, " ") != 0)
-	{
-		append_dstr(options, format_str("fillchars+=vborder:%s",
-					cfg.border_filler));
-	}
+	append_dstr(options, format_str("fillchars+=vborder:%s,hborder:%s",
+				escape_spaces(cfg.vborder_filler),
+				escape_spaces(cfg.hborder_filler)));
 	append_dstr(options, format_str("findprg=%s", escape_spaces(cfg.find_prg)));
 	append_dstr(options, format_str("%sfollowlinks",
 				cfg.follow_links ? "" : "no"));
@@ -2242,6 +2257,8 @@ store_global_options(JSON_Object *root)
 	append_dstr(options, format_str("rulerformat=%s",
 				escape_spaces(cfg.ruler_format)));
 	append_dstr(options, format_str("%srunexec", cfg.auto_execute ? "" : "no"));
+	append_dstr(options, format_str("navoptions=%s",
+				escape_spaces(vle_opts_get("navoptions", OPT_GLOBAL))));
 	append_dstr(options, format_str("previewoptions=%s",
 				escape_spaces(vle_opts_get("previewoptions", OPT_GLOBAL))));
 	append_dstr(options, format_str("%sscrollbind", cfg.scroll_bind ? "" : "no"));
@@ -2264,6 +2281,8 @@ store_global_options(JSON_Object *root)
 				cfg.sort_numbers ? "" : "no"));
 	append_dstr(options, format_str("statusline=%s",
 				escape_spaces(cfg.status_line)));
+	append_dstr(options, format_str("tabline=%s",
+				escape_spaces(vle_opts_get("tabline", OPT_GLOBAL))));
 	append_dstr(options, format_str("syncregs=%s",
 			escape_spaces(vle_opts_get("syncregs", OPT_GLOBAL))));
 	append_dstr(options, format_str("tablabel=%s",
@@ -2292,6 +2311,8 @@ store_global_options(JSON_Object *root)
 			escape_spaces(vle_opts_get("confirm", OPT_GLOBAL))));
 	append_dstr(options, format_str("dotdirs=%s",
 			escape_spaces(vle_opts_get("dotdirs", OPT_GLOBAL))));
+	append_dstr(options, format_str("mouse=%s",
+			escape_spaces(vle_opts_get("mouse", OPT_GLOBAL))));
 	append_dstr(options, format_str("caseoptions=%s",
 			escape_spaces(vle_opts_get("caseoptions", OPT_GLOBAL))));
 	append_dstr(options, format_str("suggestoptions=%s",
