@@ -101,8 +101,6 @@ static int swap_range(void);
 static int resolve_mark(char mark);
 static char * cmds_expand_macros(const char str[], int for_shell, int *usr1,
 		int *usr2);
-static int setup_extcmd_file(const char path[], const char beginning[],
-		CmdInputType type);
 static void prepare_extcmd_file(FILE *fp, const char beginning[],
 		CmdInputType type);
 static hist_t * history_by_type(CmdInputType type);
@@ -115,6 +113,7 @@ static int skip_at_beginning(int id, const char args[]);
 static void create_builtin_vars(void);
 static void session_changed_callback(const char new_session[]);
 static char * pattern_expand_hook(const char pattern[]);
+static int is_implicit_cd(view_t *view, const char cmd[], int cmd_error);
 static int cmd_should_be_processed(int cmd_id);
 TSTATIC char ** break_cmdline(const char cmdline[], int for_menu);
 static int is_out_of_arg(const char cmd[], const char pos[]);
@@ -124,10 +123,10 @@ static CmdArgsType get_cmd_args_type(const char cmd[]);
 static char * skip_to_cmd_name(const char cmd[]);
 static int repeat_command(view_t *view, CmdInputType type);
 static int is_at_scope_bottom(const int_stack_t *scope_stack);
-TSTATIC char * eval_arglist(const char args[], const char **stop_ptr);
 
 /* Settings for the cmds unit. */
 static cmds_conf_t cmds_conf = {
+	.complete_line = &complete_line,
 	.complete_args = &complete_args,
 	.swap_range = &swap_range,
 	.resolve_mark = &resolve_mark,
@@ -170,7 +169,7 @@ cmds_expand_macros(const char str[], int for_shell, int *usr1, int *usr2)
 	char *result;
 	MacroFlags flags = MF_NONE;
 
-	result = ma_expand(str, NULL, &flags, for_shell);
+	result = ma_expand(str, NULL, &flags, for_shell ? MER_SHELL_OP : MER_OP);
 
 	*usr1 = flags;
 
@@ -180,13 +179,13 @@ cmds_expand_macros(const char str[], int for_shell, int *usr1, int *usr2)
 char *
 cmds_expand_envvars(const char str[])
 {
-	return expand_envvars(str, 1);
+	return expand_envvars(str, EEF_ESCAPE_VALS | EEF_KEEP_ESCAPES);
 }
 
 void
-get_and_execute_command(const char line[], size_t line_pos, CmdInputType type)
+cmds_run_ext(const char line[], size_t line_pos, CmdInputType type)
 {
-	char *const cmd = get_ext_command(line, line_pos, type);
+	char *const cmd = cmds_get_ext(line, line_pos, type);
 	if(cmd == NULL)
 	{
 		save_extcmd(line, type);
@@ -200,45 +199,31 @@ get_and_execute_command(const char line[], size_t line_pos, CmdInputType type)
 }
 
 char *
-get_ext_command(const char beginning[], size_t line_pos, CmdInputType type)
+cmds_get_ext(const char beginning[], size_t line_pos, CmdInputType type)
 {
 	char cmd_file[PATH_MAX + 1];
-	char *cmd = NULL;
-
-	generate_tmp_file_name("vifm.cmdline", cmd_file, sizeof(cmd_file));
-
-	if(setup_extcmd_file(cmd_file, beginning, type) == 0)
+	FILE *file = make_file_in_tmp("vifm.cmdline", 0600, /*auto_delete=*/0,
+			cmd_file, sizeof(cmd_file));
+	if(file == NULL)
 	{
-		if(vim_view_file(cmd_file, 1, line_pos, 0) == 0)
-		{
-			cmd = get_file_first_line(cmd_file);
-		}
+		show_error_msgf("External Editing", "Failed to create a temporary file: %s",
+				strerror(errno));
+		return NULL;
 	}
-	else
+
+	prepare_extcmd_file(file, beginning, type);
+	fclose(file);
+
+	char *cmd = NULL;
+	if(vim_view_file(cmd_file, 1, line_pos, 0) == 0)
 	{
-		show_error_msgf("Error Creating Temporary File",
-				"Could not create file %s: %s", cmd_file, strerror(errno));
+		cmd = get_file_first_line(cmd_file);
 	}
 
 	unlink(cmd_file);
 	return cmd;
 }
 
-/* Create and fill file for external command prompt.  Returns zero on success,
- * otherwise non-zero is returned and errno contains valid value. */
-static int
-setup_extcmd_file(const char path[], const char beginning[], CmdInputType type)
-{
-	FILE *const fp = os_fopen(path, "wt");
-	if(fp == NULL)
-	{
-		return 1;
-	}
-	prepare_extcmd_file(fp, beginning, type);
-	fclose(fp);
-	return 0;
-}
-
 /* Fills the file with history (more recent goes first). */
 static void
 prepare_extcmd_file(FILE *fp, const char beginning[], CmdInputType type)
@@ -273,6 +258,8 @@ history_by_type(CmdInputType type)
 			return &curr_stats.cmd_hist;
 		case CIT_PROMPT_INPUT:
 			return &curr_stats.prompt_hist;
+		case CIT_EXPRREG_INPUT:
+			return &curr_stats.exprreg_hist;
 		case CIT_FILTER_PATTERN:
 			return &curr_stats.filter_hist;
 
@@ -303,16 +290,16 @@ execute_extcmd(const char command[], CmdInputType type)
 {
 	if(type == CIT_COMMAND)
 	{
-		commands_scope_start();
-		curr_stats.save_msg = exec_commands(command, curr_view, type);
-		if(commands_scope_finish() != 0)
+		cmds_scope_start();
+		curr_stats.save_msg = cmds_dispatch(command, curr_view, type);
+		if(cmds_scope_finish() != 0)
 		{
 			curr_stats.save_msg = 1;
 		}
 	}
 	else
 	{
-		curr_stats.save_msg = exec_command(command, curr_view, type);
+		curr_stats.save_msg = cmds_dispatch1(command, curr_view, type);
 	}
 }
 
@@ -331,14 +318,14 @@ save_extcmd(const char command[], CmdInputType type)
 }
 
 int
-is_history_command(const char command[])
+cmds_goes_to_history(const char command[])
 {
 	/* Don't add :!! or :! to history list. */
 	return strcmp(command, "!!") != 0 && strcmp(command, "!") != 0;
 }
 
 int
-command_accepts_expr(int cmd_id)
+cmds_has_expr_args(int cmd_id)
 {
 	return cmd_id == COM_ECHO
 	    || cmd_id == COM_EXE
@@ -348,16 +335,16 @@ command_accepts_expr(int cmd_id)
 }
 
 char *
-commands_escape_for_insertion(const char cmd_line[], int pos, const char str[])
+cmds_insertion_escape(const char cmd_line[], int pos, const char str[])
 {
-	const CmdLineLocation ipt = get_cmdline_location(cmd_line, cmd_line + pos);
+	const CmdLineLocation ipt = cmds_classify_pos(cmd_line, cmd_line + pos);
 	switch(ipt)
 	{
 		case CLL_R_QUOTING:
 			/* XXX: Use of filename escape, while special one might be needed. */
 		case CLL_OUT_OF_ARG:
 		case CLL_NO_QUOTING:
-			return shell_like_escape(str, 0);
+			return posix_like_escape(str, /*type=*/0);
 
 		case CLL_S_QUOTING:
 			return escape_for_squotes(str, 0);
@@ -373,7 +360,9 @@ commands_escape_for_insertion(const char cmd_line[], int pos, const char str[])
 static void
 post(int id)
 {
-	if(!keep_view_selection)
+	/* When one command in a sequence enters visual mode, we don't want to break
+	 * it by resetting selection. */
+	if(!keep_view_selection && vle_mode_is(NORMAL_MODE))
 	{
 		flist_sel_stash_if_nonempty(curr_view);
 	}
@@ -402,7 +391,7 @@ cmds_select_range(int id, const cmd_info_t *cmd_info)
 static int
 skip_at_beginning(int id, const char args[])
 {
-	if(id == COM_WINDO)
+	if(id == COM_KEEPSEL || id == COM_WINDO)
 	{
 		return 0;
 	}
@@ -419,7 +408,7 @@ skip_at_beginning(int id, const char args[])
 }
 
 void
-init_commands(void)
+cmds_init(void)
 {
 	if(cmds_conf.inner != NULL)
 	{
@@ -427,7 +416,7 @@ init_commands(void)
 		return;
 	}
 
-	/* We get here when init_commands() is called the first time. */
+	/* We get here when cmds_init() is called the first time. */
 
 	vle_cmds_init(1, &cmds_conf);
 	vle_cmds_add(cmds_list, cmds_list_size);
@@ -488,31 +477,30 @@ static char *
 pattern_expand_hook(const char pattern[])
 {
 	char *const no_tilde = expand_tilde(pattern);
-	char *const expanded_pattern = expand_envvars(no_tilde, 0);
+	char *const expanded_pattern = expand_envvars(no_tilde, EEF_NONE);
 	free(no_tilde);
 
 	return expanded_pattern;
 }
 
-/* Returns negative value in case of error */
-static int
-execute_command(view_t *view, const char command[], int menu)
+int
+cmds_exec(const char cmd[], view_t *view, int menu, int keep_sel)
 {
 	int id;
 	int result;
 
-	if(command == NULL)
+	if(cmd == NULL)
 	{
 		flist_sel_stash_if_nonempty(view);
 		return 0;
 	}
 
-	command = skip_to_cmd_name(command);
+	cmd = skip_to_cmd_name(cmd);
 
-	if(command[0] == '"')
+	if(cmd[0] == '"')
 		return 0;
 
-	if(command[0] == '\0' && !menu)
+	if(cmd[0] == '\0' && !menu)
 	{
 		flist_sel_stash_if_nonempty(view);
 		return 0;
@@ -526,7 +514,7 @@ execute_command(view_t *view, const char command[], int menu)
 		cmds_conf.end = view->list_rows - 1;
 	}
 
-	id = vle_cmds_identify(command);
+	id = vle_cmds_identify(cmd);
 
 	if(!cmd_should_be_processed(id))
 	{
@@ -538,18 +526,23 @@ execute_command(view_t *view, const char command[], int menu)
 		char undo_msg[COMMAND_GROUP_INFO_LEN];
 
 		snprintf(undo_msg, sizeof(undo_msg), "in %s: %s",
-				replace_home_part(flist_get_dir(view)), command);
+				replace_home_part(flist_get_dir(view)), cmd);
 
 		un_group_open(undo_msg);
 		un_group_close();
 	}
 
-	keep_view_selection = 0;
-	result = vle_cmds_run(command);
+	keep_view_selection = keep_sel;
+	result = vle_cmds_run(cmd);
 
 	if(result >= 0)
 		return result;
 
+	if(is_implicit_cd(view, cmd, result))
+	{
+		return cd(view, flist_get_dir(view), cmd);
+	}
+
 	switch(result)
 	{
 		case CMDS_ERR_LOOP:
@@ -613,6 +606,27 @@ execute_command(view_t *view, const char command[], int menu)
 	return -1;
 }
 
+/* Whether need to process command-line command as implicit :cd.  Returns
+ * non-zero if so, otherwise zero is returned. */
+static int
+is_implicit_cd(view_t *view, const char cmd[], int cmd_error)
+{
+	if(!cfg.auto_cd)
+	{
+		return 0;
+	}
+
+	if(cmd_error != CMDS_ERR_INVALID_CMD && cmd_error != CMDS_ERR_TRAILING_CHARS)
+	{
+		return 0;
+	}
+
+	char *expanded = expand_tilde(cmd);
+	int exists = path_exists_at(flist_get_dir(view), expanded, NODEREF);
+	free(expanded);
+	return exists;
+}
+
 /* Decides whether next command with id cmd_id should be processed or not,
  * taking state of conditional statements into account.  Returns non-zero if the
  * command should be processed, otherwise zero is returned. */
@@ -809,7 +823,7 @@ line_pos(const char begin[], const char end[], char sep, int rquoting,
 }
 
 int
-exec_commands(const char cmdline[], view_t *view, CmdInputType type)
+cmds_dispatch(const char cmdline[], view_t *view, CmdInputType type)
 {
 	int save_msg = 0;
 	char **cmds = break_cmdline(cmdline, type == CIT_MENU_COMMAND);
@@ -817,7 +831,7 @@ exec_commands(const char cmdline[], view_t *view, CmdInputType type)
 
 	while(*cmd != NULL)
 	{
-		const int ret = exec_command(*cmd, view, type);
+		const int ret = cmds_dispatch1(*cmd, view, type);
 		if(ret != 0)
 		{
 			save_msg = (ret < 0) ? -1 : 1;
@@ -945,7 +959,7 @@ finish:
 static int
 is_out_of_arg(const char cmd[], const char pos[])
 {
-	const CmdLineLocation location = get_cmdline_location(cmd, pos);
+	const CmdLineLocation location = cmds_classify_pos(cmd, pos);
 
 	if(location == CLL_NO_QUOTING)
 	{
@@ -965,7 +979,7 @@ is_out_of_arg(const char cmd[], const char pos[])
 }
 
 CmdLineLocation
-get_cmdline_location(const char cmd[], const char pos[])
+cmds_classify_pos(const char cmd[], const char *pos)
 {
 	char separator;
 	int regex_quoting;
@@ -1025,6 +1039,8 @@ get_cmd_args_type(const char cmd[])
 	switch(cmd_id)
 	{
 		case COMMAND_CMD_ID:
+		case COM_AMAP:
+		case COM_ANOREMAP:
 		case COM_AUTOCMD:
 		case COM_EXECUTE:
 		case COM_CABBR:
@@ -1036,6 +1052,7 @@ get_cmd_args_type(const char cmd[])
 		case COM_FILETYPE:
 		case COM_FILEVIEWER:
 		case COM_FILEXTYPE:
+		case COM_KEEPSEL:
 		case COM_MAP:
 		case COM_MMAP:
 		case COM_MNOREMAP:
@@ -1053,12 +1070,12 @@ get_cmd_args_type(const char cmd[])
 			return CAT_UNTIL_THE_END;
 
 		default:
-			return command_accepts_expr(cmd_id) ? CAT_EXPR : CAT_REGULAR;
+			return cmds_has_expr_args(cmd_id) ? CAT_EXPR : CAT_REGULAR;
 	}
 }
 
 const char *
-find_last_command(const char cmds[])
+cmds_find_last(const char cmds[])
 {
 	const char *p, *q;
 
@@ -1116,10 +1133,11 @@ skip_to_cmd_name(const char cmd[])
 }
 
 int
-exec_command(const char cmd[], view_t *view, CmdInputType type)
+cmds_dispatch1(const char cmd[], view_t *view, CmdInputType type)
 {
 	int menu;
 	int backward;
+	int found;
 
 	if(cmd == NULL)
 	{
@@ -1132,11 +1150,11 @@ exec_command(const char cmd[], view_t *view, CmdInputType type)
 	{
 		case CIT_BSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_FSEARCH_PATTERN:
-			return modnorm_find(view, cmd, backward, 1);
+			return modnorm_find(view, cmd, backward, /*print_msg=*/1, &found);
 
 		case CIT_VBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VFSEARCH_PATTERN:
-			return modvis_find(view, cmd, backward, 1);
+			return modvis_find(view, cmd, backward, /*print_msg=*/1, &found);
 
 		case CIT_VWBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VWFSEARCH_PATTERN:
@@ -1144,7 +1162,7 @@ exec_command(const char cmd[], view_t *view, CmdInputType type)
 
 		case CIT_MENU_COMMAND: menu = 1; /* Fall through. */
 		case CIT_COMMAND:
-			return execute_command(view, cmd, menu);
+			return cmds_exec(cmd, view, menu, /*keep_sel=*/0);
 
 		case CIT_FILTER_PATTERN:
 			if(view->custom.type != CV_DIFF)
@@ -1170,22 +1188,25 @@ static int
 repeat_command(view_t *view, CmdInputType type)
 {
 	int backward = 0;
+	int found;
 	switch(type)
 	{
 		case CIT_BSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_FSEARCH_PATTERN:
-			return modnorm_find(view, hists_search_last(), backward, 1);
+			return modnorm_find(view, hists_search_last(), backward, /*print_msg=*/1,
+					&found);
 
 		case CIT_VBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VFSEARCH_PATTERN:
-			return modvis_find(view, hists_search_last(), backward, 1);
+			return modvis_find(view, hists_search_last(), backward, /*print_msg=*/1,
+					&found);
 
 		case CIT_VWBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VWFSEARCH_PATTERN:
 			return modview_find(NULL, backward);
 
 		case CIT_COMMAND:
-			return execute_command(view, NULL, 0);
+			return cmds_exec(/*cmd=*/NULL, view, /*menu=*/0, /*keep_sel=*/0);
 
 		case CIT_FILTER_PATTERN:
 			local_filter_apply(view, "");
@@ -1198,13 +1219,13 @@ repeat_command(view_t *view, CmdInputType type)
 }
 
 void
-commands_scope_start(void)
+cmds_scope_start(void)
 {
 	(void)int_stack_push(&if_levels, IF_SCOPE_GUARD);
 }
 
 void
-commands_scope_escape(void)
+cmds_scope_escape(void)
 {
 	while(!is_at_scope_bottom(&if_levels))
 	{
@@ -1213,7 +1234,7 @@ commands_scope_escape(void)
 }
 
 int
-commands_scope_finish(void)
+cmds_scope_finish(void)
 {
 	if(!is_at_scope_bottom(&if_levels))
 	{
@@ -1306,7 +1327,7 @@ is_at_scope_bottom(const int_stack_t *scope_stack)
 }
 
 char *
-eval_arglist(const char args[], const char **stop_ptr)
+cmds_eval_args(const char args[], const char **stop_ptr)
 {
 	size_t len = 0;
 	char *eval_result = NULL;
@@ -1318,30 +1339,28 @@ eval_arglist(const char args[], const char **stop_ptr)
 		char *free_this = NULL;
 		const char *tmp_result = NULL;
 
-		var_t result = var_false();
-		const ParsingErrors parsing_error = parse(args, 1, &result);
-		if(parsing_error == PE_INVALID_EXPRESSION && is_prev_token_whitespace())
+		parsing_result_t result = vle_parser_eval(args, /*interactive=*/1);
+		if(result.error == PE_INVALID_EXPRESSION && result.ends_with_whitespace)
 		{
-			result = get_parsing_result();
-			if(result.type != VTYPE_ERROR)
+			if(result.value.type != VTYPE_ERROR)
 			{
-				tmp_result = free_this = var_to_str(result);
-				args = get_last_parsed_char();
+				tmp_result = free_this = var_to_str(result.value);
+				args = result.last_parsed_char;
 			}
 		}
-		else if(parsing_error == PE_NO_ERROR)
+		else if(result.error == PE_NO_ERROR)
 		{
-			tmp_result = free_this = var_to_str(result);
-			args = get_last_position();
+			tmp_result = free_this = var_to_str(result.value);
+			args = result.last_position;
 		}
-		else if(parsing_error != PE_INVALID_EXPRESSION)
+		else if(result.error != PE_INVALID_EXPRESSION)
 		{
-			report_parsing_error(parsing_error);
+			vle_parser_report(&result);
 		}
 
 		if(tmp_result == NULL)
 		{
-			var_free(result);
+			var_free(result.value);
 			break;
 		}
 
@@ -1351,7 +1370,7 @@ eval_arglist(const char args[], const char **stop_ptr)
 		}
 		eval_result = extend_string(eval_result, tmp_result, &len);
 
-		var_free(result);
+		var_free(result.value);
 		free(free_this);
 
 		args = skip_whitespace(args);
