@@ -31,6 +31,8 @@
 enum
 {
 	COM_CDS = NO_COMPLETION_BOUNDARY,
+	COM_AMAP,
+	COM_ANOREMAP,
 	COM_CMAP,
 	COM_CNOREMAP,
 	COM_COMMAND,
