@@ -43,9 +43,28 @@
 #include "utils/utils.h"
 #include "filelist.h"
 #include "filtering.h"
+#include "flist_sel.h"
+#include "fops_common.h"
 #include "fops_cpmv.h"
 #include "fops_misc.h"
 #include "running.h"
+#include "undo.h"
+
+/*
+ * Optimization for content-based matching.
+ *
+ * At first fingerprint uses only size of the file, then it's looked up to see
+ * if there is any other file of the same size.  If there isn't, store compare
+ * record by that fingerprint.
+ *
+ * If there is, two cases are possible:
+ *   - there is only one conflicting file and its contents fingerprint wasn't
+ *     yet computed:
+ *       * compute contents fingerprint for that file and insert it
+ *       * compute contents fingerprint for current file and insert it
+ *   - there is more than one conflicting file:
+ *       * compute contents fingerprint for current file and insert it
+ */
 
 /* This is the only unit that uses xxhash, so import it directly here. */
 #define XXH_PRIVATE_API
@@ -62,37 +81,50 @@ typedef struct compare_record_t
 {
 	char *path;                    /* Full path to file with sample content. */
 	int id;                        /* Chosen id. */
-	struct compare_record_t *next; /* Next entry in the list. */
+	int is_partial;                /* Shows that fingerprinting was lazy. */
+	struct compare_record_t *next; /* Next entry in the list of conflicts. */
 }
 compare_record_t;
 
 static void make_unique_lists(entries_t curr, entries_t other);
 static void leave_only_dups(entries_t *curr, entries_t *other);
 static int is_not_duplicate(view_t *view, const dir_entry_t *entry, void *arg);
-static void fill_side_by_side(entries_t curr, entries_t other, int group_paths);
+static void fill_side_by_side_by_paths(entries_t curr, entries_t other,
+		int flags, compare_stats_t *stats);
+static void fill_side_by_side_by_ids(entries_t curr, entries_t other, int flags,
+		compare_stats_t *stats);
+static int compare_entries(dir_entry_t *curr, dir_entry_t *other, int flags);
+static void put_side_by_side_pair(dir_entry_t *curr, dir_entry_t *other,
+		int flags, compare_stats_t *stats);
 static int id_sorter(const void *first, const void *second);
 static void put_or_free(view_t *view, dir_entry_t *entry, int id, int take);
 static entries_t make_diff_list(trie_t *trie, view_t *view, int *next_id,
-		CompareType ct, int skip_empty, int dups_only);
+		CompareType ct, int dups_only, int flags);
 static void list_view_entries(const view_t *view, strlist_t *list);
 static int append_valid_nodes(const char name[], int valid,
 		const void *parent_data, void *data, void *arg);
 static void list_files_recursively(const view_t *view, const char path[],
-		int skip_dot_files, strlist_t *list);
+		int skip_dot_files, int flags, strlist_t *list);
 static char * get_file_fingerprint(const char path[], const dir_entry_t *entry,
-		CompareType ct);
+		CompareType ct, int flags, int lazy);
 static char * get_contents_fingerprint(const char path[],
-		const dir_entry_t *entry);
-static int get_file_id(trie_t *trie, const char path[],
-		const char fingerprint[], int *id, CompareType ct);
+		unsigned long long size);
+static int add_file_to_diff(trie_t *trie, const char path[], dir_entry_t *entry,
+		CompareType ct, int dups_only, int flags, int *next_id);
 static int files_are_identical(const char a[], const char b[]);
 static void put_file_id(trie_t *trie, const char path[],
-		const char fingerprint[], int id, CompareType ct);
+		const char fingerprint[], int id, int is_partial, CompareType ct);
 static void free_compare_records(void *ptr);
+static void compare_move_entry(ops_t *ops, view_t *from, view_t *to, int idx);
 
 int
-compare_two_panes(CompareType ct, ListType lt, int group_paths, int skip_empty)
+compare_two_panes(CompareType ct, ListType lt, int flags)
 {
+	assert((flags & (CF_IGNORE_CASE | CF_RESPECT_CASE)) !=
+			(CF_IGNORE_CASE | CF_RESPECT_CASE) && "Wrong combination of flags.");
+
+	const int group_paths = flags & CF_GROUP_PATHS;
+
 	/* We don't compare lists of files, so skip the check if at least one of the
 	 * views is a custom one. */
 	if(!flist_custom_active(&lwin) && !flist_custom_active(&rwin) &&
@@ -105,23 +137,22 @@ compare_two_panes(CompareType ct, ListType lt, int group_paths, int skip_empty)
 	int next_id = 1;
 	entries_t curr, other;
 
-	trie_t *const trie = trie_create();
+	trie_t *const trie = trie_create(&free_compare_records);
 	ui_cancellation_push_on();
 
-	curr = make_diff_list(trie, curr_view, &next_id, ct, skip_empty, 0);
-	other = make_diff_list(trie, other_view, &next_id, ct, skip_empty,
-			lt == LT_DUPS);
+	curr = make_diff_list(trie, curr_view, &next_id, ct, /*dups_only=*/0, flags);
+	other = make_diff_list(trie, other_view, &next_id, ct, lt == LT_DUPS, flags);
 
 	ui_cancellation_pop();
-	trie_free_with_data(trie, &free_compare_records);
+	trie_free(trie);
 
 	/* Clear progress message displayed by make_diff_list(). */
 	ui_sb_quick_msg_clear();
 
 	if(ui_cancellation_requested())
 	{
-		free_dir_entries(curr_view, &curr.entries, &curr.nentries);
-		free_dir_entries(other_view, &other.entries, &other.nentries);
+		free_dir_entries(&curr.entries, &curr.nentries);
+		free_dir_entries(&other.entries, &other.nentries);
 		ui_sb_msg("Comparison has been cancelled");
 		return 1;
 	}
@@ -149,7 +180,21 @@ compare_two_panes(CompareType ct, ListType lt, int group_paths, int skip_empty)
 	flist_custom_start(curr_view, lt == LT_ALL ? "diff" : "dups diff");
 	flist_custom_start(other_view, lt == LT_ALL ? "diff" : "dups diff");
 
-	fill_side_by_side(curr, other, group_paths);
+	compare_stats_t stats = {};
+
+	if(group_paths)
+	{
+		fill_side_by_side_by_paths(curr, other, flags, &stats);
+	}
+	else
+	{
+		fill_side_by_side_by_ids(curr, other, flags, &stats);
+	}
+
+	/* Entries' data has been moved out of them, so need to free only the
+	 * lists. */
+	dynarray_free(curr.entries);
+	dynarray_free(other.entries);
 
 	if(flist_custom_finish(curr_view, CV_DIFF, 0) != 0)
 	{
@@ -165,8 +210,15 @@ compare_two_panes(CompareType ct, ListType lt, int group_paths, int skip_empty)
 	other_view->list_pos = 0;
 	curr_view->custom.diff_cmp_type = ct;
 	other_view->custom.diff_cmp_type = ct;
-	curr_view->custom.diff_path_group = group_paths;
-	other_view->custom.diff_path_group = group_paths;
+	curr_view->custom.diff_list_type = lt;
+	other_view->custom.diff_list_type = lt;
+	curr_view->custom.diff_cmp_flags = flags;
+	other_view->custom.diff_cmp_flags = flags;
+
+	/* Both views share the same stats, alternatively can put to status_t, but
+	 * then have to save/store per global tab. */
+	curr_view->custom.diff_stats = stats;
+	other_view->custom.diff_stats = stats;
 
 	assert(curr_view->list_rows == other_view->list_rows &&
 			"Diff views must be in sync!");
@@ -186,6 +238,8 @@ make_unique_lists(entries_t curr, entries_t other)
 	flist_custom_start(curr_view, "unique");
 	flist_custom_start(other_view, "unique");
 
+	/* Inspect entries of both sides in a manner similar to the merging procedure.
+	 * Put unique ones into custom views and purge non-unique ones in place. */
 	for(i = 0; i < other.nentries; ++i)
 	{
 		const int id = other.entries[i].id;
@@ -204,15 +258,23 @@ make_unique_lists(entries_t curr, entries_t other)
 
 		while(j < curr.nentries && curr.entries[j].id == id)
 		{
-			fentry_free(curr_view, &curr.entries[j++]);
+			fentry_free(&curr.entries[j++]);
 		}
 		while(i < other.nentries && other.entries[i].id == id)
 		{
-			fentry_free(other_view, &other.entries[i++]);
+			fentry_free(&other.entries[i++]);
 		}
+		/* Want to revisit this entry on the next iteration of the loop. */
 		--i;
 	}
 
+	/* After iterating through all elements in the other view, all entries that
+	 * remain in the current view are unique to it. */
+	while(j < curr.nentries)
+	{
+		flist_custom_put(curr_view, &curr.entries[j++]);
+	}
+
 	/* Entries' data has been moved out of them or freed, so need to free only the
 	 * lists. */
 	dynarray_free(curr.entries);
@@ -294,32 +356,74 @@ is_not_duplicate(view_t *view, const dir_entry_t *entry, void *arg)
 	return entry->id != -1;
 }
 
-/* Composes side-by-side comparison of files in two views. */
+/* Composes side-by-side comparison of files in two views centered around their
+ * relative paths. */
 static void
-fill_side_by_side(entries_t curr, entries_t other, int group_paths)
+fill_side_by_side_by_paths(entries_t curr, entries_t other, int flags,
+		compare_stats_t *stats)
+{
+	int i = 0;
+	int j = 0;
+
+	while(i < curr.nentries || j < other.nentries)
+	{
+		int cmp;
+
+		if(i < curr.nentries && j < other.nentries)
+		{
+			cmp = compare_entries(&curr.entries[i], &other.entries[j], flags);
+		}
+		else
+		{
+			cmp = (i < curr.nentries ? -1 : 1);
+		}
+
+		if(cmp == 0)
+		{
+			put_side_by_side_pair(&curr.entries[i++], &other.entries[j++], flags,
+					stats);
+		}
+		else if(cmp < 0)
+		{
+			put_side_by_side_pair(&curr.entries[i++], NULL, flags, stats);
+		}
+		else
+		{
+			put_side_by_side_pair(NULL, &other.entries[j++], flags, stats);
+		}
+	}
+}
+
+/* Composes side-by-side comparison of files in two views that is guided by
+ * comparison ids and minimizes edit script. */
+static void
+fill_side_by_side_by_ids(entries_t curr, entries_t other, int flags,
+		compare_stats_t *stats)
 {
 	enum { UP, LEFT, DIAG };
 
 	int i, j;
-	/* Describes results of solving sub-problems. */
-	int (*d)[other.nentries + 1] =
-		reallocarray(NULL, curr.nentries + 1, sizeof(*d));
+
+	/* Describes results of solving sub-problems.  Only two rows of the full
+	 * table. */
+	int (*d)[other.nentries + 1] = reallocarray(NULL, 2, sizeof(*d));
 	/* Describes paths (backtracking handles ambiguity badly). */
 	char (*p)[other.nentries + 1] =
 		reallocarray(NULL, curr.nentries + 1, sizeof(*p));
 
 	for(i = 0; i <= curr.nentries; ++i)
 	{
+		int row = i%2;
 		for(j = 0; j <= other.nentries; ++j)
 		{
 			if(i == 0)
 			{
-				d[i][j] = j;
+				d[row][j] = j;
 				p[i][j] = LEFT;
 			}
 			else if(j == 0)
 			{
-				d[i][j] = i;
+				d[row][j] = i;
 				p[i][j] = UP;
 			}
 			else
@@ -327,14 +431,12 @@ fill_side_by_side(entries_t curr, entries_t other, int group_paths)
 				const dir_entry_t *centry = &curr.entries[curr.nentries - i];
 				const dir_entry_t *oentry = &other.entries[other.nentries - j];
 
-				d[i][j] = MIN(d[i - 1][j] + 1, d[i][j - 1] + 1);
-				p[i][j] = d[i][j] == d[i - 1][j] + 1 ? UP : LEFT;
+				d[row][j] = MIN(d[1 - row][j] + 1, d[row][j - 1] + 1);
+				p[i][j] = d[row][j] == d[1 - row][j] + 1 ? UP : LEFT;
 
-				if((centry->id == oentry->id ||
-							(group_paths && stroscmp(centry->name, oentry->name) == 0)) &&
-						d[i - 1][j - 1] <= d[i][j])
+				if(centry->id == oentry->id && d[1 - row][j - 1] <= d[row][j])
 				{
-					d[i][j] = d[i - 1][j - 1];
+					d[row][j] = d[1 - row][j - 1];
 					p[i][j] = DIAG;
 				}
 			}
@@ -347,37 +449,153 @@ fill_side_by_side(entries_t curr, entries_t other, int group_paths)
 	{
 		switch(p[i][j])
 		{
-			dir_entry_t *e;
-
 			case UP:
-				e = &curr.entries[curr.nentries - 1 - --i];
-				flist_custom_put(curr_view, e);
-				flist_custom_add_separator(other_view, e->id);
+				put_side_by_side_pair(&curr.entries[curr.nentries - i--], NULL, flags,
+						stats);
 				break;
 			case LEFT:
-				e = &other.entries[other.nentries - 1 - --j];
-				flist_custom_put(other_view, e);
-				flist_custom_add_separator(curr_view, e->id);
+				put_side_by_side_pair(NULL, &other.entries[other.nentries - j--], flags,
+						stats);
 				break;
 			case DIAG:
-				flist_custom_put(curr_view, &curr.entries[curr.nentries - 1 - --i]);
-				flist_custom_put(other_view, &other.entries[other.nentries - 1 - --j]);
+				put_side_by_side_pair(&curr.entries[curr.nentries - i--],
+						&other.entries[other.nentries - j--], flags, stats);
 				break;
 		}
 	}
 
 	free(d);
 	free(p);
+}
 
-	/* Entries' data has been moved out of them, so need to free only the
-	 * lists. */
-	dynarray_free(curr.entries);
-	dynarray_free(other.entries);
+/* Adds an entry per side.  Either curr or other can be NULL. */
+static void
+put_side_by_side_pair(dir_entry_t *curr, dir_entry_t *other, int flags,
+		compare_stats_t *stats)
+{
+	/* Integer type to avoid warning about unhandled cases in switch(). */
+	int flag;
+
+	if(other == NULL)
+	{
+		flag = (curr_view == &lwin ? CF_SHOW_UNIQUE_LEFT : CF_SHOW_UNIQUE_RIGHT);
+	}
+	else if(curr == NULL)
+	{
+		flag = (other_view == &lwin ? CF_SHOW_UNIQUE_LEFT : CF_SHOW_UNIQUE_RIGHT);
+	}
+	else
+	{
+		flag = (curr->id == other->id ? CF_SHOW_IDENTICAL : CF_SHOW_DIFFERENT);
+	}
+
+	switch(flag)
+	{
+		case CF_SHOW_UNIQUE_LEFT:  ++stats->unique_left;  break;
+		case CF_SHOW_UNIQUE_RIGHT: ++stats->unique_right; break;
+		case CF_SHOW_IDENTICAL:    ++stats->identical;    break;
+		case CF_SHOW_DIFFERENT:    ++stats->different;    break;
+	}
+
+	if(flags & flag)
+	{
+		if(curr != NULL)
+		{
+			flist_custom_put(curr_view, curr);
+		}
+		else
+		{
+			flist_custom_add_separator(curr_view, other->id);
+		}
+
+		if(other != NULL)
+		{
+			flist_custom_put(other_view, other);
+		}
+		else
+		{
+			flist_custom_add_separator(other_view, curr->id);
+		}
+	}
+	else
+	{
+		if(curr != NULL)
+		{
+			fentry_free(curr);
+		}
+		if(other != NULL)
+		{
+			fentry_free(other);
+		}
+	}
+}
+
+/* Compares entries by their short paths.  Returns strcmp()-like result. */
+static int
+compare_entries(dir_entry_t *curr, dir_entry_t *other, int flags)
+{
+	char path_a[PATH_MAX + 1], path_b[PATH_MAX + 1];
+	get_full_path_of(curr, sizeof(path_a), path_a);
+	get_full_path_of(other, sizeof(path_b), path_b);
+
+	int case_sensitive;
+	if(flags & CF_IGNORE_CASE)
+	{
+		case_sensitive = 0;
+	}
+	else if(flags & CF_RESPECT_CASE)
+	{
+		case_sensitive = 1;
+	}
+	else
+	{
+		/* If at least one path is case-sensitive, don't ignore case.  Otherwise, we
+		 * would end up with multiple matching pairs of paths. */
+		case_sensitive = case_sensitive_paths(path_a)
+		              || case_sensitive_paths(path_b);
+	}
+
+	get_short_path_of(curr_view, curr, NF_NONE, 0, sizeof(path_a), path_a);
+	get_short_path_of(other_view, other, NF_NONE, 0, sizeof(path_b), path_b);
+
+	const char *a = path_a, *b = path_b;
+
+	char lower_a[PATH_MAX + 1], lower_b[PATH_MAX + 1];
+	if(!case_sensitive)
+	{
+		str_to_lower(path_a, lower_a, sizeof(lower_a));
+		str_to_lower(path_b, lower_b, sizeof(lower_b));
+
+		a = lower_a;
+		b = lower_b;
+	}
+
+	while(*a == *b && *a != '\0')
+	{
+		++a;
+		++b;
+	}
+
+	int cmp = *a - *b;
+
+	/* Correct result to reflect that a directory is smaller than a
+	 * non-directory. */
+	int dir_a = (strchr(a, '/') != NULL);
+	int dir_b = (strchr(b, '/') != NULL);
+	if(dir_a != dir_b)
+	{
+		cmp = (dir_a ? -1 : 1);
+	}
+
+	return cmp;
 }
 
 int
-compare_one_pane(view_t *view, CompareType ct, ListType lt, int skip_empty)
+compare_one_pane(view_t *view, CompareType ct, ListType lt, int flags)
 {
+	assert((flags & (CF_IGNORE_CASE | CF_RESPECT_CASE)) !=
+			(CF_IGNORE_CASE | CF_RESPECT_CASE) && "Wrong combination of flags.");
+
 	int i, dup_id;
 	view_t *other = (view == curr_view) ? other_view : curr_view;
 	const char *const title = (lt == LT_ALL)  ? "compare"
@@ -386,20 +604,20 @@ compare_one_pane(view_t *view, CompareType ct, ListType lt, int skip_empty)
 	int next_id = 1;
 	entries_t curr;
 
-	trie_t *trie = trie_create();
+	trie_t *trie = trie_create(&free_compare_records);
 	ui_cancellation_push_on();
 
-	curr = make_diff_list(trie, view, &next_id, ct, skip_empty, 0);
+	curr = make_diff_list(trie, view, &next_id, ct, /*dups_only=*/0, flags);
 
 	ui_cancellation_pop();
-	trie_free_with_data(trie, &free_compare_records);
+	trie_free(trie);
 
 	/* Clear progress message displayed by make_diff_list(). */
 	ui_sb_quick_msg_clear();
 
 	if(ui_cancellation_requested())
 	{
-		free_dir_entries(view, &curr.entries, &curr.nentries);
+		free_dir_entries(&curr.entries, &curr.nentries);
 		ui_sb_msg("Comparison has been cancelled");
 		return 1;
 	}
@@ -457,6 +675,9 @@ compare_one_pane(view_t *view, CompareType ct, ListType lt, int skip_empty)
 		rn_leave(other, 1);
 	}
 
+	curr_view->custom.diff_cmp_flags = flags;
+	other_view->custom.diff_cmp_flags = flags;
+
 	view->list_pos = 0;
 	ui_view_schedule_redraw(view);
 	return 0;
@@ -484,7 +705,7 @@ put_or_free(view_t *view, dir_entry_t *entry, int id, int take)
 	}
 	else
 	{
-		fentry_free(view, entry);
+		fentry_free(entry);
 	}
 }
 
@@ -493,8 +714,10 @@ put_or_free(view_t *view, dir_entry_t *entry, int id, int take)
  * trie. */
 static entries_t
 make_diff_list(trie_t *trie, view_t *view, int *next_id, CompareType ct,
-		int skip_empty, int dups_only)
+		int dups_only, int flags)
 {
+	const int skip_empty = flags & CF_SKIP_EMPTY;
+
 	int i;
 	strlist_t files = {};
 	entries_t r = {};
@@ -508,54 +731,40 @@ make_diff_list(trie_t *trie, view_t *view, int *next_id, CompareType ct,
 	}
 	else
 	{
-		list_files_recursively(view, flist_get_dir(view), view->hide_dot, &files);
+		list_files_recursively(view, flist_get_dir(view), view->hide_dot, flags,
+				&files);
 	}
 
 	show_progress("Querying...", 0);
 	for(i = 0; i < files.nitems && !ui_cancellation_requested(); ++i)
 	{
 		int progress;
-		int existing_id;
-		char *fingerprint;
 		const char *const path = files.items[i];
 		dir_entry_t *const entry = entry_list_add(view, &r.entries, &r.nentries,
 				path);
-
-		if(skip_empty && entry->size == 0)
+		if(entry == NULL)
 		{
-			fentry_free(view, entry);
-			--r.nentries;
+			/* Maybe the file doesn't exist anymore, maybe we've lost access to it or
+			 * we're just out of memory. */
 			continue;
 		}
 
-		fingerprint = get_file_fingerprint(path, entry, ct);
-		/* In case we couldn't obtain fingerprint (e.g., comparing by contents and
-		 * files isn't readable), ignore the file and keep going. */
-		if(is_null_or_empty(fingerprint))
+		if(skip_empty && entry->size == 0)
 		{
-			free(fingerprint);
-			fentry_free(view, entry);
+			fentry_free(entry);
 			--r.nentries;
 			continue;
 		}
 
 		entry->tag = i;
-		if(get_file_id(trie, path, fingerprint, &existing_id, ct))
-		{
-			entry->id = existing_id;
-		}
-		else if(dups_only)
-		{
-			entry->id = -1;
-		}
-		else
-		{
-			entry->id = *next_id;
-			++*next_id;
-			put_file_id(trie, path, fingerprint, entry->id, ct);
-		}
+		entry->id = add_file_to_diff(trie, path, entry, ct, dups_only, flags,
+				next_id);
 
-		free(fingerprint);
+		if(entry->id == -1)
+		{
+			fentry_free(entry);
+			--r.nentries;
+		}
 
 		progress = (i*100)/files.nitems;
 		if(progress != last_progress)
@@ -619,18 +828,31 @@ append_valid_nodes(const char name[], int valid, const void *parent_data,
 /* Collects files under specified file system tree. */
 static void
 list_files_recursively(const view_t *view, const char path[],
-		int skip_dot_files, strlist_t *list)
+		int skip_dot_files, int flags, strlist_t *list)
 {
 	int i;
 
 	/* Obtain sorted list of files. */
 	int len;
-	char **lst = list_sorted_files(path, &len);
+	char **lst = list_all_files(path, &len);
 	if(len < 0)
 	{
 		return;
 	}
 
+	if(flags & CF_IGNORE_CASE)
+	{
+		safe_qsort(lst, len, sizeof(*lst), &strcasesorter);
+	}
+	else if(flags & CF_RESPECT_CASE)
+	{
+		safe_qsort(lst, len, sizeof(*lst), &strsorter);
+	}
+	else
+	{
+		safe_qsort(lst, len, sizeof(*lst), &strossorter);
+	}
+
 	/* Visit all subdirectories ignoring symbolic links to directories. */
 	for(i = 0; i < len && !ui_cancellation_requested(); ++i)
 	{
@@ -653,7 +875,7 @@ list_files_recursively(const view_t *view, const char path[],
 		{
 			if(!is_symlink(full_path))
 			{
-				list_files_recursively(view, full_path, skip_dot_files, list);
+				list_files_recursively(view, full_path, skip_dot_files, flags, list);
 			}
 			free(full_path);
 			update_string(&lst[i], NULL);
@@ -680,27 +902,54 @@ list_files_recursively(const view_t *view, const char path[],
 }
 
 /* Computes fingerprint of the file specified by path and entry.  Type of the
- * fingerprint is determined by ct parameter.  Returns newly allocated string
- * with the fingerprint, which is empty or NULL on error. */
+ * fingerprint is determined by ct parameter.  Lazy fingerprint is an
+ * optimization which prevents computing contents fingerprint until there is
+ * more than one file of the given size.  Returns newly allocated string with
+ * the fingerprint, which is empty or NULL on error. */
 static char *
 get_file_fingerprint(const char path[], const dir_entry_t *entry,
-		CompareType ct)
+		CompareType ct, int flags, int lazy)
 {
 	switch(ct)
 	{
+		int case_sensitive;
 		char name[NAME_MAX + 1];
 
 		case CT_NAME:
-			if(case_sensitive_paths(path))
+			if(flags & CF_IGNORE_CASE)
+			{
+				case_sensitive = 0;
+			}
+			else if(flags & CF_RESPECT_CASE)
+			{
+				case_sensitive = 1;
+			}
+			else
+			{
+				case_sensitive = case_sensitive_paths(path);
+			}
+
+			if(case_sensitive)
 			{
 				return strdup(entry->name);
 			}
+
 			str_to_lower(entry->name, name, sizeof(name));
 			return strdup(name);
 		case CT_SIZE:
 			return format_str("%" PRINTF_ULL, (unsigned long long)entry->size);
 		case CT_CONTENTS:
-			return get_contents_fingerprint(path, entry);
+			if(lazy)
+			{
+				/* Comparing by contents can't be done if file can't be read. */
+				if(os_access(path, R_OK) != 0)
+				{
+					return strdup("");
+				}
+
+				return format_str("%" PRINTF_ULL, (unsigned long long)entry->size);
+			}
+			return get_contents_fingerprint(path, entry->size);
 	}
 	assert(0 && "Unexpected diffing type.");
 	return strdup("");
@@ -709,18 +958,8 @@ get_file_fingerprint(const char path[], const dir_entry_t *entry,
 /* Makes fingerprint of file contents (all or part of it of fixed size).
  * Returns the fingerprint as a string, which is empty or NULL on error. */
 static char *
-get_contents_fingerprint(const char path[], const dir_entry_t *entry)
+get_contents_fingerprint(const char path[], unsigned long long size)
 {
-#if INTPTR_MAX == INT64_MAX
-#define XX_BITS 64
-#else
-#define XX_BITS 32
-#endif
-#define XX__(name, bits) XXH ## bits ## _ ## name
-#define XX_(name, bits) XX__(name, bits)
-#define XX(name) XX_(name, XX_BITS)
-
-	XX(state_t) st;
 	char block[BLOCK_SIZE];
 	size_t to_read = PREFIX_SIZE;
 	FILE *in = os_fopen(path, "rb");
@@ -729,7 +968,20 @@ get_contents_fingerprint(const char path[], const dir_entry_t *entry)
 		return strdup("");
 	}
 
-	XX(reset)(&st, 0U);
+	XXH3_state_t *st = XXH3_createState();
+	if(st == NULL)
+	{
+		fclose(in);
+		return strdup("");
+	}
+
+	if(XXH3_64bits_reset(st) == XXH_ERROR)
+	{
+		fclose(in);
+		XXH3_freeState(st);
+		return strdup("");
+	}
+
 	while(to_read != 0U)
 	{
 		const size_t portion = MIN(sizeof(block), to_read);
@@ -739,56 +991,112 @@ get_contents_fingerprint(const char path[], const dir_entry_t *entry)
 			break;
 		}
 
-		XX(update)(&st, block, nread);
+		XXH3_64bits_update(st, block, nread);
 		to_read -= nread;
 	}
 	fclose(in);
 
-	return format_str("%" PRINTF_ULL "|%" PRINTF_ULL,
-			(unsigned long long)entry->size, (unsigned long long)XX(digest)(&st));
+	const unsigned long long digest = XXH3_64bits_digest(st);
+	XXH3_freeState(st);
 
-#undef XX_BITS
-#undef XX__
-#undef XX_
-#undef XX
+	return format_str("%" PRINTF_ULL "|%" PRINTF_ULL, size, digest);
 }
 
-/* Retrieves file from the trie by its fingerprint.  Returns non-zero if it was
- * in the trie and sets *id, otherwise zero is returned. */
+/* Looks up file in the trie by its fingerprint.  Returns id for the file or -1
+ * if it should be skipped. */
 static int
-get_file_id(trie_t *trie, const char path[], const char fingerprint[], int *id,
-		CompareType ct)
+add_file_to_diff(trie_t *trie, const char path[], dir_entry_t *entry,
+		CompareType ct, int dups_only, int flags, int *next_id)
 {
-	void *data;
-	compare_record_t *record;
-	if(trie_get(trie, fingerprint, &data) != 0)
+	char *fingerprint = get_file_fingerprint(path, entry, ct, flags, /*lazy=*/1);
+	if(is_null_or_empty(fingerprint))
 	{
-		return 0;
-	}
-	record = data;
-
-	/* Comparison by contents is the only one when we need to resolve fingerprint
-	 * conflicts. */
-	if(ct != CT_CONTENTS)
-	{
-		*id = record->id;
-		return 1;
+		/* In case we couldn't obtain fingerprint (e.g., comparing by contents and
+		 * the file isn't readable), ignore the file and keep going. */
+		free(fingerprint);
+		return -1;
 	}
 
-	/* Fingerprint does not guarantee a match, go through files and find file with
-	 * identical content. */
-	do
+	void *data = NULL;
+	(void)trie_get(trie, fingerprint, &data);
+
+	compare_record_t *record = data;
+	int is_partial = (ct == CT_CONTENTS);
+
+	/* Comparison by contents is the only one when we need to account for lazy
+	 * fingerprint computation or resolve fingerprint conflicts. */
+	if(record != NULL && ct == CT_CONTENTS)
 	{
-		if(files_are_identical(path, record->path))
+		free(fingerprint);
+		is_partial = 0;
+
+		fingerprint = get_file_fingerprint(path, entry, ct, flags, /*lazy=*/0);
+		if(is_null_or_empty(fingerprint))
 		{
-			*id = record->id;
-			return 1;
+			/* In case we couldn't obtain fingerprint (e.g., comparing by contents and
+			 * the file isn't readable), ignore the file and keep going. */
+			free(fingerprint);
+			return -1;
 		}
-		record = record->next;
-	}
-	while(record != NULL);
 
-	return 0;
+		if(record->is_partial)
+		{
+			/* There is another file of the same size whose contents fingerprint
+			 * hasn't been computed yet.  Do it here. */
+			char *other_fingerprint = get_contents_fingerprint(record->path,
+					entry->size);
+			if(is_null_or_empty(fingerprint))
+			{
+				/* That other file has issues, don't update it and skip any other file
+				 * that can conflict with it by size.  The file itself won't be skipped
+				 * though, should it be? */
+				free(other_fingerprint);
+				free(fingerprint);
+				return -1;
+			}
+
+			put_file_id(trie, record->path, other_fingerprint, record->id,
+					/*is_partial=*/0, ct);
+			free(other_fingerprint);
+
+			record->is_partial = 0;
+		}
+
+		/* Repeat trie lookup with contents fingerprint. */
+		(void)trie_get(trie, fingerprint, &data);
+		record = data;
+
+		/* Fingerprint does not guarantee a match, go through files and find file
+		 * with identical contents. */
+		do
+		{
+			if(files_are_identical(path, record->path))
+			{
+				break;
+			}
+			record = record->next;
+		}
+		while(record != NULL);
+	}
+
+	if(record != NULL)
+	{
+		free(fingerprint);
+		return record->id;
+	}
+
+	if(dups_only)
+	{
+		free(fingerprint);
+		return -1;
+	}
+
+	int id = *next_id;
+	++*next_id;
+	put_file_id(trie, path, fingerprint, id, is_partial, ct);
+
+	free(fingerprint);
+	return id;
 }
 
 /* Checks whether two files specified by their names hold identical content.
@@ -840,19 +1148,29 @@ files_are_identical(const char a[], const char b[])
 /* Stores id of a file with given fingerprint in the trie. */
 static void
 put_file_id(trie_t *trie, const char path[], const char fingerprint[], int id,
-		CompareType ct)
+		int is_partial, CompareType ct)
 {
 	compare_record_t *const record = malloc(sizeof(*record));
-	void *data = NULL;
-	(void)trie_get(trie, fingerprint, &data);
 
 	record->id = id;
-	record->next = data;
+	record->is_partial = is_partial;
+	record->next = NULL;
 
 	/* Comparison by contents is the only one when we need to resolve fingerprint
 	 * conflicts. */
 	record->path = (ct == CT_CONTENTS ? strdup(path) : NULL);
 
+	/* Just add new entry to the list if something is already there. */
+	void *data = NULL;
+	(void)trie_get(trie, fingerprint, &data);
+	compare_record_t *prev = data;
+	if(prev != NULL)
+	{
+		prev->next = record;
+		return;
+	}
+
+	/* Otherwise we're the head of the list. */
 	if(trie_set(trie, fingerprint, record) < 0)
 	{
 		free(record->path);
@@ -878,63 +1196,87 @@ free_compare_records(void *ptr)
 int
 compare_move(view_t *from, view_t *to)
 {
-	char from_path[PATH_MAX + 1], to_path[PATH_MAX + 1];
-	char *from_fingerprint, *to_fingerprint;
-
-	const CompareType ct = from->custom.diff_cmp_type;
-
-	dir_entry_t *const curr = &from->dir_entry[from->list_pos];
-	dir_entry_t *const other = &to->dir_entry[from->list_pos];
-
-	if(from->custom.type != CV_DIFF || !from->custom.diff_path_group)
+	const CompareType flags = from->custom.diff_cmp_flags;
+	if(from->custom.type != CV_DIFF || !(flags & CF_GROUP_PATHS))
 	{
 		ui_sb_err("Not in diff mode with path grouping");
 		return 1;
 	}
 
+	char *from_dir = strdup(replace_home_part(flist_get_dir(from)));
+	char *to_dir = strdup(replace_home_part(flist_get_dir(to)));
+	if(from_dir == NULL || to_dir == NULL)
+	{
+		free(from_dir);
+		free(to_dir);
+		return 1;
+	}
+
+	char undo_msg[2*PATH_MAX + 32];
+	snprintf(undo_msg, sizeof(undo_msg), "Diff apply %s -> %s", from_dir, to_dir);
+
+	ops_t *ops = fops_get_ops(OP_COPY, "Applying", from_dir, to_dir);
+	free(from_dir);
+	free(to_dir);
+
+	if(ops == NULL)
+	{
+		show_error_msg("Comparison", "Failed to initialize apply operation");
+		return 1;
+	}
+
+	/* We're going at least to try to update one of the views (which might refer
+	 * to the same directory), so schedule a reload. */
+	ui_view_schedule_reload(from);
+	ui_view_schedule_reload(to);
+
+	un_group_open(undo_msg);
+
+	dir_entry_t *entry = NULL;
+	while(iter_selection_or_current_any(curr_view, &entry) && fops_active(ops))
+	{
+		compare_move_entry(ops, from, to, entry_to_pos(curr_view, entry));
+	}
+
+	un_group_close();
+
+	fops_free_ops(ops);
+	flist_sel_stash(curr_view);
+	return 0;
+}
+
+/* Moves a file identified by an entry from one view to the other. */
+static void
+compare_move_entry(ops_t *ops, view_t *from, view_t *to, int idx)
+{
+	char from_path[PATH_MAX + 1], to_path[PATH_MAX + 1];
+	char *from_fingerprint, *to_fingerprint;
+
+	const CompareType ct = from->custom.diff_cmp_type;
+	const CompareType flags = from->custom.diff_cmp_flags;
+
+	dir_entry_t *const curr = &from->dir_entry[idx];
+	dir_entry_t *const other = &to->dir_entry[idx];
+
 	if(curr->id == other->id && !fentry_is_fake(curr) && !fentry_is_fake(other))
 	{
 		/* Nothing to do if files are already equal. */
-		return 0;
+		return;
 	}
 
-	/* We're going at least to try to update one of views (which might refer to
-	 * the same directory), so schedule a reload. */
-	ui_view_schedule_reload(from);
-	ui_view_schedule_reload(to);
-
 	if(fentry_is_fake(curr))
 	{
 		/* Just remove the other file (it can't be fake entry too). */
-		return fops_delete_current(to, 1, 0);
+		fops_delete_entry(ops, to, other, /*use_trash=*/1, /*nested=*/0);
+		return;
 	}
 
 	get_full_path_of(curr, sizeof(from_path), from_path);
 	get_full_path_of(other, sizeof(to_path), to_path);
 
-	if(fentry_is_fake(other))
-	{
-		char to_path[PATH_MAX + 1];
-		char canonical[PATH_MAX + 1];
-		snprintf(to_path, sizeof(to_path), "%s/%s/%s", flist_get_dir(to),
-				curr->origin + strlen(flist_get_dir(from)), curr->name);
-		canonicalize_path(to_path, canonical, sizeof(canonical));
-
-		/* Copy current file to position of the other one using relative path with
-		 * different base. */
-		fops_replace(from, canonical, 0);
-
-		/* Update the other entry to not be fake. */
-		remove_last_path_component(canonical);
-		replace_string(&other->name, curr->name);
-		replace_string(&other->origin, canonical);
-	}
-	else
-	{
-		/* Overwrite file in the other pane with corresponding file from current
-		 * pane. */
-		fops_replace(from, to_path, 1);
-	}
+	/* Overwrite file in the other pane with corresponding file from current
+	 * pane. */
+	fops_replace_entry(ops, from, curr, to, other);
 
 	/* Obtaining file fingerprint relies on size field of entries, so try to load
 	 * it and ignore if it fails. */
@@ -943,8 +1285,9 @@ compare_move(view_t *from, view_t *to)
 	/* Try to update id of the other entry by computing fingerprint of both files
 	 * and checking if they match. */
 
-	from_fingerprint = get_file_fingerprint(from_path, curr, ct);
-	to_fingerprint = get_file_fingerprint(to_path, other, ct);
+	from_fingerprint = get_file_fingerprint(from_path, curr, ct, flags,
+			/*lazy=*/0);
+	to_fingerprint = get_file_fingerprint(to_path, other, ct, flags, /*lazy=*/0);
 
 	if(!is_null_or_empty(from_fingerprint) && !is_null_or_empty(to_fingerprint))
 	{
@@ -961,8 +1304,6 @@ compare_move(view_t *from, view_t *to)
 
 	free(from_fingerprint);
 	free(to_fingerprint);
-
-	return 0;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
