@@ -29,6 +29,7 @@
 #include "ui/fileview.h"
 #include "ui/ui.h"
 #include "utils/fs.h"
+#include "utils/macros.h"
 #include "utils/regexp.h"
 #include "utils/path.h"
 #include "utils/str.h"
@@ -81,7 +82,7 @@ fpos_scroll_down(view_t *view, int lines_count)
 	if(!fpos_are_all_files_visible(view))
 	{
 		view->list_pos =
-			get_corrected_list_pos_down(view, lines_count*view->run_size);
+			fpos_adjust_for_scroll_back(view, lines_count*view->run_size);
 		return 1;
 	}
 	return 0;
@@ -93,12 +94,31 @@ fpos_scroll_up(view_t *view, int lines_count)
 	if(!fpos_are_all_files_visible(view))
 	{
 		view->list_pos =
-			get_corrected_list_pos_up(view, lines_count*view->run_size);
+			fpos_adjust_for_scroll_fwd(view, lines_count*view->run_size);
 		return 1;
 	}
 	return 0;
 }
 
+void
+fpos_scroll_page(view_t *view, int base, int direction)
+{
+	enum { HOR_GAP_SIZE = 2, VER_GAP_SIZE = 1 };
+	int old_pos = view->list_pos;
+	int offset = fview_is_transposed(view)
+	    ? (MAX(1, view->column_count - VER_GAP_SIZE))*view->window_rows
+	    : (view->window_rows - HOR_GAP_SIZE)*view->column_count;
+	int new_pos = base + direction*offset
+	            + old_pos%view->run_size - base%view->run_size;
+	view->list_pos = MAX(0, MIN(view->list_rows - 1, new_pos));
+	fview_scroll_by(view, direction*offset);
+
+	/* Updating list_pos ourselves doesn't take into account
+	 * synchronization/updates of the other view, so trigger them. */
+	ui_view_schedule_redraw(view);
+	fpos_set_pos(view, view->list_pos);
+}
+
 void
 fpos_set_pos(view_t *view, int pos)
 {
@@ -254,22 +274,50 @@ fpos_get_ver_step(const struct view_t *view)
 }
 
 int
-fpos_has_hidden_top(const view_t *view)
+fpos_can_scroll_back(const view_t *view)
 {
-	return (fview_is_transposed(view) ? 0 : can_scroll_up(view));
+	return (view->top_line > 0);
 }
 
 int
-fpos_has_hidden_bottom(const view_t *view)
+fpos_can_scroll_fwd(const view_t *view)
 {
-	return (fview_is_transposed(view) ? 0 : can_scroll_down(view));
+	return (fpos_get_last_visible_cell(view) < view->list_rows - 1);
+}
+
+int
+fpos_adjust_for_scroll_back(const view_t *view, int pos_delta)
+{
+	const int scroll_offset = fpos_get_offset(view);
+	if(view->list_pos <= view->top_line + scroll_offset + (MAX(pos_delta, 1) - 1))
+	{
+		const int column_correction = view->list_pos%view->run_size;
+		const int offset = scroll_offset + pos_delta + column_correction;
+		return view->top_line + offset;
+	}
+	return view->list_pos;
+}
+
+int
+fpos_adjust_for_scroll_fwd(const view_t *view, int pos_delta)
+{
+	const int scroll_offset = fpos_get_offset(view);
+	const int last = fpos_get_last_visible_cell(view);
+	if(view->list_pos >= last - scroll_offset - (MAX(pos_delta, 1) - 1))
+	{
+		const int column_correction = (view->run_size - 1)
+		                            - view->list_pos%view->run_size;
+		const int offset = scroll_offset + pos_delta + column_correction;
+		return last - offset;
+	}
+	return view->list_pos;
 }
 
 int
 fpos_get_top_pos(const view_t *view)
 {
 	return get_column_top_pos(view)
-	     + (can_scroll_up(view) ? fpos_get_offset(view) : 0);
+	     + (fpos_can_scroll_back(view) ? fpos_get_offset(view) : 0);
 }
 
 int
@@ -285,7 +333,7 @@ int
 fpos_get_bottom_pos(const view_t *view)
 {
 	return get_column_bottom_pos(view)
-	     - (can_scroll_down(view) ? fpos_get_offset(view) : 0);
+	     - (fpos_can_scroll_fwd(view) ? fpos_get_offset(view) : 0);
 }
 
 int
@@ -325,7 +373,7 @@ fpos_half_scroll(view_t *view, int down)
 
 	if(down)
 	{
-		new_pos = get_corrected_list_pos_down(view, offset);
+		new_pos = fpos_adjust_for_scroll_back(view, offset);
 		new_pos = MAX(new_pos, view->list_pos + offset);
 
 		if(new_pos >= view->list_rows)
@@ -336,7 +384,7 @@ fpos_half_scroll(view_t *view, int down)
 	}
 	else
 	{
-		new_pos = get_corrected_list_pos_up(view, offset);
+		new_pos = fpos_adjust_for_scroll_fwd(view, offset);
 		new_pos = MIN(new_pos, view->list_pos - offset);
 
 		if(new_pos < 0)
@@ -345,7 +393,7 @@ fpos_half_scroll(view_t *view, int down)
 		}
 	}
 
-	scroll_by_files(view, new_pos - view->list_pos);
+	fview_scroll_by(view, new_pos - view->list_pos);
 	return new_pos;
 }
 
@@ -735,7 +783,7 @@ static int
 is_mismatched_entry(const dir_entry_t *entry)
 {
 	/* To avoid passing view pointer here, we exploit the fact that entry_to_pos()
-	 * checks whether it's argument belongs to the given view. */
+	 * checks whether its argument belongs to the given view. */
 	int pos = entry_to_pos(&lwin, entry);
 	view_t *other = &rwin;
 	if(pos == -1)
@@ -744,7 +792,9 @@ is_mismatched_entry(const dir_entry_t *entry)
 		other = &lwin;
 	}
 
-	return (other->dir_entry[pos].id != entry->id);
+	return other->dir_entry[pos].id != entry->id
+	    || fentry_is_fake(entry)
+	    || fentry_is_fake(&other->dir_entry[pos]);
 }
 
 /* Finds position of the next entry matching the predicate.  Returns new
