@@ -43,6 +43,10 @@ int fpos_scroll_down(struct view_t *view, int lines_count);
  * position was updated. */
 int fpos_scroll_up(struct view_t *view, int lines_count);
 
+/* Scrolls the view by one view up or down.  The direction should be -1 (up)
+ * or 1 (down). */
+void fpos_scroll_page(struct view_t *view, int base, int direction);
+
 /* Moves cursor to specified position.  Normalizes it if needed, invokes fview
  * update function and can synchronize cursor position in the other view. */
 void fpos_set_pos(struct view_t *view, int pos);
@@ -98,13 +102,21 @@ int fpos_get_hor_step(const struct view_t *view);
  * step. */
 int fpos_get_ver_step(const struct view_t *view);
 
-/* Checks whether there are more elements to show above what can be seen
- * currently.  Returns non-zero if so, otherwise zero is returned. */
-int fpos_has_hidden_top(const struct view_t *view);
+/* Checks whether there are more elements to show above/on the left what can be
+ * seen currently.  Returns non-zero if so, otherwise zero is returned. */
+int fpos_can_scroll_back(const struct view_t *view);
 
-/* Checks whether there are more elements to show below what can be seen
- * currently.  Returns non-zero if so, otherwise zero is returned. */
-int fpos_has_hidden_bottom(const struct view_t *view);
+/* Checks whether there are more elements to show below/on the right what can
+ * be seen currently.  Returns non-zero if so, otherwise zero is returned. */
+int fpos_can_scroll_fwd(const struct view_t *view);
+
+/* Calculates list position corrected for scrolling down.  Returns adjusted
+ * position. */
+int fpos_adjust_for_scroll_back(const struct view_t *view, int pos_delta);
+
+/* Calculates list position corrected for scrolling up.  Returns adjusted
+ * position. */
+int fpos_adjust_for_scroll_fwd(const struct view_t *view, int pos_delta);
 
 /* Calculates position in list of files that corresponds to window top, which is
  * adjusted according to 'scrolloff' option.  Returns the position. */
