@@ -44,6 +44,7 @@ typedef struct
 	int nlines;              /* Number of user supplied file names (list size). */
 	int move;                /* Whether this is a move operation. */
 	int force;               /* Whether destination files should be removed. */
+	int skip;                /* Skip files that already exist at destination. */
 	char **sel_list;         /* Full paths of files to be processed. */
 	size_t sel_list_len;     /* Number of files to process (sel_list size). */
 	char path[PATH_MAX + 1]; /* Path at which processing should take place. */
@@ -56,11 +57,12 @@ typedef struct
 }
 bg_args_t;
 
+struct custom_prompt_t;
 struct dirent;
-struct response_variant;
 
-/* Callback for returning edited filename. */
-typedef void (*fo_prompt_cb)(const char new_filename[]);
+/* Callback for returning edited filename.  arg is user supplied value, which is
+ * passed through. */
+typedef void (*fo_prompt_cb)(const char new_filename[], void *arg);
 
 /* Line completion function.  arg is user supplied value, which is passed
  * through.  Should return completion offset. */
@@ -68,11 +70,10 @@ typedef int (*fo_complete_cmd_func)(const char cmd[], void *arg);
 
 /* Function to request filename editing. */
 typedef void (*line_prompt_func)(const char prompt[], const char filename[],
-		fo_prompt_cb cb, fo_complete_cmd_func complete, int allow_ee);
+		fo_prompt_cb cb, void *cb_arg, fo_complete_cmd_func complete, int allow_ee);
 
-/* Function to choose an option.  Returns choice. */
-typedef char (*options_prompt_func)(const char title[], const char message[],
-		const struct response_variant *variants);
+/* Function to choose an option.  Returns the choice. */
+typedef char (*options_prompt_func)(const struct custom_prompt_t *details);
 
 /* Function invoked to check whether edited list is OK.  Should return non-zero
  * if so and zero otherwise.  Should reallocate *error on error.  *data is the
@@ -165,6 +166,10 @@ void fops_bg_ops_init(ops_t *ops, bg_op_t *bg_op);
  * newly allocated structure, which should be freed by fops_free_ops(). */
 ops_t * fops_get_bg_ops(OPS main_op, const char descr[], const char dir[]);
 
+/* Checks whether operation should be carried on.  Returns zero if it was
+ * cancelled (via Ctrl-C) or aborted (via error dialog option) by the user. */
+int fops_active(const ops_t *ops);
+
 /* Frees ops structure previously obtained by call to get_ops().  ops can be
  * NULL. */
 void fops_free_ops(ops_t *ops);
@@ -214,8 +219,11 @@ int fops_is_dir_writable(DirRole dir_role, const char path[]);
 
 TSTATIC_DEFS(
 	struct ext_edit_t;
+	struct progress_data_t;
+
 	char ** edit_list(struct ext_edit_t *ext_edit, size_t orig_len, char *orig[],
 		int *edited_len, int load_always);
+	struct progress_data_t * alloc_progress_data(int bg, void *info);
 )
 
 #endif /* VIFM__FOPS_COMMON_H__ */
