@@ -69,10 +69,17 @@ static const char * get_top_dir(const view_t *view);
 static void delete_files_in_bg(bg_op_t *bg_op, void *arg);
 static void delete_file_in_bg(ops_t *ops, const char path[], int use_trash);
 static int prepare_register(int reg);
-static void change_link_cb(const char new_target[]);
+static char ** list_files_to_retarget(view_t *view, int *len);
+static int retarget_one(view_t *view);
+static void change_link_cb(const char new_target[], void *arg);
+static int retarget_many(view_t *view, char *files[], int nfiles);
+static int verify_retarget_list(char *files[], int nfiles, char *names[],
+		int nnames, char **error, void *data);
+static void change_link(ops_t *ops, const char path[], const char from[],
+		const char to[]);
 static int complete_filename(const char str[], void *arg);
-static int verify_list(char *files[], int nfiles, char *names[], int nnames,
-		char **error, void *data);
+static int verify_clone_list(char *files[], int nfiles, char *names[],
+		int nnames, char **error, void *data);
 TSTATIC const char * gen_clone_name(const char dir[], const char normal_name[]);
 static int clone_file(const dir_entry_t *entry, const char path[],
 		const char clone[], ops_t *ops);
@@ -81,13 +88,12 @@ static void go_to_first_file(view_t *view, char *names[], int count);
 static void update_dir_entry_size(dir_entry_t *entry, int force);
 static void start_dir_size_calc(const char path[], int force);
 static void dir_size_bg(bg_op_t *bg_op, void *arg);
-static void dir_size(bg_op_t *bg_op, char path[], int force);
+static void dir_size(bg_op_t *bg_op, const char path[], int force);
 static int bg_cancellation_hook(void *arg);
-static void redraw_after_path_change(view_t *view, const char path[]);
 #ifndef _WIN32
-static void change_owner_cb(const char new_owner[]);
+static void change_owner_cb(const char new_owner[], void *arg);
 static int complete_owner(const char str[], void *arg);
-static void change_group_cb(const char new_group[]);
+static void change_group_cb(const char new_group[], void *arg);
 static int complete_group(const char str[], void *arg);
 #endif
 
@@ -158,7 +164,7 @@ fops_delete(view_t *view, int reg, int use_trash)
 
 	entry = NULL;
 	i = 0;
-	while(iter_marked_entries(view, &entry) && !ui_cancellation_requested())
+	while(iter_marked_entries(view, &entry) && fops_active(ops))
 	{
 		int result;
 
@@ -192,52 +198,25 @@ fops_delete(view_t *view, int reg, int use_trash)
 	return 1;
 }
 
-int
-fops_delete_current(view_t *view, int use_trash, int nested)
+void
+fops_delete_entry(ops_t *ops, view_t *view, dir_entry_t *entry, int use_trash,
+		int nested)
 {
-	char undo_msg[COMMAND_GROUP_INFO_LEN];
-	dir_entry_t *entry;
-	ops_t *ops;
 	const char *const top_dir = get_top_dir(view);
-	const char *const curr_dir = top_dir == NULL ? flist_get_dir(view) : top_dir;
 
 	use_trash = use_trash && cfg.use_trash;
 
 	/* This check for the case when we are for sure in the trash. */
 	if(use_trash && top_dir != NULL && trash_has_path(top_dir))
 	{
-		show_error_msg("Can't perform deletion",
-				"Current directory is under trash directory");
-		return 0;
+		show_error_msgf("Can't delete to trash",
+				"Current directory is under trash directory:\n%s",
+				replace_home_part(top_dir));
+		return;
 	}
 
-	snprintf(undo_msg, sizeof(undo_msg), "%celete in %s: ", use_trash ? 'd' : 'D',
-			replace_home_part(curr_dir));
-	if(!nested)
-	{
-		un_group_open(undo_msg);
-	}
-
-	ops = fops_get_ops(OP_REMOVE, use_trash ? "deleting" : "Deleting", curr_dir,
-			curr_dir);
-
-	entry = &view->dir_entry[view->list_pos];
-
 	fops_progress_msg("Deleting files", 0, 1);
 	(void)delete_file(entry, ops, BLACKHOLE_REG_NAME, use_trash, nested);
-
-	if(!nested)
-	{
-		un_group_close();
-		ui_views_reload_filelists();
-	}
-
-	ui_sb_msgf("%d %s %celeted%s", ops->succeeded,
-			(ops->succeeded == 1) ? "file" : "files", use_trash ? 'd' : 'D',
-			fops_get_cancellation_suffix());
-
-	fops_free_ops(ops);
-	return 1;
 }
 
 /* Removes single file specified by its entry.  Returns zero on success,
@@ -252,12 +231,15 @@ delete_file(dir_entry_t *entry, ops_t *ops, int reg, int use_trash, int nested)
 
 	if(!use_trash)
 	{
-		result = perform_operation(OP_REMOVE, ops, NULL, full_path, NULL);
+		OpsResult r = perform_operation(OP_REMOVE, ops, NULL, full_path, NULL);
+		result = (r == OPS_SUCCEEDED ? 0 : -1);
+
 		/* For some reason "rm" sometimes returns 0 on cancellation. */
-		if(path_exists(full_path, NODEREF))
+		if(result == 0 && path_exists(full_path, NODEREF))
 		{
 			result = -1;
 		}
+
 		if(result == 0)
 		{
 			un_group_add_op(OP_REMOVE, NULL, NULL, full_path, "");
@@ -281,12 +263,15 @@ delete_file(dir_entry_t *entry, ops_t *ops, int reg, int use_trash, int nested)
 		char *const dest = trash_gen_path(entry->origin, entry->name);
 		if(dest != NULL)
 		{
-			result = perform_operation(op, ops, NULL, full_path, dest);
+			OpsResult r = perform_operation(op, ops, NULL, full_path, dest);
+			result = (r == OPS_SUCCEEDED ? 0 : -1);
+
 			/* For some reason "rm" sometimes returns 0 on cancellation. */
-			if(path_exists(full_path, DEREF))
+			if(result == 0 && path_exists(full_path, NODEREF))
 			{
 				result = -1;
 			}
+
 			if(result == 0)
 			{
 				un_group_add_op(op, NULL, NULL, full_path, dest);
@@ -310,7 +295,6 @@ int
 fops_delete_bg(view_t *view, int use_trash)
 {
 	char task_desc[COMMAND_GROUP_INFO_LEN];
-	bg_args_t *args;
 	const char *const top_dir = get_top_dir(view);
 	const char *const curr_dir = top_dir == NULL ? flist_get_dir(view) : top_dir;
 
@@ -328,7 +312,13 @@ fops_delete_bg(view_t *view, int use_trash)
 		return 0;
 	}
 
-	args = calloc(1, sizeof(*args));
+	bg_args_t *args = calloc(1, sizeof(*args));
+	if(args == NULL)
+	{
+		show_error_msg("Can't perform deletion", "Out of memory");
+		return 0;
+	}
+
 	args->use_trash = use_trash;
 
 	fops_prepare_for_bg_task(view, args);
@@ -445,6 +435,10 @@ delete_file_in_bg(ops_t *ops, const char path[], int use_trash)
 		return;
 	}
 
+	/* XXX: dealing with the trash unit from *background* thread here and in
+	 *      delete_files_in_bg() is likely to cause a race!
+	 *
+	 *      What to do?  Forbid bg trashing?  Do a snapshot to apply later? */
 	if(!trash_is_at_path(path))
 	{
 		const char *const fname = get_last_path_component(path);
@@ -509,10 +503,6 @@ prepare_register(int reg)
 int
 fops_retarget(view_t *view)
 {
-	char full_path[PATH_MAX + 1];
-	char linkto[PATH_MAX + 1];
-	const dir_entry_t *const entry = get_current_entry(view);
-
 	if(!symlinks_available())
 	{
 		show_error_msg("Symbolic Links Error",
@@ -524,6 +514,67 @@ fops_retarget(view_t *view)
 		return 0;
 	}
 
+	int nfiles;
+	char **files = list_files_to_retarget(view, &nfiles);
+	if(nfiles == -1)
+	{
+		/* An error has occurred. */
+		flist_sel_stash(view);
+		return 1;
+	}
+
+	int result;
+	if(nfiles == 0)
+	{
+		result = retarget_one(view);
+	}
+	else
+	{
+		result = retarget_many(view, files, nfiles);
+	}
+
+	flist_sel_stash(view);
+	free_string_array(files, nfiles);
+	return result;
+}
+
+/* Makes list of symbolic links to be retargeted.  Always sets *len.  Returns
+ * list of files (NULL if empty) or NULL and sets *len to -1 on error. */
+static char **
+list_files_to_retarget(view_t *view, int *len)
+{
+	*len = 0;
+
+	char **files = NULL;
+	dir_entry_t *entry = NULL;
+	while(iter_marked_entries(view, &entry))
+	{
+		char path[PATH_MAX + 1];
+		get_short_path_of(view, entry, NF_NONE, 0, sizeof(path), path);
+
+		if(entry->type != FT_LINK)
+		{
+			ui_sb_errf("File is not a symbolic link: %s", path);
+			free_string_array(files, *len);
+			*len = -1;
+			return NULL;
+		}
+
+		*len = add_to_string_array(&files, *len, path);
+	}
+
+	return files;
+}
+
+/* Changes target of a symbolic link under the cursor.  Returns new value for
+ * save_msg. */
+static int
+retarget_one(view_t *view)
+{
+	char full_path[PATH_MAX + 1];
+	char linkto[PATH_MAX + 1];
+	const dir_entry_t *const entry = get_current_entry(view);
+
 	if(fentry_is_fake(entry))
 	{
 		ui_sb_err("Entry doesn't correspond to a file");
@@ -542,14 +593,14 @@ fops_retarget(view_t *view)
 		return 0;
 	}
 
-	fops_line_prompt("Link target: ", linkto, &change_link_cb, &complete_filename,
-			0);
+	fops_line_prompt("Link target: ", linkto, &change_link_cb, /*cb_arg=*/NULL,
+			&complete_filename, 0);
 	return 0;
 }
 
 /* Handles users response for new link target prompt. */
 static void
-change_link_cb(const char new_target[])
+change_link_cb(const char new_target[], void *arg)
 {
 	char undo_msg[2*PATH_MAX + 32];
 	char full_path[PATH_MAX + 1];
@@ -579,14 +630,7 @@ change_link_cb(const char new_target[])
 			replace_home_part(flist_get_dir(curr_view)), fname, linkto, new_target);
 	un_group_open(undo_msg);
 
-	if(perform_operation(OP_REMOVESL, ops, NULL, full_path, NULL) == 0)
-	{
-		un_group_add_op(OP_REMOVESL, NULL, NULL, full_path, linkto);
-	}
-	if(perform_operation(OP_SYMLINK2, ops, NULL, new_target, full_path) == 0)
-	{
-		un_group_add_op(OP_SYMLINK2, NULL, NULL, new_target, full_path);
-	}
+	change_link(ops, full_path, linkto, new_target);
 
 	un_group_close();
 
@@ -601,6 +645,107 @@ complete_filename(const char str[], void *arg)
 	return name_begin - str + filename_completion(str, CT_ALL_WOE, 0);
 }
 
+/* Changes target of marked fifles.  Returns new value for save_msg. */
+static int
+retarget_many(view_t *view, char *files[], int nfiles)
+{
+	int nfrom = 0;
+	char **from = NULL;
+
+	dir_entry_t *entry = NULL;
+	while(iter_marked_entries(view, &entry) && !ui_cancellation_requested())
+	{
+		char full_path[PATH_MAX + 1];
+		get_full_path_of(entry, sizeof(full_path), full_path);
+
+		char linkto[PATH_MAX + 1];
+		if(get_link_target(full_path, linkto, sizeof(linkto)) != 0)
+		{
+			free_string_array(from, nfrom);
+			show_error_msgf("Error", "Failed to read target of %s", full_path);
+			return 1;
+		}
+
+		nfrom = add_to_string_array(&from, nfrom, linkto);
+	}
+
+	int nto;
+	char **to = fops_query_list(nfrom, from, &nto, /*load_always=*/0,
+			&verify_retarget_list, NULL);
+	if(nto == 0)
+	{
+		free(to);
+		free_string_array(from, nfrom);
+		ui_sb_msg("0 links retargeted");
+		return 1;
+	}
+
+	const char *curr_dir = flist_get_dir(view);
+	ops_t *ops = fops_get_ops(OP_SYMLINK2, "re-targeting", curr_dir, curr_dir);
+
+	char undo_msg[2*PATH_MAX + 32];
+	snprintf(undo_msg, sizeof(undo_msg), "cl in %s: ",
+			replace_home_part(flist_get_dir(view)));
+	fops_append_marked_files(view, undo_msg, to);
+	un_group_open(undo_msg);
+
+	entry = NULL;
+	while(iter_marked_entries(view, &entry) && !ui_cancellation_requested())
+	{
+		char full_path[PATH_MAX + 1];
+		get_full_path_of(entry, sizeof(full_path), full_path);
+		ops_enqueue(ops, full_path, full_path);
+	}
+
+	int i = 0;
+	entry = NULL;
+	while(iter_marked_entries(view, &entry) && !ui_cancellation_requested())
+	{
+		char full_path[PATH_MAX + 1];
+		get_full_path_of(entry, sizeof(full_path), full_path);
+
+		change_link(ops, full_path, from[i], to[i]);
+		ops_advance(ops, /*succeeded=*/1);
+
+		++i;
+	}
+
+	un_group_close();
+
+	ui_sb_msgf("%d link%s retargeted%s", ops->succeeded,
+			(ops->succeeded == 1) ? "" : "s", fops_get_cancellation_suffix());
+
+	fops_free_ops(ops);
+
+	free_string_array(from, nfrom);
+	free_string_array(to, nto);
+	return 1;
+}
+
+/* Checks that retargeting can be performed.  Returns non-zero if so, otherwise
+ * zero is returned along with setting *error. */
+static int
+verify_retarget_list(char *files[], int nfiles, char *names[], int nnames,
+		char **error, void *data)
+{
+	return 1;
+}
+
+/* Changes target of a symbolic link. */
+static void
+change_link(ops_t *ops, const char path[], const char from[], const char to[])
+{
+	if(perform_operation(OP_REMOVESL, ops, NULL, path, NULL) == OPS_SUCCEEDED)
+	{
+		un_group_add_op(OP_REMOVESL, NULL, NULL, path, from);
+	}
+
+	if(perform_operation(OP_SYMLINK2, ops, NULL, to, path) == OPS_SUCCEEDED)
+	{
+		un_group_add_op(OP_SYMLINK2, NULL, NULL, to, path);
+	}
+}
+
 int
 fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 {
@@ -647,7 +792,7 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 	const int from_file = (nlines < 0);
 	if(from_file)
 	{
-		list = fops_query_list(nmarked, marked, &nlines, 0, &verify_list,
+		list = fops_query_list(nmarked, marked, &nlines, 0, &verify_clone_list,
 				&verify_args);
 		if(list == NULL)
 		{
@@ -661,7 +806,7 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 
 	char *error_str = NULL;
 	if(nlines > 0 &&
-			!verify_list(NULL, nmarked, list, nlines, &error_str, &verify_args))
+			!verify_clone_list(NULL, nmarked, list, nlines, &error_str, &verify_args))
 	{
 		if(error_str != NULL)
 		{
@@ -757,8 +902,8 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 /* Checks that cloning can be performed.  Returns non-zero if so, otherwise zero
  * is returned along with setting *error. */
 static int
-verify_list(char *files[], int nfiles, char *names[], int nnames, char **error,
-		void *data)
+verify_clone_list(char *files[], int nfiles, char *names[], int nnames,
+		char **error, void *data)
 {
 	verify_args_t *args = data;
 	return fops_is_name_list_ok(nfiles, nnames, names, NULL, error)
@@ -819,7 +964,8 @@ clone_file(const dir_entry_t *entry, const char path[], const char clone[],
 	to_canonic_path(clone, path, clone_name, sizeof(clone_name));
 	if(path_exists(clone_name, DEREF))
 	{
-		if(perform_operation(OP_REMOVESL, NULL, NULL, clone_name, NULL) != 0)
+		if(perform_operation(OP_REMOVESL, NULL, NULL, clone_name, NULL) !=
+				OPS_SUCCEEDED)
 		{
 			return 1;
 		}
@@ -827,7 +973,8 @@ clone_file(const dir_entry_t *entry, const char path[], const char clone[],
 
 	get_full_path_of(entry, sizeof(full_path), full_path);
 
-	if(perform_operation(OP_COPY, ops, NULL, full_path, clone_name) != 0)
+	if(perform_operation(OP_COPY, ops, NULL, full_path, clone_name) !=
+			OPS_SUCCEEDED)
 	{
 		return 1;
 	}
@@ -875,6 +1022,8 @@ fops_mkdirs(view_t *view, int at, char **names, int count, int create_parent)
 		}
 	}
 
+	ops_t *ops = fops_get_ops(OP_MKDIR, "making dirs", dst_dir, dst_dir);
+
 	snprintf(buf, sizeof(buf), "mkdir in %s: ", replace_home_part(dst_dir));
 
 	get_group_file_list(names, count, buf);
@@ -885,7 +1034,7 @@ fops_mkdirs(view_t *view, int at, char **names, int count, int create_parent)
 		char full[PATH_MAX + 1];
 		to_canonic_path(names[i], dst_dir, full, sizeof(full));
 
-		if(perform_operation(OP_MKDIR, NULL, cp, full, NULL) == 0)
+		if(perform_operation(OP_MKDIR, ops, cp, full, NULL) == OPS_SUCCEEDED)
 		{
 			un_group_add_op(OP_MKDIR, cp, NULL, full, "");
 			++n;
@@ -913,6 +1062,8 @@ fops_mkdirs(view_t *view, int at, char **names, int count, int create_parent)
 
 	ui_sb_msgf("%d director%s created%s", n, (n == 1) ? "y" : "ies",
 			fops_get_cancellation_suffix());
+
+	fops_free_ops(ops);
 	return 1;
 }
 
@@ -965,7 +1116,7 @@ fops_mkfiles(view_t *view, int at, char *names[], int count)
 		char full[PATH_MAX + 1];
 		to_canonic_path(names[i], dst_dir, full, sizeof(full));
 
-		if(perform_operation(OP_MKFILE, ops, NULL, full, NULL) == 0)
+		if(perform_operation(OP_MKFILE, ops, NULL, full, NULL) == OPS_SUCCEEDED)
 		{
 			un_group_add_op(OP_MKFILE, NULL, NULL, full, "");
 			++n;
@@ -1108,6 +1259,12 @@ start_dir_size_calc(const char path[], int force)
 	dir_size_args_t *args;
 
 	args = malloc(sizeof(*args));
+	if(args == NULL)
+	{
+		show_error_msg("Can't calculate size", "Out of memory");
+		return;
+	}
+
 	args->path = strdup(path);
 	args->force = force;
 
@@ -1139,7 +1296,7 @@ dir_size_bg(bg_op_t *bg_op, void *arg)
 /* Calculates directory size and triggers view updates if necessary.  Changes
  * path. */
 static void
-dir_size(bg_op_t *bg_op, char path[], int force)
+dir_size(bg_op_t *bg_op, const char path[], int force)
 {
 	const cancellation_t bg_cancellation_info = {
 		.arg = bg_op,
@@ -1148,10 +1305,10 @@ dir_size(bg_op_t *bg_op, char path[], int force)
 
 	(void)fops_dir_size(path, force, &bg_cancellation_info);
 
-	remove_last_path_component(path);
-
-	redraw_after_path_change(&lwin, path);
-	redraw_after_path_change(&rwin, path);
+	/* Redraw the views unconditionally, because checking their location from a
+	 * background thread will cause a data race. */
+	ui_view_schedule_redraw(&lwin);
+	ui_view_schedule_redraw(&rwin);
 }
 
 /* Implementation of cancellation hook for background tasks. */
@@ -1161,16 +1318,6 @@ bg_cancellation_hook(void *arg)
 	return bg_op_cancelled(arg);
 }
 
-/* Schedules view redraw in case path change might have affected it. */
-static void
-redraw_after_path_change(view_t *view, const char path[])
-{
-	if(path_starts_with(view->curr_dir, path) || flist_custom_active(view))
-	{
-		ui_view_schedule_redraw(view);
-	}
-}
-
 uint64_t
 fops_dir_size(const char path[], int force_update,
 		const cancellation_t *cancellation)
@@ -1181,14 +1328,24 @@ fops_dir_size(const char path[], int force_update,
 
 	time_t mtime = 0;
 	uint64_t inode = DCACHE_UNKNOWN;
-#ifndef _WIN32
 	struct stat s;
 	if(os_stat(path, &s) == 0)
 	{
 		mtime = s.st_mtime;
 		inode = s.st_ino;
 	}
-#endif
+
+	/* The check is at the top and not in the loop to do only one stat() for each
+	 * path. */
+	if(!force_update)
+	{
+		uint64_t dir_size;
+		dcache_get_at(path, mtime, inode, &dir_size, NULL);
+		if(dir_size != DCACHE_UNKNOWN)
+		{
+			return dir_size;
+		}
+	}
 
 	DIR *dir = os_opendir(path);
 	if(dir == NULL)
@@ -1211,13 +1368,7 @@ fops_dir_size(const char path[], int force_update,
 				dentry->d_name);
 		if(fops_is_dir_entry(full_path, dentry))
 		{
-			uint64_t dir_size;
-			dcache_get_at(full_path, mtime, inode, &dir_size, NULL);
-			if(dir_size == DCACHE_UNKNOWN || force_update)
-			{
-				dir_size = fops_dir_size(full_path, force_update, cancellation);
-			}
-			size += dir_size;
+			size += fops_dir_size(full_path, force_update, cancellation);
 		}
 		else
 		{
@@ -1233,6 +1384,8 @@ fops_dir_size(const char path[], int force_update,
 
 	os_closedir(dir);
 
+	/* Could calculate nitems here, but they aren't recursive and might only take
+	 * up memory, because interest in size sort of excludes interest in nitems. */
 	(void)dcache_set_at(path, inode, size, DCACHE_UNKNOWN);
 	return size;
 }
@@ -1243,7 +1396,7 @@ int
 fops_chown(int u, int g, uid_t uid, gid_t gid)
 {
 /* Integer to pointer conversion. */
-#define V(e) (void *)(long)(e)
+#define V(e) ((void *)(uintptr_t)(e))
 
 	view_t *const view = curr_view;
 	char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
@@ -1267,7 +1420,8 @@ fops_chown(int u, int g, uid_t uid, gid_t gid)
 		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
 
-		if(u && perform_operation(OP_CHOWN, ops, V(uid), full_path, NULL) == 0)
+		if(u && perform_operation(OP_CHOWN, ops, V(uid), full_path, NULL) ==
+				OPS_SUCCEEDED)
 		{
 			un_group_add_op(OP_CHOWN, V(uid), V(entry->uid), full_path, "");
 		}
@@ -1276,7 +1430,8 @@ fops_chown(int u, int g, uid_t uid, gid_t gid)
 			full_success -= (u != 0);
 		}
 
-		if(g && perform_operation(OP_CHGRP, ops, V(gid), full_path, NULL) == 0)
+		if(g && perform_operation(OP_CHGRP, ops, V(gid), full_path, NULL) ==
+				OPS_SUCCEEDED)
 		{
 			un_group_add_op(OP_CHGRP, V(gid), V(entry->gid), full_path, "");
 		}
@@ -1306,12 +1461,13 @@ fops_chuser(void)
 		show_error_msg("Change owner", "No files to process");
 		return;
 	}
-	fops_line_prompt("New owner: ", "", &change_owner_cb, &complete_owner, 0);
+	fops_line_prompt("New owner: ", "", &change_owner_cb, /*cb_arg=*/NULL,
+			&complete_owner, 0);
 }
 
 /* Handles users response for new file owner name prompt. */
 static void
-change_owner_cb(const char new_owner[])
+change_owner_cb(const char new_owner[], void *arg)
 {
 	uid_t uid;
 
@@ -1347,12 +1503,13 @@ fops_chgroup(void)
 		show_error_msg("Change group", "No files to process");
 		return;
 	}
-	fops_line_prompt("New group: ", "", &change_group_cb, &complete_group, 0);
+	fops_line_prompt("New group: ", "", &change_group_cb, /*cb_arg=*/NULL,
+			&complete_group, 0);
 }
 
 /* Handles users response for new file group name prompt. */
 static void
-change_group_cb(const char new_group[])
+change_group_cb(const char new_group[], void *arg)
 {
 	gid_t gid;
 
