@@ -21,6 +21,7 @@
 
 #include <assert.h> /* assert() */
 #include <ctype.h> /* tolower() */
+#include <limits.h> /* INT_MAX */
 #include <string.h> /* memmove() memset() strdup() */
 
 #include "cfg/config.h"
@@ -37,6 +38,7 @@
 #include "utils/str.h"
 #include "utils/string_array.h"
 #include "utils/test_helpers.h"
+#include "utils/utf8.h"
 #include "utils/utils.h"
 #include "background.h"
 #include "filelist.h"
@@ -48,6 +50,14 @@
 #include "trash.h"
 #include "undo.h"
 
+/* Information necessary for composing conflict message's body. */
+typedef struct
+{
+	const char *src; /* Full path of the source file. */
+	const char *dst; /* Full path of the destination file. */
+}
+conflict_prompt_data_t;
+
 static void put_files_in_bg(bg_op_t *bg_op, void *arg);
 static int initiate_put_files(view_t *view, int at, CopyMoveLikeOp op,
 		const char descr[], int reg_name);
@@ -59,20 +69,27 @@ static int is_dir_clash(const char src_path[], const char dst_dir[]);
 static int put_files_i(view_t *view, int start);
 static void update_cursor_position(view_t *view);
 static int put_next(int force);
+static int unprocessed_dirs_present(void);
 TSTATIC int merge_dirs(const char src[], const char dst[], ops_t *ops);
 static int handle_clashing(int move, const char src[], const char dst[]);
 static void prompt_what_to_do(const char fname[], const char caused_by[]);
 static void handle_prompt_response(const char fname[], const char caused_by[],
 		char response);
 static void prompt_dst_name(const char src_name[]);
-static void prompt_dst_name_cb(const char dst_name[]);
+static void prompt_dst_name_cb(const char dst_name[], void *arg);
 static void put_continue(int force);
+static char * make_conflict_title(CopyMoveLikeOp op);
+static char * make_conflict_message(int max_w, int max_h, void *data);
+static char * make_conflict_prompt(const conflict_prompt_data_t *data,
+		int max_w, int *block_center);
+static char * prettify_fname(const char full_path[], const struct stat *stat);
+static char cmp_mark(int cmp);
 
 /* Global state for file putting and name conflicts resolution that happen in
  * the process. */
 static struct
 {
-	reg_t *reg;          /* Register used for the operation. */
+	const reg_t *reg;    /* Register used for the operation. */
 	int *file_order;     /* Defines custom ordering of files in register. */
 	view_t *view;        /* View in which operation takes place. */
 	CopyMoveLikeOp op;   /* Type of current operation. */
@@ -82,6 +99,7 @@ static struct
 	int overwrite_all;   /* Overwrite all future conflicting files/directories. */
 	int append;          /* Whether we're appending ending of a file or not. */
 	int allow_merge;     /* Allow merging of files in directories. */
+	int allow_merge_all; /* Allow merging of files in directories by default. */
 	int merge;           /* Merge conflicting directory once. */
 	int merge_all;       /* Merge all conflicting directories. */
 	ops_t *ops;          /* Currently running operation. */
@@ -107,7 +125,6 @@ fops_put_bg(view_t *view, int at, int reg_name, int move)
 	size_t task_desc_len;
 	int i;
 	bg_args_t *args;
-	reg_t *reg;
 	const char *const dst_dir = fops_get_dst_dir(view, at);
 
 	/* Check that operation generally makes sense given our input. */
@@ -118,7 +135,7 @@ fops_put_bg(view_t *view, int at, int reg_name, int move)
 	}
 
 	regs_sync_from_shared_memory();
-	reg = regs_find(tolower(reg_name));
+	const reg_t *reg = regs_find(tolower(reg_name));
 	if(reg == NULL || reg->nfiles < 1)
 	{
 		ui_sb_err(reg == NULL ? "No such register" : "Register is empty");
@@ -174,7 +191,10 @@ fops_put_bg(view_t *view, int at, int reg_name, int move)
 
 		if(!paths_are_equal(src, dst) && path_exists(dst, NODEREF))
 		{
-			ui_sb_errf("File \"%s\" already exists", dst);
+			char *escaped_dst = escape_unreadable(dst);
+			ui_sb_errf("File \"%s\" already exists", escaped_dst);
+			free(escaped_dst);
+
 			fops_free_bg_args(args);
 			return 1;
 		}
@@ -264,7 +284,6 @@ static int
 initiate_put_files(view_t *view, int at, CopyMoveLikeOp op, const char descr[],
 		int reg_name)
 {
-	reg_t *reg;
 	int i;
 	const char *const dst_dir = fops_get_dst_dir(view, at);
 
@@ -274,7 +293,7 @@ initiate_put_files(view_t *view, int at, CopyMoveLikeOp op, const char descr[],
 	}
 
 	regs_sync_from_shared_memory();
-	reg = regs_find(tolower(reg_name));
+	const reg_t *reg = regs_find(tolower(reg_name));
 	if(reg == NULL || reg->nfiles < 1)
 	{
 		ui_sb_err("Register is empty");
@@ -295,7 +314,7 @@ initiate_put_files(view_t *view, int at, CopyMoveLikeOp op, const char descr[],
 		put_confirm.file_order[i] = i;
 	}
 
-	/* If slashes are harmful, order files by descending depth in the file system
+	/* If clashes are harmful, order files by descending depth in the file system
 	 * and then move all that will clash to the tail of the array. */
 	if(op == CMLO_MOVE || op == CMLO_COPY)
 	{
@@ -405,7 +424,7 @@ is_dir_clash(const char src_path[], const char dst_dir[])
 {
 	char dst_path[PATH_MAX + 1];
 
-	snprintf(dst_path, sizeof(dst_path), "%s/%s", dst_dir,
+	build_path(dst_path, sizeof(dst_path), dst_dir,
 			fops_get_dst_name(src_path, trash_has_path(src_path)));
 	chosp(dst_path);
 
@@ -542,7 +561,7 @@ put_next(int force)
 	int move;
 	int success;
 	int merge;
-	int safe_operation = 0;
+	int clashing_op = 0;
 
 	/* TODO: refactor this function (put_next()) */
 
@@ -579,7 +598,7 @@ put_next(int force)
 		dst_name = fops_get_dst_name(src_buf, from_trash);
 	}
 
-	snprintf(dst_buf, sizeof(dst_buf), "%s/%s", dst_dir, dst_name);
+	build_path(dst_buf, sizeof(dst_buf), dst_dir, dst_name);
 	chosp(dst_buf);
 
 	if(!put_confirm.append && path_exists(dst_buf, NODEREF))
@@ -607,12 +626,12 @@ put_next(int force)
 					if(is_in_subtree(src_buf, dst_buf, 0))
 					{
 						/* Don't delete /a/b before moving /a/b/c to /a/b. */
-						safe_operation = 1;
+						clashing_op = 1;
 					}
 				}
 
-				if(!safe_operation && perform_operation(OP_REMOVESL, put_confirm.ops,
-							NULL, dst_buf, NULL) != 0)
+				if(!clashing_op && perform_operation(OP_REMOVESL, put_confirm.ops, NULL,
+							dst_buf, NULL) != OPS_SUCCEEDED)
 				{
 					show_error_msgf("Can't replace a file",
 							"Failed to remove a file:\n%s", dst_buf);
@@ -630,11 +649,25 @@ put_next(int force)
 		{
 			return 0;
 		}
-		else
+		else if(put_confirm.op == CMLO_COPY || put_confirm.op == CMLO_MOVE)
 		{
 			struct stat dst_st;
-			put_confirm.allow_merge = os_lstat(dst_buf, &dst_st) == 0 &&
-					S_ISDIR(dst_st.st_mode) && S_ISDIR(src_st.st_mode);
+			put_confirm.allow_merge = S_ISDIR(src_st.st_mode)
+			                       && (os_lstat(dst_buf, &dst_st) == 0)
+			                       && S_ISDIR(dst_st.st_mode);
+
+			/* Enable "merge all" in conflict resolution options if at least one of
+			 * source paths left to process (including current) is a directory. */
+			put_confirm.allow_merge_all = unprocessed_dirs_present();
+
+			prompt_what_to_do(dst_name, src_buf);
+			return 1;
+		}
+		else
+		{
+			/* Merging makes sense only for copy and move. */
+			put_confirm.allow_merge = 0;
+			put_confirm.allow_merge_all = 0;
 			prompt_what_to_do(dst_name, src_buf);
 			return 1;
 		}
@@ -675,7 +708,7 @@ put_next(int force)
 
 		un_group_reopen_last();
 
-		snprintf(dst_path, sizeof(dst_path), "%s/%s", dst_dir, dst_name);
+		build_path(dst_path, sizeof(dst_path), dst_dir, dst_name);
 
 		if(merge_dirs(src_buf, dst_path, put_confirm.ops) != 0)
 		{
@@ -684,34 +717,37 @@ put_next(int force)
 
 		un_group_close();
 	}
-	else if(safe_operation)
+	else if(clashing_op)
 	{
+		/* We're replacing parent with its child.  To make this possible use a
+		 * temporary path next to the parent first, remove the parent, then move
+		 * data to where the parent was. */
 		const char *const unique_dst = make_name_unique(dst_buf);
 
-		/* An optimization: if we're going to remove destination anyway, don't
-		 * bother copying it, just move. */
-		if(op == OP_COPY)
+		/* An optimization: if we're going to remove original anyway, don't bother
+		 * copying it, just move. */
+		if(op == OP_COPY || op == OP_COPYF)
 		{
 			op = OP_MOVE;
 		}
 
 		success = (perform_operation(op, put_confirm.ops, NULL, src_buf,
-					unique_dst) == 0);
+					unique_dst) == OPS_SUCCEEDED);
 		if(success)
 		{
 			success = (perform_operation(OP_REMOVESL, put_confirm.ops, NULL, dst_buf,
-						NULL) == 0);
+						NULL) == OPS_SUCCEEDED);
 		}
 		if(success)
 		{
 			success = (perform_operation(OP_MOVE, put_confirm.ops, NULL, unique_dst,
-						dst_buf) == 0);
+						dst_buf) == OPS_SUCCEEDED);
 		}
 	}
 	else
 	{
 		success = (perform_operation(op, put_confirm.ops, NULL, src_buf,
-					dst_buf) == 0);
+					dst_buf) == OPS_SUCCEEDED);
 	}
 
 	ops_advance(put_confirm.ops, success);
@@ -757,7 +793,7 @@ put_next(int force)
 		}
 
 		char dst_path[PATH_MAX + 1];
-		snprintf(dst_path, sizeof(dst_path), "%s/%s", dst_dir, dst_name);
+		build_path(dst_path, sizeof(dst_path), dst_dir, dst_name);
 		put_confirm.put.nitems = add_to_string_array(&put_confirm.put.items,
 				put_confirm.put.nitems, dst_path);
 	}
@@ -765,6 +801,30 @@ put_next(int force)
 	return 0;
 }
 
+/* Checks whether current or any other unprocessed paths point to directories.
+ * Returns non-zero if so, otherwise zero is returned. */
+static int
+unprocessed_dirs_present(void)
+{
+	int i;
+	for(i = put_confirm.index; i < put_confirm.reg->nfiles; ++i)
+	{
+		const char *path = put_confirm.reg->files[put_confirm.file_order[i]];
+		if(path == NULL)
+		{
+			/* This file has been excluded from processing. */
+			continue;
+		}
+
+		struct stat src_st;
+		if(os_lstat(path, &src_st) == 0 && S_ISDIR(src_st.st_mode))
+		{
+			return 1;
+		}
+	}
+	return 0;
+}
+
 /* Merges src into dst.  Returns zero on success, otherwise non-zero is
  * returned. */
 TSTATIC int
@@ -773,7 +833,6 @@ merge_dirs(const char src[], const char dst[], ops_t *ops)
 	struct stat st;
 	DIR *dir;
 	struct dirent *d;
-	int result;
 
 	if(os_stat(src, &st) != 0)
 	{
@@ -801,8 +860,8 @@ merge_dirs(const char src[], const char dst[], ops_t *ops)
 			continue;
 		}
 
-		snprintf(src_path, sizeof(src_path), "%s/%s", src, d->d_name);
-		snprintf(dst_path, sizeof(dst_path), "%s/%s", dst, d->d_name);
+		build_path(src_path, sizeof(src_path), src, d->d_name);
+		build_path(dst_path, sizeof(dst_path), dst, d->d_name);
 
 		if(fops_is_dir_entry(dst_path, d))
 		{
@@ -814,7 +873,7 @@ merge_dirs(const char src[], const char dst[], ops_t *ops)
 		else
 		{
 			if(perform_operation(OP_MOVEF, put_confirm.ops, NULL, src_path,
-						dst_path) != 0)
+						dst_path) != OPS_SUCCEEDED)
 			{
 				break;
 			}
@@ -828,8 +887,9 @@ merge_dirs(const char src[], const char dst[], ops_t *ops)
 		return 1;
 	}
 
-	result = perform_operation(OP_RMDIR, put_confirm.ops, NULL, src, NULL);
-	if(result == 0)
+	OpsResult result = perform_operation(OP_RMDIR, put_confirm.ops, NULL, src,
+			NULL);
+	if(result == OPS_SUCCEEDED)
 	{
 		un_group_add_op(OP_RMDIR, NULL, NULL, src, "");
 	}
@@ -839,7 +899,7 @@ merge_dirs(const char src[], const char dst[], ops_t *ops)
 	clone_attribs(dst, src, &st);
 	(void)chmod(dst, st.st_mode);
 
-	return result;
+	return (result == OPS_SUCCEEDED ? 0 : 1);
 }
 
 /* Goes through the rest of files in the register to see whether they are inside
@@ -872,7 +932,7 @@ handle_clashing(int move, const char src[], const char dst[])
 		int i;
 		char msg[PATH_MAX + 1];
 		response_variant responses[] = {
-			{ .key = 'y', .descr = "[y]es " },
+			{ .key = 'y', .descr = "[y]es /" },
 			{ .key = 'n', .descr = " [n]o\n" },
 			{ .key = NC_C_c, .descr = "\nEsc or Ctrl-C to abort" },
 			{}
@@ -884,7 +944,13 @@ handle_clashing(int move, const char src[], const char dst[])
 		snprintf(msg, sizeof(msg), "Overwriting\n%s\nwith\n%s\nwill result "
 				"in loss of the following files.  Are you sure?\n%s",
 				replace_home_part(dst), replace_home_part(src), vle_tb_get_data(lost));
-		switch(fops_options_prompt("Possible data loss", msg, responses))
+
+		const custom_prompt_t prompt = {
+			.title = "Possible data loss",
+			.message = msg,
+			.variants = responses,
+		};
+		switch(fops_options_prompt(&prompt))
 		{
 			case 'y':
 				/* Do nothing. */
@@ -942,32 +1008,35 @@ prompt_what_to_do(const char fname[], const char caused_by[])
 	/* Strange spacing is for left alignment.  Doesn't look nice here, but it is
 	 * problematic to get such alignment otherwise. */
 	static const response_variant
-		rename        = { .key = 'r', .descr = "[r]ename (also Enter)        \n" },
-		enter         = { .key = '\r', .descr = "" },
-		skip          = { .key = 's', .descr = "[s]kip " },
-		skip_all      = { .key = 'S', .descr = " [S]kip all          \n" },
-		append        = { .key = 'a', .descr = "[a]ppend the tail            \n" },
-		overwrite     = { .key = 'o', .descr = "[o]verwrite " },
-		overwrite_all = { .key = 'O', .descr = " [O]verwrite all\n" },
-		merge         = { .key = 'm', .descr = "[m]erge " },
-		merge_all     = { .key = 'M', .descr = " [M]erge all        \n" },
-		escape        = { .key = NC_C_c, .descr = "\nEsc or Ctrl-C to cancel" };
+		rename         = { .key = 'r', .descr = "[r]ename (also Enter)\n" },
+		enter          = { .key = '\r', .descr = "" },
+		skip           = { .key = 's', .descr = "[s]kip /" },
+		skip_all       = { .key = 'S', .descr = " [S]kip all\n" },
+		append         = { .key = 'a', .descr = "[a]ppend the tail\n" },
+		overwrite      = { .key = 'o', .descr = "[o]verwrite /" },
+		overwrite_all  = { .key = 'O', .descr = " [O]verwrite all\n" },
+		merge          = { .key = 'm', .descr = "[m]erge /" },
+		merge_all      = { .key = 'M', .descr = " [M]erge all\n" },
+		merge_all_only = { .key = 'M', .descr = "[M]erge all\n" },
+		escape         = { .key = NC_C_c, .descr = "\n   Esc or Ctrl-C to abort" };
 
-	char response;
-	response_variant responses[11] = {};
+	/* Last element is a terminator. */
+	response_variant responses[12] = {};
 	size_t i = 0;
 
 	char dst_buf[PATH_MAX + 1];
-	snprintf(dst_buf, sizeof(dst_buf), "%s/%s", put_confirm.dst_dir, fname);
+	build_path(dst_buf, sizeof(dst_buf), put_confirm.dst_dir, fname);
 	const int same_file = paths_are_equal(dst_buf, caused_by);
 
 	responses[i++] = rename;
 	responses[i++] = enter;
+
 	responses[i++] = skip;
 	responses[i++] = skip_all;
+
 	if(!same_file)
 	{
-		if(cfg.use_system_calls && is_regular_file_noderef(fname) &&
+		if(cfg.use_system_calls && is_regular_file_noderef(dst_buf) &&
 				is_regular_file_noderef(caused_by))
 		{
 			responses[i++] = append;
@@ -977,30 +1046,45 @@ prompt_what_to_do(const char fname[], const char caused_by[])
 		if(put_confirm.allow_merge)
 		{
 			responses[i++] = merge;
-			responses[i++] = merge_all;
+		}
+		if(put_confirm.allow_merge_all)
+		{
+			responses[i++] = (put_confirm.allow_merge ? merge_all : merge_all_only);
 		}
 	}
 
 	responses[i++] = escape;
-	assert(i < ARRAY_LEN(responses) && "Array is too small.");
+	assert(i + 1 <= ARRAY_LEN(responses) && "Array is too small.");
 
 	/* Screen needs to be restored after displaying progress dialog. */
 	modes_update();
 
-	char msg[PATH_MAX*3];
-	if(same_file)
+	/* Invoking make_conflict_prompt() to get block_center flag and validate the
+	 * operation (we're not showing the prompt if NULL is returned). */
+	conflict_prompt_data_t prompt_data = {
+		.src = caused_by,
+		.dst = dst_buf,
+	};
+	int block_center;
+	char *msg =
+		make_conflict_prompt(&prompt_data, /*max_w=*/INT_MAX, &block_center);
+	free(msg);
+
+	char response = NC_C_c;
+	char *title = make_conflict_title(put_confirm.op);
+	if(title != NULL && msg != NULL)
 	{
-		snprintf(msg, sizeof(msg),
-				"Same file is both source and destination:\n%s\nWhat to do?",
-				replace_home_part(caused_by));
+		const custom_prompt_t prompt = {
+			.title = title,
+			.make_message = &make_conflict_message,
+			.user_data = &prompt_data,
+			.variants = responses,
+			.block_center = block_center,
+		};
+		response = fops_options_prompt(&prompt);
 	}
-	else
-	{
-		snprintf(msg, sizeof(msg),
-				"Name conflict for %s.  Caused by:\n%s\nWhat to do?", fname,
-				replace_home_part(caused_by));
-	}
-	response = fops_options_prompt("File Conflict", msg, responses);
+	free(title);
+
 	handle_prompt_response(fname, caused_by, response);
 }
 
@@ -1010,7 +1094,7 @@ handle_prompt_response(const char fname[], const char caused_by[],
 		char response)
 {
 	char dst_path[PATH_MAX + 1];
-	snprintf(dst_path, sizeof(dst_path), "%s/%s", put_confirm.dst_dir, fname);
+	build_path(dst_path, sizeof(dst_path), put_confirm.dst_dir, fname);
 
 	/* Record last conflict to position cursor at it later. */
 	update_string(&put_confirm.last_conflict, dst_path);
@@ -1048,7 +1132,7 @@ handle_prompt_response(const char fname[], const char caused_by[],
 		put_confirm.merge = 1;
 		put_continue(1);
 	}
-	else if(put_confirm.allow_merge && response == 'M')
+	else if(put_confirm.allow_merge_all && response == 'M')
 	{
 		put_confirm.merge_all = 1;
 		put_continue(1);
@@ -1077,12 +1161,13 @@ prompt_dst_name(const char src_name[])
 	char prompt[128 + PATH_MAX];
 
 	snprintf(prompt, ARRAY_LEN(prompt), "New name for %s: ", src_name);
-	fops_line_prompt(prompt, src_name, &prompt_dst_name_cb, NULL, 0);
+	fops_line_prompt(prompt, src_name, &prompt_dst_name_cb, /*cb_arg=*/NULL, NULL,
+			0);
 }
 
 /* Callback for line prompt result. */
 static void
-prompt_dst_name_cb(const char dst_name[])
+prompt_dst_name_cb(const char dst_name[], void *arg)
 {
 	if(is_null_or_empty(dst_name))
 	{
@@ -1091,7 +1176,7 @@ prompt_dst_name_cb(const char dst_name[])
 
 	/* Record new destination path. */
 	char dst_path[PATH_MAX + 1];
-	snprintf(dst_path, sizeof(dst_path), "%s/%s", put_confirm.dst_dir, dst_name);
+	build_path(dst_path, sizeof(dst_path), put_confirm.dst_dir, dst_name);
 	update_string(&put_confirm.last_conflict, dst_path);
 
 	if(replace_string(&put_confirm.dst_name, dst_name) != 0)
@@ -1118,5 +1203,156 @@ put_continue(int force)
 	}
 }
 
+/* Formats a title for conflict message.  Returns newly allocated string. */
+static char *
+make_conflict_title(CopyMoveLikeOp op)
+{
+	const char *action = "?";
+	switch(op)
+	{
+		case CMLO_COPY:
+			action = "Copying";
+			break;
+		case CMLO_MOVE:
+			action = "Moving";
+			break;
+		case CMLO_LINK_REL:
+		case CMLO_LINK_ABS:
+			action = "Symlinking";
+			break;
+	}
+
+	return format_str("File Conflict on %s", action);
+}
+
+/* Adapts make_conflict_prompt() for use with fops_options_prompt().  Returns
+ * newly allocated string. */
+static char *
+make_conflict_message(int max_w, int max_h, void *data)
+{
+	int block_center;
+	return make_conflict_prompt(data, max_w, &block_center);
+}
+
+/* Produces text for a conflict prompt.  Returns newly allocated string. */
+static char *
+make_conflict_prompt(const conflict_prompt_data_t *data, int max_w,
+		int *block_center)
+{
+	*block_center = 1;
+
+	struct stat src;
+	if(os_lstat(data->src, &src) != 0)
+	{
+		show_error_msgf("Conflict handling error", "Unable to query metadata of %s",
+				data->src);
+		return NULL;
+	}
+
+	char *pretty_src = prettify_fname(data->src, &src);
+	if(pretty_src == NULL)
+	{
+		return NULL;
+	}
+
+	if(paths_are_equal(data->src, data->dst))
+	{
+		*block_center = 0;
+		char *text =
+			format_str("Same file is both source and destination:\n%s", pretty_src);
+		free(pretty_src);
+		return text;
+	}
+
+	struct stat dst;
+	if(os_lstat(data->dst, &dst) != 0)
+	{
+		show_error_msgf("Conflict handling error", "Unable to query metadata of %s",
+				data->dst);
+		free(pretty_src);
+		return NULL;
+	}
+
+	char *pretty_dst = prettify_fname(data->dst, &dst);
+	if(pretty_dst == NULL)
+	{
+		free(pretty_src);
+		return NULL;
+	}
+
+	char src_size[64], dst_size[64];
+	(void)friendly_size_notation(src.st_size, sizeof(src_size), src_size);
+	(void)friendly_size_notation(dst.st_size, sizeof(dst_size), dst_size);
+
+	char src_bsize[64], dst_bsize[64];
+	snprintf(src_bsize, sizeof(src_bsize), " (%" PRINTF_ULL ")",
+			(unsigned long long)src.st_size);
+	snprintf(dst_bsize, sizeof(dst_bsize), " (%" PRINTF_ULL ")",
+			(unsigned long long)dst.st_size);
+
+	char src_mtime[64], dst_mtime[64];
+	format_iso_time(src.st_mtime, src_mtime, sizeof(src_mtime));
+	format_iso_time(dst.st_mtime, dst_mtime, sizeof(dst_mtime));
+
+	int fields[3] = { };
+	fields[0] = MAX(strlen(src_size), strlen(dst_size));
+	fields[1] = MAX(strlen(src_bsize), strlen(dst_bsize));
+	fields[2] = MAX(utf8_strsw(src_mtime), utf8_strsw(dst_mtime));
+
+	const char *info_fmt = "      %c %*s%*s   %c %*s";
+	int l = snprintf(NULL, 0, info_fmt,
+			'x', fields[0], "", fields[1], "",
+			'y', fields[2], "");
+	if(l > max_w)
+	{
+		info_fmt = "      %c %*s%*s\n      %c %*s";
+	}
+
+#define CMP(a, b) ((a) == (b) ? 0 : (a) < (b) ? -1 : 1)
+	int size_cmp = CMP(src.st_size, dst.st_size);
+	int mtime_cmp = CMP(src.st_mtime, dst.st_mtime);
+#undef CMP
+
+	vle_textbuf *text = vle_tb_create();
+
+	vle_tb_append_linef(text, "From: %s", pretty_src);
+	vle_tb_append_linef(text, info_fmt,
+			cmp_mark(size_cmp), fields[0], src_size, fields[1], src_bsize,
+			cmp_mark(mtime_cmp), fields[2], src_mtime);
+
+	vle_tb_append_line(text, " ");
+
+	vle_tb_append_linef(text, "  To: %s", pretty_dst);
+	vle_tb_append_linef(text, info_fmt,
+			cmp_mark(-size_cmp), fields[0], dst_size, fields[1], dst_bsize,
+			cmp_mark(-mtime_cmp), fields[2], dst_mtime);
+
+	free(pretty_src);
+	free(pretty_dst);
+
+	return vle_tb_release(text);
+}
+
+/* Prepares file name for being displayed in a dialog.  Returns newly allocated
+ * string. */
+static char *
+prettify_fname(const char full_path[], const struct stat *stat)
+{
+	char *pretty = escape_unreadable(replace_home_part(full_path));
+	if(pretty != NULL && S_ISDIR(stat->st_mode))
+	{
+		put_string(&pretty, format_str("%s/", pretty));
+	}
+	return pretty;
+}
+
+/* Translates comparison result to one of three marks: <, =, >.  Returns the
+ * mark. */
+static char
+cmp_mark(int cmp)
+{
+	return (cmp == 0 ? '=' : cmp < 0 ? '<' : '>');
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
