@@ -40,6 +40,7 @@
 #include "../utils/fsddata.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
+#include "../utils/utils.h"
 #include "../filetype.h"
 #include "../status.h"
 #include "desktop.h"
@@ -158,8 +159,19 @@ update_cache(fsddata_t *cache, const char path[], const char mimetype[],
 	(void)filemon_from_file(path, FMT_MODIFIED, filemon);
 
 	data = malloc(sizeof(*data));
-	data->mime = strdup(mimetype);
+	if(data == NULL)
+	{
+		return;
+	}
+
 	data->filemon = *filemon;
+	data->mime = strdup(mimetype);
+	if(data->mime == NULL)
+	{
+		free(data);
+		return;
+	}
+
 	fsddata_set(cache, path, data);
 }
 
@@ -244,7 +256,8 @@ get_file_mimetype(const char filename[], char buf[], size_t buf_sz)
 #ifdef HAVE_FILE_PROG
 	FILE *pipe;
 	char command[1024];
-	char *const escaped_filename = shell_like_escape(filename, 0);
+	ShellType shell_type = (get_env_type() == ET_UNIX ? ST_POSIX : ST_CMD);
+	char *const escaped_filename = shell_arg_escape(filename, shell_type);
 
 	/* Use the file command to get mimetype */
 	snprintf(command, sizeof(command), "file %s -b --mime-type",
@@ -277,6 +290,11 @@ get_handlers(const char mime_type[])
 	static assoc_records_t handlers;
 	ft_assoc_records_free(&handlers);
 
+	if(mime_type == NULL)
+	{
+		return handlers;
+	}
+
 #if !defined(_WIN32) && defined(ENABLE_DESKTOP_FILES)
 	parse_app_dir("/usr/share/applications", mime_type, &handlers);
 	parse_app_dir("/usr/local/share/applications", mime_type, &handlers);
