@@ -24,6 +24,7 @@
 #include "../cfg/config.h"
 #include "../modes/cmdline.h"
 #include "../modes/menu.h"
+#include "../modes/normal.h"
 #include "../ui/ui.h"
 #include "../utils/hist.h"
 #include "../utils/string_array.h"
@@ -38,6 +39,7 @@ typedef enum
 	BSEARCHHISTORY, /* Backward search history. */
 	PROMPTHISTORY,  /* Prompt input history. */
 	FILTERHISTORY,  /* Local filter history. */
+	EXPRREGHISTORY, /* Expression register history. */
 }
 HistoryType;
 
@@ -82,6 +84,13 @@ show_filterhistory_menu(view_t *view)
 			"Filter History");
 }
 
+int
+show_exprreghistory_menu(struct view_t *view)
+{
+	return show_history(view, EXPRREGHISTORY, &curr_stats.exprreg_hist,
+			"Expression Register History");
+}
+
 /* Returns non-zero if status bar message should be saved. */
 static int
 show_history(view_t *view, HistoryType type, hist_t *hist, const char title[])
@@ -113,20 +122,23 @@ execute_history_cb(view_t *view, menu_data_t *m)
 	{
 		case CMDHISTORY:
 			hists_commands_save(line);
-			exec_commands(line, view, CIT_COMMAND);
+			cmds_dispatch(line, view, CIT_COMMAND);
 			break;
 		case FSEARCHHISTORY:
 			hists_search_save(line);
-			exec_commands(line, view, CIT_FSEARCH_PATTERN);
+			modnorm_set_search_attrs(/*count=*/1, /*last_search_backward=*/0);
+			cmds_dispatch1(line, view, CIT_FSEARCH_PATTERN);
 			break;
 		case BSEARCHHISTORY:
 			hists_search_save(line);
-			exec_commands(line, view, CIT_BSEARCH_PATTERN);
+			modnorm_set_search_attrs(/*count=*/1, /*last_search_backward=*/1);
+			cmds_dispatch1(line, view, CIT_BSEARCH_PATTERN);
 			break;
 		case FILTERHISTORY:
 			hists_filter_save(line);
-			exec_commands(line, view, CIT_FILTER_PATTERN);
+			cmds_dispatch1(line, view, CIT_FILTER_PATTERN);
 			break;
+		case EXPRREGHISTORY:
 		case PROMPTHISTORY:
 			/* Can't replay prompt input. */
 			break;
@@ -146,11 +158,22 @@ history_khandler(view_t *view, menu_data_t *m, const wchar_t keys[])
 		CmdLineSubmode submode = CLS_COMMAND;
 		switch((HistoryType)m->extra_data)
 		{
-			case CMDHISTORY:     submode = CLS_COMMAND; break;
-			case FSEARCHHISTORY: submode = CLS_FSEARCH; break;
-			case BSEARCHHISTORY: submode = CLS_BSEARCH; break;
-			case FILTERHISTORY:  submode = CLS_FILTER; break;
+			case CMDHISTORY:
+				submode = CLS_COMMAND;
+				break;
+			case FSEARCHHISTORY:
+				submode = CLS_FSEARCH;
+				modnorm_set_search_attrs(/*count=*/1, /*last_search_backward=*/0);
+				break;
+			case BSEARCHHISTORY:
+				submode = CLS_BSEARCH;
+				modnorm_set_search_attrs(/*count=*/1, /*last_search_backward=*/1);
+				break;
+			case FILTERHISTORY:
+				submode = CLS_FILTER;
+				break;
 
+			case EXPRREGHISTORY:
 			case PROMPTHISTORY:
 				/* Can't edit prompt input. */
 				return KHR_UNHANDLED;
