@@ -33,6 +33,7 @@
 #include "../compat/fs_limits.h"
 #include "../compat/os.h"
 #include "../compat/reallocarray.h"
+#include "../engine/mode.h"
 #include "../int/term_title.h"
 #include "../int/vim.h"
 #include "../modes/dialogs/msg_dialog.h"
@@ -299,16 +300,21 @@ menus_set_pos(menu_state_t *ms, int pos)
 	else
 	{
 		draw_menu_item(ms, m->pos, ms->current, 0);
+		show_position_in_menu(m);
 	}
-	checked_wmove(menu_win, ms->current, 2);
 
-	show_position_in_menu(m);
+	checked_wmove(menu_win, ms->current, 2);
 }
 
 /* Displays current menu position on a ruler. */
 static void
 show_position_in_menu(const menu_data_t *m)
 {
+	if(modes_is_cmdline_like())
+	{
+		return;
+	}
+
 	char pos_buf[32];
 	snprintf(pos_buf, sizeof(pos_buf), " %d-%d ", m->pos + 1, m->len);
 
@@ -441,6 +447,8 @@ menus_partial_redraw(menu_state_t *m)
 	{
 		draw_menu_item(m, pos, i + 1, 0);
 	}
+
+	show_position_in_menu(m->d);
 }
 
 /* Draws single menu item at position specified by line argument.  Non-zero
@@ -467,15 +475,17 @@ draw_menu_item(menu_state_t *ms, int pos, int line, int clear)
 	}
 	int color_pair = cs_load_color(&col);
 
+	char *escaped = escape_unreadable(m->items[pos]);
+
 	/* Calculate offset of m->hor_pos's character in item text. */
 	off = 0;
 	i = m->hor_pos;
-	while(i-- > 0 && m->items[pos][off] != '\0')
+	while(i-- > 0 && escaped[0] != '\0')
 	{
-		off += utf8_chrw(m->items[pos] + off);
+		off += utf8_chrw(escaped + off);
 	}
 
-	item_tail = strdup(m->items[pos] + off);
+	item_tail = escaped + off;
 	replace_char(item_tail, '\t', ' ');
 
 	ui_set_attr(menu_win, &col, color_pair);
@@ -491,7 +501,8 @@ draw_menu_item(menu_state_t *ms, int pos, int line, int clear)
 	if(utf8_strsw(item_tail) > (size_t)(width - 2))
 	{
 		char *ellipsed = right_ellipsis(item_tail, width - 2, curr_stats.ellipsis);
-		free(item_tail);
+		free(escaped);
+		escaped = ellipsed;
 		item_tail = ellipsed;
 	}
 	else
@@ -506,11 +517,11 @@ draw_menu_item(menu_state_t *ms, int pos, int line, int clear)
 	if(ms->search_highlight && ms->matches != NULL && ms->matches[pos][0] >= 0)
 	{
 		const cchar_t cch = cs_color_to_cchar(&col, color_pair);
-		draw_search_match(item_tail, ms->matches[pos][0] - m->hor_pos,
-				ms->matches[pos][1] - m->hor_pos, line, width, &cch);
+		draw_search_match(item_tail, ms->matches[pos][0] - off,
+				ms->matches[pos][1] - off, line, width, &cch);
 	}
 
-	free(item_tail);
+	free(escaped);
 }
 
 /* Draws search match highlight on the element. */
@@ -518,18 +529,27 @@ static void
 draw_search_match(char str[], int start, int end, int line, int width,
 		const cchar_t *attrs)
 {
+	if(start == end)
+	{
+		/* Empty match is not drawn. */
+		return;
+	}
+
 	const int len = strlen(str);
 
+	/* XXX: conditions for displaying <<< and >>> are for some reason different in
+	 *      file views. */
+
 	if(end <= 0)
 	{
-		/* Match is completely at the left. */
+		/* Match is completely on the left. */
 
 		checked_wmove(menu_win, line, 2);
 		wprinta(menu_win, "<<<", attrs, A_REVERSE);
 	}
 	else if(start >= len)
 	{
-		/* Match is completely at the right. */
+		/* Match is completely on the right. */
 
 		checked_wmove(menu_win, line, width - 3);
 		wprinta(menu_win, ">>>", attrs, A_REVERSE);
@@ -689,7 +709,7 @@ menus_get_targets(view_t *view)
 	if(view->selected_files > 0 ||
 			(view->pending_marking && flist_count_marked(view) > 0))
 	{
-		return ma_expand("%f", NULL, NULL, 1);
+		return ma_expand("%f", NULL, NULL, MER_SHELL_OP);
 	}
 
 	if(!flist_custom_active(view))
@@ -844,7 +864,7 @@ menus_capture(view_t *view, const char cmd[], int user_sh, menu_data_t *m,
 	if(ma_flags_present(flags, MF_CUSTOMVIEW_OUTPUT) ||
 			ma_flags_present(flags, MF_VERYCUSTOMVIEW_OUTPUT))
 	{
-		rn_for_flist(view, cmd, m->title, flags);
+		rn_for_flist(view, cmd, m->title, user_sh, flags);
 		menus_reset_data(m);
 		return 0;
 	}
@@ -960,7 +980,7 @@ search_menu(menu_state_t *ms, int start_pos, int print_errors)
 	}
 
 	cflags = get_regexp_cflags(ms->regexp);
-	err = regcomp(&re, ms->regexp, cflags);
+	err = regexp_compile(&re, ms->regexp, cflags);
 	if(err != 0)
 	{
 		if(print_errors)
@@ -974,10 +994,13 @@ search_menu(menu_state_t *ms, int start_pos, int print_errors)
 	for(i = 0; i < m->len; ++i)
 	{
 		regmatch_t matches[1];
-		if(regexec(&re, m->items[i], 1, matches, 0) == 0)
+		const char *item = m->items[i];
+		if(regexec(&re, item, 1, matches, 0) == 0)
 		{
-			ms->matches[i][0] = matches[0].rm_so;
-			ms->matches[i][1] = matches[0].rm_eo;
+			ms->matches[i][0] = matches[0].rm_so
+			                  + escape_unreadableo(item, matches[0].rm_so);
+			ms->matches[i][1] = matches[0].rm_eo
+			                  + escape_unreadableo(item, matches[0].rm_eo);
 
 			++ms->matching_entries;
 		}
@@ -1104,7 +1127,7 @@ menus_search_print_msg(const menu_data_t *m)
 	}
 
 	cflags = get_regexp_cflags(ms->regexp);
-	err = regcomp(&re, ms->regexp, cflags);
+	err = regexp_compile(&re, ms->regexp, cflags);
 
 	if(err != 0)
 	{
@@ -1154,9 +1177,9 @@ menus_search_reset_hilight(menu_state_t *m)
 }
 
 int
-menus_search_matched(menu_state_t *m)
+menus_search_matched(const menu_data_t *m)
 {
-	return m->matching_entries;
+	return m->state->matching_entries;
 }
 
 void
