@@ -44,6 +44,7 @@
 #include "../../utils/str.h"
 #include "../../utils/test_helpers.h"
 #include "../../utils/utf8.h"
+#include "../../utils/utils.h"
 #include "../../filelist.h"
 #include "../../flist_sel.h"
 #include "../../ops.h"
@@ -178,7 +179,8 @@ enter_attr_mode(view_t *active_view)
 	}
 
 	vle_mode_set(ATTR_MODE, VMT_SECONDARY);
-	clear_input_bar();
+	ui_hide_graphics();
+	modes_input_bar_clear();
 	curr_stats.use_input_bar = 0;
 
 	/* Values:
@@ -268,7 +270,7 @@ redraw_attr_dialog(void)
 	free(title);
 
 	checked_wmove(change_win, curr, col);
-	curs_set(1);
+	ui_set_cursor(/*visibility=*/1);
 	ui_refresh_win(change_win);
 }
 
@@ -299,7 +301,6 @@ get_perm_mark(int i)
 static char *
 get_title(int max_width)
 {
-	char *ellipsed, *title;
 	const int first_file_index = get_first_file_index();
 
 	if(!is_one_file_selected(first_file_index))
@@ -307,10 +308,14 @@ get_title(int max_width)
 		return format_str(" %d files ", get_selection_size(first_file_index));
 	}
 
-	ellipsed = right_ellipsis(view->dir_entry[first_file_index].name,
-			max_width - 2, curr_stats.ellipsis);
-	title = format_str(" %s ", ellipsed);
+	char *escaped = escape_unreadable(view->dir_entry[first_file_index].name);
+
+	char *ellipsed = right_ellipsis(escaped, max_width - 2, curr_stats.ellipsis);
+	free(escaped);
+
+	char *title = format_str(" %s ", ellipsed);
 	free(ellipsed);
+
 	return title;
 }
 
@@ -366,7 +371,7 @@ static void
 leave_attr_mode(void)
 {
 	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
-	curs_set(0);
+	ui_set_cursor(/*visibility=*/0);
 	curr_stats.use_input_bar = 1;
 
 	flist_sel_stash(view);
@@ -400,11 +405,17 @@ set_perm_string(view_t *view, const int perms[13], const int origin_perms[13],
 		int adv_perms[3])
 {
 	int i = 0;
-	char *add_perm[] = {"u+r", "u+w", "u+x", "u+s", "g+r", "g+w", "g+x", "g+s",
-											"o+r", "o+w", "o+x", "o+t"};
-	char *sub_perm[] = {"u-r", "u-w", "u-x", "u-s", "g-r", "g-w", "g-x", "g-s",
-											"o-r", "o-w", "o-x", "o-t"};
-	char *add_adv_perm[] = {"u-x+X", "g-x+X", "o-x+X"};
+	const char *add_perm[] = {
+		"u+r", "u+w", "u+x", "u+s",
+		"g+r", "g+w", "g+x", "g+s",
+		"o+r", "o+w", "o+x", "o+t"
+	};
+	const char *sub_perm[] = {
+		"u-r", "u-w", "u-x", "u-s",
+		"g-r", "g-w", "g-x", "g-s",
+		"o-r", "o-w", "o-x", "o-t"
+	};
+	const char *add_adv_perm[] = { "u-x+X", "g-x+X", "o-x+X" };
 	char perm_str[64] = " ";
 	size_t perm_str_len = strlen(perm_str);
 
@@ -505,7 +516,7 @@ file_chmod(char *path, const char *mode, const char *inv_mode, int recurse_dirs)
 {
 	int op = recurse_dirs ? OP_CHMODR : OP_CHMOD;
 
-	if(perform_operation(op, NULL, (void *)mode, path, NULL) == 0)
+	if(perform_operation(op, NULL, (void *)mode, path, NULL) == OPS_SUCCEEDED)
 	{
 		un_group_add_op(op, strdup(mode), strdup(inv_mode), path, "");
 	}
@@ -650,13 +661,10 @@ cmd_space(key_info_t key_info, keys_info_t *keys_info)
 		{
 			perms[permnum] = -1;
 		}
-		else if(perms[permnum] == 0)
-		{
-			perms[permnum] = 1;
-		}
 		else
 		{
-			perms[permnum] = 0;
+			/* perms[permnum] < 0 is handled above, so it must be 0 here. */
+			perms[permnum] = 1;
 		}
 	}
 	else
