@@ -37,6 +37,7 @@
 #include "../../utils/path.h"
 #include "../../utils/str.h"
 #include "../../utils/utf8.h"
+#include "../../utils/utils.h"
 #include "../../filelist.h"
 #include "../../flist_sel.h"
 #include "../../status.h"
@@ -161,7 +162,8 @@ enter_attr_mode(view_t *active_view)
 
 	view = active_view;
 	vle_mode_set(ATTR_MODE, VMT_SECONDARY);
-	clear_input_bar();
+	ui_hide_graphics();
+	modes_input_bar_clear();
 	curr_stats.use_input_bar = 0;
 
 	init();
@@ -227,7 +229,6 @@ get_attrs(void)
 static char *
 get_title(int max_width)
 {
-	char *ellipsed, *title;
 	const int first_file_index = get_first_file_index();
 
 	if(!is_one_file_selected(first_file_index))
@@ -235,10 +236,14 @@ get_title(int max_width)
 		return format_str(" %d files ", get_selection_size(first_file_index));
 	}
 
-	ellipsed = right_ellipsis(view->dir_entry[first_file_index].name,
-			max_width - 2, curr_stats.ellipsis);
-	title = format_str(" %s ", ellipsed);
+	char *escaped = escape_unreadable(view->dir_entry[first_file_index].name);
+
+	char *ellipsed = right_ellipsis(escaped, max_width - 2, curr_stats.ellipsis);
+	free(escaped);
+
+	char *title = format_str(" %s ", ellipsed);
 	free(ellipsed);
+
 	return title;
 }
 
@@ -439,7 +444,8 @@ file_attrib(char *path, DWORD add, DWORD sub, int recurse_dirs)
 	if(add != 0)
 	{
 		const size_t wadd = add;
-		if(perform_operation(OP_ADDATTR, NULL, (void *)wadd, path, NULL) == 0)
+		if(perform_operation(OP_ADDATTR, NULL, (void *)wadd, path, NULL) ==
+				OPS_SUCCEEDED)
 		{
 			un_group_add_op(OP_ADDATTR, (void *)wadd, (void *)(~attrs & wadd), path,
 					"");
@@ -448,7 +454,8 @@ file_attrib(char *path, DWORD add, DWORD sub, int recurse_dirs)
 	if(sub != 0)
 	{
 		const size_t wsub = sub;
-		if(perform_operation(OP_SUBATTR, NULL, (void *)wsub, path, NULL) == 0)
+		if(perform_operation(OP_SUBATTR, NULL, (void *)wsub, path, NULL) ==
+				OPS_SUCCEEDED)
 		{
 			un_group_add_op(OP_SUBATTR, (void *)wsub, (void *)(~attrs & wsub), path,
 					"");
@@ -512,14 +519,10 @@ cmd_space(key_info_t key_info, keys_info_t *keys_info)
 		}
 		else if(attrs[attr_num] == 0)
 		{
+			/* attrs[attr_num] < 0 is handled above, so it must be 0 here. */
 			c = '*';
 			attrs[attr_num] = 1;
 		}
-		else
-		{
-			c = ' ';
-			attrs[attr_num] = 0;
-		}
 	}
 	else
 	{
