@@ -31,6 +31,7 @@
 #include "../../engine/keys.h"
 #include "../../engine/mode.h"
 #include "../../modes/modes.h"
+#include "../../ui/quickview.h"
 #include "../../ui/statusline.h"
 #include "../../ui/ui.h"
 #include "../../utils/macros.h"
@@ -40,13 +41,18 @@
 #include "../../status.h"
 #include "../wk.h"
 
+/* Both vertical and horizontal margin for the dialog. */
+#define MARGIN 1
+
 /* Kinds of dialogs. */
 typedef enum
 {
-	D_ERROR,              /* Error message. */
-	D_QUERY_WITHOUT_LIST, /* User query with all lines centered. */
-	D_QUERY_WITH_LIST,    /* User query with left aligned list and one line
-	                         centered at the top. */
+	D_ERROR,              /* Error message.  All lines are left-aligned unless
+	                         message is a single line that doesn't wrap, in which
+	                         case it's centered. */
+	D_QUERY_CENTER_EACH,  /* Query with each line centered on its own. */
+	D_QUERY_CENTER_FIRST, /* Query with the first line centered. */
+	D_QUERY_CENTER_BLOCK, /* Query with all lines centered as a block. */
 }
 Dialog;
 
@@ -61,29 +67,46 @@ typedef enum
 }
 DialogResult;
 
+/* Internal information about a prompt. */
+typedef struct
+{
+	const custom_prompt_t details; /* Essential parts of prompt description. */
+
+	/* Configuration. */
+	const Dialog kind;     /* Internal dialog kind. */
+	const int prompt_skip; /* Whether to allow skipping future errors. */
+	const int accept_mask; /* Mask of allowed DR_* results. */
+
+	/* State. */
+	int exit_requested;  /* Main loop quit flag. */
+	DialogResult result; /* Dialog result. */
+	char custom_result;  /* One of user-defined input keys. */
+}
+dialog_data_t;
+
 static int def_handler(wchar_t key);
 static void cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_m(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_n(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_y(key_info_t key_info, keys_info_t *keys_info);
-static void handle_response(DialogResult dr);
-static void leave(DialogResult dr);
+static void handle_response(dialog_data_t *data, DialogResult dr);
+static void leave(dialog_data_t *data, DialogResult dr, char custom_result);
 static int prompt_error_msg_internalv(const char title[], const char format[],
 		int prompt_skip, va_list pa);
 static int prompt_error_msg_internal(const char title[], const char message[],
 		int prompt_skip);
-static void prompt_msg_internal(const char title[], const char message[],
-		const response_variant variants[], int with_list);
-static void enter(int result_mask);
-static void redraw_error_msg(const char title_arg[], const char message_arg[],
-		int prompt_skip, int lazy);
-static const char * get_control_msg(Dialog msg_kind, int global_skip);
+static void prompt_msg_internal(dialog_data_t *data);
+static void enter(dialog_data_t *data);
+static void redraw_error_msg(const dialog_data_t *data, int lazy);
+static const char * get_control_msg(const dialog_data_t *data);
 static const char * get_custom_control_msg(const response_variant responses[]);
 static void draw_msg(const char title[], const char msg[],
-		const char ctrl_msg[], int lines_to_center, int recommended_width);
-static size_t measure_sub_lines(const char msg[], size_t *max_len);
-static size_t determine_width(const char msg[]);
+		const char ctrl_msg[], int lines_to_center, int block_center,
+		int recommended_width);
+static size_t measure_sub_lines(const char msg[], int skip_empty,
+		size_t *max_len);
+static size_t measure_text_width(const char msg[]);
 
 /* List of builtin key bindings. */
 static keys_add_info_t builtin_cmds[] = {
@@ -95,19 +118,8 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_y,   {{&cmd_y},      .descr = "confirm the query"}},
 };
 
-/* Main loop quit flag. */
-static int quit;
-/* Dialog result. */
-static DialogResult dialog_result;
-/* Bit mask of R_* kinds of results that are allowed. */
-static int accept_mask;
-/* Type of active dialog message. */
-static Dialog msg_kind;
-
-/* Possible responses for custom prompt message. */
-static const response_variant *responses;
-/* One of user-defined input keys. */
-static char custom_result;
+/* Currently active dialog or NULL. */
+static dialog_data_t *current_dialog;
 
 void
 init_msg_dialog_mode(void)
@@ -132,13 +144,12 @@ def_handler(wchar_t key)
 		return 0;
 	}
 
-	const response_variant *response = responses;
+	const response_variant *response = current_dialog->details.variants;
 	while(response != NULL && response->key != '\0')
 	{
 		if(response->key == (char)key)
 		{
-			custom_result = key;
-			leave(DR_CUSTOM);
+			leave(current_dialog, DR_CUSTOM, key);
 			break;
 		}
 		++response;
@@ -150,7 +161,7 @@ def_handler(wchar_t key)
 static void
 cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info)
 {
-	handle_response(DR_CANCEL);
+	handle_response(current_dialog, DR_CANCEL);
 }
 
 /* Redraws the screen. */
@@ -164,27 +175,27 @@ cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_m(key_info_t key_info, keys_info_t *keys_info)
 {
-	handle_response(DR_OK);
+	handle_response(current_dialog, DR_OK);
 }
 
 /* Denies the query. */
 static void
 cmd_n(key_info_t key_info, keys_info_t *keys_info)
 {
-	handle_response(DR_NO);
+	handle_response(current_dialog, DR_NO);
 }
 
 /* Confirms the query. */
 static void
 cmd_y(key_info_t key_info, keys_info_t *keys_info)
 {
-	handle_response(DR_YES);
+	handle_response(current_dialog, DR_YES);
 }
 
 /* Processes users choice.  Leaves the mode if the result is found among the
  * list of expected results. */
 static void
-handle_response(DialogResult dr)
+handle_response(dialog_data_t *data, DialogResult dr)
 {
 	/* Map result to corresponding input key to omit branching per handler. */
 	static const char r_to_c[] = {
@@ -196,27 +207,28 @@ handle_response(DialogResult dr)
 
 	(void)def_handler(r_to_c[dr]);
 	/* Default handler might have already requested quitting. */
-	if(!quit)
+	if(!data->exit_requested)
 	{
-		if(accept_mask & MASK(dr))
+		if(data->accept_mask & MASK(dr))
 		{
-			leave(dr);
+			leave(data, dr, /*custom_result=*/'?');
 		}
 	}
 }
 
 /* Leaves the mode with given result. */
 static void
-leave(DialogResult dr)
+leave(dialog_data_t *data, DialogResult dr, char custom_result)
 {
-	dialog_result = dr;
-	quit = 1;
+	data->result = dr;
+	data->custom_result = custom_result;
+	data->exit_requested = 1;
 }
 
 void
 redraw_msg_dialog(int lazy)
 {
-	redraw_error_msg(NULL, NULL, 0, lazy);
+	redraw_error_msg(/*data=*/NULL, lazy);
 }
 
 void
@@ -297,27 +309,40 @@ prompt_error_msg_internal(const char title[], const char message[],
 		return 0;
 	}
 
-	msg_kind = D_ERROR;
-
-	redraw_error_msg(title, message, prompt_skip, 0);
-
-	enter(MASK(DR_OK) | (prompt_skip ? MASK(DR_CANCEL) : 0));
+	dialog_data_t data = {
+		.details = {
+			.title = title,
+			.message = message,
+		},
+		.kind = D_ERROR,
+		.prompt_skip = prompt_skip,
+		.accept_mask = MASK(DR_OK) | (prompt_skip ? MASK(DR_CANCEL) : 0),
+	};
+	enter(&data);
 
 	if(curr_stats.load_stage < 2)
 	{
-		skip_until_started = (dialog_result == DR_CANCEL);
+		skip_until_started = (data.result == DR_CANCEL);
 	}
 
 	modes_redraw();
 
-	return (dialog_result == DR_CANCEL);
+	return (data.result == DR_CANCEL);
 }
 
 int
 prompt_msg(const char title[], const char message[])
 {
-	prompt_msg_internal(title, message, NULL, 0);
-	return (dialog_result == DR_YES);
+	dialog_data_t data = {
+		.details = {
+			.title = title,
+			.message = message,
+		},
+		.kind = D_QUERY_CENTER_EACH,
+		.accept_mask = MASK(DR_YES, DR_NO),
+	};
+	prompt_msg_internal(&data);
+	return (data.result == DR_YES);
 }
 
 int
@@ -335,89 +360,106 @@ prompt_msgf(const char title[], const char format[], ...)
 }
 
 char
-prompt_msg_custom(const char title[], const char message[],
-		const response_variant variants[])
+prompt_msg_custom(const custom_prompt_t *details)
 {
-	assert(variants[0].key != '\0' && "Variants should have at least one item.");
-	prompt_msg_internal(title, message, variants, 0);
-	return custom_result;
+	assert(details->variants[0].key != '\0' &&
+			"Variants should have at least one item.");
+
+	dialog_data_t data = {
+		.details = *details,
+		.kind = details->block_center ? D_QUERY_CENTER_BLOCK : D_QUERY_CENTER_EACH,
+	};
+
+	prompt_msg_internal(&data);
+	return data.custom_result;
 }
 
-/* Common implementation of prompt message.  The variants can be NULL. */
+/* Common implementation of prompt message. */
 static void
-prompt_msg_internal(const char title[], const char message[],
-		const response_variant variants[], int with_list)
+prompt_msg_internal(dialog_data_t *data)
 {
-	responses = variants;
-	msg_kind = (with_list ? D_QUERY_WITH_LIST : D_QUERY_WITHOUT_LIST);
-
-	redraw_error_msg(title, message, 0, 0);
-
-	enter(variants == NULL ? MASK(DR_YES, DR_NO) : 0);
-
+	enter(data);
 	modes_redraw();
 }
 
 /* Enters the mode, which won't be left until one of expected results specified
  * by the mask is picked by the user. */
 static void
-enter(int result_mask)
+enter(dialog_data_t *data)
 {
+	dialog_data_t *prev_dialog = current_dialog;
 	const int prev_use_input_bar = curr_stats.use_input_bar;
 	const vle_mode_t prev_mode = vle_mode_get();
 
-	/* Message must be displayed to the user for him to close it.  It won't happen
+	/* Message must be visible to the user for him to close it.  It won't happen
 	 * without user interaction as new event loop is out of scope of a mapping
 	 * that started it. */
 	stats_unsilence_ui();
 
-	accept_mask = result_mask;
+	ui_hide_graphics();
+
+	current_dialog = data;
 	curr_stats.use_input_bar = 0;
 	vle_mode_set(MSG_MODE, VMT_SECONDARY);
 
-	quit = 0;
+	redraw_error_msg(data, /*lazy=*/0);
+
+	data->exit_requested = 0;
 	/* Avoid starting nested loop in tests. */
 	if(curr_stats.load_stage > 0)
 	{
-		event_loop(&quit);
+		event_loop(&data->exit_requested, /*manage_marking=*/0);
 	}
 
 	vle_mode_set(prev_mode, VMT_SECONDARY);
 	curr_stats.use_input_bar = prev_use_input_bar;
+	current_dialog = prev_dialog;
 }
 
 /* Draws error message on the screen or redraws the last message when both
  * title_arg and message_arg are NULL. */
 static void
-redraw_error_msg(const char title_arg[], const char message_arg[],
-		int prompt_skip, int lazy)
+redraw_error_msg(const dialog_data_t *data, int lazy)
 {
-	/* TODO: refactor this function redraw_error_msg() */
+	static const dialog_data_t *last_data;
 
-	static const char *title;
-	static const char *message;
-	static int ctrl_c;
-
-	const char *ctrl_msg;
-	const int lines_to_center = msg_kind == D_QUERY_WITHOUT_LIST ? INT_MAX
-	                          : msg_kind == D_QUERY_WITH_LIST ? 1 : 0;
-
-	if(title_arg != NULL && message_arg != NULL)
+	if(data == NULL)
 	{
-		title = title_arg;
-		message = message_arg;
-		ctrl_c = prompt_skip;
+		data = last_data;
+	}
+	else
+	{
+		last_data = data;
+	}
+	assert(data != NULL && "Invalid dialog redraw request!");
+
+	const char *message = data->details.message;
+	char *free_me = NULL;
+	if(data->details.make_message != NULL)
+	{
+		int sw, sh;
+		getmaxyx(stdscr, sh, sw);
+		int max_w = sw - 2 - 2 - MARGIN*2;
+		int max_h = sh - 2 - ui_stat_height() - 2 - MARGIN*2;
+
+		free_me = data->details.make_message(max_w, max_h, data->details.user_data);
+		if(free_me == NULL)
+		{
+			return;
+		}
+
+		message = free_me;
 	}
 
-	if(title == NULL || message == NULL)
-	{
-		assert(title != NULL && "Asked to redraw dialog, but no title is set.");
-		assert(message != NULL && "Asked to redraw dialog, but no message is set.");
-		return;
-	}
+	const char *ctrl_msg = get_control_msg(data);
+	int lines_to_center = (data->kind == D_QUERY_CENTER_EACH) ? INT_MAX
+	                    : (data->kind == D_QUERY_CENTER_BLOCK) ? INT_MAX
+	                    : (data->kind == D_QUERY_CENTER_FIRST) ? 1
+	                    : 0;
+	draw_msg(data->details.title, message, ctrl_msg, lines_to_center,
+			data->details.block_center, /*recommended_width=*/0);
 
-	ctrl_msg = get_control_msg(msg_kind, ctrl_c);
-	draw_msg(title, message, ctrl_msg, lines_to_center, 0);
+	free(free_me);
 
 	if(lazy)
 	{
@@ -430,20 +472,21 @@ redraw_error_msg(const char title_arg[], const char message_arg[],
 }
 
 /* Picks control message (information on available actions) basing on dialog
- * kind and previous actions.  Returns the message. */
+ * kind.  Returns the message. */
 static const char *
-get_control_msg(Dialog msg_kind, int global_skip)
+get_control_msg(const dialog_data_t *data)
 {
-	if(msg_kind == D_QUERY_WITHOUT_LIST || msg_kind == D_QUERY_WITH_LIST)
+	if(data->kind != D_ERROR)
 	{
-		if(responses == NULL)
+		if(data->details.variants == NULL)
 		{
 			return "Enter [y]es or [n]o";
 		}
 
-		return get_custom_control_msg(responses);
+		return get_custom_control_msg(data->details.variants);
 	}
-	else if(global_skip)
+
+	if(data->prompt_skip)
 	{
 		return "Press Return to continue or "
 		       "Ctrl-C to skip its future error messages";
@@ -467,10 +510,6 @@ get_custom_control_msg(const response_variant responses[])
 		if(response->descr[0] != '\0')
 		{
 			(void)sstrappend(msg_buf, &len, sizeof(msg_buf), response->descr);
-			if(response[1].key != '\0')
-			{
-				(void)sstrappendch(msg_buf, &len, sizeof(msg_buf), '/');
-			}
 		}
 
 		++response;
@@ -490,7 +529,8 @@ draw_msgf(const char title[], const char ctrl_msg[], int recommended_width,
 	vsnprintf(msg, sizeof(msg), format, pa);
 	va_end(pa);
 
-	draw_msg(title, msg, ctrl_msg, 0, recommended_width);
+	draw_msg(title, msg, ctrl_msg, /*lines_to_center=*/0, /*block_center=*/0,
+			recommended_width);
 	touch_all_windows();
 	ui_refresh_win(error_win);
 }
@@ -499,119 +539,130 @@ draw_msgf(const char title[], const char ctrl_msg[], int recommended_width,
  * specified title and control message on error_win. */
 static void
 draw_msg(const char title[], const char msg[], const char ctrl_msg[],
-		int lines_to_center, int recommended_width)
+		int lines_to_center, int block_center, int recommended_width)
 {
-	enum { margin = 1 };
+	/* TODO: refactor this function draw_msg() */
 
 	int sw, sh;
-	int w, h;
-	int max_h;
-	int len;
+	int w;
 	size_t ctrl_msg_n;
 	size_t wctrl_msg;
 	int first_line_x = 1;
 	const int first_line_y = 2;
+	int block_margin;
 
 	if(curr_stats.load_stage < 1)
 	{
 		return;
 	}
 
-	curs_set(0);
+	ui_set_cursor(/*visibility=*/0);
 
 	getmaxyx(stdscr, sh, sw);
 
-	ctrl_msg_n = MAX(measure_sub_lines(ctrl_msg, &wctrl_msg), 1U);
+	ctrl_msg_n = MAX(measure_sub_lines(ctrl_msg, /*skip_empty=*/0, &wctrl_msg),
+	                 1U);
 
-	max_h = sh - 2 - ctrl_msg_n - ui_stat_height();
-	h = max_h;
+	int wmsg = measure_text_width(msg);
+
+	/* We start with maximum height and reduce is later. */
+	int max_h = sh - 2 - ui_stat_height();
 	/* The outermost condition is for VLA below (to calm static analyzers). */
-	w = MAX(2 + 2*margin, MIN(sw - 2,
-	        MAX(MAX(recommended_width, sw/3),
-	            (int)MAX(wctrl_msg, determine_width(msg)) + 4)));
-	wresize(error_win, h, w);
+	w = MAX(2 + 2*MARGIN, MIN(sw - 2,
+	        MAX(MAX(recommended_width, sw/3), MAX((int)wctrl_msg, wmsg) + 4)));
+	wresize(error_win, max_h, w);
 
 	werase(error_win);
 
-	len = strlen(msg);
-	if(len <= w - 2 && strchr(msg, '\n') == NULL)
+	block_margin = 0;
+	if(block_center)
 	{
-		first_line_x = (w - len)/2;
-		h = 5 + ctrl_msg_n;
-		wresize(error_win, h, w);
-		mvwin(error_win, (sh - h)/2, (sw - w)/2);
-		checked_wmove(error_win, first_line_y, first_line_x);
-		wprint(error_win, msg);
+		block_margin = MAX(w - wmsg, 2 + 2*MARGIN)/2;
 	}
-	else
+
+	if(strchr(msg, '\n') == NULL && wmsg <= w - 2 - 2*MARGIN)
 	{
-		int i = 0;
-		int cy = first_line_y;
-		while(i < len)
+		lines_to_center = 1;
+	}
+
+	const char *curr = msg;
+	int cy = first_line_y;
+	while(*curr != '\0')
+	{
+		int max_width = w - 2 - 2*MARGIN;
+
+		/* Screen line stops at new line or when there is no more space. */
+		const char *nl = until_first(curr, '\n');
+		const char *sl = curr + utf8_strsnlen(curr, max_width);
+		const char *end = (nl < sl ? nl : sl);
+
+		char buf[max_width*10 + 1];
+		copy_str(buf, MIN((int)sizeof(buf), end - curr + 1), curr);
+		curr = (end[0] == '\n' ? end + 1 : end);
+		if(buf[0] == '\0')
+			continue;
+
+		if(cy >= max_h - (int)ctrl_msg_n - 3)
 		{
-			int j;
-			char buf[w - 2 - 2*margin + 1];
-			int cx;
-
-			copy_str(buf, sizeof(buf), msg + i);
-
-			for(j = 0; buf[j] != '\0'; j++)
-				if(buf[j] == '\n')
-					break;
-
-			if(buf[j] != '\0')
-				i++;
-			buf[j] = '\0';
-			i += j;
-
-			if(buf[0] == '\0')
-				continue;
-
-			if(cy >= max_h - (int)ctrl_msg_n - 3)
+			/* Skip trailing part of the message if it's too long, just print how
+			 * many lines we're omitting. */
+			size_t max_len;
+			int more_lines = 1U + measure_sub_lines(curr, /*skip_empty=*/1, &max_len);
+			if(more_lines > 1)
 			{
-				/* Skip trailing part of the message if it's too long, just print how
-				 * many lines we're omitting. */
-				size_t max_len;
-				const int more_lines = 1U + measure_sub_lines(msg + i, &max_len);
-				snprintf(buf, sizeof(buf), "<<%d more line%s not shown>>", more_lines,
-						(more_lines == 1) ? "" : "s");
+				snprintf(buf, sizeof(buf), "<<%d more lines not shown>>", more_lines);
 				/* Make sure this is the last iteration of the loop. */
-				i = len;
+				curr += strlen(curr);
 			}
-
-			h = 1 + cy + 1 + ctrl_msg_n + 1;
-			wresize(error_win, h, w);
-			mvwin(error_win, (sh - h)/2, (sw - w)/2);
-
-			cx = lines_to_center-- > 0 ? (w - utf8_strsw(buf))/2 : (1 + margin);
-			if(cy == first_line_y)
-			{
-				first_line_x = cx;
-			}
-			checked_wmove(error_win, cy++, cx);
-			wprint(error_win, buf);
 		}
+
+		int cx = 1 + MARGIN;
+		if(lines_to_center-- > 0)
+		{
+			if(block_center)
+			{
+				cx = block_margin;
+			}
+			else
+			{
+				cx = (w - utf8_strsw(buf))/2;
+			}
+		}
+
+		if(cy == first_line_y)
+		{
+			first_line_x = cx;
+		}
+		checked_wmove(error_win, cy++, cx);
+		wprint(error_win, buf);
 	}
 
+	int h = 1 + cy + ctrl_msg_n + 1;
+	wresize(error_win, h, w);
+	mvwin(error_win, (sh - h)/2, (sw - w)/2);
+
 	box(error_win, 0, 0);
 	if(title[0] != '\0')
 	{
 		mvwprintw(error_win, 0, (w - strlen(title) - 2)/2, " %s ", title);
 	}
 
+	int ctrl_msg_width = measure_text_width(ctrl_msg);
+	block_margin = MAX(w - ctrl_msg_width, 2 + 2*MARGIN)/2;
+
 	/* Print control message line by line. */
-	size_t i = ctrl_msg_n;
+	size_t ctrl_i = ctrl_msg_n;
 	while(1)
 	{
 		const size_t len = strcspn(ctrl_msg, "\n");
-		mvwaddnstr(error_win, h - i - 1, MAX(0, (w - (int)len)/2), ctrl_msg, len);
+		mvwaddnstr(error_win, h - ctrl_i - 1, block_margin, ctrl_msg, len);
 
-		if(--i == 0)
+		if(--ctrl_i == 0)
 		{
 			break;
 		}
 
-		ctrl_msg = skip_char(ctrl_msg + len + 1U, '/');
+		ctrl_msg += len + (ctrl_msg[len] == '\n' ? 1 : 0);
 	}
 
 	checked_wmove(error_win, first_line_y, first_line_x);
@@ -621,7 +672,7 @@ draw_msg(const char title[], const char msg[], const char ctrl_msg[],
  * *max_len to the length of the longest sub-line.  Returns total number of
  * sub-lines, which can be zero is msg is an empty line. */
 static size_t
-measure_sub_lines(const char msg[], size_t *max_len)
+measure_sub_lines(const char msg[], int skip_empty, size_t *max_len)
 {
 	size_t nlines = 0U;
 	*max_len = 0U;
@@ -632,38 +683,32 @@ measure_sub_lines(const char msg[], size_t *max_len)
 		{
 			*max_len = len;
 		}
-		++nlines;
+		/* Empty lines are not displayed. */
+		if(len != 0 || !skip_empty)
+		{
+			++nlines;
+		}
 		msg += len + (msg[len] == '\n' ? 1U : 0U);
 	}
 	return nlines;
 }
 
-/* Determines maximum width of line in the message.  Returns the width. */
+/* Measures maximum on-screen width of a line in a message.  Returns the
+ * width. */
 static size_t
-determine_width(const char msg[])
+measure_text_width(const char msg[])
 {
 	size_t max_width = 0U;
-
 	while(*msg != '\0')
 	{
-		size_t width = 0U;
-		while(*msg != '\n' && *msg != '\0')
-		{
-			++width;
-			++msg;
-		}
-
+		const size_t len = strcspn(msg, "\n");
+		const size_t width = utf8_nstrsw(msg, len);
 		if(width > max_width)
 		{
 			max_width = width;
 		}
-
-		if(*msg == '\n')
-		{
-			++msg;
-		}
+		msg += len + (msg[len] == '\n' ? 1U : 0U);
 	}
-
 	return max_width;
 }
 
@@ -705,10 +750,18 @@ confirm_deletion(char *files[], int nfiles, int use_trash)
 		}
 	}
 
-	prompt_msg_internal(title, msg, NULL, 1);
+	dialog_data_t data = {
+		.details = {
+			.title = title,
+			.message = msg,
+		},
+		.kind = D_QUERY_CENTER_FIRST,
+		.accept_mask = MASK(DR_YES, DR_NO),
+	};
+	prompt_msg_internal(&data);
 	free(msg);
 
-	if(dialog_result != DR_YES)
+	if(data.result != DR_YES)
 	{
 		return 0;
 	}
