@@ -31,6 +31,7 @@
 #include "../../ui/colors.h"
 #include "../../ui/ui.h"
 #include "../../utils/macros.h"
+#include "../../utils/utils.h"
 #include "../../filelist.h"
 #include "../../status.h"
 #include "../modes.h"
@@ -200,6 +201,7 @@ enter_sort_mode(view_t *active_view)
 	view = active_view;
 	descending = (view->sort[0] < 0);
 	vle_mode_set(SORT_MODE, VMT_SECONDARY);
+	ui_hide_graphics();
 
 	top = 4;
 	bottom = top + SK_COUNT - 1;
@@ -266,7 +268,7 @@ leave_sort_mode(void)
 
 	ui_view_reset_selection_and_reload(view);
 
-	update_all_windows();
+	stats_redraw_later();
 }
 
 /* Redraws the dialog. */
@@ -329,11 +331,8 @@ cmd_h(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_j(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(key_info.count == NO_COUNT_GIVEN)
-		key_info.count = 1;
-
 	clear_at_pos();
-	curr += key_info.count;
+	curr += def_count(key_info.count);
 	if(curr > bottom)
 		curr = bottom;
 
@@ -344,11 +343,8 @@ cmd_j(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_k(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(key_info.count == NO_COUNT_GIVEN)
-		key_info.count = 1;
-
 	clear_at_pos();
-	curr -= key_info.count;
+	curr -= def_count(key_info.count);
 	if(curr < top)
 		curr = top;
 
