@@ -41,6 +41,7 @@
 #include "../engine/keys.h"
 #include "../engine/mode.h"
 #include "../engine/variables.h"
+#include "../menus/filetypes_menu.h"
 #include "../modes/dialogs/msg_dialog.h"
 #include "../ui/cancellation.h"
 #include "../ui/fileview.h"
@@ -87,7 +88,6 @@ static void cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_e(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_f(key_info_t key_info, keys_info_t *keys_info);
-static void page_scroll(int base, int direction);
 static void cmd_ctrl_g(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_space(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_emarkemark(key_info_t key_info, keys_info_t *keys_info);
@@ -247,6 +247,8 @@ static void pick_files(view_t *view, int end, keys_info_t *keys_info);
 static void selector_S(key_info_t key_info, keys_info_t *keys_info);
 static void selector_a(key_info_t key_info, keys_info_t *keys_info);
 static void selector_s(key_info_t key_info, keys_info_t *keys_info);
+static void set_pos_in_curr_view(int pos);
+static void handle_mouse_event(key_info_t key_info, keys_info_t *keys_info);
 
 static int last_fast_search_char;
 static int last_fast_search_backward = -1;
@@ -345,8 +347,8 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_c WK_w,        {{&cmd_cw}, .descr = "rename files"}},
 	{WK_D WK_D,        {{&cmd_DD}, .nim = 1, .descr = "remove files permanently"}},
 	{WK_d WK_d,        {{&cmd_dd}, .nim = 1, .descr = "remove files"}},
-	{WK_d WK_o,        {{&cmd_do}, .descr = "obtain current file"}},
-	{WK_d WK_p,        {{&cmd_dp}, .descr = "put current file"}},
+	{WK_d WK_o,        {{&cmd_do}, .descr = "obtain files"}},
+	{WK_d WK_p,        {{&cmd_dp}, .descr = "put files"}},
 	{WK_D,             {{&cmd_D_selector}, FOLLOWED_BY_SELECTOR, .descr = "remove files permanently"}},
 	{WK_d,             {{&cmd_d_selector}, FOLLOWED_BY_SELECTOR, .descr = "remove files"}},
 	{WK_e,             {{&cmd_e}, .descr = "explore file contents"}},
@@ -407,8 +409,8 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_RP,            {{&cmd_right_paren}, .descr = "go to next group of files"}},
 	{WK_z WK_k,        {{&cmd_z_k},  .descr = "go to previous sibling dir"}},
 	{WK_z WK_j,        {{&cmd_z_j},  .descr = "go to next sibling dir"}},
-	{WK_LB WK_c,       {{&cmd_lb_c}, .descr = "go to previous mismatch"}},
-	{WK_RB WK_c,       {{&cmd_rb_c}, .descr = "go to next mismatch"}},
+	{WK_LB WK_c,       {{&cmd_lb_c}, .descr = "go to previous compare change"}},
+	{WK_RB WK_c,       {{&cmd_rb_c}, .descr = "go to next compare change"}},
 	{WK_LB WK_d,       {{&cmd_lb_d}, .descr = "go to previous dir"}},
 	{WK_RB WK_d,       {{&cmd_rb_d}, .descr = "go to next dir"}},
 	{WK_LB WK_r,       {{&cmd_lb_r}, .descr = "navigate to previous sibling dir"}},
@@ -441,6 +443,7 @@ static keys_add_info_t builtin_cmds[] = {
 #else
 	{WK_ESC L"[Z",               {{&cmd_shift_tab}, .descr = "switch to view pane"}},
 #endif /* ENABLE_EXTENDED_KEYS */
+	{{K(KEY_MOUSE)}, {{&handle_mouse_event}, FOLLOWED_BY_NONE}},
 };
 
 static keys_add_info_t selectors[] = {
@@ -471,8 +474,8 @@ static keys_add_info_t selectors[] = {
 	{WK_RP,      {{&cmd_right_paren}, .descr = "to next group of files"}},
 	{WK_z WK_k,  {{&cmd_z_k},  .descr = "to previous sibling dir"}},
 	{WK_z WK_j,  {{&cmd_z_j},  .descr = "to next sibling dir"}},
-	{WK_LB WK_c, {{&cmd_lb_c}, .descr = "to previous mismatch"}},
-	{WK_RB WK_c, {{&cmd_rb_c}, .descr = "to next mismatch"}},
+	{WK_LB WK_c, {{&cmd_lb_c}, .descr = "to previous compare change"}},
+	{WK_RB WK_c, {{&cmd_rb_c}, .descr = "to next compare change"}},
 	{WK_LB WK_d, {{&cmd_lb_d}, .descr = "to previous dir"}},
 	{WK_RB WK_d, {{&cmd_rb_d}, .descr = "to next dir"}},
 	{WK_LB WK_s, {{&cmd_lb_s}, .descr = "to previous selected entry"}},
@@ -516,10 +519,7 @@ cmd_ctrl_a(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_b(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(can_scroll_up(curr_view))
-	{
-		page_scroll(fpos_get_last_visible_cell(curr_view), -1);
-	}
+	fview_scroll_page_up(curr_view);
 }
 
 /* Resets selection and search highlight. */
@@ -546,9 +546,9 @@ cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_e(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(fpos_has_hidden_bottom(curr_view) && fpos_scroll_down(curr_view, 1))
+	if(fpos_can_scroll_fwd(curr_view) && fpos_scroll_down(curr_view, 1))
 	{
-		scroll_down(curr_view, 1);
+		fview_scroll_fwd_by(curr_view, 1);
 		redraw_current_view();
 	}
 }
@@ -556,31 +556,7 @@ cmd_ctrl_e(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_f(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(can_scroll_down(curr_view))
-	{
-		page_scroll(curr_view->top_line, 1);
-	}
-}
-
-/* Scrolls pane by one view in both directions.  The direction should be 1 or
- * -1. */
-static void
-page_scroll(int base, int direction)
-{
-	enum { HOR_GAP_SIZE = 2, VER_GAP_SIZE = 1 };
-	int old_pos = curr_view->list_pos;
-	int offset = fview_is_transposed(curr_view)
-	    ? (MAX(1, curr_view->column_count - VER_GAP_SIZE))*curr_view->window_rows
-	    : (curr_view->window_rows - HOR_GAP_SIZE)*curr_view->column_count;
-	int new_pos = base + direction*offset
-	            + old_pos%curr_view->run_size - base%curr_view->run_size;
-	curr_view->list_pos = MAX(0, MIN(curr_view->list_rows - 1, new_pos));
-	scroll_by_files(curr_view, direction*offset);
-
-	/* Updating list_pos ourselves doesn't take into account
-	 * synchronization/updates of the other view, so trigger them. */
-	ui_view_schedule_redraw(curr_view);
-	fpos_set_pos(curr_view, curr_view->list_pos);
+	fview_scroll_page_down(curr_view);
 }
 
 static void
@@ -618,7 +594,7 @@ cmd_emarkemark(key_info_t key_info, keys_info_t *keys_info)
 	}
 
 	cmds_vars_set_count(key_info.count);
-	modcline_enter(CLS_COMMAND, prefix, NULL);
+	modcline_enter(CLS_COMMAND, prefix);
 }
 
 /* Processes !<selector> normal mode command.  Processes results of applying
@@ -930,9 +906,9 @@ cmd_ctrl_x(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_y(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(fpos_has_hidden_top(curr_view) && fpos_scroll_up(curr_view, 1))
+	if(fpos_can_scroll_back(curr_view) && fpos_scroll_up(curr_view, 1))
 	{
-		scroll_up(curr_view, 1);
+		fview_scroll_back_by(curr_view, 1);
 		redraw_current_view();
 	}
 }
@@ -1092,8 +1068,6 @@ cmd_gr(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_gs(key_info_t key_info, keys_info_t *keys_info)
 {
-	reg_t *reg;
-
 	if(key_info.reg == NO_REG_GIVEN)
 	{
 		flist_sel_restore(curr_view, NULL);
@@ -1101,7 +1075,7 @@ cmd_gs(key_info_t key_info, keys_info_t *keys_info)
 	}
 
 	regs_sync_from_shared_memory();
-	reg = regs_find(tolower(key_info.reg));
+	const reg_t *reg = regs_find(tolower(key_info.reg));
 	if(reg == NULL || reg->nfiles < 1)
 	{
 		ui_sb_err(reg == NULL ? "No such register" : "Register is empty");
@@ -1329,7 +1303,7 @@ cmd_percent(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_equal(key_info_t key_info, keys_info_t *keys_info)
 {
-	modcline_enter(CLS_FILTER, curr_view->local_filter.filter.raw, NULL);
+	modcline_enter(CLS_FILTER, curr_view->local_filter.filter.raw);
 }
 
 /* Continues navigation to word which starts with specified character in
@@ -1348,7 +1322,7 @@ cmd_comma(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_dot(key_info_t key_info, keys_info_t *keys_info)
 {
-	curr_stats.save_msg = exec_commands(curr_stats.last_cmdline_command,
+	curr_stats.save_msg = cmds_dispatch(curr_stats.last_cmdline_command,
 			curr_view, CIT_COMMAND);
 }
 
@@ -1374,7 +1348,7 @@ cmd_colon(key_info_t key_info, keys_info_t *keys_info)
 	}
 
 	cmds_vars_set_count(key_info.count);
-	modcline_enter(CLS_COMMAND, prefix, NULL);
+	modcline_enter(CLS_COMMAND, prefix);
 }
 
 /* Continues navigation to word which starts with specified character in initial
@@ -1433,10 +1407,18 @@ cmd_cg(key_info_t key_info, keys_info_t *keys_info)
 }
 #endif
 
-/* Change symbolic link. */
+/* Change symbolic link(s). */
 static void
 cmd_cl(key_info_t key_info, keys_info_t *keys_info)
 {
+	if(curr_view->selected_files > 1)
+	{
+		flist_set_marking(curr_view, /*prefer_current=*/0);
+	}
+	else
+	{
+		clear_marking(curr_view);
+	}
 	curr_stats.save_msg = fops_retarget(curr_view);
 }
 
@@ -1710,42 +1692,9 @@ cmd_n(key_info_t key_info, keys_info_t *keys_info)
 static void
 search(key_info_t key_info, int backward)
 {
-	/* TODO: extract common part of this function and visual.c:search(). */
-
-	int found;
-
-	if(hist_is_empty(&curr_stats.search_hist))
-	{
-		return;
-	}
-
-	if(key_info.count == NO_COUNT_GIVEN)
-		key_info.count = 1;
-
-	found = 0;
-	if(curr_view->matches == 0)
-	{
-		const char *const pattern = hists_search_last();
-		curr_stats.save_msg = (find_pattern(curr_view, pattern, backward, 1, &found,
-				0) != 0);
-		--key_info.count;
-	}
-
-	while(key_info.count-- > 0)
-	{
-		found += goto_search_match(curr_view, backward) != 0;
-	}
-
-	if(found)
-	{
-		print_search_next_msg(curr_view, backward);
-	}
-	else
-	{
-		print_search_fail_msg(curr_view, backward);
-	}
-
-	curr_stats.save_msg = 1;
+	curr_stats.save_msg = search_next(curr_view, backward,
+			/*stash_selection=*/cfg.hl_search, /*select_matches=*/cfg.hl_search,
+			def_count(key_info.count), &set_pos_in_curr_view);
 }
 
 /* Put files. */
@@ -1783,7 +1732,7 @@ call_put_links(key_info_t key_info, int relative)
 static void
 cmd_q_colon(key_info_t key_info, keys_info_t *keys_info)
 {
-	get_and_execute_command("", 0U, CIT_COMMAND);
+	cmds_run_ext("", 0U, CIT_COMMAND);
 }
 
 /* Runs external editor to get search pattern and then executes it. */
@@ -1806,17 +1755,17 @@ activate_search(int count, int back, int external)
 {
 	/* TODO: generalize with visual.c:activate_search(). */
 
-	search_repeat = (count == NO_COUNT_GIVEN) ? 1 : count;
+	search_repeat = def_count(count);
 	curr_stats.last_search_backward = back;
 	if(external)
 	{
 		CmdInputType type = back ? CIT_BSEARCH_PATTERN : CIT_FSEARCH_PATTERN;
-		get_and_execute_command("", 0U, type);
+		cmds_run_ext("", 0U, type);
 	}
 	else
 	{
 		const CmdLineSubmode submode = back ? CLS_BSEARCH : CLS_FSEARCH;
-		modcline_enter(submode, "", NULL);
+		modcline_enter(submode, "");
 	}
 }
 
@@ -1824,7 +1773,7 @@ activate_search(int count, int back, int external)
 static void
 cmd_q_equals(key_info_t key_info, keys_info_t *keys_info)
 {
-	get_and_execute_command("", 0U, CIT_FILTER_PATTERN);
+	cmds_run_ext("", 0U, CIT_FILTER_PATTERN);
 }
 
 /* Toggles selection of the current file. */
@@ -2008,10 +1957,10 @@ cmd_zd(key_info_t key_info, keys_info_t *keys_info)
 void
 modnorm_zb(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(can_scroll_up(curr_view))
+	if(fpos_can_scroll_back(curr_view))
 	{
 		const int bottom = fpos_get_bottom_pos(curr_view);
-		scroll_up(curr_view, bottom - curr_view->list_pos);
+		fview_scroll_back_by(curr_view, bottom - curr_view->list_pos);
 		redraw_current_view();
 	}
 }
@@ -2202,10 +2151,10 @@ cmd_right_curly_bracket(key_info_t key_info, keys_info_t *keys_info)
 void
 modnorm_zt(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(can_scroll_down(curr_view))
+	if(fpos_can_scroll_fwd(curr_view))
 	{
 		const int top = fpos_get_top_pos(curr_view);
-		scroll_down(curr_view, curr_view->list_pos - top);
+		fview_scroll_fwd_by(curr_view, curr_view->list_pos - top);
 		redraw_current_view();
 	}
 }
@@ -2216,7 +2165,7 @@ modnorm_zz(key_info_t key_info, keys_info_t *keys_info)
 	if(!fpos_are_all_files_visible(curr_view))
 	{
 		const int middle = fpos_get_middle_pos(curr_view);
-		scroll_by_files(curr_view, curr_view->list_pos - middle);
+		fview_scroll_by(curr_view, curr_view->list_pos - middle);
 		redraw_current_view();
 	}
 }
@@ -2333,30 +2282,133 @@ selector_s(key_info_t key_info, keys_info_t *keys_info)
 }
 
 int
-modnorm_find(view_t *view, const char pattern[], int backward, int print_errors)
+modnorm_find(view_t *view, const char pattern[], int backward, int print_msg,
+		int *found)
 {
-	const int nrepeats = search_repeat - 1;
-	int i;
-	int save_msg;
-	int found;
+	return search_find(view, pattern, backward, /*stash_selection=*/cfg.hl_search,
+			/*select_matches=*/cfg.hl_search, search_repeat, &set_pos_in_curr_view,
+			print_msg, found);
+}
 
-	/* Reset number of repeats so that future calls are not affected by the
-	 * previous ones. */
-	search_repeat = 1;
+/* Moves cursor to pos, redraws cursor and schedules redraw of curr_view. */
+static void
+set_pos_in_curr_view(int pos)
+{
+	fpos_set_pos(curr_view, pos);
+}
 
-	save_msg = find_pattern(view, pattern, backward, 1, &found, print_errors);
-	if(!print_errors && save_msg < 0)
+void
+modnorm_set_search_attrs(int count, int last_search_backward)
+{
+	search_repeat = count;
+	curr_stats.last_search_backward = last_search_backward;
+}
+
+/* Processes events from the mouse. */
+static void
+handle_mouse_event(key_info_t key_info, keys_info_t *keys_info)
+{
+	MEVENT e;
+	if(ui_get_mouse(&e) != OK)
 	{
-		/* If we're not printing messages, we might be interested in broken
-		 * pattern. */
-		return -1;
+		return;
 	}
 
-	for(i = 0; i < nrepeats; ++i)
+	int on_tab_line = !cfg.pane_tabs && wenclose(tab_line, e.y, e.x);
+
+	if(ui_wenclose(&lwin, lwin.win, e.x, e.y) ||
+			ui_wenclose(&lwin, lwin.title, e.x, e.y))
 	{
-		save_msg += (goto_search_match(view, backward) != 0);
+		if(curr_view != &lwin)
+		{
+			go_to_other_window();
+			return;
+		}
+	}
+	else if(ui_wenclose(&rwin, rwin.win, e.x, e.y) ||
+			ui_wenclose(&rwin, rwin.title, e.x, e.y))
+	{
+		if(curr_view != &rwin)
+		{
+			go_to_other_window();
+			return;
+		}
+	}
+	else if(!on_tab_line)
+	{
+		return;
+	}
+
+	if(on_tab_line || ui_wenclose(curr_view, curr_view->title, e.y, e.x))
+	{
+		if(!on_tab_line && !cfg.pane_tabs)
+		{
+			/* Do nothing. */
+		}
+		else if(e.bstate & BUTTON1_PRESSED)
+		{
+			wmouse_trafo(ui_get_tab_line_win(curr_view), &e.y, &e.x, FALSE);
+
+			int tab_num = ui_map_tab_line(curr_view, e.x);
+			if(tab_num >= 0)
+			{
+				tabs_goto(tab_num);
+			}
+		}
+		else if(e.bstate & BUTTON4_PRESSED)
+		{
+			tabs_previous(1);
+		}
+		else if(e.bstate & (BUTTON2_PRESSED | BUTTON5_PRESSED))
+		{
+			tabs_next(1);
+		}
+	}
+	else if(e.bstate & BUTTON1_PRESSED)
+	{
+		wmouse_trafo(curr_view->win, &e.y, &e.x, FALSE);
+
+		/* Only handle clicks on non-blank lines. */
+		int list_pos = fview_map_coordinates(curr_view, e.x, e.y);
+		if(list_pos >= 0)
+		{
+			int old_pos = curr_view->list_pos;
+			fpos_set_pos(curr_view, list_pos);
+
+			if(curr_view->list_pos == old_pos)
+			{
+				cmd_return(key_info, keys_info);
+			}
+		}
+		else if(list_pos == FVM_LEAVE)
+		{
+			cmd_gh(key_info, keys_info);
+		}
+		else if(list_pos == FVM_OPEN)
+		{
+			cmd_i(key_info, keys_info);
+		}
+	}
+	else if(e.bstate & BUTTON3_PRESSED)
+	{
+		wmouse_trafo(curr_view->win, &e.y, &e.x, FALSE);
+
+		/* Only handle clicks on non-blank lines. */
+		int list_pos = fview_map_coordinates(curr_view, e.x, e.y);
+		if(list_pos >= 0)
+		{
+			fpos_set_pos(curr_view, list_pos);
+			curr_stats.save_msg = show_file_menu(curr_view, 0);
+		}
+	}
+	else if(e.bstate & BUTTON4_PRESSED)
+	{
+		cmd_ctrl_y(key_info, keys_info);
+	}
+	else if(e.bstate & (BUTTON2_PRESSED | BUTTON5_PRESSED))
+	{
+		cmd_ctrl_e(key_info, keys_info);
 	}
-	return save_msg;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
