@@ -19,12 +19,8 @@
 
 #include "running.h"
 
-#include <curses.h> /* FALSE curs_set() */
-
 #include <sys/stat.h> /* stat */
-#ifndef _WIN32
-#include <sys/wait.h> /* WEXITSTATUS() */
-#else
+#ifdef _WIN32
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <shellapi.h>
@@ -108,7 +104,6 @@ static void run_implicit_prog(view_t *view, const char prog_spec[], int pause,
 		int force_bg);
 static void view_current_file(const view_t *view);
 static void follow_link(view_t *view, int follow_dirs, int ultimate);
-static void enter_dir(struct view_t *view);
 static int cd_to_parent_dir(view_t *view);
 static void extract_last_path_component(const char path[], char buf[]);
 static char * run_shell_prepare(const char command[], ShellPause pause,
@@ -129,7 +124,6 @@ static int try_run_with_filetype(view_t *view, const assoc_records_t assocs,
 static void output_to_statusbar(const char cmd[], view_t *view,
 		MacroFlags flags);
 static int output_to_preview(view_t *view, const char cmd[], MacroFlags flags);
-static void output_to_nowhere(const char cmd[], view_t *view, MacroFlags flags);
 static void run_in_split(const view_t *view, const char cmd[], int vert_split);
 static void path_handler(const char line[], void *arg);
 static void line_handler(const char line[], void *arg);
@@ -189,7 +183,7 @@ handle_file(view_t *view, FileHandleExec exec, FileHandleLink follow)
 		int dir_like_entry = (is_dir(full_path) || is_unc_root(view->curr_dir));
 		if(dir_like_entry)
 		{
-			enter_dir(view);
+			(void)rn_enter_dir(view);
 			return;
 		}
 	}
@@ -279,7 +273,7 @@ run_win_executable(char full_path[], int elevate)
 	else
 	{
 		int returned_exit_code;
-		const int error = win_exec_cmd(full_path, &returned_exit_code);
+		const int error = win_exec_cmd(full_path, full_path, &returned_exit_code);
 		if(error != 0 && !returned_exit_code)
 		{
 			if(error == ERROR_ELEVATION_REQUIRED && is_vista_and_above())
@@ -374,16 +368,18 @@ static void
 execute_file(const char full_path[], int elevate)
 {
 #ifndef _WIN32
-	char *const escaped = shell_like_escape(full_path, 0);
+	char *const escaped = shell_arg_escape(full_path, curr_stats.shell_type);
 	rn_shell(escaped, PAUSE_ALWAYS, 1, SHELL_BY_APP);
 	free(escaped);
 #else
-	char *const dquoted_full_path = strdup(enclose_in_dquotes(full_path));
+	char *copy = strdup(full_path);
+	internal_to_system_slashes(copy);
 
-	internal_to_system_slashes(dquoted_full_path);
-	run_win_executable(dquoted_full_path, elevate);
+	char *escaped = shell_arg_escape(copy, curr_stats.shell_type);
+	free(copy);
 
-	free(dquoted_full_path);
+	run_win_executable(escaped, elevate);
+	free(escaped);
 #endif
 }
 
@@ -469,13 +465,13 @@ run_with_defaults(view_t *view)
 {
 	if(get_current_entry(view)->type == FT_DIR)
 	{
-		enter_dir(view);
+		(void)rn_enter_dir(view);
 		return;
 	}
 
 	/* Reuse marking second time. */
 	view->pending_marking = 1;
-	if(vim_edit_marking() != 0)
+	if(vim_edit_marking(view) != 0)
 	{
 		show_error_msg("Running error", "Can't edit selection");
 	}
@@ -557,7 +553,7 @@ rn_open_with(view_t *view, const char prog_spec[], int dont_execute,
 	}
 	else if(strcmp(prog_spec, VIFM_PSEUDO_CMD) == 0)
 	{
-		enter_dir(view);
+		(void)rn_enter_dir(view);
 	}
 	else if(strchr(prog_spec, '%') != NULL)
 	{
@@ -575,7 +571,7 @@ static void
 run_explicit_prog(view_t *view, const char prog_spec[], int pause, int force_bg)
 {
 	MacroFlags flags;
-	char *const cmd = ma_expand(prog_spec, NULL, &flags, 1);
+	char *const cmd = ma_expand(prog_spec, NULL, &flags, MER_SHELL_OP);
 
 	const ShellPause pause_shell = (pause ? PAUSE_ALWAYS : PAUSE_ON_ERROR);
 
@@ -623,7 +619,7 @@ run_implicit_prog(view_t *view, const char prog_spec[], int pause, int force_bg)
 	strcpy(spec, prog_spec);
 	bg = cut_suffix(spec, " &") || force_bg;
 
-	if(curr_stats.shell_type == ST_CMD)
+	if(curr_stats.shell_type == ST_CMD || curr_stats.shell_type == ST_YORI)
 	{
 		name_macro = (view == curr_view) ? "%\"c" : "%\"C";
 	}
@@ -632,7 +628,7 @@ run_implicit_prog(view_t *view, const char prog_spec[], int pause, int force_bg)
 		name_macro = (view == curr_view) ? "%c" : "%C";
 	}
 
-	file_name = ma_expand(name_macro, NULL, NULL, 1);
+	file_name = ma_expand(name_macro, NULL, NULL, MER_SHELL_OP);
 	snprintf(cmd, sizeof(cmd), "%s %s", spec, file_name);
 	free(file_name);
 
@@ -745,27 +741,29 @@ follow_link(view_t *view, int follow_dirs, int ultimate)
 	free(dir);
 }
 
-/* Handles opening of current entry of the view as a directory. */
-static void
-enter_dir(view_t *view)
+int
+rn_enter_dir(view_t *view)
 {
 	dir_entry_t *const curr = get_current_entry(view);
 
 	if(is_parent_dir(curr->name) && !curr->owns_origin)
 	{
 		rn_leave(view, 1);
-		return;
+		return 0;
 	}
 
 	char full_path[PATH_MAX + 1];
 	get_full_path_of(curr, sizeof(full_path), full_path);
-
-	if(cd_is_possible(full_path))
+	if(!cd_is_possible(full_path))
 	{
-		curr_stats.ch_pos = (cfg_ch_pos_on(CHPOS_ENTER) ? 1 : 0);
-		navigate_to(view, full_path);
-		curr_stats.ch_pos = 1;
+		return 1;
 	}
+
+	curr_stats.ch_pos = (cfg_ch_pos_on(CHPOS_ENTER) ? 1 : 0);
+	int result = navigate_to(view, full_path);
+	curr_stats.ch_pos = 1;
+
+	return result;
 }
 
 void
@@ -896,7 +894,7 @@ run_shell_finish(const char cmd[], const char final_cmd[], ShellPause pause,
 {
 	cleanup_shellout_env();
 
-	int exit_code = WEXITSTATUS(status);
+	int exit_code = status_to_exit_code(status);
 
 	if(pause == PAUSE_ALWAYS && final_cmd != NULL && ends_with(final_cmd, "&"))
 	{
@@ -928,10 +926,7 @@ run_shell_finish(const char cmd[], const char final_cmd[], ShellPause pause,
 		stats_redraw_later();
 	}
 
-	if(curr_stats.load_stage > 0)
-	{
-		curs_set(0);
-	}
+	ui_set_cursor(/*visibility=*/0);
 
 	if(cmd == NULL)
 	{
@@ -972,7 +967,7 @@ setup_shellout_env(void)
 			return;
 	}
 
-	escaped_path = shell_like_escape(mount_file, 0);
+	escaped_path = shell_arg_escape(mount_file, curr_stats.shell_type);
 	cmd = format_str(term_multiplexer_fmt, FUSE_FILE_ENVVAR, escaped_path);
 	(void)vifm_system(cmd, SHELL_BY_APP);
 	free(cmd);
@@ -1044,7 +1039,7 @@ gen_term_multiplexer_cmd(const char cmd[], int pause, ShellRequester by)
 	title_arg = gen_term_multiplexer_title_arg(cmd);
 
 	raw_shell_cmd = format_str("%s%s", cmd, pause ? PAUSE_STR : "");
-	escaped_shell_cmd = shell_like_escape(raw_shell_cmd, 0);
+	escaped_shell_cmd = shell_arg_escape(raw_shell_cmd, curr_stats.shell_type);
 
 	const char *sh_flag = (by == SHELL_BY_USER ? cfg.shell_cmd_flag : "-c");
 
@@ -1052,7 +1047,7 @@ gen_term_multiplexer_cmd(const char cmd[], int pause, ShellRequester by)
 	{
 		char *const arg = format_str("%s %s %s", cfg.shell, sh_flag,
 				escaped_shell_cmd);
-		char *const escaped_arg = shell_like_escape(arg, 0);
+		char *const escaped_arg = shell_arg_escape(arg, curr_stats.shell_type);
 
 		shell_cmd = format_str("tmux new-window %s %s", title_arg, escaped_arg);
 
@@ -1117,7 +1112,7 @@ gen_term_multiplexer_title_arg(const char cmd[])
 	else
 	{
 		const char opt_c = (curr_stats.term_multiplexer == TM_SCREEN) ? 't' : 'n';
-		char *const escaped_title = shell_like_escape(title, 0);
+		char *const escaped_title = shell_arg_escape(title, curr_stats.shell_type);
 		title_arg = format_str("-%c %s", opt_c, escaped_title);
 		free(escaped_title);
 	}
@@ -1136,7 +1131,7 @@ gen_normal_cmd(const char cmd[], int pause)
 	{
 		const char *cmd_with_pause_fmt;
 
-		if(curr_stats.shell_type == ST_CMD)
+		if(curr_stats.shell_type == ST_CMD || curr_stats.shell_type == ST_YORI)
 		{
 			cmd_with_pause_fmt = "%s" PAUSE_STR;
 		}
@@ -1163,7 +1158,7 @@ gen_normal_cmd(const char cmd[], int pause)
 static void
 set_pwd_in_screen(const char path[])
 {
-	char *const escaped_dir = shell_like_escape(path, 0);
+	char *const escaped_dir = shell_arg_escape(path, curr_stats.shell_type);
 	char *const set_pwd = format_str("screen -X setenv PWD %s", escaped_dir);
 
 	(void)vifm_system(set_pwd, SHELL_BY_APP);
@@ -1248,24 +1243,31 @@ rn_ext(view_t *view, const char cmd[], const char title[], MacroFlags flags,
 	}
 	else if(ma_flags_present(flags, MF_IGNORE))
 	{
-		*save_msg = 0;
-		if(bg)
+		/* TODO: consider merging with rn_start_bg_command(). */
+
+		const int supply_input = ma_flags_present(flags, MF_PIPE_FILE_LIST)
+		                      || ma_flags_present(flags, MF_PIPE_FILE_LIST_Z);
+		FILE *input = NULL;
+		FILE **input_ptr = (supply_input ? &input : NULL);
+		int error = 0;
+
+		setup_shellout_env();
+		if(bg_run_external(cmd, 1, SHELL_BY_USER, input_ptr) != 0)
 		{
-			int error;
-
-			setup_shellout_env();
-			error = (bg_run_external(cmd, 1, SHELL_BY_USER, NULL) != 0);
-			cleanup_shellout_env();
-
-			if(error)
-			{
-				ui_sb_errf("Failed to start in bg: %s", cmd);
-				*save_msg = 1;
-			}
+			error = 1;
 		}
-		else
+		cleanup_shellout_env();
+
+		if(input != NULL)
 		{
-			output_to_nowhere(cmd, view, flags);
+			const int null_sep = ma_flags_present(flags, MF_PIPE_FILE_LIST_Z);
+			write_marked_paths(input, view, null_sep);
+			fclose(input);
+		}
+
+		if(error)
+		{
+			show_error_msgf("Trouble running command", "Unable to run: %s", cmd);
 		}
 		return -1;
 	}
@@ -1288,7 +1290,7 @@ rn_ext(view_t *view, const char cmd[], const char title[], MacroFlags flags,
 	        ma_flags_present(flags, MF_CUSTOMVIEW_IOUTPUT) ||
 	        ma_flags_present(flags, MF_VERYCUSTOMVIEW_IOUTPUT))
 	{
-		rn_for_flist(view, cmd, title, flags);
+		rn_for_flist(view, cmd, title, /*user_sh=*/1, flags);
 	}
 	else
 	{
@@ -1328,6 +1330,9 @@ output_to_statusbar(const char cmd[], view_t *view, MacroFlags flags)
 
 	lines = NULL;
 	len = 0;
+	/* XXX: the loop can potentially never end if error pipe gets filled, might
+	 *      need to redirect stderr to /dev/null instead of opening and not using
+	 *      it. */
 	while(fgets(buf, sizeof(buf), file) == buf)
 	{
 		char *p;
@@ -1362,39 +1367,6 @@ output_to_preview(view_t *view, const char cmd[], MacroFlags flags)
 	return 0;
 }
 
-/* Executes the cmd ignoring its output. */
-static void
-output_to_nowhere(const char cmd[], view_t *view, MacroFlags flags)
-{
-	FILE *file, *err;
-	FILE *input_tmp = make_in_file(view, flags);
-
-	setup_shellout_env();
-	int error = 0;
-	if(bg_run_and_capture((char *)cmd, 1, input_tmp, &file, &err) == (pid_t)-1)
-	{
-		error = 1;
-	}
-	cleanup_shellout_env();
-
-	if(input_tmp != NULL)
-	{
-		fclose(input_tmp);
-	}
-
-	if(error)
-	{
-		show_error_msgf("Trouble running command", "Unable to run: %s", cmd);
-		return;
-	}
-
-	/* FIXME: better way of doing this would be to redirect these streams to
-	 *        /dev/null rather than closing them, but not sure about Windows (NUL
-	 *        device might work). */
-	fclose(file);
-	fclose(err);
-}
-
 /* Runs the cmd in a split window of terminal multiplexer.  Runs shell, if cmd
  * is NULL. */
 static void
@@ -1402,7 +1374,7 @@ run_in_split(const view_t *view, const char cmd[], int vert_split)
 {
 	char *const escaped_cmd = (cmd == NULL)
 	                        ? strdup(cfg.shell)
-	                        : shell_like_escape(cmd, 0);
+	                        : shell_arg_escape(cmd, curr_stats.shell_type);
 
 	setup_shellout_env();
 
@@ -1419,7 +1391,8 @@ run_in_split(const view_t *view, const char cmd[], int vert_split)
 
 		/* "eval" executes each argument as a separate argument, but escaping rules
 		 * are not exactly like in shell, so last command is run separately. */
-		char *const escaped_dir = shell_like_escape(flist_get_dir(view), 0);
+		char *const escaped_dir =
+			shell_arg_escape(flist_get_dir(view), curr_stats.shell_type);
 		if(vert_split)
 		{
 			snprintf(cmd, sizeof(cmd), "screen -X eval chdir\\ %s 'focus right' "
@@ -1450,6 +1423,8 @@ run_in_split(const view_t *view, const char cmd[], int vert_split)
 void
 rn_start_bg_command(view_t *view, const char cmd[], MacroFlags flags)
 {
+	/* TODO: similar code in rn_ext(). */
+
 	const int supply_input = ma_flags_present(flags, MF_PIPE_FILE_LIST)
 	                      || ma_flags_present(flags, MF_PIPE_FILE_LIST_Z);
 	FILE *input = NULL;
@@ -1466,7 +1441,7 @@ rn_start_bg_command(view_t *view, const char cmd[], MacroFlags flags)
 }
 
 int
-rn_for_flist(view_t *view, const char cmd[], const char title[],
+rn_for_flist(view_t *view, const char cmd[], const char title[], int user_sh,
 		MacroFlags flags)
 {
 	enum { MAX_TITLE_WIDTH = 80 };
@@ -1494,8 +1469,8 @@ rn_for_flist(view_t *view, const char cmd[], const char title[],
 	FILE *input_tmp = make_in_file(view, flags);
 
 	setup_shellout_env();
-	int error = (process_cmd_output("Loading custom view", cmd, input_tmp, 1,
-				interactive, &path_handler, view) != 0);
+	int error = (process_cmd_output("Loading custom view", cmd, input_tmp,
+				user_sh, interactive, &path_handler, view) != 0);
 	cleanup_shellout_env();
 
 	if(input_tmp != NULL)
