@@ -46,6 +46,10 @@ struct view_t;
 /* Handles opening of current file/selection of the view. */
 void rn_open(struct view_t *view, FileHandleExec exec);
 
+/* Handles opening of current entry of the view as a directory.  Returns zero on
+ * success. */
+int rn_enter_dir(struct view_t *view);
+
 /* Follows file to find its true location (e.g. target of symbolic link) or just
  * opens it. */
 void rn_follow(struct view_t *view, int ultimate);
@@ -89,7 +93,7 @@ void rn_start_bg_command(struct view_t *view, const char cmd[],
  * or very custom view.  Returns zero on success, otherwise non-zero is
  * returned. */
 int rn_for_flist(struct view_t *view, const char cmd[], const char title[],
-		MacroFlags flags);
+		int user_sh, MacroFlags flags);
 
 /* Executes external command capturing its output as list of lines.  Sets *lines
  * and *nlines.  Returns zero on success, otherwise non-zero is returned. */
