@@ -31,28 +31,34 @@
 #include <assert.h> /* assert() */
 #include <limits.h> /* INT_MIN */
 #include <stddef.h> /* NULL */
-#include <string.h>
+#include <string.h> /* memcpy() memmove() */
 #include <time.h> /* time_t time() */
 
 #include "cfg/config.h"
 #include "compat/fs_limits.h"
 #include "compat/pthread.h"
 #include "compat/reallocarray.h"
+#include "lua/vlua.h"
 #include "modes/modes.h"
 #include "ui/colors.h"
 #include "ui/ui.h"
 #include "utils/env.h"
+#include "utils/fs.h"
 #include "utils/fsdata.h"
 #include "utils/log.h"
 #include "utils/macros.h"
 #include "utils/path.h"
 #include "utils/str.h"
+#include "utils/string_array.h"
+#include "utils/test_helpers.h"
 #include "utils/utils.h"
 #include "cmd_completion.h"
 #include "cmd_core.h"
 #include "filelist.h"
 #include "filetype.h"
+#include "ipc.h"
 #include "opt_handlers.h"
+#include "plugins.h"
 
 /* Environment variables by which application hosted by terminal multiplexer can
  * identify the host. */
@@ -70,6 +76,16 @@ typedef struct
 }
 dcache_data_t;
 
+/* Saved view selection. */
+typedef struct
+{
+	char *location; /* Path at which selection was done. */
+
+	char **paths; /* Names of selected files. */
+	int length;   /* Number of items in paths. */
+}
+selection_t;
+
 static void load_def_values(status_t *stats, config_t *config);
 static void determine_fuse_umount_cmd(status_t *stats);
 static void set_gtk_available(status_t *stats);
@@ -78,6 +94,9 @@ static void set_last_cmdline_command(const char cmd[]);
 static void dcache_get(const char path[], time_t mtime, uint64_t inode,
 		dcache_result_t *size, dcache_result_t *nitems);
 static void size_updater(void *data, void *arg);
+TSTATIC time_t dcache_get_size_timestamp(const char path[]);
+TSTATIC void dcache_set_size_timestamp(const char path[], time_t ts);
+static void rotate_right(void *ptr, size_t count, size_t item_len);
 
 status_t curr_stats;
 
@@ -103,6 +122,9 @@ static int silent_ui;
 /* Whether silencing UI led to skipping of screen updates. */
 static int silence_skipped_updates;
 
+/* Selection history.  Most recently updated entry comes first. */
+static selection_t sel_hist[10];
+
 int
 stats_init(config_t *config)
 {
@@ -118,6 +140,7 @@ stats_init(config_t *config)
 	update_string(&curr_stats.term_name, env_get("TERM"));
 
 	(void)hist_init(&curr_stats.cmd_hist, config->history_len);
+	(void)hist_init(&curr_stats.exprreg_hist, config->history_len);
 	(void)hist_init(&curr_stats.search_hist, config->history_len);
 	(void)hist_init(&curr_stats.prompt_hist, config->history_len);
 	(void)hist_init(&curr_stats.filter_hist, config->history_len);
@@ -189,7 +212,7 @@ load_def_values(status_t *stats, config_t *config)
 
 	stats->ellipsis = "...";
 
-	stats->shell_type = ST_NORMAL;
+	stats->shell_type = ST_POSIX;
 
 	stats->fuse_umount_cmd = "";
 
@@ -201,12 +224,22 @@ load_def_values(status_t *stats, config_t *config)
 
 	update_string(&stats->on_choose, NULL);
 
+	update_string(&stats->last_session, NULL);
+
 	stats->preview_hint = NULL;
 
+	free_string_array(stats->plugins_dirs.items, stats->plugins_dirs.nitems);
+	stats->plugins_dirs.items = NULL;
+	stats->plugins_dirs.nitems = 0;
+
 	stats->global_local_settings = 0;
 
 	stats->history_size = 0;
 
+	ipc_free(stats->ipc);
+	plugs_free(stats->plugs);
+	vlua_finish(stats->vlua);
+
 	stats->ipc = NULL;
 	stats->vlua = NULL;
 	stats->plugs = NULL;
@@ -494,6 +527,7 @@ hists_resize(int new_size)
 {
 	curr_stats.history_size = new_size;
 
+	hist_resize(&curr_stats.exprreg_hist, new_size);
 	hist_resize(&curr_stats.search_hist, new_size);
 	hist_resize(&curr_stats.cmd_hist, new_size);
 	hist_resize(&curr_stats.prompt_hist, new_size);
@@ -503,7 +537,7 @@ hists_resize(int new_size)
 void
 hists_commands_save(const char command[])
 {
-	if(is_history_command(command))
+	if(cmds_goes_to_history(command))
 	{
 		if(!curr_stats.restart_in_progress && curr_stats.load_stage == 3)
 		{
@@ -526,6 +560,12 @@ set_last_cmdline_command(const char cmd[])
 			"The field was not initialized properly");
 }
 
+void
+hists_exprreg_save(const char pattern[])
+{
+	hist_add(&curr_stats.exprreg_hist, pattern, -1);
+}
+
 void
 hists_search_save(const char pattern[])
 {
@@ -684,5 +724,107 @@ dcache_set_at(const char path[], uint64_t inode, uint64_t size, uint64_t nitems)
 	return ret;
 }
 
+TSTATIC time_t
+dcache_get_size_timestamp(const char path[])
+{
+	dcache_data_t size_data;
+	if(fsdata_get(dcache_size, path, &size_data, sizeof(size_data)) == 0)
+	{
+		return size_data.timestamp;
+	}
+
+	return -1;
+}
+
+TSTATIC void
+dcache_set_size_timestamp(const char path[], time_t ts)
+{
+	dcache_data_t size_data;
+	if(fsdata_get(dcache_size, path, &size_data, sizeof(size_data)) == 0)
+	{
+		size_data.timestamp = ts;
+		(void)fsdata_set(dcache_size, path, &size_data, sizeof(size_data));
+	}
+}
+
+void
+selhist_put(const char location[], char *paths[], int path_count)
+{
+	if(path_count == 0)
+	{
+		free_string_array(paths, 0);
+		return;
+	}
+
+	char *location_copy = strdup(location);
+	if(location_copy == NULL)
+	{
+		free_string_array(paths, path_count);
+		return;
+	}
+
+	unsigned int i;
+	for(i = 0; i < ARRAY_LEN(sel_hist) - 1; ++i)
+	{
+		if(sel_hist[i].location != NULL &&
+				paths_are_same(sel_hist[i].location, location))
+		{
+			break;
+		}
+	}
+
+	/* At this point we have either found a match among all but the last element
+	 * or will use the last element (whether it's a match doesn't matter). */
+
+	free(sel_hist[i].location);
+	free_string_array(sel_hist[i].paths, sel_hist[i].length);
+
+	sel_hist[i].location = location_copy;
+	sel_hist[i].paths = paths;
+	sel_hist[i].length = path_count;
+
+	rotate_right(sel_hist, i + 1, sizeof(sel_hist[0]));
+}
+
+/* Rotates array's elements to the right once: 0 -> 1, ..., (n - 1) -> 0. */
+static void
+rotate_right(void *ptr, size_t count, size_t item_len)
+{
+	assert(count > 0 && "Can't rotate if item count is 0!");
+	assert(item_len > 0 && "Can't rotate if item size is 0!");
+
+	size_t slice_size = item_len*(count - 1);
+	char *p = ptr;
+
+	/* Stash last item. */
+	uint8_t buf[item_len];
+	memcpy(buf, p + slice_size, item_len);
+
+	/* Move all but the last item. */
+	memmove(p + item_len, p, slice_size);
+
+	/* Put last item in front. */
+	memcpy(p, buf, item_len);
+}
+
+int
+selhist_get(const char location[], char ***paths, int *path_count)
+{
+	unsigned int i;
+
+	for(i = 0; i < ARRAY_LEN(sel_hist); ++i)
+	{
+		if(sel_hist[i].location != NULL &&
+				paths_are_same(sel_hist[i].location, location))
+		{
+			*paths = copy_string_array(sel_hist[i].paths, sel_hist[i].length);
+			*path_count = sel_hist[i].length;
+			return 0;
+		}
+	}
+
+	return 1;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
