@@ -67,6 +67,8 @@ enum
 	JOB_LINE_COLOR,     /* Line that displays status of background jobs. */
 	SUGGEST_BOX_COLOR,  /* Style of suggestion box. */
 	MISMATCH_COLOR,     /* File entries that don't match each other in diff. */
+	UNMATCHED_COLOR,    /* Diff file entry that has no pair in the other pane. */
+	BLANK_COLOR,        /* Fake entry in a diff. */
 	AUX_WIN_COLOR,      /* Auxiliary part of window. */
 	TAB_LINE_COLOR,     /* Tab line. */
 	TAB_LINE_SEL_COLOR, /* Tip of selected tab. */
@@ -79,12 +81,26 @@ enum
 	USER7_COLOR,        /* User color #7. */
 	USER8_COLOR,        /* User color #8. */
 	USER9_COLOR,        /* User color #9. */
+	USER10_COLOR,       /* User color #10. */
+	USER11_COLOR,       /* User color #11. */
+	USER12_COLOR,       /* User color #12. */
+	USER13_COLOR,       /* User color #13. */
+	USER14_COLOR,       /* User color #14. */
+	USER15_COLOR,       /* User color #15. */
+	USER16_COLOR,       /* User color #16. */
+	USER17_COLOR,       /* User color #17. */
+	USER18_COLOR,       /* User color #18. */
+	USER19_COLOR,       /* User color #19. */
+	USER20_COLOR,       /* User color #20. */
 	OTHER_WIN_COLOR,    /* Background and highlighting for inactive pane. */
 	LINE_NUM_COLOR,     /* Color of line number column of panes. */
 	ODD_LINE_COLOR,     /* Color of every second entry line in a pane. */
 	MAXNUM_COLOR        /* Number of elements of a color scheme. */
 };
 
+/* The last of USER*_COLOR groups. */
+#define LAST_USER_COLOR 20
+
 #endif /* VIFM__UI__COLORS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
