@@ -21,7 +21,7 @@
 
 #include <curses.h>
 
-#include <regex.h> /* regmatch_t regcomp() regexec() */
+#include <regex.h> /* regmatch_t regexec() */
 
 #ifndef _WIN32
 #include <pwd.h>
@@ -49,6 +49,7 @@
 #include "../flist_pos.h"
 #include "../opt_handlers.h"
 #include "../sort.h"
+#include "../vifm.h"
 #include "color_scheme.h"
 #include "column_view.h"
 #include "quickview.h"
@@ -59,8 +60,9 @@
 
 static void draw_left_column(view_t *view);
 static void draw_right_column(view_t *view);
-static void print_column(view_t *view, entries_t entries, const char current[],
-		const char path[], int width, int offset, int number_width);
+static void print_side_column(view_t *view, entries_t entries,
+		const char current[], const char path[], int width, int offset,
+		int number_width);
 static void fill_column(view_t *view, int start_line, int top, int width,
 		int offset);
 static void calculate_table_conf(view_t *view, size_t *count, size_t *width);
@@ -69,10 +71,7 @@ static int calculate_number_width(const view_t *view, int list_length,
 static int count_digits(int num);
 static int calculate_top_position(view_t *view, int top);
 static int get_line_color(const view_t *view, const dir_entry_t *entry);
-static size_t calculate_print_width(const view_t *view, int i,
-		size_t max_width);
-static void draw_cell(columns_t *columns, column_data_t *cdt, size_t col_width,
-		size_t print_width);
+static void draw_cell(columns_t *columns, column_data_t *cdt, size_t col_width);
 static columns_t * get_view_columns(const view_t *view, int truncated);
 static columns_t * get_name_column(int truncated);
 static void consider_scroll_bind(view_t *view);
@@ -80,7 +79,7 @@ static cchar_t prepare_inactive_color(view_t *view, dir_entry_t *entry,
 		int line_color);
 static void redraw_cell(view_t *view, int top, int cursor, int is_current);
 static void compute_and_draw_cell(column_data_t *cdt, int cell,
-		size_t col_width);
+		size_t col_count, size_t col_width);
 static void column_line_print(const char buf[], size_t offset, AlignType align,
 		const char full_column[], const format_info_t *info);
 static void draw_line_number(const column_data_t *cdt, int column);
@@ -135,6 +134,7 @@ static void format_id(void *data, size_t buf_len, char buf[],
 		const format_info_t *info);
 static size_t calculate_column_width(view_t *view);
 static size_t calculate_columns_count(view_t *view);
+static int has_extra_tls_col(const view_t *view, int col_width);
 static preview_area_t get_miller_preview_area(view_t *view);
 static size_t get_max_filename_width(const view_t *view);
 static size_t get_filename_width(const view_t *view, int i);
@@ -213,7 +213,7 @@ fview_init(view_t *view)
 
 	view->sort_groups = strdup("");
 	view->sort_groups_g = strdup("");
-	(void)regcomp(&view->primary_group, view->sort_groups,
+	(void)regexp_compile(&view->primary_group, view->sort_groups,
 			REG_EXTENDED | REG_ICASE);
 
 	view->preview_prg = strdup("");
@@ -228,6 +228,7 @@ fview_reset(view_t *view)
 {
 	view->ls_view_g = view->ls_view = 0;
 	view->ls_transposed_g = view->ls_transposed = 0;
+	view->ls_cols_g = view->ls_cols = 0;
 	/* Invalidate maximum file name widths cache. */
 	view->max_filename_width = 0;;
 	view->column_count = 1;
@@ -237,7 +238,7 @@ fview_reset(view_t *view)
 	view->miller_ratios_g[0] = view->miller_ratios[0] = 1;
 	view->miller_ratios_g[1] = view->miller_ratios[1] = 1;
 	view->miller_ratios_g[2] = view->miller_ratios[2] = 1;
-	view->miller_preview_files_g = view->miller_preview_files = 0;
+	view->miller_preview_g = view->miller_preview = MP_DIRS;
 
 	view->num_type_g = view->num_type = NT_NONE;
 	view->num_width_g = view->num_width = 4;
@@ -274,7 +275,7 @@ draw_dir_list_only(view_t *view)
 	size_t col_width, col_count;
 	int visible_cells;
 
-	if(curr_stats.load_stage < 2)
+	if(curr_stats.load_stage < 2 || vifm_testing())
 	{
 		return;
 	}
@@ -297,10 +298,8 @@ draw_dir_list_only(view_t *view)
 	draw_left_column(view);
 
 	visible_cells = view->window_cells;
-	if(fview_is_transposed(view) &&
-			view->column_count*(int)col_width < ui_view_available_width(view))
+	if(has_extra_tls_col(view, col_width))
 	{
-		/* Add extra visual column to display more context. */
 		visible_cells += view->window_rows;
 	}
 
@@ -315,7 +314,7 @@ draw_dir_list_only(view_t *view)
 			.current_pos = view->list_pos,
 		};
 
-		compute_and_draw_cell(&cdt, cell, col_width);
+		compute_and_draw_cell(&cdt, cell, col_count, col_width);
 	}
 
 	draw_right_column(view);
@@ -345,7 +344,7 @@ draw_left_column(view_t *view)
 	               - (cfg.extra_padding ? 1 : 0) - 1;
 	if(lcol_width <= 0)
 	{
-		flist_free_cache(view, &view->left_column);
+		flist_free_cache(&view->left_column);
 		return;
 	}
 
@@ -359,7 +358,7 @@ draw_left_column(view_t *view)
 
 	if(view->left_column.entries.nentries >= 0)
 	{
-		print_column(view, view->left_column.entries, dir, path, lcol_width, 0,
+		print_side_column(view, view->left_column.entries, dir, path, lcol_width, 0,
 				number_width);
 	}
 }
@@ -379,7 +378,7 @@ draw_right_column(view_t *view)
 	const int rcol_width = ui_view_right_reserved(view) - padding - 1;
 	if(rcol_width <= 0)
 	{
-		flist_free_cache(view, &view->right_column);
+		flist_free_cache(&view->right_column);
 		return;
 	}
 
@@ -398,13 +397,18 @@ draw_right_column(view_t *view)
 	};
 
 	dir_entry_t *const entry = get_current_entry(view);
-	if(view->miller_preview_files && !fentry_is_dir(entry))
+	if(view->miller_preview != MP_DIRS && !fentry_is_dir(entry))
 	{
 		const char *clear_cmd = qv_draw_on(entry, &parea);
 		update_string(&view->file_preview_clear_cmd, clear_cmd);
 		return;
 	}
 
+	if(view->miller_preview == MP_FILES && fentry_is_dir(entry))
+	{
+		return;
+	}
+
 	if(displayed_graphics)
 	{
 		/* Do this even if there is no clear command. */
@@ -418,7 +422,7 @@ draw_right_column(view_t *view)
 
 	if(view->right_column.entries.nentries >= 0)
 	{
-		print_column(view, view->right_column.entries, NULL, path, rcol_width,
+		print_side_column(view, view->right_column.entries, NULL, path, rcol_width,
 				offset, 0);
 	}
 }
@@ -426,7 +430,7 @@ draw_right_column(view_t *view)
 /* Prints column full of entry names.  Current is a hint that tells which column
  * has to be selected (otherwise position from history record is used). */
 static void
-print_column(view_t *view, entries_t entries, const char current[],
+print_side_column(view_t *view, entries_t entries, const char current[],
 		const char path[], int width, int offset, int number_width)
 {
 	columns_t *const columns = get_name_column(0);
@@ -476,6 +480,8 @@ print_column(view_t *view, entries_t entries, const char current[],
 		flist_hist_update(view, path, get_last_path_component(current), pos - top);
 	}
 
+	int padding = (cfg.extra_padding ? 1 : 0);
+
 	for(i = top; i < entries.nentries && i - top < view->window_rows; ++i)
 	{
 		size_t prefix_len = 0U;
@@ -492,7 +498,7 @@ print_column(view_t *view, entries_t entries, const char current[],
 			.prefix_len = &prefix_len,
 		};
 
-		draw_cell(columns, &cdt, width, width - 1);
+		draw_cell(columns, &cdt, width - padding);
 	}
 
 	fill_column(view, i, top, number_width + width, offset);
@@ -664,28 +670,28 @@ get_line_color(const view_t *view, const dir_entry_t *entry)
 	}
 }
 
-/* Calculates width of the column using entry and maximum width. */
-static size_t
-calculate_print_width(const view_t *view, int i, size_t max_width)
+/* Draws a full cell of the file list.  col_width doesn't include extra padding!
+ * The total printed widths will be that plus two empty cells (one before and
+ * one after). */
+static void
+draw_cell(columns_t *columns, column_data_t *cdt, size_t col_width)
 {
-	if(!ui_view_displays_columns(view))
+	size_t width_left;
+	if(cdt->view->ls_view)
 	{
-		const size_t raw_name_width = get_filename_width(view, i);
-		return MIN(max_width - 1, raw_name_width);
+		width_left = cdt->view->window_cols
+		           - cdt->column_offset
+		           - ui_view_right_reserved(cdt->view)
+		           - (cfg.extra_padding ? 2 : 0);
+	}
+	else
+	{
+		width_left = cdt->is_main
+		           ? ui_view_available_width(cdt->view) -
+		             (cdt->column_offset - ui_view_left_reserved(cdt->view))
+		           : col_width + 1U;
 	}
 
-	return max_width;
-}
-
-/* Draws a full cell of the file list.  print_width <= col_width. */
-static void
-draw_cell(columns_t *columns, column_data_t *cdt, size_t col_width,
-		size_t print_width)
-{
-	size_t width_left = cdt->is_main
-	                  ? ui_view_available_width(cdt->view) - (cdt->column_offset -
-	                    ui_view_left_reserved(cdt->view))
-	                  : col_width + 1U;
 	const format_info_t info = {
 		.data = cdt,
 		.id = FILL_COLUMN_ID
@@ -698,9 +704,9 @@ draw_cell(columns_t *columns, column_data_t *cdt, size_t col_width,
 
 	columns_format_line(columns, cdt, MIN(col_width, width_left));
 
-	if(cfg.extra_padding && width_left >= col_width)
+	if(cfg.extra_padding)
 	{
-		column_line_print(" ", print_width, AT_LEFT, " ", &info);
+		column_line_print(" ", col_width, AT_LEFT, " ", &info);
 	}
 }
 
@@ -796,11 +802,11 @@ consider_scroll_bind(view_t *view)
 		other->top_line *= other->column_count;
 		other->top_line = calculate_top_position(other, other->top_line);
 
-		if(can_scroll_up(other))
+		if(fpos_can_scroll_back(other))
 		{
 			(void)fpos_scroll_down(other, 0);
 		}
-		if(can_scroll_down(other))
+		if(fpos_can_scroll_fwd(other))
 		{
 			(void)fpos_scroll_up(other, 0);
 		}
@@ -868,6 +874,23 @@ fview_cursor_redraw(view_t *view)
 	}
 }
 
+void
+fview_clear_miller_preview(view_t *view)
+{
+	if(!view->miller_view || view->miller_preview == MP_DIRS)
+	{
+		return;
+	}
+
+	const int padding = (cfg.extra_padding ? 1 : 0);
+	const int rcol_width = ui_view_right_reserved(view) - padding - 1;
+	if(rcol_width > 0)
+	{
+		const preview_area_t parea = get_miller_preview_area(view);
+		qv_cleanup_area(&parea, view->file_preview_clear_cmd);
+	}
+}
+
 void
 fview_draw_inactive_cursor(view_t *view)
 {
@@ -878,6 +901,11 @@ fview_draw_inactive_cursor(view_t *view)
 	 * be drawn next time. */
 	invalidate_cursor_pos_cache(view);
 
+	if(curr_stats.load_stage < 2)
+	{
+		return;
+	}
+
 	if(!ui_view_displays_columns(view))
 	{
 		/* Inactive cell in ls-like view usually takes less space than an active
@@ -947,22 +975,21 @@ redraw_cell(view_t *view, int top, int cursor, int is_current)
 		.line_pos = pos,
 		.current_pos = is_current ? view->list_pos : -1,
 	};
-	compute_and_draw_cell(&cdt, cursor, col_width);
+	compute_and_draw_cell(&cdt, cursor, col_count, col_width);
 }
 
 /* Fills in fields of cdt based on passed in arguments and
  * view/entry/line_pos/current_pos fields of cdt.  Then draws the cell. */
 static void
-compute_and_draw_cell(column_data_t *cdt, int cell, size_t col_width)
+compute_and_draw_cell(column_data_t *cdt, int cell, size_t col_count,
+		size_t col_width)
 {
 	size_t prefix_len = 0U;
 
-	const size_t print_width = calculate_print_width(cdt->view, cdt->line_pos,
-			col_width);
+	int col = fpos_get_col(cdt->view, cell);
 
 	cdt->current_line = fpos_get_line(cdt->view, cell);
-	cdt->column_offset = ui_view_left_reserved(cdt->view)
-	                   + fpos_get_col(cdt->view, cell)*col_width;
+	cdt->column_offset = ui_view_left_reserved(cdt->view) + col*col_width;
 	cdt->line_hi_group = get_line_color(cdt->view, cdt->entry);
 	cdt->number_width = cdt->view->real_num_width;
 	cdt->total_width = ui_view_available_width(cdt->view);
@@ -971,31 +998,28 @@ compute_and_draw_cell(column_data_t *cdt, int cell, size_t col_width)
 
 	if(cfg.extra_padding && !ui_view_displays_columns(cdt->view))
 	{
-		/* Padding in ls-like view adds additional empty single character between
-		 * columns, on which we shouldn't draw anything here. */
-		--col_width;
+		if(cdt->view->ls_cols != 0 && col_count > 1 && col == (int)col_count - 1)
+		{
+			/* Reserve one character column after the last ls column. */
+			col_width -= 1;
+		}
+		else
+		{
+			/* Reserve two character columns between two ls columns to draw padding or
+			 * before and after single column if it's the only one. */
+			col_width -= 2;
+		}
 	}
 
-	draw_cell(get_view_columns(cdt->view, cell >= cdt->view->window_cells), cdt,
-			col_width, print_width);
+	int truncated = (cell >= cdt->view->window_cells);
+	columns_t *columns = get_view_columns(cdt->view, truncated);
+	draw_cell(columns, cdt, col_width);
 
 	cdt->prefix_len = NULL;
 }
 
-int
-can_scroll_up(const view_t *view)
-{
-	return view->top_line > 0;
-}
-
-int
-can_scroll_down(const view_t *view)
-{
-	return fpos_get_last_visible_cell(view) < view->list_rows - 1;
-}
-
 void
-scroll_up(view_t *view, int by)
+fview_scroll_back_by(view_t *view, int by)
 {
 	/* Round it up, so 1 will cause one line scrolling. */
 	view->top_line -= view->run_size*DIV_ROUND_UP(by, view->run_size);
@@ -1009,7 +1033,7 @@ scroll_up(view_t *view, int by)
 }
 
 void
-scroll_down(view_t *view, int by)
+fview_scroll_fwd_by(view_t *view, int by)
 {
 	/* Round it up, so 1 will cause one line scrolling. */
 	view->top_line += view->run_size*DIV_ROUND_UP(by, view->run_size);
@@ -1018,36 +1042,21 @@ scroll_down(view_t *view, int by)
 	view->curr_line = view->list_pos - view->top_line;
 }
 
-int
-get_corrected_list_pos_down(const view_t *view, int pos_delta)
+void
+fview_scroll_by(view_t *view, int by)
 {
-	const int scroll_offset = fpos_get_offset(view);
-	if(view->list_pos <= view->top_line + scroll_offset + (MAX(pos_delta, 1) - 1))
+	if(by > 0)
 	{
-		const int column_correction = view->list_pos%view->column_count;
-		const int offset = scroll_offset + pos_delta + column_correction;
-		return view->top_line + offset;
+		fview_scroll_fwd_by(view, by);
+	}
+	else if(by < 0)
+	{
+		fview_scroll_back_by(view, -by);
 	}
-	return view->list_pos;
 }
 
 int
-get_corrected_list_pos_up(const view_t *view, int pos_delta)
-{
-	const int scroll_offset = fpos_get_offset(view);
-	const int last = fpos_get_last_visible_cell(view);
-	if(view->list_pos >= last - scroll_offset - (MAX(pos_delta, 1) - 1))
-	{
-		const int column_correction = (view->column_count - 1)
-		                            - view->list_pos%view->column_count;
-		const int offset = scroll_offset + pos_delta + column_correction;
-		return last - offset;
-	}
-	return view->list_pos;
-}
-
-int
-consider_scroll_offset(view_t *view)
+fview_enforce_scroll_offset(view_t *view)
 {
 	int need_redraw = 0;
 	int pos = view->list_pos;
@@ -1055,18 +1064,18 @@ consider_scroll_offset(view_t *view)
 	{
 		const int s = fpos_get_offset(view);
 		/* Check scroll offset at the top. */
-		if(can_scroll_up(view) && pos - view->top_line < s)
+		if(fpos_can_scroll_back(view) && pos - view->top_line < s)
 		{
-			scroll_up(view, s - (pos - view->top_line));
+			fview_scroll_back_by(view, s - (pos - view->top_line));
 			need_redraw = 1;
 		}
 		/* Check scroll offset at the bottom. */
-		if(can_scroll_down(view))
+		if(fpos_can_scroll_fwd(view))
 		{
 			const int last = fpos_get_last_visible_cell(view);
 			if(pos > last - s)
 			{
-				scroll_down(view, s + (pos - last));
+				fview_scroll_fwd_by(view, s + (pos - last));
 				need_redraw = 1;
 			}
 		}
@@ -1075,24 +1084,21 @@ consider_scroll_offset(view_t *view)
 }
 
 void
-scroll_by_files(view_t *view, int by)
+fview_scroll_page_up(view_t *view)
 {
-	if(by > 0)
+	if(fpos_can_scroll_back(view))
 	{
-		scroll_down(view, by);
-	}
-	else if(by < 0)
-	{
-		scroll_up(view, -by);
+		fpos_scroll_page(view, fpos_get_last_visible_cell(view), -1);
 	}
 }
 
 void
-update_scroll_bind_offset(void)
+fview_scroll_page_down(view_t *view)
 {
-	const int rwin_pos = rwin.top_line/rwin.column_count;
-	const int lwin_pos = lwin.top_line/lwin.column_count;
-	curr_stats.scroll_bind_off = rwin_pos - lwin_pos;
+	if(fpos_can_scroll_fwd(view))
+	{
+		fpos_scroll_page(view, view->top_line, 1);
+	}
 }
 
 /* Print callback for column_view unit. */
@@ -1116,6 +1122,8 @@ column_line_print(const char buf[], size_t offset, AlignType align,
 	                 || info->id == SK_BY_INAME
 	                 || info->id == SK_BY_ROOT
 	                 || info->id == SK_BY_FILEROOT
+	                 || info->id == SK_BY_EXTENSION
+	                 || info->id == SK_BY_FILEEXT
 	                 || vlua_viewcolumn_is_primary(curr_stats.vlua, info->id);
 	const cchar_t line_attrs = prepare_col_color(view, primary, 0, cdt);
 
@@ -1374,12 +1382,24 @@ mix_in_common_colors(col_attr_t *col, const view_t *view, dir_entry_t *entry,
 	const col_scheme_t *const cs = ui_view_get_cs(view);
 	view_t *const other = (view == &lwin) ? &rwin : &lwin;
 
-	/* If two files on the same line in side-by-side comparison have different
-	 * ids, that's a mismatch. */
-	if(view->custom.type == CV_DIFF &&
-			other->dir_entry[entry_to_pos(view, entry)].id != entry->id)
+	if(view->custom.type == CV_DIFF)
 	{
-		cs_mix_colors(col, &cs->color[MISMATCH_COLOR]);
+		const dir_entry_t *oentry = &other->dir_entry[entry_to_pos(view, entry)];
+
+		/* If two files on the same line in side-by-side comparison have different
+		 * ids, that's a mismatch. */
+		if(oentry->id != entry->id)
+		{
+			cs_mix_colors(col, &cs->color[MISMATCH_COLOR]);
+		}
+		else if(fentry_is_fake(entry))
+		{
+			cs_mix_colors(col, &cs->color[BLANK_COLOR]);
+		}
+		else if(fentry_is_fake(oentry))
+		{
+			cs_mix_colors(col, &cs->color[UNMATCHED_COLOR]);
+		}
 	}
 
 	if(entry->selected)
@@ -1745,7 +1765,7 @@ fview_is_transposed(const view_t *view)
 int
 fview_previews(view_t *view, const char path[])
 {
-	if(!view->miller_view || !view->miller_preview_files)
+	if(!view->miller_view || view->miller_preview == MP_DIRS)
 	{
 		return 0;
 	}
@@ -1773,13 +1793,86 @@ fview_set_millerview(view_t *view, int enabled)
 static size_t
 calculate_column_width(view_t *view)
 {
-	const int column_gap = (cfg.extra_padding ? 2 : 1);
-	if(view->max_filename_width == 0)
+	size_t max_width = view->window_cols
+	                 - ui_view_left_reserved(view) - ui_view_right_reserved(view);
+
+	size_t column_width;
+	if(view->ls_cols == 0)
 	{
-		view->max_filename_width = get_max_filename_width(view);
+		if(view->max_filename_width == 0)
+		{
+			view->max_filename_width = get_max_filename_width(view);
+		}
+
+		const int column_gap = (cfg.extra_padding ? 2 : 1);
+		column_width = view->max_filename_width + column_gap;
 	}
-	return MIN(view->max_filename_width + column_gap,
-	           (size_t)ui_view_available_width(view));
+	else
+	{
+		column_width = max_width/view->ls_cols;
+	}
+
+	return MIN(column_width, max_width);
+}
+
+int
+fview_map_coordinates(view_t *view, int x, int y)
+{
+	if(view->miller_view)
+	{
+		const int padding = (cfg.extra_padding ? 1 : 0);
+		const int lcol_end = ui_view_left_reserved(view);
+		const int rcol_start = lcol_end + padding
+		                     + ui_view_available_width(view) + padding;
+
+		if(x < lcol_end)
+		{
+			return FVM_LEAVE;
+		}
+		if(x >= rcol_start)
+		{
+			return FVM_OPEN;
+		}
+	}
+
+	int pos;
+	if(ui_view_displays_columns(view))
+	{
+	  pos = view->top_line + y;
+	}
+	else
+	{
+		size_t col_count, col_width;
+		calculate_table_conf(view, &col_count, &col_width);
+
+		if(has_extra_tls_col(view, col_width))
+		{
+			++col_count;
+		}
+
+		size_t x_offset = x/col_width;
+		if(x_offset >= col_count)
+		{
+			return FVM_NONE;
+		}
+
+		pos = fview_is_transposed(view)
+		    ? view->top_line + view->run_size*x_offset + y
+		    : view->top_line + view->run_size*y + x_offset;
+	}
+
+	return (pos < view->list_rows ? pos : FVM_NONE);
+}
+
+/* Whether there is an extra visual column to transposed ls-like view to display
+ * more context (when available and unless user requested fixed number of
+ * columns).  Returns non-zero if so. */
+static int
+has_extra_tls_col(const view_t *view, int col_width)
+{
+	return fview_is_transposed(view)
+			&& view->ls_cols == 0
+			&& view->column_count*col_width < ui_view_available_width(view);
 }
 
 void
@@ -1795,17 +1888,7 @@ void
 fview_dir_updated(view_t *view)
 {
 	view->local_cs = cs_load_local(view == &lwin, view->curr_dir);
-
-	if(view->miller_view && view->miller_preview_files)
-	{
-		const int padding = (cfg.extra_padding ? 1 : 0);
-		const int rcol_width = ui_view_right_reserved(view) - padding - 1;
-		if(rcol_width > 0)
-		{
-			const preview_area_t parea = get_miller_preview_area(view);
-			qv_cleanup_area(&parea, view->file_preview_clear_cmd);
-		}
-	}
+	fview_clear_miller_preview(view);
 }
 
 /* Computes area description for miller preview.  Returns the area. */
@@ -1856,7 +1939,9 @@ calculate_columns_count(view_t *view)
 	if(!ui_view_displays_columns(view))
 	{
 		const size_t column_width = calculate_column_width(view);
-		return ui_view_available_width(view)/column_width;
+		size_t max_width = view->window_cols - ui_view_left_reserved(view)
+		                 - ui_view_right_reserved(view);
+		return max_width/column_width;
 	}
 	return 1U;
 }
@@ -2037,10 +2122,12 @@ position_hardware_cursor(view_t *view)
 		return;
 	}
 
+	int cell = view->list_pos - view->top_line;
+
 	calculate_table_conf(view, &col_count, &col_width);
-	current_line = view->curr_line/col_count;
+	current_line = fpos_get_line(view, cell);
 	column_offset = ui_view_left_reserved(view)
-	              + (view->curr_line%col_count)*col_width;
+	              + fpos_get_col(view, cell)*col_width;
 	format_name(NULL, sizeof(buf) - 1U, buf, &info);
 
 	checked_wmove(view->win, current_line,
@@ -2080,16 +2167,16 @@ move_curr_line(view_t *view)
 	}
 	else if(pos > last)
 	{
-		scroll_down(view, pos - last);
+		fview_scroll_fwd_by(view, pos - last);
 		redraw++;
 	}
 	else if(pos < view->top_line)
 	{
-		scroll_up(view, view->top_line - pos);
+		fview_scroll_back_by(view, view->top_line - pos);
 		redraw++;
 	}
 
-	if(consider_scroll_offset(view))
+	if(fview_enforce_scroll_offset(view))
 	{
 		redraw++;
 	}
