@@ -86,6 +86,7 @@ typedef struct
 	int nfiles;        /* Number of seen files. */
 	int max;           /* Maximum line number. */
 	int full_stats;    /* Collect statistics for the whole tree. */
+	int depth;         /* Current depth of the traversal. */
 	char prefix[4096]; /* Prefix character for each tree level. */
 }
 tree_print_state_t;
@@ -317,6 +318,14 @@ view_file(const char path[], const preview_area_t *parea,
 	const char *viewer = qv_get_viewer(path);
 	const char *clear_cmd = (viewer != NULL ? ma_get_clear_cmd(viewer) : NULL);
 
+	/* Don't draw graphics while dialog is shown as they aren't combined nicely
+	 * on the screen. */
+	const ViewerKind kind = ft_viewer_kind(viewer);
+	if(kind != VK_TEXTUAL && modes_is_dialog_like())
+	{
+		return NULL;
+	}
+
 	if(is_cache_valid(cache, path, viewer, parea))
 	{
 		/* Update area as we might draw preview at a different location. */
@@ -326,7 +335,6 @@ view_file(const char path[], const preview_area_t *parea,
 		return clear_cmd;
 	}
 
-	ViewerKind kind = ft_viewer_kind(viewer);
 	int max_lines = is_dir(path) ? ui_qv_height(parea->view)
 	                             : MAX_PREVIEW_LINES;
 
@@ -423,9 +431,12 @@ get_lines(const quickview_cache_t *cache)
 
 	const char *error;
 
-	char *expanded = (cache->viewer == NULL) ? NULL
-	                                         : qv_expand_viewer(cache->viewer);
-	strlist_t lines = vcache_lookup(cache->path, expanded, MF_NONE, cache->kind,
+	MacroFlags flags = MF_NONE;
+	char *expanded = (cache->viewer == NULL)
+	               ? NULL
+	               : qv_expand_viewer(cache->pa.source, cache->viewer, &flags);
+
+	strlist_t lines = vcache_lookup(cache->path, expanded, flags, cache->kind,
 			cache->max_lines, VC_ASYNC, &error);
 	free(expanded);
 
@@ -523,6 +534,15 @@ print_dir_tree(tree_print_state_t *s, const char path[], int last)
 		return 1;
 	}
 
+	/* No need to check cfg.max_tree_depth for 0, after enter_dir s->depth is
+	 * greater than 0. */
+	if(s->depth == cfg.max_tree_depth)
+	{
+		free_string_array(lst, len);
+		leave_dir(s);
+		return 0;
+	}
+
 	int i;
 	int reached_limit = 0;
 	for(i = 0; i < len && !reached_limit && !ui_cancellation_requested(); ++i)
@@ -652,6 +672,7 @@ enter_dir(tree_print_state_t *s, const char path[], int last)
 	indent_prefix(s);
 	set_prefix_char(s, '|');
 
+	++s->depth;
 	return (++s->n >= s->max);
 }
 
@@ -686,6 +707,7 @@ static void
 leave_dir(tree_print_state_t *s)
 {
 	unindent_prefix(s);
+	--s->depth;
 }
 
 /* Adds one indentation level for the following elements of tree. */
@@ -833,21 +855,22 @@ qv_hide(void)
 	}
 }
 
-/* Returns a pointer to newly allocated memory, which should be released by the
- * caller. */
 char *
-qv_expand_viewer(const char viewer[])
+qv_expand_viewer(view_t *view, const char viewer[], MacroFlags *flags)
 {
+	ma_flags_set(flags, MF_NONE);
+
 	char *result;
 	if(strchr(viewer, '%') == NULL)
 	{
-		char *escaped = shell_like_escape(get_current_file_name(curr_view), 0);
+		char *escaped =
+			shell_arg_escape(get_current_file_name(view), curr_stats.shell_type);
 		result = format_str("%s %s", viewer, escaped);
 		free(escaped);
 	}
 	else
 	{
-		result = ma_expand(viewer, NULL, NULL, 1);
+		result = ma_expand(viewer, NULL, flags, MER_SHELL);
 	}
 	return result;
 }
@@ -882,7 +905,7 @@ qv_cleanup_area(const preview_area_t *parea, const char cmd[])
 	curr_stats.preview.clearing = 1;
 	curr_stats.preview_hint = parea;
 
-	char *expanded = qv_expand_viewer(cmd);
+	char *expanded = qv_expand_viewer(parea->view, cmd, NULL);
 	if(vlua_handler_cmd(curr_stats.vlua, expanded))
 	{
 		char path[PATH_MAX + 1];
