@@ -24,7 +24,7 @@
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
 #include <stddef.h> /* NULL size_t */
-#include <stdlib.h> /* RAND_MAX free() rand() */
+#include <stdlib.h> /* free() */
 #include <string.h> /* strcat() strdup() strlen() */
 #include <time.h> /* time() */
 #include <unistd.h>
@@ -71,7 +71,7 @@ static const char * format_job_bar(void);
 static char ** take_job_descr_snapshot(void);
 
 /* List of macros that are expanded in the status line. */
-static const char STATUS_LINE_MACROS[] = "tTfacAugsEdD-xlLPSz%[]{*";
+static const char STATUS_LINE_MACROS[] = "tTfacAugsEdD-xlLoPSz%[]{*";
 
 /* Number of background jobs. */
 static size_t nbar_jobs;
@@ -123,6 +123,10 @@ static void
 split_and_print_status_line(view_t *view, int width)
 {
 	char *status_line = fetch_status_line(view, width);
+	if(status_line == NULL)
+	{
+		return;
+	}
 
 	const char *part = status_line;
 	int line = 0;
@@ -381,6 +385,11 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 				get_perm_string(buf, sizeof(buf), curr->mode);
 #else
 				copy_str(buf, sizeof(buf), attr_str_long(curr->attrs));
+#endif
+				break;
+			case 'o':
+#ifndef _WIN32
+				snprintf(buf, sizeof(buf), "%03o", curr->mode & 0777);
 #endif
 				break;
 			case 'u':
@@ -477,8 +486,6 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 					 * closing brackets */
 					const char *e = strchr(*format, '}');
 					char *expr = NULL, *resstr = NULL;
-					var_t res = var_false();
-					ParsingErrors parsing_error;
 
 					/* If there's no matching closing bracket, just add the opening one
 					 * literally */
@@ -492,13 +499,18 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 					 * TODO: we could temporarily use buf for that, to avoid extra
 					 * allocation, but explicitly named variable reads better. */
 					expr = calloc(e - (*format) + 1 /* NUL-term */, 1);
+					if(expr == NULL)
+					{
+						ok = 0;
+						break;
+					}
 					memcpy(expr, *format, e - (*format));
 
-					/* Try to parse expr, and convert the res to string if succeed. */
-					parsing_error = parse(expr, 0, &res);
-					if(parsing_error == PE_NO_ERROR)
+					/* Try to parse expr and convert the result to string on success. */
+					parsing_result_t result = vle_parser_eval(expr, /*interactive=*/0);
+					if(result.error == PE_NO_ERROR)
 					{
-						resstr = var_to_str(res);
+						resstr = var_to_str(result.value);
 					}
 
 					if(resstr != NULL)
@@ -510,7 +522,7 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 						copy_str(buf, sizeof(buf), "<Invalid expr>");
 					}
 
-					var_free(res);
+					var_free(result.value);
 					free(resstr);
 					free(expr);
 
@@ -518,13 +530,13 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 				}
 				break;
 			case '*':
-				if(width > 9)
+				if(width > LAST_USER_COLOR)
 				{
 					snprintf(buf, sizeof(buf), "%%%d*", (int)width);
 					width = 0;
 					break;
 				}
-				cline_set_attr(&result, '0' + width);
+				cline_set_attr(&result, /*user_color=*/width);
 				width = 0;
 				break;
 
@@ -605,6 +617,8 @@ get_tip(void)
 	  "c key in :file menu puts command into command-line to edit before running",
 	  ":copen reopens the last list of files found by a :grep-like command",
 	  "Up/down arrows on command-line load history entries with identical prefix",
+	  ":copy/:move commands accept absolute paths",
+	  "Using dp on blank compare entry removes the other file",
 	};
 
 	if(need_to_shuffle)
@@ -614,8 +628,7 @@ get_tip(void)
 		unsigned int i;
 		for(i = 0U; i < ARRAY_LEN(tips) - 1U; ++i)
 		{
-			const unsigned int j =
-				i + (rand()/(RAND_MAX + 1.0))*(ARRAY_LEN(tips) - i);
+			const unsigned int j = vifm_rand(i, ARRAY_LEN(tips) - 1);
 			const char *const t = tips[i];
 			tips[i] = tips[j];
 			tips[j] = t;
@@ -780,7 +793,6 @@ find_view_macro(const char **format, const char macros[], char macro, int opt)
 			const char *e = strchr(*format, '}');
 			if(e == NULL)
 			{
-				ok = 0;
 				break;
 			}
 
@@ -805,8 +817,8 @@ ui_stat_refresh(void)
 {
 	ui_stat_job_bar_check_for_updates();
 
-	ui_refresh_win(job_bar);
-	ui_refresh_win(stat_win);
+	wnoutrefresh(job_bar);
+	wnoutrefresh(stat_win);
 }
 
 int
@@ -927,7 +939,7 @@ is_job_bar_visible(void)
 {
 	/* Pretend that bar isn't visible in tests. */
 	return curr_stats.load_stage >= 2
-	    && ui_stat_job_bar_height() != 0 && !is_in_menu_like_mode();
+	    && ui_stat_job_bar_height() != 0 && !modes_is_menu_like();
 }
 
 void
@@ -964,11 +976,15 @@ format_job_bar(void)
 	size_t max_width;
 	char **descrs;
 
-	descrs = take_job_descr_snapshot();
-
 	bar_text[0] = '\0';
 	text_width = 0U;
 
+	descrs = take_job_descr_snapshot();
+	if(descrs == NULL)
+	{
+		return bar_text;
+	}
+
 	/* The check of stage is for tests. */
 	max_width = (curr_stats.load_stage < 2) ? 80 : getmaxx(job_bar);
 	width_used = 0U;
@@ -1007,22 +1023,26 @@ format_job_bar(void)
 }
 
 /* Makes snapshot of current job descriptions.  Returns array of length
- * nbar_jobs which should be freed via free_string_array(). */
+ * nbar_jobs which should be freed via free_string_array() or NULL.  The array
+ * can contain NULLs. */
 static char **
 take_job_descr_snapshot(void)
 {
-	size_t i;
-	char **descrs;
+	char **descrs = reallocarray(NULL, nbar_jobs, sizeof(*descrs));
+	if(descrs == NULL)
+	{
+		return NULL;
+	}
 
-	descrs = reallocarray(NULL, nbar_jobs, sizeof(*descrs));
+	size_t i;
 	for(i = 0U; i < nbar_jobs; ++i)
 	{
-		const char *descr;
-
-		bg_op_lock(bar_jobs[i]);
-		descr = bar_jobs[i]->descr;
-		descrs[i] = strdup((descr == NULL) ? "UNKNOWN" : descr);
-		bg_op_unlock(bar_jobs[i]);
+		if(bg_op_lock(bar_jobs[i]))
+		{
+			const char *descr = bar_jobs[i]->descr;
+			descrs[i] = strdup((descr == NULL) ? "UNKNOWN" : descr);
+			bg_op_unlock(bar_jobs[i]);
+		}
 	}
 
 	return descrs;
