@@ -48,10 +48,12 @@
 #include "../compat/pthread.h"
 #include "../engine/mode.h"
 #include "../int/term_title.h"
+#include "../lua/vlua.h"
 #include "../modes/dialogs/msg_dialog.h"
 #include "../modes/modes.h"
 #include "../modes/view.h"
 #include "../modes/wk.h"
+#include "../utils/darray.h"
 #include "../utils/fs.h"
 #include "../utils/log.h"
 #include "../utils/macros.h"
@@ -61,6 +63,7 @@
 #include "../utils/string_array.h"
 #include "../utils/utf8.h"
 #include "../utils/utils.h"
+#include "../compare.h"
 #include "../event_loop.h"
 #include "../filelist.h"
 #include "../flist_sel.h"
@@ -80,6 +83,17 @@
 #include "statusline.h"
 #include "tabs.h"
 
+/* List of formatted tab labels with some extra information. */
+typedef struct
+{
+	cline_t *labels; /* List of labels (might miss some tabs). */
+	int count;       /* Number of labels. */
+	int skipped;     /* Number of leading tabs skipped due to lack of space. */
+	int current;     /* Index of the current tab (does not take skipped count into
+	                    account). */
+}
+tab_line_info_t;
+
 /* Information for formatting tab title. */
 typedef struct
 {
@@ -103,10 +117,14 @@ static WINDOW *no_delay_window;
  * screen, sometimes used for reading input. */
 static WINDOW *inf_delay_window;
 
-static WINDOW *ltop_line1;
-static WINDOW *ltop_line2;
-static WINDOW *rtop_line1;
-static WINDOW *rtop_line2;
+/* Pieces of top line for the left/top view. */
+static WINDOW *ltop_line1; /* On top of left border. */
+static WINDOW *ltop_line2; /* Middle of right border in horizontal view. */
+/* On top of middle border. */
+static WINDOW *mtop_line;
+/* Pieces of top line for the right/bottom view. */
+static WINDOW *rtop_line1; /* On top of right border. */
+static WINDOW *rtop_line2; /* Middle of right border in horizontal view. */
 
 unsigned int ui_next_view_id = 3;
 
@@ -125,9 +143,7 @@ WINDOW *menu_win;
 WINDOW *sort_win;
 WINDOW *change_win;
 WINDOW *error_win;
-
-static WINDOW *top_line;
-static WINDOW *tab_line;
+WINDOW *tab_line;
 
 static WINDOW *lborder;
 static WINDOW *mborder;
@@ -164,6 +180,8 @@ static int view_shows_tabline(const view_t *view);
 static int get_tabline_height(void);
 static void print_tabline(WINDOW *win, view_t *view, col_attr_t base_col,
 		path_func pf);
+static tab_line_info_t format_tab_labels(view_t *view, int max_width,
+		path_func pf);
 static void compute_avg_width(int *avg_width, int *spare_width,
 		int min_widths[], int max_width, view_t *view, path_func pf);
 TSTATIC cline_t make_tab_title(const tab_title_info_t *title_info);
@@ -227,7 +245,7 @@ setup_ncurses_interface(void)
 	nonl();
 	raw();
 
-	curs_set(0);
+	ui_set_cursor(/*visibility=*/0);
 
 	getmaxyx(stdscr, screen_y, screen_x);
 	/* Screen is too small to be useful. */
@@ -369,6 +387,25 @@ move_pair(int from, int to)
 	}
 }
 
+void
+ui_set_mouse_active(int active)
+{
+	if(vifm_testing())
+	{
+		return;
+	}
+
+	if(active)
+	{
+		mousemask(ALL_MOUSE_EVENTS, /*oldmask=*/NULL);
+		mouseinterval(0);
+	}
+	else
+	{
+		mousemask(/*newmask=*/0, /*oldmask=*/NULL);
+	}
+}
+
 /* Initializes all WINDOW variables by calling newwin() to create ncurses
  * windows and configures hardware cursor. */
 static void
@@ -401,7 +438,7 @@ create_windows(void)
 	ltop_line1 = newwin(1, 1, 0, 0);
 	ltop_line2 = newwin(1, 1, 0, 0);
 
-	top_line = newwin(1, 1, 0, 0);
+	mtop_line = newwin(1, 1, 0, 0);
 	tab_line = newwin(1, 1, 0, 0);
 
 	rtop_line1 = newwin(1, 1, 0, 0);
@@ -431,7 +468,7 @@ create_windows(void)
 	leaveok(mborder, TRUE);
 	leaveok(ltop_line1, TRUE);
 	leaveok(ltop_line2, TRUE);
-	leaveok(top_line, TRUE);
+	leaveok(mtop_line, TRUE);
 	leaveok(tab_line, TRUE);
 	leaveok(rtop_line1, TRUE);
 	leaveok(rtop_line2, TRUE);
@@ -561,7 +598,6 @@ vertical_layout(int screen_x)
 	wresize(lwin.win, border_height, splitter_pos + size_correction);
 	mvwin(lwin.win, y + 1, pos_correction);
 
-	ui_set_bg(mborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
 	wresize(mborder, border_height, splitter_width);
 	mvwin(mborder, y + 1, splitter_pos);
 
@@ -571,8 +607,8 @@ vertical_layout(int screen_x)
 	wresize(tab_line, 1, screen_x);
 	mvwin(tab_line, 0, 0);
 
-	wresize(top_line, 1, splitter_width);
-	mvwin(top_line, y, splitter_pos);
+	wresize(mtop_line, 1, splitter_width);
+	mvwin(mtop_line, y, splitter_pos);
 
 	mvwin(rtop_line1, y, screen_x - 1);
 	mvwin(rtop_line2, y, screen_x - 1);
@@ -596,44 +632,41 @@ horizontal_layout(int screen_x, int screen_y)
 	const int y = get_tabline_height();
 
 	int splitter_pos;
-
 	if(curr_stats.splitter_pos < 0)
 		splitter_pos = screen_y/2 - 1;
 	else
 		splitter_pos = curr_stats.splitter_pos;
+	int splitter_height = (middle_border_is_visible() ? 1 : 0);
 	if(splitter_pos < 2)
 		splitter_pos = 2;
-	if(splitter_pos > get_working_area_height() - 1)
-		splitter_pos = get_working_area_height() - 1;
+	if(splitter_pos > get_working_area_height() - 1 - splitter_height)
+		splitter_pos = get_working_area_height() - 1 - splitter_height;
 	if(curr_stats.splitter_pos >= 0)
 		stats_set_splitter_pos(splitter_pos);
 
 	wresize(lwin.title, 1, screen_x + size_correction);
 	mvwin(lwin.title, y, pos_correction);
 
-	wresize(rwin.title, 1, screen_x + size_correction);
-	mvwin(rwin.title, splitter_pos, pos_correction);
-
 	wresize(lwin.win, splitter_pos - (y + 1), screen_x + size_correction);
 	mvwin(lwin.win, y + 1, pos_correction);
 
-	wresize(rwin.win, get_working_area_height() - splitter_pos + y,
-			screen_x + size_correction);
-	mvwin(rwin.win, splitter_pos + 1, pos_correction);
-
-	ui_set_bg(mborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
-	wresize(mborder, 1, screen_x);
+	wresize(mborder, splitter_height, screen_x);
 	mvwin(mborder, splitter_pos, 0);
 
+	wresize(rwin.title, 1, screen_x + size_correction);
+	mvwin(rwin.title, splitter_pos + splitter_height, pos_correction);
+
+	wresize(rwin.win,
+			get_working_area_height() - splitter_pos - splitter_height + y,
+			screen_x + size_correction);
+	mvwin(rwin.win, splitter_pos + splitter_height + 1, pos_correction);
+
 	mvwin(ltop_line1, y, 0);
 	mvwin(ltop_line2, splitter_pos, 0);
 
 	wresize(tab_line, 1, screen_x);
 	mvwin(tab_line, 0, 0);
 
-	wresize(top_line, 1, 2 - screen_x%2);
-	mvwin(top_line, y, screen_x/2 - 1 + screen_x%2);
-
 	mvwin(rtop_line1, y, screen_x - 1);
 	mvwin(rtop_line2, splitter_pos, screen_x - 1);
 
@@ -642,6 +675,10 @@ horizontal_layout(int screen_x, int screen_y)
 
 	wresize(rborder, screen_y - 1, 1);
 	mvwin(rborder, y, screen_x - 1);
+
+	/* Unused in this layout. */
+	wresize(mtop_line, 1, 1);
+	mvwin(mtop_line, 0, 0);
 }
 
 /* Calculates height available for main area that contains file lists.  Returns
@@ -711,6 +748,29 @@ cv_tree(CVType type)
 	return type == CV_TREE || type == CV_CUSTOM_TREE;
 }
 
+const char *
+cv_describe(CVType type)
+{
+	switch(type)
+	{
+		case CV_REGULAR:
+			return "custom";
+
+		case CV_VERY:
+			return "very-custom";
+
+		case CV_CUSTOM_TREE:
+		case CV_TREE:
+			return "tree";
+
+		case CV_DIFF:
+		case CV_COMPARE:
+			return "compare";
+	}
+
+	return "UNKNOWN";
+}
+
 void
 update_screen(UpdateType update_kind)
 {
@@ -750,16 +810,6 @@ update_start(UpdateType update_kind)
 
 	update_attributes();
 
-	if(cfg.side_borders_visible)
-	{
-		clear_border(lborder);
-		clear_border(rborder);
-	}
-	if(middle_border_is_visible())
-	{
-		clear_border(mborder);
-	}
-
 	if(curr_stats.term_state != TS_NORMAL)
 	{
 		return 0;
@@ -778,16 +828,9 @@ update_start(UpdateType update_kind)
 
 	ui_stat_update(curr_view, 0);
 
-	if(!ui_sb_multiline())
+	if(curr_stats.save_msg == 0 && !ui_sb_multiline())
 	{
-		if(curr_view->selected_files)
-		{
-			print_selected_msg();
-		}
-		else
-		{
-			ui_sb_clear();
-		}
+		modes_statusbar_update();
 
 		if(vle_mode_is(VIEW_MODE))
 		{
@@ -803,6 +846,10 @@ update_start(UpdateType update_kind)
 	{
 		ui_sb_clear();
 	}
+	else
+	{
+		ui_sb_msg(NULL);
+	}
 
 	if(vle_mode_is(VIEW_MODE) ||
 			(curr_stats.number_of_windows == 2 && other_view->explore_mode))
@@ -887,7 +934,7 @@ ui_resize_all(void)
 
 	update_statusbar_layout();
 
-	curs_set(0);
+	ui_set_cursor(/*visibility=*/0);
 }
 
 /* Adjusts splitter position after screen resize. */
@@ -917,19 +964,34 @@ static void
 clear_border(WINDOW *border)
 {
 	int i;
-	int height;
+	int height, width;
 
 	werase(border);
 
-	if(strcmp(cfg.border_filler, " ") == 0)
+	getmaxyx(border, height, width);
+	if(height > width)
 	{
-		return;
+		/* Vertical split */
+		if(strcmp(cfg.vborder_filler, " ") == 0 || *cfg.vborder_filler == '\0')
+		{
+			return;
+		}
+		for(i = 0; i < height; ++i)
+		{
+			mvwaddstr(border, i, 0, cfg.vborder_filler);
+		}
 	}
-
-	height = getmaxy(border);
-	for(i = 0; i < height; ++i)
+	else
 	{
-		mvwaddstr(border, i, 0, cfg.border_filler);
+		/* Horizontal split */
+		if(strcmp(cfg.hborder_filler, " ") == 0 || *cfg.hborder_filler == '\0')
+		{
+			return;
+		}
+		for(i = 0; i < width; ++i)
+		{
+			mvwaddstr(border, 0, i, cfg.hborder_filler);
+		}
 	}
 
 	wnoutrefresh(border);
@@ -940,7 +1002,19 @@ clear_border(WINDOW *border)
 static int
 middle_border_is_visible(void)
 {
-	return (curr_stats.number_of_windows == 2 && curr_stats.split == VSPLIT);
+	if(curr_stats.number_of_windows != 2)
+	{
+		return 0;
+	}
+
+	if(curr_stats.split == VSPLIT)
+	{
+		/* Always visible for vertical split. */
+		return 1;
+	}
+
+	/* Optionally visible for horizontal split. */
+	return (strlen(cfg.hborder_filler) != 0);
 }
 
 /* Updates (redraws or reloads) views. */
@@ -1059,10 +1133,16 @@ touch_all_windows(void)
 		return;
 	}
 
-	if(!is_in_menu_like_mode())
+	if(!modes_is_menu_like())
 	{
 		update_window_lazy(tab_line);
 
+		if(middle_border_is_visible())
+		{
+			update_window_lazy(mborder);
+			update_window_lazy(mtop_line);
+		}
+
 		if(curr_stats.number_of_windows == 1)
 		{
 			/* In one window view. */
@@ -1071,9 +1151,6 @@ touch_all_windows(void)
 		else
 		{
 			/* Two pane View. */
-			update_window_lazy(mborder);
-			update_window_lazy(top_line);
-
 			update_view(&lwin);
 			update_view(&rwin);
 		}
@@ -1248,9 +1325,9 @@ update_attributes(void)
 	if(cfg.side_borders_visible)
 	{
 		ui_set_bg(lborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
-		werase(lborder);
+		clear_border(lborder);
 		ui_set_bg(rborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
-		werase(rborder);
+		clear_border(rborder);
 
 		ui_set_bg(ltop_line1, &cfg.cs.color[TOP_LINE_COLOR],
 				cfg.cs.pair[TOP_LINE_COLOR]);
@@ -1260,10 +1337,6 @@ update_attributes(void)
 				cfg.cs.pair[TOP_LINE_COLOR]);
 		werase(ltop_line2);
 
-		ui_set_bg(top_line, &cfg.cs.color[TOP_LINE_COLOR],
-				cfg.cs.pair[TOP_LINE_COLOR]);
-		werase(top_line);
-
 		ui_set_bg(rtop_line1, &cfg.cs.color[TOP_LINE_COLOR],
 				cfg.cs.pair[TOP_LINE_COLOR]);
 		werase(rtop_line1);
@@ -1273,8 +1346,19 @@ update_attributes(void)
 		werase(rtop_line2);
 	}
 
-	ui_set_bg(mborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
-	werase(mborder);
+	if(middle_border_is_visible())
+	{
+		ui_set_bg(mtop_line, &cfg.cs.color[TOP_LINE_COLOR],
+				cfg.cs.pair[TOP_LINE_COLOR]);
+		werase(mtop_line);
+		/* For some reason this wnoutrefresh is needed to make this window appear
+		 * at the same time along with others during startup.  Might be an
+		 * indication that all windows need a refresh, but that need is masked. */
+		wnoutrefresh(mtop_line);
+
+		ui_set_bg(mborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
+		clear_border(mborder);
+	}
 
 	ui_set_bg(tab_line, &cfg.cs.color[TAB_LINE_COLOR],
 			cfg.cs.pair[TAB_LINE_COLOR]);
@@ -1349,7 +1433,10 @@ resize_for_menu_like(void)
 {
 	int screen_x, screen_y;
 
-	ui_update_term_state();
+	/* Resizing everything to avoid a situation when windows that are not visible
+	 * are of wrong size and using redrawwin() on them causes a memory corruption.
+	 * I'm pretty sure it's a bug in ncurses. */
+	ui_resize_all();
 	if(curr_stats.term_state == TS_TOO_SMALL)
 	{
 		return 1;
@@ -1380,7 +1467,7 @@ ui_setup_for_menu_like(void)
 	if(curr_stats.load_stage > 0)
 	{
 		scrollok(menu_win, FALSE);
-		curs_set(0);
+		ui_set_cursor(/*visibility=*/0);
 		werase(menu_win);
 		werase(status_bar);
 		werase(ruler_win);
@@ -1586,6 +1673,9 @@ ui_swap_view_data(view_t *left, view_t *right)
 	WINDOW *tmp;
 	int t;
 
+	/* Data in some fields doesn't need to be swapped.  Swap it beforehand so that
+	 * swapping structures will put it back. */
+
 	tmp = left->win;
 	left->win = right->win;
 	right->win = tmp;
@@ -1602,6 +1692,24 @@ ui_swap_view_data(view_t *left, view_t *right)
 	left->title = right->title;
 	right->title = tmp;
 
+	/* Swap these fields so they reflect updated layout. */
+
+	t = left->custom.diff_stats.unique_left;
+	left->custom.diff_stats.unique_left = left->custom.diff_stats.unique_right;
+	left->custom.diff_stats.unique_right = t;
+
+	t = right->custom.diff_stats.unique_left;
+	right->custom.diff_stats.unique_left = right->custom.diff_stats.unique_right;
+	right->custom.diff_stats.unique_right = t;
+
+	const int unique_lr = (CF_SHOW_UNIQUE_LEFT | CF_SHOW_UNIQUE_RIGHT);
+	if((left->custom.diff_cmp_flags & unique_lr) == CF_SHOW_UNIQUE_LEFT ||
+			(left->custom.diff_cmp_flags & unique_lr) == CF_SHOW_UNIQUE_RIGHT)
+	{
+		left->custom.diff_cmp_flags ^= unique_lr;
+		right->custom.diff_cmp_flags ^= unique_lr;
+	}
+
 	tmp_view = *left;
 	*left = *right;
 	*right = tmp_view;
@@ -1623,7 +1731,7 @@ split_view(SPLIT orientation)
 	curr_stats.split = orientation;
 	curr_stats.number_of_windows = 2;
 
-	if(curr_stats.number_of_windows == 2 && curr_stats.splitter_pos > 0)
+	if(curr_stats.splitter_pos > 0)
 	{
 		stats_set_splitter_ratio(curr_stats.splitter_ratio);
 	}
@@ -1662,6 +1770,14 @@ move_splitter(int by, int fact)
 	set_splitter(pos + fact*by);
 }
 
+void
+ui_remember_scroll_offset(void)
+{
+	const int rwin_pos = rwin.top_line/rwin.column_count;
+	const int lwin_pos = lwin.top_line/lwin.column_count;
+	curr_stats.scroll_bind_off = rwin_pos - lwin_pos;
+}
+
 void
 ui_view_resize(view_t *view, int to)
 {
@@ -1693,16 +1809,17 @@ void
 format_entry_name(const dir_entry_t *entry, NameFormat fmt, size_t buf_len,
 		char buf[])
 {
-	const char *prefix, *suffix;
-	char tmp_buf[strlen(entry->name) + 1];
-	const char *name = entry->name;
-
 	if(fmt == NF_NONE)
 	{
-		copy_str(buf, buf_len, entry->name);
+		char *escaped = escape_unreadable(entry->name);
+		copy_str(buf, buf_len, escaped);
+		free(escaped);
 		return;
 	}
 
+	char tmp_buf[strlen(entry->name) + 1];
+	const char *name = entry->name;
+
 	if(fmt == NF_ROOT)
 	{
 		int root_len;
@@ -1713,9 +1830,14 @@ format_entry_name(const dir_entry_t *entry, NameFormat fmt, size_t buf_len,
 		name = tmp_buf;
 	}
 
+	char *escaped = escape_unreadable(name);
+
+	const char *prefix, *suffix;
 	ui_get_decors(entry, &prefix, &suffix);
 	snprintf(buf, buf_len, "%s%s%s", prefix,
-			(is_root_dir(entry->name) && suffix[0] == '/') ? "" : name, suffix);
+			(is_root_dir(escaped) && suffix[0] == '/') ? "" : escaped, suffix);
+
+	free(escaped);
 }
 
 void
@@ -1819,6 +1941,97 @@ checked_wmove(WINDOW *win, int y, int x)
 	}
 }
 
+void
+ui_set_cursor(int visibility)
+{
+	/* PDCurses crashes if curs_set() is called for uninitialized library. */
+	if(!vifm_testing())
+	{
+		(void)curs_set(visibility);
+	}
+}
+
+int
+ui_get_mouse(MEVENT *event)
+{
+	int ret = getmouse(event);
+	if(ret != OK)
+	{
+		return ret;
+	}
+
+	/* Positions after 222 can become negative due to a combination of protocol
+	 * limitations and implementation.  This workaround can extend the range at
+	 * least a bit when SGR 1006 isn't available. */
+	if(event->x < 0)
+	{
+		event->x &= 0xff;
+	}
+	if(event->y < 0)
+	{
+		event->y &= 0xff;
+	}
+
+	int mask = M_ALL_MODES;
+	switch(vle_mode_get())
+	{
+		case NORMAL_MODE:  mask |= M_NORMAL_MODE; break;
+		case NAV_MODE:
+		case CMDLINE_MODE: mask |= M_CMDLINE_MODE; break;
+		case VISUAL_MODE:  mask |= M_VISUAL_MODE; break;
+		case MENU_MODE:    mask |= M_MENU_MODE; break;
+		case VIEW_MODE:    mask |= M_VIEW_MODE; break;
+
+		default:           mask = 0; break;
+	}
+
+	return (cfg.mouse & mask ? OK : ERR);
+}
+
+WINDOW *
+ui_get_tab_line_win(const view_t *view)
+{
+	return (view_shows_tabline(view) ? view->title : tab_line);
+}
+
+int
+ui_map_tab_line(view_t *view, int x)
+{
+	if(!is_null_or_empty(cfg.tab_line))
+	{
+		/* No mouse mapping for custom tabline. */
+		return -1;
+	}
+
+	path_func pf = cfg.shorten_title_paths ? &replace_home_part : &path_identity;
+
+	const int max_width = getmaxx(ui_get_tab_line_win(view));
+	tab_line_info_t info = format_tab_labels(view, max_width, pf);
+
+	int tab_idx = -1;
+
+	int i;
+	for(i = 0; i < info.count; ++i)
+	{
+		if(tab_idx == -1 && x < (int)info.labels[i].attrs_len)
+		{
+			tab_idx = info.skipped + i;
+		}
+		x -= info.labels[i].attrs_len;
+
+		cline_dispose(&info.labels[i]);
+	}
+	free(info.labels);
+
+	return tab_idx;
+}
+
+int
+ui_wenclose(const view_t *view, WINDOW *win, int x, int y)
+{
+	return (ui_view_is_visible(view) && wenclose(win, y, x));
+}
+
 void
 ui_display_too_small_term_msg(void)
 {
@@ -1840,7 +2053,7 @@ ui_view_win_changed(view_t *view)
 	refresh_bottom_lines();
 }
 
-/* Makes sure that statusline and statusbar are drawn over view window after
+/* Makes sure that status line and status bar are drawn over view window after
  * view refresh. */
 static void
 refresh_bottom_lines(void)
@@ -1976,41 +2189,95 @@ get_tabline_height(void)
 static void
 print_tabline(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
 {
-	int i;
-	tab_info_t tab_info;
-
 	const int max_width = (vifm_testing() ? cfg.columns : getmaxx(win));
-	int width_used = 0;
-	int avg_width, spare_width;
-
-	int min_widths[tabs_count(view)];
 
 	ui_set_bg(win, &base_col, -1);
 	werase(win);
 	checked_wmove(win, 0, 0);
 
+	if(is_null_or_empty(cfg.tab_line))
+	{
+		tab_line_info_t info = format_tab_labels(view, max_width, pf);
+
+		int i;
+		for(i = 0; i < info.count; ++i)
+		{
+			col_attr_t col = base_col;
+			if(i == info.current - info.skipped)
+			{
+				cs_mix_colors(&col, &cfg.cs.color[TAB_LINE_SEL_COLOR]);
+			}
+
+			cline_print(&info.labels[i], win, &col);
+			cline_dispose(&info.labels[i]);
+		}
+		free(info.labels);
+	}
+	else
+	{
+		char *fmt;
+		if(vlua_handler_cmd(curr_stats.vlua, cfg.tab_line))
+		{
+			int other = (view == other_view);
+			fmt = vlua_make_tab_line(curr_stats.vlua, cfg.tab_line, other, max_width);
+		}
+		else
+		{
+			fmt = strdup(cfg.tab_line);
+		}
+
+		if(fmt != NULL)
+		{
+			cline_t title = ma_expand_colored_custom(fmt, /*nmacros=*/0,
+					/*macros=*/NULL, MA_OPT);
+			free(fmt);
+
+			cline_print(&title, win, &base_col);
+			cline_dispose(&title);
+		}
+	}
+
+	wnoutrefresh(win);
+}
+
+/* Computes layout of the tab line and formats tab labels.  Returns list of tab
+ * labels along with supplementary information. */
+static tab_line_info_t
+format_tab_labels(view_t *view, int max_width, path_func pf)
+{
+	int i;
+	int tab_count = tabs_count(view);
+	int width_used = 0;
+
+	int avg_width, spare_width;
+	int min_widths[tab_count];
 	compute_avg_width(&avg_width, &spare_width, min_widths, max_width, view, pf);
 
-	int tab_count = tabs_count(view);
 	int min_width = 0;
 	for(i = 0; i < tab_count; ++i)
 	{
 		min_width += min_widths[i];
 	}
 
-	int before_current = 1;
+	cline_t *tab_labels = NULL;
+	DA_INSTANCE(tab_labels);
 
+	int current_tab = -1;
+	int skipped_tabs = 0;
+	tab_info_t tab_info;
 	for(i = 0; tabs_get(view, i, &tab_info) && width_used < max_width; ++i)
 	{
 		int current = (tab_info.view == view);
 		if(current)
 		{
-			before_current = 0;
+			current_tab = i;
 		}
-		else if(before_current && min_width > max_width)
+		else if(current_tab == -1 && min_width > max_width)
 		{
+			/* Skip a tab that precedes the current one because it doesn't fit in. */
 			min_width -= min_widths[i];
 			spare_width = max_width - min_width;
+			++skipped_tabs;
 			continue;
 		}
 
@@ -2025,12 +2292,6 @@ print_tabline(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
 		const int extra_width = prefix.attrs_len + suffix.attrs_len;
 		int width = max_width;
 
-		col_attr_t col = base_col;
-		if(current)
-		{
-			cs_mix_colors(&col, &cfg.cs.color[TAB_LINE_SEL_COLOR]);
-		}
-
 		if(!current)
 		{
 			width = tab_info.last ? (max_width - width_used)
@@ -2049,8 +2310,6 @@ print_tabline(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
 					curr_stats.ellipsis);
 		}
 
-		ui_set_attr(win, &col, -1);
-
 		int real_width = prefix.attrs_len + title.attrs_len + suffix.attrs_len;
 
 		if(width < real_width && max_width - width_used >= real_width)
@@ -2059,19 +2318,26 @@ print_tabline(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
 		}
 		if(width >= real_width)
 		{
-			cline_print(&prefix, win, &col);
-			cline_print(&title, win, &col);
-			cline_print(&suffix, win, &col);
+			cline_t *tab_label = DA_EXTEND(tab_labels);
+			if(tab_label != NULL)
+			{
+				*tab_label = prefix;
+				cline_append(tab_label, &title);
+				cline_append(tab_label, &suffix);
+				DA_COMMIT(tab_labels);
+			}
 		}
 
 		width_used += real_width;
-
-		cline_dispose(&prefix);
-		cline_dispose(&title);
-		cline_dispose(&suffix);
 	}
 
-	wnoutrefresh(win);
+	tab_line_info_t result = {
+		.labels = tab_labels,
+		.count = DA_SIZE(tab_labels),
+		.skipped = skipped_tabs,
+		.current = current_tab,
+	};
+	return result;
 }
 
 /* Computes average width of tab tips as well as number of spare character
@@ -2368,8 +2634,8 @@ fixup_titles_attributes(const view_t *view, int active_view)
 		ui_set_bg(view->title, &col, cfg.cs.pair[TOP_LINE_COLOR]);
 		ui_set_attr(view->title, &col, cfg.cs.pair[TOP_LINE_COLOR]);
 
-		ui_set_bg(top_line, &col, cfg.cs.pair[TOP_LINE_COLOR]);
-		werase(top_line);
+		ui_set_bg(mtop_line, &col, cfg.cs.pair[TOP_LINE_COLOR]);
+		werase(mtop_line);
 	}
 
 	return col;
@@ -2475,13 +2741,21 @@ int
 ui_view_right_reserved(const view_t *view)
 {
 	dir_entry_t *const entry = get_current_entry(view);
+
+	if(!is_in_miller_view(view) || is_parent_dir(entry->name))
+	{
+		return 0;
+	}
+
+	if(view->miller_preview != MP_ALL &&
+			fentry_is_dir(entry) != (view->miller_preview == MP_DIRS))
+	{
+		return 0;
+	}
+
 	const int total = view->miller_ratios[0] + view->miller_ratios[1]
 	                + view->miller_ratios[2];
-	return is_in_miller_view(view)
-	    && !is_parent_dir(entry->name)
-	    && (fentry_is_dir(entry) || view->miller_preview_files)
-	     ? (view->window_cols*view->miller_ratios[2])/total
-	     : 0;
+	return (view->window_cols*view->miller_ratios[2])/total;
 }
 
 /* Whether miller columns should be displayed.  Returns non-zero if so,
@@ -2533,10 +2807,26 @@ ui_qv_width(const view_t *view)
 void
 ui_qv_cleanup_if_needed(void)
 {
-	if(curr_stats.preview.on && curr_stats.preview.cleanup_cmd != NULL)
+	if(curr_stats.preview.on && curr_stats.preview.kind != VK_TEXTUAL)
 	{
 		qv_cleanup(other_view, curr_stats.preview.cleanup_cmd);
 	}
+
+	if(ui_view_is_visible(&lwin))
+	{
+		fview_clear_miller_preview(&lwin);
+	}
+	if(ui_view_is_visible(&rwin))
+	{
+		fview_clear_miller_preview(&rwin);
+	}
+}
+
+void
+ui_hide_graphics(void)
+{
+	ui_qv_cleanup_if_needed();
+	modview_hide_graphics();
 }
 
 void
@@ -2600,13 +2890,22 @@ ui_shutdown(void)
 void
 ui_pause(void)
 {
+#ifdef _WIN32
+	/* Refresh the window, because otherwise curses redraws the screen on call to
+	 * `compat_wget_wch()` (why does it do this?). */
+	use_wrefresh(inf_delay_window);
+#endif
+
 	/* Show previous screen state. */
 	ui_shutdown();
+
+#ifndef _WIN32
 	/* Yet restore program mode to read input without waiting for Enter. */
 	reset_prog_mode();
 	/* Refresh the window, because otherwise curses redraws the screen on call to
 	 * `compat_wget_wch()` (why does it do this?). */
 	wnoutrefresh(inf_delay_window);
+#endif
 
 	/* Ignore window resize. */
 	wint_t pressed;
