@@ -53,7 +53,7 @@
 
 /* Menus don't look like menus as all if height is less than 5. */
 #define MIN_TERM_HEIGHT 5
-/* There is a lower limit on statusbar width. */
+/* There is a lower limit on status bar width. */
 #define MIN_TERM_WIDTH (INPUT_WIN_WIDTH + 1 + POS_WIN_MIN_WIDTH)
 
 /* Width of the ruler and input windows. */
@@ -165,6 +165,15 @@ typedef enum
 }
 CompareType;
 
+/* Type of files to list after a comparison. */
+typedef enum
+{
+	LT_ALL,    /* All files. */
+	LT_DUPS,   /* Files that have at least 1 dup on other side or in this view. */
+	LT_UNIQUE, /* Files unique to this view or within this view. */
+}
+ListType;
+
 /* Type of scheduled view update event. */
 typedef enum
 {
@@ -183,6 +192,15 @@ typedef enum
 }
 NameFormat;
 
+/* What kinds of entries should be previewed by miller views. */
+typedef enum
+{
+	MP_ALL,   /* All entries. */
+	MP_DIRS,  /* Directories only. */
+	MP_FILES, /* Files only. */
+}
+MillerPreview;
+
 /* Single entry of directory history. */
 typedef struct
 {
@@ -254,15 +272,27 @@ typedef struct
 }
 entries_t;
 
+/* Entry with comparison results. */
+typedef struct
+{
+	int identical;    /* Number of matched files judged identical. */
+	int different;    /* Number of matched files judged different. */
+	int unique_left;  /* Number of unmatched files on the left. */
+	int unique_right; /* Number of unmatched files on the right. */
+}
+compare_stats_t;
+
 /* Data related to custom filling. */
 struct cv_data_t
 {
 	/* Type of the custom view. */
 	CVType type;
 
-	/* Additional data about CV_DIFF type. */
-	CompareType diff_cmp_type; /* Type of comparison. */
-	int diff_path_group;       /* Whether entries are grouped by paths. */
+	/* Additional data for CV_DIFF type. */
+	CompareType diff_cmp_type;  /* Type of comparison. */
+	ListType diff_list_type;    /* Type of results. */
+	int diff_cmp_flags;         /* Flags used to build the diff. */
+	compare_stats_t diff_stats; /* List of comparison results */
 
 	/* This is temporary storage for custom list entries used during its
 	 * construction. */
@@ -353,8 +383,8 @@ struct view_t
 	int miller_view, miller_view_g;
 	/* Proportions of columns. */
 	int miller_ratios[3], miller_ratios_g[3];
-	/* Whether right column should also preview files. */
-	int miller_preview_files, miller_preview_files_g;
+	/* What entries are to be previewed in the right column. */
+	MillerPreview miller_preview, miller_preview_g;
 	/* Caches of file lists for miller mode. */
 	cached_entries_t left_column;
 	cached_entries_t right_column;
@@ -416,7 +446,7 @@ struct view_t
 	signed char sort[SK_COUNT], sort_g[SK_COUNT];
 	/* Sorting groups (comma-separated list of regular expressions). */
 	char *sort_groups, *sort_groups_g;
-	/* Primary group in compiled form. */
+	/* Primary group of sort_groups (not sort_groups_g) in compiled form. */
 	regex_t primary_group;
 
 	int history_num;    /* Number of used history elements. */
@@ -438,6 +468,7 @@ struct view_t
 	/* ls-like view related fields. */
 	int ls_view, ls_view_g;             /* Non-zero if ls-like view is enabled. */
 	int ls_transposed, ls_transposed_g; /* Non-zero for transposed ls-view. */
+	int ls_cols, ls_cols_g;             /* Non-zero if column count is limited. */
 	size_t max_filename_width; /* Maximum filename width (length in character
 	                            * positions on the screen) among all entries of
 	                            * the file list.  Zero if not calculated. */
@@ -485,6 +516,7 @@ extern WINDOW *menu_win;
 extern WINDOW *sort_win;
 extern WINDOW *change_win;
 extern WINDOW *error_win;
+extern WINDOW *tab_line;
 
 /* Updates the ruler with information from the view (possibly lazily). */
 void ui_ruler_update(view_t *view, int lazy_redraw);
@@ -524,6 +556,10 @@ int cv_compare(CVType type);
  * non-zero if so, otherwise zero is returned. */
 int cv_tree(CVType type);
 
+/* Retrieves textual description of the specified custom view type.  Returns the
+ * description. */
+const char * cv_describe(CVType type);
+
 /* Resizes all windows according to current screen size and TUI
  * configuration. */
 void ui_resize_all(void);
@@ -618,6 +654,9 @@ void only(void);
  * given factor. */
 void move_splitter(int by, int fact);
 
+/* Recalculates difference of two panes scroll positions. */
+void ui_remember_scroll_offset(void);
+
 /* Sets size of the view to specified value. */
 void ui_view_resize(view_t *view, int to);
 
@@ -638,6 +677,25 @@ void ui_view_reset_decor_cache(const view_t *view);
  * movement. */
 void checked_wmove(WINDOW *win, int y, int x);
 
+/* Changes visibility of hardware cursor. */
+void ui_set_cursor(int visibility);
+
+/* Retrieves mouse event.  Adjusts and filters events in the process.  Returns
+ * ERR or OK curses error codes. */
+int ui_get_mouse(MEVENT *event);
+
+/* Determines curses window that displays tabs for the view or global tabs if
+ * they are active.  Returns the window pointer. */
+WINDOW * ui_get_tab_line_win(const view_t *view);
+
+/* Determines index of a tab at the specified coordinate.  Returns tab number
+ * base zero or -1 if tab label is not present at that location. */
+int ui_map_tab_line(view_t *view, int x);
+
+/* Determines whether coordinates are with the window which is part of the view.
+ * Returns non-zero if so, otherwise zero is returned. */
+int ui_wenclose(const view_t *view, WINDOW *win, int x, int y);
+
 /* Displays "Terminal is too small" kind of message instead of UI. */
 void ui_display_too_small_term_msg(void);
 
@@ -719,6 +777,9 @@ int ui_qv_width(const view_t *view);
  * isn't enough. */
 void ui_qv_cleanup_if_needed(void);
 
+/* Hides any visible graphics. */
+void ui_hide_graphics(void);
+
 /* Invalidates views-specific knowledge about given color scheme. */
 void ui_invalidate_cs(const col_scheme_t *cs);
 
@@ -741,6 +802,9 @@ int ui_view_unsorted(const view_t *view);
  * or when terminal might be used by another application that vifm runs). */
 void ui_shutdown(void);
 
+/* Enables/disables mouse support. */
+void ui_set_mouse_active(int active);
+
 /* Temporarily shuts down UI until a key is pressed. */
 void ui_pause(void);
 
