@@ -32,7 +32,7 @@
 #include <unistd.h> /* pathconf() readlink() */
 
 #include <ctype.h> /* isalpha() */
-#include <errno.h> /* errno */
+#include <errno.h> /* EINVAL ERANGE errno */
 #include <stddef.h> /* NULL */
 #include <stdio.h> /* snprintf() remove() */
 #include <stdlib.h> /* free() */
@@ -336,7 +336,7 @@ make_path(const char dir_name[], mode_t mode)
 		.arg3.mode = mode,
 	};
 
-	return iop_mkdir(&args);
+	return (iop_mkdir(&args) == IO_RES_SUCCEEDED ? 0 : 1);
 }
 
 int
@@ -505,28 +505,13 @@ list_sorted_files(const char path[], int *len)
 int
 is_regular_file(const char path[])
 {
-	char resolved_link[PATH_MAX + 1];
-	if(is_symlink(path))
+	char path_real[PATH_MAX + 1];
+	if(os_realpath(path, path_real) != path_real)
 	{
-		char *const symlink_base = strdup(path);
-
-		if(!is_root_dir(symlink_base))
-		{
-			remove_last_path_component(symlink_base);
-		}
-
-		if(get_link_target_abs(path, symlink_base, resolved_link,
-					sizeof(resolved_link)) != 0)
-		{
-			free(symlink_base);
-			return 0;
-		}
-		free(symlink_base);
-
-		path = resolved_link;
+		return 0;
 	}
 
-	return is_regular_file_noderef(path);
+	return is_regular_file_noderef(path_real);
 }
 
 int
@@ -814,6 +799,48 @@ restore_cwd(char saved_cwd[])
 	}
 }
 
+FILE *
+make_tmp_file(char path[], mode_t mode, int auto_delete)
+{
+	int fd = create_unique_file(path, mode, auto_delete);
+	if(fd == -1)
+	{
+		return NULL;
+	}
+
+	FILE *file = fdopen(fd, "w+b");
+	if(file == NULL)
+	{
+		int error = errno;
+		(void)close(fd);
+		errno = error;
+	}
+
+	return file;
+}
+
+FILE *
+make_file_in_tmp(const char prefix[], mode_t mode, int auto_delete,
+		char full_path[], size_t full_path_len)
+{
+	if(contains_slash(prefix))
+	{
+		errno = EINVAL;
+		return NULL;
+	}
+
+	int len = snprintf(full_path, full_path_len, "%s/%s-XXXXXX", get_tmpdir(),
+			prefix);
+	if(len < 0 || (size_t)len >= full_path_len)
+	{
+		errno = ERANGE;
+		return NULL;
+	}
+
+	system_to_internal_slashes(full_path);
+	return make_tmp_file(full_path, mode, auto_delete);
+}
+
 #ifndef _WIN32
 
 /* Checks if path (dereferenced for a symbolic link) is an existing directory.
