@@ -24,6 +24,7 @@
 
 #include <stddef.h> /* size_t */
 #include <stdint.h> /* uint32_t uint64_t */
+#include <stdio.h> /* FILE */
 
 /* Functions to deal with file system objects */
 
@@ -212,6 +213,17 @@ char * save_cwd(void);
  * was remembered, does nothing. */
 void restore_cwd(char saved_cwd[]);
 
+/* Creates a unique file by replacing trailing "XXXXXX" in the path with some
+ * unique sequence.  Updates the path in place.  Returns file descriptor on
+ * success or -1 on failure (including when there is no trailing "XXXXXX").
+ * errno is meaningful on failure. */
+FILE * make_tmp_file(char path[], mode_t mode, int auto_delete);
+
+/* Same as make_tmp_file(), but always creates a file in a temporary directory.
+ * The prefix must not contain slashes.  errno is meaningful on failure. */
+FILE * make_file_in_tmp(const char prefix[], mode_t mode, int auto_delete,
+		char full_path[], size_t full_path_len);
+
 #ifdef _WIN32
 
 int S_ISLNK(mode_t mode);
