@@ -31,9 +31,9 @@
 #include <errno.h> /* errno */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h>  /* snprintf() */
-#include <stdlib.h> /* malloc() free() */
+#include <stdlib.h> /* free() */
 #include <string.h> /* memset() strcat() strcmp() strdup() strncmp() strncat()
-                       strchr() strcpy() strlen() strrchr() */
+                       strchr() strcpy() strlen() strpbrk() strrchr() */
 
 #include "../cfg/config.h"
 #include "../compat/fs_limits.h"
@@ -354,85 +354,6 @@ is_unc_root(const char *path)
 #endif
 }
 
-char *
-shell_like_escape(const char string[], int type)
-{
-	size_t len;
-	size_t i;
-	char *ret, *dup;
-
-	len = strlen(string);
-
-	dup = ret = malloc(len*3 + 2 + 1);
-
-	if(*string == '-')
-	{
-		*dup++ = '.';
-		*dup++ = '/';
-	}
-
-	for(i = 0; i < len; i++, string++, dup++)
-	{
-		switch(*string)
-		{
-			case '%':
-				if(type == 1)
-				{
-					*dup++ = '%';
-				}
-				break;
-
-			/* Escape the following characters anywhere in the line. */
-			case '\'':
-			case '\\':
-			case '\r':
-			case '\t':
-			case '"':
-			case ';':
-			case ' ':
-			case '?':
-			case '|':
-			case '[':
-			case ']':
-			case '{':
-			case '}':
-			case '<':
-			case '>':
-			case '`':
-			case '!':
-			case '$':
-			case '&':
-			case '*':
-			case '(':
-			case ')':
-			case '#':
-				*dup++ = '\\';
-				break;
-
-			case '\n':
-				if(type != 0)
-				{
-					break;
-				}
-
-				*dup++ = '"';
-				*dup++ = '\n';
-				*dup = '"';
-				continue;
-
-			/* Escape the following characters only at the beginning of the line. */
-			case '~':
-			case '=': /* Command-path expansion in zsh. */
-				if(dup == ret)
-					*dup++ = '\\';
-				break;
-		}
-		*dup = *string;
-	}
-	*dup = '\0';
-	return ret;
-}
-
 char *
 replace_home_part(const char path[])
 {
@@ -491,13 +412,6 @@ replace_tilde(char path[])
 static char *
 try_replace_tilde(const char path[])
 {
-#ifndef _WIN32
-	char name[NAME_MAX + 1];
-	const char *p;
-	char *result;
-	struct passwd *pw;
-#endif
-
 	if(path[0] != '~')
 	{
 		return (char *)path;
@@ -511,26 +425,24 @@ try_replace_tilde(const char path[])
 	}
 
 #ifndef _WIN32
-	if((p = strchr(path, '/')) == NULL)
-	{
-		p = path + strlen(path);
-		copy_str(name, sizeof(name), path + 1);
-	}
-	else
-	{
-		copy_str(name, p - (path + 1) + 1, path + 1);
-		p++;
-	}
+	char user_name[NAME_MAX + 1];
 
-	if((pw = getpwnam(name)) == NULL)
+	const char *p = until_first(path + 1, '/');
+	int user_name_len = p - (path + 1);
+	if(user_name_len + 1 > (int)sizeof(user_name))
 	{
 		return (char *)path;
 	}
 
-	chosp(pw->pw_dir);
-	result = join_paths(pw->pw_dir, p);
+	copy_str(user_name, user_name_len + 1, path + 1);
 
-	return result;
+	struct passwd *pw = getpwnam(user_name);
+	if(pw == NULL)
+	{
+		return (char *)path;
+	}
+
+	return join_paths(pw->pw_dir, p);
 #else
 	return (char *)path;
 #endif
@@ -623,12 +535,13 @@ void
 to_canonic_path(const char path[], const char base[], char buf[],
 		size_t buf_len)
 {
-	char *normalized = strdup(path);
+	char normalized[PATH_MAX + 1];
+	copy_str(normalized, sizeof(normalized), path);
 	system_to_internal_slashes(normalized);
 
 	if(!is_path_absolute(normalized))
 	{
-		char full_path[PATH_MAX + 1];
+		char full_path[PATH_MAX*2 + 1];
 		/* Assert is not level above to keep "." in curr_dir in tests, but this
 		 * should be possible to change. */
 		assert(is_path_absolute(base) && "Base path has to be absolute.");
@@ -640,8 +553,6 @@ to_canonic_path(const char path[], const char base[], char buf[],
 		canonicalize_path(normalized, buf, buf_len);
 	}
 
-	free(normalized);
-
 	if(!is_root_dir(buf))
 	{
 		chosp(buf);
@@ -649,14 +560,13 @@ to_canonic_path(const char path[], const char base[], char buf[],
 }
 
 int
-contains_slash(const char *path)
+contains_slash(const char path[])
 {
-	char *slash_pos = strchr(path, '/');
-#ifdef _WIN32
-	if(slash_pos == NULL)
-		slash_pos = strchr(path, '\\');
+#ifndef _WIN32
+	return (strchr(path, '/') != NULL);
+#else
+	return (strpbrk(path, "/\\") != NULL);
 #endif
-	return slash_pos != NULL;
 }
 
 char *
@@ -789,14 +699,6 @@ find_cmd_in_path(const char cmd[], size_t path_len, char path[])
 	return 1;
 }
 
-void
-generate_tmp_file_name(const char prefix[], char buf[], size_t buf_len)
-{
-	snprintf(buf, buf_len, "%s/%s", get_tmpdir(), prefix);
-	system_to_internal_slashes(buf);
-	copy_str(buf, buf_len, make_name_unique(buf));
-}
-
 const char *
 get_tmpdir(void)
 {
@@ -807,7 +709,15 @@ get_tmpdir(void)
 void
 build_path(char buf[], size_t buf_len, const char p1[], const char p2[])
 {
-	snprintf(buf, buf_len, "%s%s%s", p1, ends_with_slash(p1) ? "" : "/", p2);
+	p2 = skip_char(p2, '/');
+	if(p2[0] == '\0')
+	{
+		copy_str(buf, buf_len, p1);
+	}
+	else
+	{
+		snprintf(buf, buf_len, "%s%s%s", p1, ends_with_slash(p1) ? "" : "/", p2);
+	}
 }
 
 char *
