@@ -384,6 +384,18 @@ strnoscmp(const char *s, const char *t, size_t n)
 #endif
 }
 
+int
+strsorter(const void *s, const void *t)
+{
+	return strcmp(*(const char **)s, *(const char **)t);
+}
+
+int
+strcasesorter(const void *s, const void *t)
+{
+	return strcasecmp(*(const char **)s, *(const char **)t);
+}
+
 int
 strossorter(const void *s, const void *t)
 {
@@ -612,13 +624,20 @@ break_in_two(char str[], size_t max, const char separator[])
 {
 	char *break_point = strstr(str, separator);
 	if(break_point == NULL)
+	{
 		return str;
+	}
 
 	const size_t separator_len = strlen(separator);
 	const size_t len = utf8_strsw(str) - separator_len;
 	const size_t size = MAX(strlen(str), max);
 
 	char *result = malloc(size*4 + 2);
+	if(result == NULL)
+	{
+		return NULL;
+	}
+
 	copy_str(result, break_point - str + 1, str);
 
 	if(len > max)
@@ -700,6 +719,10 @@ escape_chars(const char string[], const char chars[])
 	len = strlen(string);
 
 	dup = ret = malloc(len*2 + 2 + 1);
+	if(dup == NULL)
+	{
+		return NULL;
+	}
 
 	for(i = 0; i < len; i++)
 	{
@@ -808,7 +831,8 @@ get_first_wchar(const char str[])
 	     ? (unsigned char)str[0]
 	     : wc[0];
 #else
-	return utf8_first_char(str);
+	int len;
+	return utf8_first_char(str, &len);
 #endif
 }
 
@@ -1101,6 +1125,11 @@ char *
 double_char(const char str[], char c)
 {
 	char *doubled = malloc(strlen(str) + chars_in_str(str, c) + 1);
+	if(doubled == NULL)
+	{
+		return NULL;
+	}
+
 	char *p = doubled;
 	while(*str != '\0')
 	{
