@@ -50,6 +50,12 @@ size_t utf8_nstrsnlen(const char str[], size_t max_screen_width);
  * number. */
 size_t utf8_strsw(const char str[]);
 
+/* Counts number of screen characters in a utf-8 encoded str of the specified
+ * length (or less if NUL byte is found earlier).  If length ends in the middle
+ * of a utf-8 character, the character's screen width is counted as well.
+ * Returns the number. */
+size_t utf8_nstrsw(const char str[], int len);
+
 /* Counts number of screen characters in a utf-8 encoded str expanding
  * tabulation according to specified tab stops.  tab_stops must be positive.
  * Returns the number. */
@@ -72,15 +78,16 @@ size_t utf8_strso(const char str[]);
  * including terminating null character. */
 size_t utf8_strcpy(char dst[], const char src[], size_t dst_len);
 
+/* Extracts first utf-8 character from the string converting it to wide
+ * character representation.  Returns the wide character and sets *len to width
+ * of input character, which can be 0. */
+wchar_t utf8_first_char(const char utf8[], int *len);
+
 #ifdef _WIN32
 
 /* Converts utf-8 to utf-16 string.  Returns newly allocated utf-8 string. */
 wchar_t * utf8_to_utf16(const char utf8[]);
 
-/* Extracts first utf-8 character from the string converting it to wide
- * character representation.  Returns the wide character. */
-wchar_t utf8_first_char(const char utf8[]);
-
 /* Calculates how many utf-16 chars are needed to store given utf-8 string.
  * Returns the number. */
 size_t utf8_widen_len(const char utf8[]);
