@@ -26,6 +26,7 @@
 #include <stddef.h> /* size_t wchar_t */
 #include <stdint.h> /* uint64_t */
 #include <stdio.h> /* FILE */
+#include <time.h> /* time_t */
 
 #include "../macros.h"
 #include "../status.h"
@@ -46,6 +47,25 @@ typedef enum
 }
 ShellRequester;
 
+/* Fine tuning of expand_envvars() behaviour. */
+typedef enum
+{
+	EEF_NONE            = 0x00, /* None of the other options. */
+	EEF_ESCAPE_VALS     = 0x01, /* Escape value suitable for internal use. */
+	EEF_KEEP_ESCAPES    = 0x02, /* Only remove \ in front of $ and keep others. */
+	EEF_DOUBLE_PERCENTS = 0x04, /* Double percents to make the resulting string
+	                               suitable for further macro expansion. */
+}
+ExpandEnvFlags;
+
+/* Defines an integer interval. */
+typedef struct
+{
+	int first; /* Left inclusive bound. */
+	int last;  /* Right inclusive bound. */
+}
+interval_t;
+
 /* Forward declarations. */
 struct dir_entry_t;
 struct view_t;
@@ -101,7 +121,7 @@ int refers_to_slower_fs(const char from[], const char to[]);
 int friendly_size_notation(uint64_t num, int str_size, char str[]);
 
 /* Returns pointer to a statically allocated buffer. */
-const char * enclose_in_dquotes(const char str[]);
+const char * enclose_in_dquotes(const char str[], ShellType shell_type);
 
 /* Changes current working directory of the process.  Does nothing if we already
  * at path.  Returns zero on success, otherwise -1 is returned. */
@@ -111,10 +131,10 @@ int vifm_chdir(const char path[]);
  * memory, that should be freed by the caller. */
 char * expand_path(const char path[]);
 
-/* Expands all environment variables in the str of form "$envvar".  Non-zero
- * escape_vals means escaping suitable for internal use.  Allocates and returns
- * memory that should be freed by the caller. */
-char * expand_envvars(const char str[], int escape_vals);
+/* Expands all environment variables in the str of the form "$envvar".  Flags is
+ * a combination of EEF_* values.  Allocates and returns memory that should be
+ * freed by the caller. */
+char * expand_envvars(const char str[], int flags);
 
 /* Makes filename unique by adding an unique suffix to it.
  * Returns pointer to a statically allocated buffer */
@@ -147,10 +167,18 @@ char * escape_for_squotes(const char string[], size_t offset);
  * quoted string, prefix is not escaped.  Returns newly allocated string. */
 char * escape_for_dquotes(const char string[], size_t offset);
 
-/* Escapes characters aren't visible or don't looks nice to the user.  Returns
- * newly allocated string. */
+/* Escapes characters that aren't visible or don't look nice to the user.
+ * Returns newly allocated string. */
 char * escape_unreadable(const char str[]);
 
+/* Calculates escaping overhead for some prefix of the string.  Returns
+ * byte length difference between unchanged prefix and prefix after escaping,
+ * which might be positive, zero or negative. */
+int escape_unreadableo(const char str[], int prefix_len);
+
+/* Wide version of escape_unreadable(). */
+wchar_t * escape_unreadablew(const wchar_t str[]);
+
 /* Expands double percent sequences into single percent character in place. */
 void expand_percent_escaping(char s[]);
 
@@ -186,6 +214,10 @@ FILE * make_in_file(struct view_t *view, MacroFlags flags);
  * null characters. */
 void write_marked_paths(FILE *file, struct view_t *view, int null_sep);
 
+/* Formats time as a string independent of settings.  Writes empty string on
+ * error. */
+void format_iso_time(time_t t, char buf[], size_t buf_size);
+
 /* Checks line for path in it.  Ignores empty lines and attempts to parse it as
  * location line (path followed by a colon and optional line and column
  * numbers).  Returns canonicalized path as a newly allocated string or NULL. */
@@ -235,6 +267,16 @@ ExecEnvType get_exec_env_type(void);
 /* Determines kind of the shell by its invocation command.  Returns the kind. */
 ShellType get_shell_type(const char shell_cmd[]);
 
+/* Escapes the string for the purpose of inserting it into a POSIX-like shell or
+ * command-line.  type == 1 enables prepending percent sign with a percent
+ * sign and not escaping newline, because we do only worse.  type == 2 only
+ * skips escaping of newline.  Returns new string, caller should free it. */
+char * posix_like_escape(const char string[], int type);
+
+/* Escapes a string so that it can be used as a command argument in a shell
+ * invocation.  Returns newly allocated string. */
+char * shell_arg_escape(const char what[], ShellType shell_type);
+
 /* Formats command to view documentation in plain-text format.  Returns non-zero
  * if command that should be run in background, otherwise zero is returned. */
 int format_help_cmd(char cmd[], size_t cmd_size);
@@ -281,8 +323,10 @@ FILE * read_cmd_output(const char cmd[], int preserve_stdin);
 const char * get_installed_data_dir(void);
 
 /* Gets path to directory where global configuration files of Vifm are stored.
- * Returns pointer to a statically allocated buffer. */
-const char * get_sys_conf_dir(void);
+ * In case there are multiple such directories, index can be used to enumerate
+ * them starting with 0.  Returns pointer to a statically allocated buffer or
+ * NULL on error or invalid index. */
+const char * get_sys_conf_dir(int idx);
 
 /* Clones attributes from file specified by from to file at path.  st is a hint
  * to omit extra file system requests if possible, can be NULL on Windows. */
@@ -297,6 +341,25 @@ int get_drive_info(const char at[], uint64_t *total_bytes,
  * links if necessary.  Returns the inode number. */
 uint64_t get_true_inode(const struct dir_entry_t *entry);
 
+/* Auxiliary function for binary search in interval table.  Returns non-zero
+ * if search was successful and zero otherwise. */
+int unichar_bisearch(wchar_t ucs, const interval_t table[], int max);
+
+/* Checks whether Unicode character is printable.  Returns non-zero if so,
+ * otherwise zero is returned. */
+int unichar_isprint(wchar_t ucs);
+
+/* Creates a temporary file by replacing trailing "XXXXXX" in the path with some
+ * unique sequence.  Updates the path in place.  Returns file descriptor on
+ * success or -1 on failure.  errno is meaningful on failure. */
+int create_unique_file(char path[], mode_t mode, int auto_delete);
+
+/* Initializes random number generator with the seed. */
+void vifm_srand(unsigned int seed);
+
+/* Produces a random number in the ranage [min; max].  Returns the number. */
+int vifm_rand(int min, int max);
+
 #ifdef _WIN32
 #include "utils_win.h"
 #else
