@@ -39,8 +39,8 @@
 #include <grp.h> /* getgrnam() getgrgid_r() */
 #include <pthread.h> /* pthread_sigmask() */
 #include <pwd.h> /* getpwnam() getpwuid_r() */
-#include <unistd.h> /* X_OK chown() dup() dup2() getpid() isatty() pause()
-                       sysconf() ttyname() */
+#include <unistd.h> /* X_OK chown() close() dup() dup2() getpid() isatty()
+                       pause() sysconf() ttyname() */
 
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
@@ -93,6 +93,8 @@ typedef struct
 get_mount_point_traverser_state;
 
 static int get_mount_info_traverser(struct mntent *entry, void *arg);
+static void process_cancel_request(pid_t pid,
+		const cancellation_t *cancellation);
 static void free_mnt_entries(struct mntent *entries, unsigned int nentries);
 static struct mntent * read_mnt_entries(unsigned int *nentries);
 static int clone_mnt_entry(struct mntent *lhs, const struct mntent *rhs);
@@ -168,7 +170,7 @@ run_with_input(char command[], FILE *input, ShellRequester by)
 		_Exit(127);
 	}
 
-	result = get_proc_exit_status(pid);
+	result = get_proc_exit_status(pid, &no_cancellation);
 
 	sigaction(SIGTSTP, &old, NULL);
 
@@ -187,7 +189,7 @@ recover_after_shellout(void)
 
 void
 wait_for_data_from(pid_t pid, FILE *f, int fd,
-		const struct cancellation_t *cancellation)
+		const cancellation_t *cancellation)
 {
 	const struct timeval ts_init = { .tv_sec = 0, .tv_usec = 1000 };
 	struct timeval ts;
@@ -217,8 +219,10 @@ block_all_thread_signals(void)
 	pthread_sigmask(SIG_SETMASK, &set, NULL);
 }
 
-void
-process_cancel_request(pid_t pid, const struct cancellation_t *cancellation)
+/* Checks whether cancelling of current operation is requested and sends SIGINT
+ * to process specified by its process id to request cancellation. */
+static void
+process_cancel_request(pid_t pid, const cancellation_t *cancellation)
 {
 	if(cancellation_requested(cancellation))
 	{
@@ -231,7 +235,7 @@ process_cancel_request(pid_t pid, const struct cancellation_t *cancellation)
 }
 
 int
-get_proc_exit_status(pid_t pid)
+get_proc_exit_status(pid_t pid, const cancellation_t *cancellation)
 {
 	while(1)
 	{
@@ -240,9 +244,11 @@ get_proc_exit_status(pid_t pid)
 		{
 			if(errno == EINTR)
 			{
+				process_cancel_request(pid, cancellation);
 				continue;
 			}
-			LOG_SERROR_MSG(errno, "waitpid()");
+			LOG_SERROR_MSG(errno, "waitpid(%" PRINTF_ULL ") has failed",
+					(unsigned long long)pid);
 			break;
 		}
 
@@ -399,14 +405,15 @@ make_execv_array(char shell[], char shell_flag[], char cmd[])
 	}
 	(void)strappend(&eval_cmd, &len, "\"");
 
-	args[i++] = shell;
+	int j = 0;
+	args[j++] = shell;
 	if(with_sh_arg)
 	{
-		args[i++] = sh_arg;
+		args[j++] = sh_arg;
 	}
-	args[i++] = shell_flag;
-	args[i++] = eval_cmd;
-	args[i++ + npieces] = NULL;
+	args[j++] = shell_flag;
+	args[j++] = eval_cmd;
+	args[j + npieces] = NULL;
 
 	return args;
 }
@@ -812,14 +819,21 @@ get_exec_env_type(void)
 ShellType
 get_shell_type(const char shell_cmd[])
 {
-	return ST_NORMAL;
+	return ST_POSIX;
+}
+
+char *
+shell_arg_escape(const char what[], ShellType shell_type)
+{
+	assert(shell_type == ST_POSIX && "Unexpected shell type.");
+	return posix_like_escape(what, /*type=*/0);
 }
 
 int
 format_help_cmd(char cmd[], size_t cmd_size)
 {
 	int bg;
-	char *const escaped = shell_like_escape(cfg.config_dir, 0);
+	char *const escaped = posix_like_escape(cfg.config_dir, /*type=*/0);
 	snprintf(cmd, cmd_size, "%s %s/" VIFM_HELP, cfg_get_vicmd(&bg), escaped);
 	free(escaped);
 	return bg;
@@ -1084,15 +1098,44 @@ get_installed_data_dir(void)
 }
 
 const char *
-get_sys_conf_dir(void)
+get_sys_conf_dir(int idx)
 {
 	static char conf_dir[PATH_MAX + 1];
+	static int init_done;
+
+	if(!init_done)
+	{
+		init_done = 1;
+
+		const char *prefix = env_get("VIFM_APPDIR_ROOT");
+		if(!is_null_or_empty(prefix))
+		{
+			int written = snprintf(conf_dir, sizeof(conf_dir), "%s%s", prefix,
+					PACKAGE_SYSCONF_DIR);
+			if(written < 0 || written >= (int)sizeof(conf_dir))
+			{
+				conf_dir[0] = '\0';
+			}
+		}
+	}
+
 	if(conf_dir[0] == '\0')
 	{
-		const char *prefix = env_get_def("VIFM_APPDIR_ROOT", "");
-		snprintf(conf_dir, sizeof(conf_dir), "%s%s", prefix, PACKAGE_SYSCONF_DIR);
+		/* Offset indices by one to skip AppImage configuration directory which
+		 * isn't present. */
+		++idx;
 	}
-	return conf_dir;
+
+	if(idx == 0)
+	{
+		return conf_dir;
+	}
+	if(idx == 1)
+	{
+		return PACKAGE_SYSCONF_DIR;
+	}
+
+	return NULL;
 }
 
 void
@@ -1215,7 +1258,7 @@ get_drive_info(const char at[], uint64_t *total_bytes, uint64_t *free_bytes)
 }
 
 uint64_t
-get_true_inode(const struct dir_entry_t *entry)
+get_true_inode(const dir_entry_t *entry)
 {
 	if(entry->type != FT_LINK)
 	{
@@ -1233,5 +1276,43 @@ get_true_inode(const struct dir_entry_t *entry)
 	return entry->inode;
 }
 
+void
+bind_pipe_or_die(int fd, int pipe_end, int pipe_other)
+{
+	if(dup2(pipe_end, fd) == -1)
+	{
+		perror("dup2");
+		_Exit(EXIT_FAILURE);
+	}
+
+	/* Close original input pipe descriptors. */
+	if(pipe_end != fd)
+	{
+		close(pipe_end);
+	}
+	close(pipe_other);
+}
+
+int
+create_new_file(const char path[], mode_t mode, int auto_delete)
+{
+	int fd = open(path, O_RDWR | O_CREAT | O_EXCL, mode);
+	if(fd == -1)
+	{
+		return -1;
+	}
+
+	if(auto_delete && unlink(path) != 0)
+	{
+		/* Something weird has happened. */
+		int error = errno;
+		(void)close(fd);
+		errno = error;
+		return -1;
+	}
+
+	return fd;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
