@@ -32,6 +32,7 @@
 #include <unistd.h> /* _dup2() _pipe() _spawnvp() close() dup() pipe() */
 
 #include <ctype.h> /* toupper() */
+#include <errno.h> /* EEXIST ENOMEM errno */
 #include <stddef.h> /* NULL size_t */
 #include <stdint.h> /* uint32_t */
 #include <stdlib.h> /* EXIT_SUCCESS free() */
@@ -80,7 +81,7 @@ static BOOL CALLBACK close_app_enum(HWND hwnd, LPARAM lParam);
 void
 pause_shell(void)
 {
-	if(curr_stats.shell_type == ST_CMD)
+	if(curr_stats.shell_type == ST_CMD || curr_stats.shell_type == ST_YORI)
 	{
 		run_in_shell_no_cls("pause", SHELL_BY_APP);
 	}
@@ -97,22 +98,13 @@ pause_shell(void)
 int
 run_in_shell_no_cls(char command[], ShellRequester by)
 {
-	int ret;
 	char *const sh_cmd = win_make_sh_cmd(command, by);
 
-	/* XXX: why do we use different functions for different cases? */
-	if(curr_stats.shell_type == ST_CMD || curr_stats.shell_type == ST_PS)
+	int returned_exit_code;
+	int ret = win_exec_cmd(command, sh_cmd, &returned_exit_code);
+	if(!returned_exit_code)
 	{
-		ret = os_system(sh_cmd);
-	}
-	else
-	{
-		int returned_exit_code;
-		ret = win_exec_cmd(sh_cmd, &returned_exit_code);
-		if (!returned_exit_code)
-		{
-			ret = -1;
-		}
+		ret = -1;
 	}
 
 	free(sh_cmd);
@@ -248,9 +240,9 @@ wcwidth(wchar_t c)
 }
 
 int
-win_exec_cmd(char cmd[], int *returned_exit_code)
+win_exec_cmd(const char cmd[], char full_cmd[], int *returned_exit_code)
 {
-	wchar_t *utf16_cmd;
+	wchar_t *utf16_full_cmd;
 	BOOL ret;
 	DWORD code;
 	STARTUPINFOW startup = {};
@@ -258,10 +250,10 @@ win_exec_cmd(char cmd[], int *returned_exit_code)
 
 	*returned_exit_code = 0;
 
-	utf16_cmd = utf8_to_utf16(cmd);
-	ret = CreateProcessW(NULL, utf16_cmd, NULL, NULL, 0, 0, NULL, NULL, &startup,
-			&pinfo);
-	free(utf16_cmd);
+	utf16_full_cmd = utf8_to_utf16(full_cmd);
+	ret = CreateProcessW(NULL, utf16_full_cmd, NULL, NULL, 0, 0, NULL, NULL,
+			&startup, &pinfo);
+	free(utf16_full_cmd);
 
 	if(ret == 0)
 	{
@@ -296,6 +288,11 @@ win_make_sh_cmd(const char cmd[], ShellRequester by)
 		 * special characters work at the same time. */
 		fmt = (cmd[0] == '"') ? "%s %s \"%s\"" : "%s %s %s";
 	}
+	else if(curr_stats.shell_type == ST_YORI)
+	{
+		sh_flag = (by == SHELL_BY_USER ? cfg.shell_cmd_flag : "-c");
+		fmt = "%s %s %s";
+	}
 	else if(curr_stats.shell_type == ST_PS)
 	{
 		fmt = "%s %s %s";
@@ -396,6 +393,8 @@ handle_process(const char cmd[], HANDLE proc, int *got_exit_code)
 
 	if(cmd != NULL && !should_wait_for_program(cmd))
 	{
+		/* Simulate program finishing normally. */
+		*got_exit_code = 1;
 		return 0;
 	}
 
@@ -694,6 +693,10 @@ get_shell_type(const char shell_cmd[])
 	{
 		return ST_CMD;
 	}
+	if(stroscmp(shell_name, "yori") == 0 || stroscmp(shell_name, "yori.exe") == 0)
+	{
+		return ST_YORI;
+	}
 	if(stroscmp(shell_name, "powershell") == 0 ||
 			stroscmp(shell_name, "powershell.exe") == 0 ||
 			stroscmp(shell_name, "pwsh") == 0 ||
@@ -701,7 +704,13 @@ get_shell_type(const char shell_cmd[])
 	{
 		return ST_PS;
 	}
-	return ST_NORMAL;
+	return ST_POSIX;
+}
+
+char *
+shell_arg_escape(const char what[], ShellType shell_type)
+{
+	return strdup(enclose_in_dquotes(what, shell_type));
 }
 
 int
@@ -744,7 +753,7 @@ update_terminal_settings(void)
 	/* We need ENABLE_WINDOW_INPUT flag to get terminal resize event. */
 	SetConsoleMode(GetStdHandle(STD_INPUT_HANDLE), ENABLE_ECHO_INPUT |
 			ENABLE_EXTENDED_FLAGS | ENABLE_INSERT_MODE | ENABLE_LINE_INPUT |
-			ENABLE_MOUSE_INPUT | ENABLE_QUICK_EDIT_MODE | ENABLE_WINDOW_INPUT);
+			ENABLE_MOUSE_INPUT | ENABLE_WINDOW_INPUT);
 }
 
 void
@@ -931,51 +940,9 @@ get_installed_data_dir(void)
 }
 
 const char *
-get_sys_conf_dir(void)
+get_sys_conf_dir(int idx)
 {
-	return get_installed_data_dir();
-}
-
-FILE *
-win_tmpfile(void)
-{
-	char dir[PATH_MAX + 1];
-	char file[PATH_MAX + 1];
-	HANDLE h;
-	int fd;
-	FILE *f;
-
-	if(GetTempPathA(sizeof(dir), dir) == 0)
-	{
-		return NULL;
-	}
-
-	if(GetTempFileNameA(dir, "dir-view", 0U, file) == 0)
-	{
-		return NULL;
-	}
-
-	h = CreateFileA(file, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING,
-			FILE_FLAG_DELETE_ON_CLOSE, NULL);
-	if(h == INVALID_HANDLE_VALUE)
-	{
-		return NULL;
-	}
-
-	fd = _open_osfhandle((intptr_t)h, _O_RDWR);
-	if(fd == -1)
-	{
-		CloseHandle(h);
-		return NULL;
-	}
-
-	f = fdopen(fd, "r+");
-	if(f == NULL)
-	{
-		close(fd);
-	}
-
-	return f;
+	return (idx == 0 ? get_installed_data_dir() : NULL);
 }
 
 void
@@ -1353,5 +1320,27 @@ get_true_inode(const struct dir_entry_t *entry)
 	return 0;
 }
 
+int
+create_new_file(const char path[], mode_t mode, int auto_delete)
+{
+	DWORD attrs = auto_delete ? FILE_FLAG_DELETE_ON_CLOSE : FILE_ATTRIBUTE_NORMAL;
+	HANDLE h = CreateFileA(path, GENERIC_READ | GENERIC_WRITE, 0, NULL,
+			CREATE_NEW, attrs, NULL);
+	if(h == INVALID_HANDLE_VALUE)
+	{
+		errno = EEXIST;
+		return -1;
+	}
+
+	int fd = _open_osfhandle((intptr_t)h, _O_RDWR);
+	if(fd == -1)
+	{
+		CloseHandle(h);
+		errno = ENOMEM;
+	}
+
+	return fd;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
