@@ -63,6 +63,8 @@ typedef struct vcache_entry_t
 	size_t size;       /* Size taken up by this entry (lower bound). */
 	int max_lines;     /* Number of lines requested. */
 
+	/* Value of maxtreedepth for this entry. */
+	int max_tree_depth;
 	/* Whether cache contains complete output of the viewer. */
 	unsigned int complete : 1;
 	/* Whether last line is truncated. */
@@ -163,9 +165,10 @@ vcache_lookup(const char full_path[], const char viewer[], MacroFlags flags,
 {
 	*error = NULL;
 
-	if(kind == VK_GRAPHICAL)
+	/* Skip caching of data if we can't really cache it or when user doesn't want
+	 * it to be cached. */
+	if(kind == VK_GRAPHICAL || ma_flags_present(flags, MF_NO_CACHE))
 	{
-		/* Skip caching of data we can't really cache. */
 		static vcache_entry_t non_cache;
 		free_cache_entry(&non_cache);
 
@@ -173,7 +176,7 @@ vcache_lookup(const char full_path[], const char viewer[], MacroFlags flags,
 		replace_string(&non_cache.path, full_path);
 		update_string(&non_cache.viewer, viewer);
 
-		non_cache.lines = view_entry(&non_cache, MF_NONE, error);
+		non_cache.lines = view_entry(&non_cache, flags, error);
 		wait_async_finish(&non_cache);
 
 		return non_cache.lines;
@@ -423,7 +426,8 @@ is_cache_valid(const vcache_entry_t *centry, const char path[],
 		}
 	}
 
-	if(centry->top_tree_stats != cfg.top_tree_stats && is_dir(path))
+	if((centry->top_tree_stats != cfg.top_tree_stats ||
+			centry->max_tree_depth != cfg.max_tree_depth) && is_dir(path))
 	{
 		return 0;
 	}
@@ -670,6 +674,7 @@ view_builtin(vcache_entry_t *centry, const char **error)
 	if(dir)
 	{
 		centry->top_tree_stats = cfg.top_tree_stats;
+		centry->max_tree_depth = cfg.max_tree_depth;
 		fp = qv_view_dir(centry->path, centry->max_lines);
 	}
 	else
@@ -719,6 +724,11 @@ view_external(vcache_entry_t *centry, MacroFlags flags, const char **error)
 		bg_flags |= BJF_SUPPLY_INPUT;
 	}
 
+	if(ma_flags_present(flags, MF_KEEP_IN_FG))
+	{
+		bg_flags |= BJF_KEEP_IN_FG;
+	}
+
 	centry->job = bg_run_external_job(centry->viewer, bg_flags);
 	if(centry->job == NULL)
 	{
