@@ -31,7 +31,7 @@
 #include <locale.h> /* setlocale() LC_ALL */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* fprintf() fputs() puts() snprintf() */
-#include <stdlib.h> /* EXIT_FAILURE EXIT_SUCCESS exit() srand() system() */
+#include <stdlib.h> /* EXIT_FAILURE EXIT_SUCCESS exit() system() */
 #include <string.h>
 #include <time.h> /* time() */
 
@@ -97,7 +97,7 @@
 
 static int vifm_main(int argc, char *argv[]);
 static int get_start_cwd(char buf[], size_t buf_len);
-static int undo_perform_func(OPS op, void *data, const char src[],
+static OpsResult undo_perform_func(OPS op, void *data, const char src[],
 		const char dst[]);
 static void parse_received_arguments(char *args[]);
 static char * eval_received_expression(const char expr[]);
@@ -176,7 +176,7 @@ vifm_main(int argc, char *argv[])
 	json_set_check_strings(0);
 
 	args_parse(&vifm_args, argc, argv, dir);
-	args_process(&vifm_args, 1);
+	args_process(&vifm_args, AS_GENERAL, curr_stats.ipc);
 
 	lwin_cv = (strcmp(vifm_args.lwin_path, "-") == 0 && vifm_args.lwin_handle);
 	rwin_cv = (strcmp(vifm_args.rwin_path, "-") == 0 && vifm_args.rwin_handle);
@@ -191,15 +191,26 @@ vifm_main(int argc, char *argv[])
 	}
 
 	(void)setlocale(LC_ALL, "");
-	srand(time(NULL));
-
-	cfg_init();
+	vifm_srand((unsigned int)time(NULL));
 
 	if(vifm_args.logging)
 	{
 		init_logger(1, vifm_args.startup_log_path);
 	}
 
+	/* Process --remote* parameters even before initializing configuration as it
+	 * indirectly depends on terminal initialization and IPC interaction mustn't
+	 * need a terminal. */
+	struct ipc_t *ipc = ipc_init(vifm_args.server_name, &parse_received_arguments,
+			&eval_received_expression);
+	if(ipc_enabled() && ipc == NULL)
+	{
+		fputs("Failed to initialize IPC unit", stderr);
+		return -1;
+	}
+	args_process(&vifm_args, AS_IPC, ipc);
+
+	cfg_init();
 	init_filelists();
 	tabs_init();
 	regs_init();
@@ -207,7 +218,7 @@ vifm_main(int argc, char *argv[])
 	reinit_logger(cfg.log_file);
 
 	/* Commands module also initializes bracket notation and variables. */
-	init_commands();
+	cmds_init();
 
 	init_builtin_functions();
 	update_path_env(1);
@@ -220,6 +231,8 @@ vifm_main(int argc, char *argv[])
 		return -1;
 	}
 
+	curr_stats.ipc = ipc;
+
 	/* Tell file type module what function to use to check availability of
 	 * external programs. */
 	ft_init(&external_command_exists);
@@ -235,23 +248,25 @@ vifm_main(int argc, char *argv[])
 		state_load(0);
 	}
 
-	curr_stats.ipc = ipc_init(vifm_args.server_name, &parse_received_arguments,
-			&eval_received_expression);
-	if(ipc_enabled() && curr_stats.ipc == NULL)
-	{
-		fputs("Failed to initialize IPC unit", stderr);
-		return -1;
-	}
-	/* Export chosen server name to parsing unit. */
+	/* Export chosen IPC server name to parsing unit. */
 	{
 		var_t var = var_from_str(ipc_get_name(curr_stats.ipc));
 		setvar("v:servername", var);
 		var_free(var);
 	}
 
-	args_process(&vifm_args, 0);
+	args_process(&vifm_args, AS_OTHER, curr_stats.ipc);
 
-	bg_init();
+	/* $VIFM/plugins is to be searched for plugins last. */
+	curr_stats.plugins_dirs.nitems = put_into_string_array(
+			&curr_stats.plugins_dirs.items, curr_stats.plugins_dirs.nitems,
+			format_str("%s/plugins", cfg.config_dir));
+
+	if(bg_init() != 0)
+	{
+		fprintf(stderr, "Failed to initialize threads.\n");
+		return -1;
+	}
 
 	fops_init(&modcline_prompt, &prompt_msg_custom);
 
@@ -286,7 +301,7 @@ vifm_main(int argc, char *argv[])
 		return -1;
 	}
 
-	init_modes();
+	modes_init();
 	un_init(&undo_perform_func, NULL, &ui_cancellation_requested,
 			&cfg.undo_levels);
 	load_view_options(curr_view);
@@ -318,7 +333,7 @@ vifm_main(int argc, char *argv[])
 		(void)trash_set_specs(cfg.trash_dir);
 	}
 
-	plugs_load(curr_stats.plugs, cfg.config_dir);
+	plugs_load(curr_stats.plugs, curr_stats.plugins_dirs);
 
 	check_path_for_file(&lwin, vifm_args.lwin_path, vifm_args.lwin_handle);
 	check_path_for_file(&rwin, vifm_args.rwin_path, vifm_args.rwin_handle);
@@ -352,7 +367,11 @@ vifm_main(int argc, char *argv[])
 
 	curr_stats.load_stage = 3;
 
-	event_loop(&quit);
+	/* Update screen after startup commands while in load state 3 so CHPOS_STARTUP
+	 * has no effect and doesn't reset cursor position after `+"goto path"`. */
+	update_screen(stats_update_fetch());
+
+	event_loop(&quit, /*manage_marking=*/1);
 
 	return 0;
 }
@@ -380,7 +399,7 @@ get_start_cwd(char buf[], size_t buf_len)
 }
 
 /* perform_operation() interface adaptor for the undo unit. */
-static int
+static OpsResult
 undo_perform_func(OPS op, void *data, const char src[], const char dst[])
 {
 	return perform_operation(op, NULL, data, src, dst);
@@ -395,10 +414,13 @@ parse_received_arguments(char *argv[])
 	(void)vifm_chdir(argv[0]);
 	opterr = 0;
 	args_parse(&args, count_strings(argv), argv, argv[0]);
-	args_process(&args, 0);
+	args_process(&args, AS_IPC, curr_stats.ipc);
+	/* XXX: why AS_OTHER invocation is used with IPC args, is this a mistake? */
+	args_process(&args, AS_OTHER, curr_stats.ipc);
 
-	abort_menu_like_mode();
+	modes_abort_menu_like();
 	exec_startup_commands(&args);
+	update_screen(stats_update_fetch());
 
 	if(NONE(vle_mode_is, NORMAL_MODE, VIEW_MODE))
 	{
@@ -429,7 +451,7 @@ parse_received_arguments(char *argv[])
 
 	args_free(&args);
 
-	/* XXX: why force clearing of the statusbar? */
+	/* XXX: why force clearing of the status bar? */
 	ui_sb_clear();
 	curr_stats.save_msg = 0;
 }
@@ -498,16 +520,15 @@ need_to_switch_active_pane(const char lwin_path[], const char rwin_path[])
 static char *
 eval_received_expression(const char expr[])
 {
-	char *result_str;
-
-	var_t result;
-	if(parse(expr, 1, &result) != PE_NO_ERROR)
+	parsing_result_t result = vle_parser_eval(expr, /*interactive=*/1);
+	if(result.error != PE_NO_ERROR)
 	{
+		var_free(result.value);
 		return NULL;
 	}
 
-	result_str = var_to_str(result);
-	var_free(result);
+	char *result_str = var_to_str(result.value);
+	var_free(result.value);
 	return result_str;
 }
 
@@ -530,6 +551,7 @@ void
 vifm_reexec_startup_commands(void)
 {
 	exec_startup_commands(&vifm_args);
+	update_screen(stats_update_fetch());
 }
 
 /* Executes list of startup commands. */
@@ -542,10 +564,8 @@ exec_startup_commands(const args_t *args)
 		/* Make sure we're executing commands in correct directory. */
 		(void)vifm_chdir(flist_get_dir(curr_view));
 
-		(void)exec_commands(args->cmds[i], curr_view, CIT_COMMAND);
+		(void)cmds_dispatch(args->cmds[i], curr_view, CIT_COMMAND);
 	}
-
-	update_screen(stats_update_fetch());
 }
 
 void
@@ -560,6 +580,10 @@ vifm_try_leave(int store_state, int cquit, int force)
 		}
 	}
 
+	/* Has to be run early when UI is up and state can still be changed. */
+	vlua_events_app_exit(curr_stats.vlua);
+	vlua_process_callbacks(curr_stats.vlua);
+
 	fuse_unmount_all();
 
 	if(store_state)
@@ -583,6 +607,10 @@ vifm_try_leave(int store_state, int cquit, int force)
 void _gnuc_noreturn
 vifm_choose_files(view_t *view, int nfiles, char *files[])
 {
+	/* Has to be run early when UI is up and state can still be changed. */
+	vlua_events_app_exit(curr_stats.vlua);
+	vlua_process_callbacks(curr_stats.vlua);
+
 	/* As curses can do something with terminal on shutting down, disable it
 	 * before writing anything to the screen. */
 	ui_shutdown();
