@@ -1,10 +1,9 @@
 #include <stic.h>
 
-#include <sys/stat.h> /* chmod() */
-#include <unistd.h> /* F_OK access() chdir() rmdir() symlink() unlink() */
+#include <unistd.h> /* chdir() rmdir() symlink() unlink() */
 
 #include <limits.h> /* INT_MAX */
-#include <stdio.h> /* FILE fclose() fopen() fprintf() remove() */
+#include <stdio.h> /* remove() */
 #include <string.h> /* strcpy() strdup() */
 
 #include <test-utils.h>
@@ -13,24 +12,24 @@
 #include "../../src/compat/fs_limits.h"
 #include "../../src/compat/os.h"
 #include "../../src/engine/keys.h"
-#include "../../src/engine/functions.h"
 #include "../../src/lua/vlua.h"
 #include "../../src/modes/modes.h"
 #include "../../src/ui/statusbar.h"
 #include "../../src/ui/ui.h"
 #include "../../src/utils/dynarray.h"
-#include "../../src/utils/env.h"
 #include "../../src/utils/fs.h"
 #include "../../src/utils/macros.h"
 #include "../../src/utils/path.h"
 #include "../../src/utils/str.h"
 #include "../../src/utils/string_array.h"
-#include "../../src/builtin_functions.h"
 #include "../../src/cmd_core.h"
+#include "../../src/compare.h"
 #include "../../src/filelist.h"
 #include "../../src/flist_hist.h"
 #include "../../src/plugins.h"
 #include "../../src/registers.h"
+#include "../../src/running.h"
+#include "../../src/status.h"
 
 static char *saved_cwd;
 
@@ -43,7 +42,6 @@ static void strings_list_is(const strlist_t expected, const strlist_t actual);
 SETUP_ONCE()
 {
 	cfg.sizefmt.base = 1;
-	cfg.dot_dirs = DD_TREE_LEAFS_PARENT;
 
 	assert_non_null(get_cwd(cwd, sizeof(cwd)));
 
@@ -59,24 +57,9 @@ SETUP()
 	curr_view = &lwin;
 	other_view = &rwin;
 
-	cfg.cd_path = strdup("");
-	cfg.fuse_home = strdup("");
-	cfg.slow_fs_list = strdup("");
-	cfg.use_system_calls = 1;
-
-#ifndef _WIN32
-	replace_string(&cfg.shell, "/bin/sh");
-	update_string(&cfg.shell_cmd_flag, "-c");
-#else
-	replace_string(&cfg.shell, "cmd");
-	update_string(&cfg.shell_cmd_flag, "/C");
-#endif
-
-	stats_update_shell_type(cfg.shell);
-
-	init_commands();
-
+	conf_setup();
 	undo_setup();
+	cmds_init();
 
 	saved_cwd = save_cwd();
 }
@@ -85,19 +68,11 @@ TEARDOWN()
 {
 	restore_cwd(saved_cwd);
 
-	update_string(&cfg.cd_path, NULL);
-	update_string(&cfg.fuse_home, NULL);
-	update_string(&cfg.slow_fs_list, NULL);
-
-	stats_update_shell_type("/bin/sh");
-	update_string(&cfg.shell_cmd_flag, NULL);
-	update_string(&cfg.shell, NULL);
-
 	view_teardown(&lwin);
 	view_teardown(&rwin);
 
+	conf_teardown();
 	vle_cmds_reset();
-
 	undo_teardown();
 }
 
@@ -108,7 +83,7 @@ TEST(cd_in_root_works)
 	strcpy(lwin.curr_dir, test_data);
 
 	assert_false(is_root_dir(lwin.curr_dir));
-	assert_success(exec_commands("cd /", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("cd /", &lwin, CIT_COMMAND));
 	assert_true(is_root_dir(lwin.curr_dir));
 }
 
@@ -121,7 +96,7 @@ TEST(double_cd_uses_same_base_for_rel_paths)
 	strcpy(lwin.curr_dir, test_data);
 	strcpy(rwin.curr_dir, "..");
 
-	assert_success(exec_commands("cd read rename", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("cd read rename", &lwin, CIT_COMMAND));
 
 	snprintf(path, sizeof(path), "%s/read", test_data);
 	assert_true(paths_are_equal(lwin.curr_dir, path));
@@ -129,114 +104,6 @@ TEST(double_cd_uses_same_base_for_rel_paths)
 	assert_true(paths_are_equal(rwin.curr_dir, path));
 }
 
-TEST(cds_does_the_replacement)
-{
-	char path[PATH_MAX + 1];
-	snprintf(path, sizeof(path), "%s/syntax-highlight", test_data);
-	assert_success(chdir(path));
-	strcpy(lwin.curr_dir, path);
-
-	assert_success(exec_commands("cds syntax-highlight rename", &lwin,
-				CIT_COMMAND));
-
-	snprintf(path, sizeof(path), "%s/rename", test_data);
-	assert_string_equal(path, lwin.curr_dir);
-}
-
-TEST(cds_aborts_on_broken_)
-{
-	char dst[PATH_MAX + 1];
-	snprintf(dst, sizeof(dst), "%s/rename", test_data);
-
-	assert_success(chdir(dst));
-
-	strcpy(lwin.curr_dir, dst);
-
-	assert_failure(exec_commands("cds/rename/read/t", &lwin, CIT_COMMAND));
-
-	assert_string_equal(dst, lwin.curr_dir);
-}
-
-TEST(cds_acts_like_substitute)
-{
-	char path[PATH_MAX + 1];
-	snprintf(path, sizeof(path), "%s/syntax-highlight", test_data);
-	assert_success(chdir(path));
-	strcpy(lwin.curr_dir, path);
-
-	assert_success(exec_commands("cds/SYNtax-?hi[a-z]*/rename/i", &lwin,
-				CIT_COMMAND));
-
-	snprintf(path, sizeof(path), "%s/rename", test_data);
-	assert_string_equal(path, lwin.curr_dir);
-}
-
-TEST(cds_can_change_path_of_both_panes)
-{
-	char path[PATH_MAX + 1];
-	snprintf(path, sizeof(path), "%s/syntax-highlight", test_data);
-	assert_success(chdir(path));
-	strcpy(lwin.curr_dir, path);
-
-	assert_success(exec_commands("cds! syntax-highlight rename", &lwin,
-				CIT_COMMAND));
-
-	snprintf(path, sizeof(path), "%s/rename", test_data);
-	assert_string_equal(path, lwin.curr_dir);
-	assert_string_equal(path, rwin.curr_dir);
-}
-
-TEST(cds_is_noop_when_pattern_not_found)
-{
-	assert_success(chdir(test_data));
-
-	strcpy(lwin.curr_dir, test_data);
-	strcpy(rwin.curr_dir, sandbox);
-
-	assert_failure(exec_commands("cds asdlfkjasdlkfj rename", &lwin,
-				CIT_COMMAND));
-
-	assert_string_equal(test_data, lwin.curr_dir);
-	assert_string_equal(sandbox, rwin.curr_dir);
-}
-
-TEST(cpmv_does_not_crash_on_wrong_list_access)
-{
-	char path[PATH_MAX + 1];
-	snprintf(path, sizeof(path), "%s/existing-files", test_data);
-
-	assert_success(chdir(path));
-
-	strcpy(lwin.curr_dir, path);
-	strcpy(rwin.curr_dir, sandbox);
-
-	lwin.list_rows = 3;
-	lwin.list_pos = 0;
-	lwin.dir_entry = dynarray_cextend(NULL,
-			lwin.list_rows*sizeof(*lwin.dir_entry));
-	lwin.dir_entry[0].name = strdup("a");
-	lwin.dir_entry[0].origin = &lwin.curr_dir[0];
-	lwin.dir_entry[0].selected = 1;
-	lwin.dir_entry[1].name = strdup("b");
-	lwin.dir_entry[1].origin = &lwin.curr_dir[0];
-	lwin.dir_entry[1].selected = 1;
-	lwin.dir_entry[2].name = strdup("c");
-	lwin.dir_entry[2].origin = &lwin.curr_dir[0];
-	lwin.dir_entry[2].selected = 1;
-	lwin.selected_files = 3;
-
-	/* cpmv used to use presence of the argument as indication of availability of
-	 * file list and access memory beyond array boundaries. */
-	(void)exec_commands("co .", &lwin, CIT_COMMAND);
-
-	snprintf(path, sizeof(path), "%s/a", sandbox);
-	assert_success(remove(path));
-	snprintf(path, sizeof(path), "%s/b", sandbox);
-	assert_success(remove(path));
-	snprintf(path, sizeof(path), "%s/c", sandbox);
-	assert_success(remove(path));
-}
-
 TEST(tr_extends_second_field)
 {
 	char path[PATH_MAX + 1];
@@ -255,7 +122,7 @@ TEST(tr_extends_second_field)
 	lwin.dir_entry[0].name = strdup("a b");
 	lwin.dir_entry[0].origin = &lwin.curr_dir[0];
 
-	(void)exec_commands("tr/ ?<>\\\\:*|\"/_", &lwin, CIT_COMMAND);
+	(void)cmds_dispatch("tr/ ?<>\\\\:*|\"/_", &lwin, CIT_COMMAND);
 
 	snprintf(path, sizeof(path), "%s/a_b", sandbox);
 	assert_success(remove(path));
@@ -283,7 +150,7 @@ TEST(substitute_works)
 	lwin.dir_entry[1].name = strdup("B c");
 	lwin.dir_entry[1].origin = &lwin.curr_dir[0];
 
-	(void)exec_commands("%substitute/b/c/Iig", &lwin, CIT_COMMAND);
+	(void)cmds_dispatch("%substitute/b/c/Iig", &lwin, CIT_COMMAND);
 
 	snprintf(path, sizeof(path), "%s/a c c", sandbox);
 	assert_success(remove(path));
@@ -313,7 +180,7 @@ TEST(chmod_works, IF(not_windows))
 	lwin.dir_entry[1].name = strdup("file2");
 	lwin.dir_entry[1].origin = &lwin.curr_dir[0];
 
-	(void)exec_commands("1,2chmod +x", &lwin, CIT_COMMAND);
+	(void)cmds_dispatch("1,2chmod +x", &lwin, CIT_COMMAND);
 
 	populate_dir_list(&lwin, 1);
 	assert_int_equal(FT_EXEC, lwin.dir_entry[0].type);
@@ -325,43 +192,6 @@ TEST(chmod_works, IF(not_windows))
 	assert_success(remove(path));
 }
 
-TEST(edit_handles_ranges, IF(not_windows))
-{
-	create_file(SANDBOX_PATH "/file1");
-	create_file(SANDBOX_PATH "/file2");
-
-	char script_path[PATH_MAX + 1];
-	make_abs_path(script_path, sizeof(script_path), sandbox, "script", NULL);
-	update_string(&cfg.vi_command, script_path);
-	update_string(&cfg.vi_x_command, "");
-
-	FILE *fp = fopen(SANDBOX_PATH "/script", "w");
-	fprintf(fp, "#!/bin/sh\n");
-	fprintf(fp, "for arg; do echo \"$arg\" >> %s/vi-list; done\n", SANDBOX_PATH);
-	fclose(fp);
-	assert_success(chmod(SANDBOX_PATH "/script", 0777));
-
-	strcpy(lwin.curr_dir, sandbox);
-	lwin.list_rows = 2;
-	lwin.list_pos = 0;
-	lwin.dir_entry = dynarray_cextend(NULL,
-			lwin.list_rows*sizeof(*lwin.dir_entry));
-	lwin.dir_entry[0].name = strdup("file1");
-	lwin.dir_entry[0].origin = &lwin.curr_dir[0];
-	lwin.dir_entry[1].name = strdup("file2");
-	lwin.dir_entry[1].origin = &lwin.curr_dir[0];
-
-	(void)exec_commands("%edit", &lwin, CIT_COMMAND);
-
-	const char *lines[] = { "file1", "file2" };
-	file_is(SANDBOX_PATH "/vi-list", lines, ARRAY_LEN(lines));
-
-	assert_success(remove(SANDBOX_PATH "/script"));
-	assert_success(remove(SANDBOX_PATH "/file1"));
-	assert_success(remove(SANDBOX_PATH "/file2"));
-	assert_success(remove(SANDBOX_PATH "/vi-list"));
-}
-
 TEST(putting_files_works)
 {
 	char path[PATH_MAX + 1];
@@ -375,7 +205,7 @@ TEST(putting_files_works)
 	assert_success(regs_append(DEFAULT_REG_NAME, path));
 	lwin.list_pos = 1;
 
-	assert_true(exec_commands("put", &lwin, CIT_COMMAND) != 0);
+	assert_true(cmds_dispatch("put", &lwin, CIT_COMMAND) != 0);
 	restore_cwd(saved_cwd);
 	saved_cwd = save_cwd();
 
@@ -385,68 +215,10 @@ TEST(putting_files_works)
 	regs_reset();
 }
 
-TEST(wincmd_can_switch_views)
-{
-	init_modes();
-	opt_handlers_setup();
-	assert_success(stats_init(&cfg));
-
-	curr_view = &rwin;
-	other_view = &lwin;
-	assert_success(exec_commands("wincmd h", curr_view, CIT_COMMAND));
-	assert_true(curr_view == &lwin);
-
-	curr_view = &rwin;
-	other_view = &lwin;
-	assert_success(exec_commands("execute 'wincmd h'", curr_view, CIT_COMMAND));
-	assert_true(curr_view == &lwin);
-
-	init_builtin_functions();
-
-	curr_view = &rwin;
-	other_view = &lwin;
-	assert_success(
-			exec_commands("if paneisat('left') == 0 | execute 'wincmd h' | endif",
-				curr_view, CIT_COMMAND));
-	assert_true(curr_view == &lwin);
-
-	curr_view = &rwin;
-	other_view = &lwin;
-	assert_success(
-			exec_commands("if paneisat('left') == 0 "
-			             "|    execute 'wincmd h' "
-			             "|    let $a = paneisat('left') "
-			             "|endif",
-				curr_view, CIT_COMMAND));
-	assert_true(curr_view == &lwin);
-	assert_string_equal("1", env_get("a"));
-
-	function_reset_all();
-
-	opt_handlers_teardown();
-	assert_success(stats_reset(&cfg));
-	vle_keys_reset();
-}
-
-TEST(wincmd_ignores_mappings)
-{
-	init_modes();
-	opt_handlers_setup();
-
-	curr_view = &rwin;
-	other_view = &lwin;
-	assert_success(exec_commands("nnoremap <c-w> <nop>", curr_view, CIT_COMMAND));
-	assert_success(exec_commands("wincmd H", curr_view, CIT_COMMAND));
-	assert_true(curr_view == &lwin);
-
-	opt_handlers_teardown();
-	vle_keys_reset();
-}
-
 TEST(yank_works_with_ranges)
 {
 	char path[PATH_MAX + 1];
-	reg_t *reg;
+	const reg_t *reg;
 
 	regs_init();
 
@@ -459,7 +231,7 @@ TEST(yank_works_with_ranges)
 	assert_non_null(reg);
 
 	assert_int_equal(0, reg->nfiles);
-	(void)exec_commands("%yank", &lwin, CIT_COMMAND);
+	(void)cmds_dispatch("%yank", &lwin, CIT_COMMAND);
 	assert_int_equal(1, reg->nfiles);
 
 	regs_reset();
@@ -491,7 +263,7 @@ TEST(symlinks_in_paths_are_not_resolved, IF(not_windows))
 			sizeof(canonic_path));
 
 	/* :mkdir */
-	(void)exec_commands("mkdir ../dir", &lwin, CIT_COMMAND);
+	(void)cmds_dispatch("mkdir ../dir", &lwin, CIT_COMMAND);
 	restore_cwd(saved_cwd);
 	saved_cwd = save_cwd();
 	assert_success(rmdir(SANDBOX_PATH "/dir"));
@@ -499,7 +271,7 @@ TEST(symlinks_in_paths_are_not_resolved, IF(not_windows))
 	/* :clone file name. */
 	create_file(SANDBOX_PATH "/dir-link/file");
 	populate_dir_list(&lwin, 1);
-	(void)exec_commands("clone ../file-clone", &lwin, CIT_COMMAND);
+	(void)cmds_dispatch("clone ../file-clone", &lwin, CIT_COMMAND);
 	restore_cwd(saved_cwd);
 	saved_cwd = save_cwd();
 	assert_success(remove(SANDBOX_PATH "/file-clone"));
@@ -510,11 +282,11 @@ TEST(symlinks_in_paths_are_not_resolved, IF(not_windows))
 			"scripts/", saved_cwd);
 	snprintf(buf, sizeof(buf), "colorscheme set-env %s/../dir-link/..",
 			sandbox);
-	assert_success(exec_commands(buf, &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch(buf, &lwin, CIT_COMMAND));
 	cs_load_defaults();
 
 	/* :cd */
-	assert_success(exec_commands("cd ../dir-link/..", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("cd ../dir-link/..", &lwin, CIT_COMMAND));
 	assert_string_equal(canonic_path, lwin.curr_dir);
 
 	restore_cwd(saved_cwd);
@@ -534,18 +306,18 @@ TEST(grep_command, IF(not_windows))
 	assert_success(chdir(TEST_DATA_PATH "/scripts"));
 	assert_non_null(get_cwd(lwin.curr_dir, sizeof(lwin.curr_dir)));
 
-	assert_success(exec_commands("set grepprg='grep -n -H -r %i %a %s %u'", &lwin,
+	assert_success(cmds_dispatch("set grepprg='grep -n -H -r %i %a %s %u'", &lwin,
 				CIT_COMMAND));
 
 	/* Nothing to repeat. */
-	assert_failure(exec_commands("grep", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("grep", &lwin, CIT_COMMAND));
 
-	assert_success(exec_commands("grep command", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("grep command", &lwin, CIT_COMMAND));
 	assert_int_equal(2, lwin.list_rows);
 	assert_string_equal("Grep command", lwin.custom.title);
 
 	/* Repeat last grep, but add inversion. */
-	assert_success(exec_commands("grep!", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("grep!", &lwin, CIT_COMMAND));
 	assert_int_equal(5, lwin.list_rows);
 	assert_string_equal("Grep command", lwin.custom.title);
 
@@ -555,7 +327,7 @@ TEST(grep_command, IF(not_windows))
 TEST(touch)
 {
 	to_canonic_path(SANDBOX_PATH, cwd, lwin.curr_dir, sizeof(lwin.curr_dir));
-	(void)exec_commands("touch file", &lwin, CIT_COMMAND);
+	(void)cmds_dispatch("touch file", &lwin, CIT_COMMAND);
 
 	assert_success(remove(SANDBOX_PATH "/file"));
 }
@@ -563,22 +335,58 @@ TEST(touch)
 TEST(compare)
 {
 	opt_handlers_setup();
-
 	create_file(SANDBOX_PATH "/file");
 
 	to_canonic_path(SANDBOX_PATH, cwd, lwin.curr_dir, sizeof(lwin.curr_dir));
 
 	/* The file is empty so nothing to do when "skipempty" is specified. */
-	assert_success(exec_commands("compare ofone skipempty", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("compare ofone skipempty", &lwin, CIT_COMMAND));
 	assert_false(flist_custom_active(&lwin));
 
-	(void)exec_commands("compare byname bysize bycontents listall listdups "
+	/* Verify that later arguments override the former ones. */
+	(void)cmds_dispatch("compare byname bysize bycontents listall listdups "
 			"listunique ofboth ofone groupids grouppaths", &lwin, CIT_COMMAND);
 	assert_true(flist_custom_active(&lwin));
 	assert_int_equal(CV_REGULAR, lwin.custom.type);
+	rn_leave(&lwin, /*levels=*/1);
 
+	/* Can't toggle without !. */
+	(void)cmds_dispatch("compare byname", &lwin, CIT_COMMAND);
+	assert_int_equal(CF_GROUP_PATHS | CF_SHOW, lwin.custom.diff_cmp_flags);
+	(void)cmds_dispatch("compare showdifferent", &lwin, CIT_COMMAND);
+	assert_int_equal(CF_GROUP_PATHS | CF_SHOW, lwin.custom.diff_cmp_flags);
+	rn_leave(&lwin, /*levels=*/1);
+
+	/* No toggling. */
+	(void)cmds_dispatch("compare! showdifferent", &lwin, CIT_COMMAND);
+	assert_string_equal("Toggling requires active compare view", ui_sb_last());
+	/* Verify that two-pane compare gets correct arguments. */
+	make_abs_path(rwin.curr_dir, sizeof(rwin.curr_dir), TEST_DATA_PATH, "rename",
+			cwd);
+	(void)cmds_dispatch("compare byname withrcase withicase", &lwin, CIT_COMMAND);
+	assert_true(flist_custom_active(&lwin));
+	assert_true(flist_custom_active(&rwin));
+	assert_int_equal(CT_NAME, lwin.custom.diff_cmp_type);
+	assert_int_equal(LT_ALL, lwin.custom.diff_list_type);
+	assert_int_equal(CF_GROUP_PATHS | CF_IGNORE_CASE | CF_SHOW,
+			lwin.custom.diff_cmp_flags);
+	/* Toggling. */
+	(void)cmds_dispatch("compare! showidentical showdifferent", &lwin,
+			CIT_COMMAND);
+	assert_true(flist_custom_active(&lwin));
+	assert_true(flist_custom_active(&rwin));
+	assert_int_equal(CT_NAME, lwin.custom.diff_cmp_type);
+	assert_int_equal(LT_ALL, lwin.custom.diff_list_type);
+	assert_int_equal(CF_GROUP_PATHS | CF_IGNORE_CASE | CF_SHOW_UNIQUE_LEFT |
+			CF_SHOW_UNIQUE_RIGHT, lwin.custom.diff_cmp_flags);
+	/* Bad toggling. */
+	(void)cmds_dispatch("compare! byname", &lwin, CIT_COMMAND);
+	assert_int_equal(CF_GROUP_PATHS | CF_IGNORE_CASE | CF_SHOW_UNIQUE_LEFT |
+			CF_SHOW_UNIQUE_RIGHT, lwin.custom.diff_cmp_flags);
+	assert_string_equal("Unexpected property for toggling: byname", ui_sb_last());
+
+	assert_success(chdir(cwd));
 	assert_success(remove(SANDBOX_PATH "/file"));
-
 	opt_handlers_teardown();
 }
 
@@ -587,15 +395,15 @@ TEST(screen)
 	assert_false(cfg.use_term_multiplexer);
 
 	/* :screen toggles the option. */
-	assert_success(exec_commands("screen", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("screen", &lwin, CIT_COMMAND));
 	assert_true(cfg.use_term_multiplexer);
-	assert_success(exec_commands("screen", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("screen", &lwin, CIT_COMMAND));
 	assert_false(cfg.use_term_multiplexer);
 
 	/* :screen! sets it to on. */
-	assert_success(exec_commands("screen!", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("screen!", &lwin, CIT_COMMAND));
 	assert_true(cfg.use_term_multiplexer);
-	assert_success(exec_commands("screen!", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("screen!", &lwin, CIT_COMMAND));
 	assert_true(cfg.use_term_multiplexer);
 
 	cfg.use_term_multiplexer = 0;
@@ -612,9 +420,9 @@ TEST(hist_next_and_prev)
 	flist_hist_setup(&lwin, sandbox, ".", 0, 1);
 	flist_hist_setup(&lwin, test_data, ".", 0, 1);
 
-	assert_success(exec_commands("histprev", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("histprev", &lwin, CIT_COMMAND));
 	assert_true(paths_are_same(lwin.curr_dir, sandbox));
-	assert_success(exec_commands("histnext", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("histnext", &lwin, CIT_COMMAND));
 	assert_true(paths_are_same(lwin.curr_dir, test_data));
 
 	cfg_resize_histories(0);
@@ -622,7 +430,7 @@ TEST(hist_next_and_prev)
 
 TEST(normal_command_does_not_reset_selection)
 {
-	init_modes();
+	modes_init();
 	opt_handlers_setup();
 
 	lwin.list_rows = 2;
@@ -637,17 +445,17 @@ TEST(normal_command_does_not_reset_selection)
 	lwin.dir_entry[1].selected = 0;
 	lwin.selected_files = 1;
 
-	assert_success(exec_commands(":normal! t", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch(":normal! t", &lwin, CIT_COMMAND));
 	assert_int_equal(0, lwin.selected_files);
 	assert_false(lwin.dir_entry[0].selected);
 	assert_false(lwin.dir_entry[1].selected);
 
-	assert_success(exec_commands(":normal! vG\r", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch(":normal! vG\r", &lwin, CIT_COMMAND));
 	assert_int_equal(2, lwin.selected_files);
 	assert_true(lwin.dir_entry[0].selected);
 	assert_true(lwin.dir_entry[1].selected);
 
-	assert_success(exec_commands(":normal! t", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch(":normal! t", &lwin, CIT_COMMAND));
 	assert_int_equal(1, lwin.selected_files);
 	assert_true(lwin.dir_entry[0].selected);
 	assert_false(lwin.dir_entry[1].selected);
@@ -656,18 +464,35 @@ TEST(normal_command_does_not_reset_selection)
 	vle_keys_reset();
 }
 
+TEST(keepsel_preserves_selection)
+{
+	init_view_list(&lwin);
+
+	lwin.dir_entry[0].selected = 1;
+	lwin.selected_files = 1;
+	assert_failure(cmds_dispatch("echo 'hi'", &lwin, CIT_COMMAND));
+	assert_int_equal(0, lwin.selected_files);
+	assert_false(lwin.dir_entry[0].selected);
+
+	lwin.dir_entry[0].selected = 1;
+	lwin.selected_files = 1;
+	assert_failure(cmds_dispatch("keepsel echo 'hi'", &lwin, CIT_COMMAND));
+	assert_int_equal(1, lwin.selected_files);
+	assert_true(lwin.dir_entry[0].selected);
+}
+
 TEST(goto_command)
 {
-	assert_failure(exec_commands("goto /", &lwin, CIT_COMMAND));
-	assert_failure(exec_commands("goto /no-such-path", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("goto /", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("goto /no-such-path", &lwin, CIT_COMMAND));
 
 	char cmd[PATH_MAX*2];
 	snprintf(cmd, sizeof(cmd), "goto %s/compare", test_data);
-	assert_success(exec_commands(cmd, &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch(cmd, &lwin, CIT_COMMAND));
 	assert_true(paths_are_same(lwin.curr_dir, test_data));
 	assert_string_equal("compare", get_current_file_name(&lwin));
 
-	assert_success(exec_commands("goto tree", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("goto tree", &lwin, CIT_COMMAND));
 	assert_true(paths_are_same(lwin.curr_dir, test_data));
 	assert_string_equal("tree", get_current_file_name(&lwin));
 }
@@ -676,7 +501,7 @@ TEST(goto_normalizes_slashes, IF(windows))
 {
 	char cmd[PATH_MAX*2];
 	snprintf(cmd, sizeof(cmd), "goto %s\\\\compare", test_data);
-	assert_success(exec_commands(cmd, &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch(cmd, &lwin, CIT_COMMAND));
 	assert_true(paths_are_same(lwin.curr_dir, test_data));
 	assert_string_equal("compare", get_current_file_name(&lwin));
 }
@@ -689,14 +514,14 @@ TEST(echo_reports_all_errors)
 	           "Invalid expression: \"hi";
 
 	ui_sb_msg("");
-	assert_failure(exec_commands("echo \"hi", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("echo \"hi", &lwin, CIT_COMMAND));
 	assert_string_equal(expected, ui_sb_last());
 
 	expected = "Expression is missing closing parenthesis: (1\n"
 	           "Invalid expression: (1";
 
 	ui_sb_msg("");
-	assert_failure(exec_commands("echo (1", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("echo (1", &lwin, CIT_COMMAND));
 	assert_string_equal(expected, ui_sb_last());
 
 	char zeroes[8192] = "echo ";
@@ -704,20 +529,33 @@ TEST(echo_reports_all_errors)
 	zeroes[sizeof(zeroes) - 1U] = '\0';
 
 	ui_sb_msg("");
-	assert_failure(exec_commands(zeroes, &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch(zeroes, &lwin, CIT_COMMAND));
 	assert_true(strchr(ui_sb_last(), '\n') != NULL);
 }
 
+TEST(echo_without_arguments_prints_nothing)
+{
+	ui_sb_msg("");
+
+	/* First, print some message to record it as the last one. */
+	assert_failure(cmds_dispatch("echo 'previous'", &lwin, CIT_COMMAND));
+	assert_string_equal("previous", ui_sb_last());
+
+	/* Now, no message.  The last one could popup here. */
+	assert_failure(cmds_dispatch("echo", &lwin, CIT_COMMAND));
+	assert_string_equal("", ui_sb_last());
+}
+
 TEST(zero_count_is_rejected)
 {
 	const char *expected = "Count argument can't be zero";
 
 	ui_sb_msg("");
-	assert_failure(exec_commands("delete a 0", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("delete a 0", &lwin, CIT_COMMAND));
 	assert_string_equal(expected, ui_sb_last());
 
 	ui_sb_msg("");
-	assert_failure(exec_commands("yank a 0", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("yank a 0", &lwin, CIT_COMMAND));
 	assert_string_equal(expected, ui_sb_last());
 }
 
@@ -726,29 +564,29 @@ TEST(tree_command)
 	strcpy(lwin.curr_dir, sandbox);
 
 	/* Invalid input. */
-	assert_failure(exec_commands("tree nesting=0", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("tree nesting=0", &lwin, CIT_COMMAND));
 	assert_false(flist_custom_active(&lwin));
 	assert_string_equal("Invalid argument: nesting=0", ui_sb_last());
-	assert_failure(exec_commands("tree depth=0", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("tree depth=0", &lwin, CIT_COMMAND));
 	assert_false(flist_custom_active(&lwin));
 	assert_string_equal("Invalid depth: 0", ui_sb_last());
 
 	/* :tree enters tree mode. */
-	assert_success(exec_commands("tree", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("tree", &lwin, CIT_COMMAND));
 	assert_true(flist_custom_active(&lwin));
 	assert_true(cv_tree(lwin.custom.type));
 
 	/* Repeating :tree leaves view in tree mode. */
-	assert_success(exec_commands("tree", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("tree", &lwin, CIT_COMMAND));
 	assert_true(flist_custom_active(&lwin));
 	assert_true(cv_tree(lwin.custom.type));
 
 	/* :tree! can leave tree mode. */
-	assert_success(exec_commands("tree!", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("tree!", &lwin, CIT_COMMAND));
 	assert_false(flist_custom_active(&lwin));
 
 	/* :tree! can enter tree mode. */
-	assert_success(exec_commands("tree!", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("tree!", &lwin, CIT_COMMAND));
 	assert_true(flist_custom_active(&lwin));
 	assert_true(cv_tree(lwin.custom.type));
 
@@ -762,7 +600,7 @@ TEST(tree_command)
 	snprintf(sub_sub_path, sizeof(sub_sub_path), "%s/sub/sub", sandbox);
 	create_dir(sub_sub_path);
 
-	assert_success(exec_commands("tree depth=1", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("tree depth=1", &lwin, CIT_COMMAND));
 	assert_true(flist_custom_active(&lwin));
 	assert_true(cv_tree(lwin.custom.type));
 	assert_int_equal(1, lwin.list_rows);
@@ -776,16 +614,16 @@ TEST(regular_command)
 	strcpy(lwin.curr_dir, sandbox);
 
 	/* :tree enters tree mode. */
-	assert_success(exec_commands("tree", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("tree", &lwin, CIT_COMMAND));
 	assert_true(flist_custom_active(&lwin));
 	assert_true(cv_tree(lwin.custom.type));
 
 	/* :regular leaves tree mode. */
-	assert_success(exec_commands("regular", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("regular", &lwin, CIT_COMMAND));
 	assert_false(flist_custom_active(&lwin));
 
 	/* Repeated :regular does nothing. */
-	assert_success(exec_commands("regular", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("regular", &lwin, CIT_COMMAND));
 	assert_false(flist_custom_active(&lwin));
 }
 
@@ -795,29 +633,29 @@ TEST(plugin_command)
 	curr_stats.plugs = plugs_create(curr_stats.vlua);
 
 	ui_sb_msg("");
-	assert_failure(exec_commands("plugin load all", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("plugin load all", &lwin, CIT_COMMAND));
 	assert_string_equal("Trailing characters", ui_sb_last());
-	assert_failure(exec_commands("plugin wrong arg", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("plugin wrong arg", &lwin, CIT_COMMAND));
 	assert_string_equal("Unknown subcommand: wrong", ui_sb_last());
-	assert_failure(exec_commands("plugin args-count", &lwin, CIT_COMMAND));
+	assert_failure(cmds_dispatch("plugin args-count", &lwin, CIT_COMMAND));
 	assert_string_equal("Too few arguments", ui_sb_last());
 
-	assert_success(exec_commands("plugin load", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("plugin load", &lwin, CIT_COMMAND));
 
 	strlist_t empty_list = {};
 	char *plug_items[] = { "plug" };
 	strlist_t plug_list = { .items = plug_items, .nitems = 1 };
 
 	ui_sb_msg("");
-	assert_success(exec_commands("plugin blacklist plug", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("plugin blacklist plug", &lwin, CIT_COMMAND));
 	assert_string_equal("", ui_sb_last());
 
 	strings_list_is(plug_list, plugs_get_blacklist(curr_stats.plugs));
 	strings_list_is(empty_list, plugs_get_whitelist(curr_stats.plugs));
 
 	ui_sb_msg("");
-	assert_success(exec_commands("plugin whitelist plug", &lwin, CIT_COMMAND));
-	assert_success(exec_commands("plugin whitelist plug", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("plugin whitelist plug", &lwin, CIT_COMMAND));
+	assert_success(cmds_dispatch("plugin whitelist plug", &lwin, CIT_COMMAND));
 	assert_string_equal("", ui_sb_last());
 
 	strings_list_is(plug_list, plugs_get_blacklist(curr_stats.plugs));
@@ -829,6 +667,101 @@ TEST(plugin_command)
 	curr_stats.vlua = NULL;
 }
 
+TEST(help_command)
+{
+	curr_stats.exec_env_type = EET_EMULATOR;
+	update_string(&cfg.vi_command, "#vifmtest#editor");
+	cfg.config_dir[0] = '\0';
+
+	curr_stats.vlua = vlua_init();
+
+	assert_success(vlua_run_string(curr_stats.vlua,
+				"function handler(info) ginfo = info; return { success = false } end"));
+	assert_success(vlua_run_string(curr_stats.vlua,
+				"vifm.addhandler{ name = 'editor', handler = handler }"));
+
+	cfg.use_vim_help = 0;
+
+	assert_success(cmds_dispatch("help", &lwin, CIT_COMMAND));
+
+	assert_success(vlua_run_string(curr_stats.vlua, "print(ginfo.action)"));
+	assert_string_equal("edit-one", ui_sb_last());
+	assert_success(vlua_run_string(curr_stats.vlua, "print(ginfo.path)"));
+	assert_string_equal("/vifm-help.txt", ui_sb_last());
+	assert_success(vlua_run_string(curr_stats.vlua, "print(ginfo.mustwait)"));
+	assert_string_equal("false", ui_sb_last());
+	assert_success(vlua_run_string(curr_stats.vlua, "print(ginfo.line)"));
+	assert_string_equal("nil", ui_sb_last());
+	assert_success(vlua_run_string(curr_stats.vlua, "print(ginfo.column)"));
+	assert_string_equal("nil", ui_sb_last());
+
+	cfg.use_vim_help = 1;
+
+	assert_success(cmds_dispatch("help", &lwin, CIT_COMMAND));
+
+	assert_success(vlua_run_string(curr_stats.vlua, "print(ginfo.action)"));
+	assert_string_equal("open-help", ui_sb_last());
+	assert_success(vlua_run_string(curr_stats.vlua, "print(ginfo.topic)"));
+	assert_string_equal("vifm-app.txt", ui_sb_last());
+	assert_success(vlua_run_string(curr_stats.vlua, "print(ginfo.vimdocdir)"));
+	assert_string_ends_with("/vim-doc", ui_sb_last());
+
+	cfg.use_vim_help = 0;
+
+	vlua_finish(curr_stats.vlua);
+	curr_stats.vlua = NULL;
+}
+
+TEST(view_command)
+{
+	opt_handlers_setup();
+
+	curr_stats.preview.on = 0;
+
+	assert_success(cmds_dispatch("view", &lwin, CIT_COMMAND));
+	assert_true(curr_stats.preview.on);
+
+	assert_success(cmds_dispatch("view", &lwin, CIT_COMMAND));
+	assert_false(curr_stats.preview.on);
+
+	assert_success(cmds_dispatch("view!", &lwin, CIT_COMMAND));
+	assert_true(curr_stats.preview.on);
+
+	assert_success(cmds_dispatch("view!", &lwin, CIT_COMMAND));
+	assert_true(curr_stats.preview.on);
+
+	assert_success(cmds_dispatch("view", &lwin, CIT_COMMAND));
+	assert_false(curr_stats.preview.on);
+
+	opt_handlers_teardown();
+}
+
+TEST(invert_command)
+{
+	opt_handlers_setup();
+
+	ui_sb_msg("");
+	assert_failure(cmds_dispatch("set sort? sortorder?", &lwin, CIT_COMMAND));
+	assert_string_equal("  sort=+name\n  sortorder=ascending", ui_sb_last());
+
+	assert_success(cmds_dispatch("invert o", &lwin, CIT_COMMAND));
+
+	ui_sb_msg("");
+	assert_failure(cmds_dispatch("set sort? sortorder?", &lwin, CIT_COMMAND));
+	assert_string_equal("  sort=-name\n  sortorder=descending", ui_sb_last());
+
+	opt_handlers_teardown();
+}
+
+TEST(locate_command)
+{
+	ui_sb_msg("");
+
+	/* Nothing to repeat. */
+	assert_failure(cmds_dispatch("locate", &lwin, CIT_COMMAND));
+	assert_string_equal("Nothing to repeat", ui_sb_last());
+}
+
 static void
 strings_list_is(const strlist_t expected, const strlist_t actual)
 {
