@@ -10,6 +10,7 @@
 
 #include "../../src/cfg/config.h"
 #include "../../src/compat/fs_limits.h"
+#include "../../src/modes/dialogs/msg_dialog.h"
 #include "../../src/utils/fs.h"
 #include "../../src/utils/path.h"
 #include "../../src/filelist.h"
@@ -20,29 +21,22 @@
 #include "../../src/trash.h"
 
 static void line_prompt(const char prompt[], const char filename[],
-		fo_prompt_cb cb, fo_complete_cmd_func complete, int allow_ee);
+		fo_prompt_cb cb, void *cb_arg, fo_complete_cmd_func complete, int allow_ee);
 static void line_prompt_rec(const char prompt[], const char filename[],
-		fo_prompt_cb cb, fo_complete_cmd_func complete, int allow_ee);
-static char options_prompt_rename(const char title[], const char message[],
-		const struct response_variant *variants);
-static char options_prompt_rename_rec(const char title[], const char message[],
-		const struct response_variant *variants);
-static char options_prompt_overwrite(const char title[], const char message[],
-		const struct response_variant *variants);
-static char options_prompt_abort(const char title[], const char message[],
-		const struct response_variant *variants);
-static char options_prompt_skip_all(const char title[], const char message[],
-		const struct response_variant *variants);
-static char cm_overwrite(const char title[], const char message[],
-		const struct response_variant *variants);
-static char cm_no(const char title[], const char message[],
-		const struct response_variant *variants);
-static char cm_skip(const char title[], const char message[],
-		const struct response_variant *variants);
+		fo_prompt_cb cb, void *cb_arg, fo_complete_cmd_func complete, int allow_ee);
+static char options_prompt_rename(const custom_prompt_t *details);
+static char options_prompt_rename_rec(const custom_prompt_t *details);
+static char options_prompt_overwrite(const custom_prompt_t *details);
+static char options_prompt_abort(const custom_prompt_t *details);
+static char options_prompt_skip_all(const custom_prompt_t *details);
+static char cm_overwrite(const custom_prompt_t *details);
+static char cm_no(const custom_prompt_t *details);
+static char cm_skip(const custom_prompt_t *details);
 static void parent_overwrite_with_put(int move);
 static void double_clash_with_put(int move);
 
 static fo_prompt_cb rename_cb;
+static int options_count;
 
 static char *saved_cwd;
 
@@ -67,78 +61,80 @@ TEARDOWN()
 	view_teardown(&lwin);
 	regs_reset();
 	restore_cwd(saved_cwd);
+	fops_init(NULL, NULL);
 }
 
 static void
 line_prompt(const char prompt[], const char filename[], fo_prompt_cb cb,
-		fo_complete_cmd_func complete, int allow_ee)
+		void *cb_arg, fo_complete_cmd_func complete, int allow_ee)
 {
-	cb("b");
+	cb("b", cb_arg);
 }
 
 static void
 line_prompt_rec(const char prompt[], const char filename[], fo_prompt_cb cb,
-		fo_complete_cmd_func complete, int allow_ee)
+		void *cb_arg, fo_complete_cmd_func complete, int allow_ee)
 {
 	rename_cb = cb;
 }
 
 static char
-options_prompt_rename(const char title[], const char message[],
-		const struct response_variant *variants)
+options_prompt_rename(const custom_prompt_t *details)
 {
 	fops_init(&line_prompt, &options_prompt_overwrite);
 	return 'r';
 }
 
 static char
-options_prompt_rename_rec(const char title[], const char message[],
-		const struct response_variant *variants)
+options_prompt_rename_rec(const custom_prompt_t *details)
 {
 	fops_init(&line_prompt_rec, &options_prompt_overwrite);
 	return 'r';
 }
 
 static char
-options_prompt_overwrite(const char title[], const char message[],
-		const struct response_variant *variants)
+options_prompt_overwrite(const custom_prompt_t *details)
 {
 	return 'o';
 }
 
 static char
-options_prompt_abort(const char title[], const char message[],
-		const struct response_variant *variants)
+options_prompt_abort(const custom_prompt_t *details)
 {
+	const response_variant *variants = details->variants;
+
+	options_count = 0;
+	while(variants->key != '\0')
+	{
+		++options_count;
+		++variants;
+	}
+
 	return '\x03';
 }
 
 static char
-options_prompt_skip_all(const char title[], const char message[],
-		const struct response_variant *variants)
+options_prompt_skip_all(const custom_prompt_t *details)
 {
 	return 'S';
 }
 
 static char
-cm_overwrite(const char title[], const char message[],
-		const struct response_variant *variants)
+cm_overwrite(const custom_prompt_t *details)
 {
 	fops_init(&line_prompt, &cm_no);
 	return 'o';
 }
 
 static char
-cm_no(const char title[], const char message[],
-		const struct response_variant *variants)
+cm_no(const custom_prompt_t *details)
 {
 	fops_init(&line_prompt, &cm_skip);
 	return 'n';
 }
 
 static char
-cm_skip(const char title[], const char message[],
-		const struct response_variant *variants)
+cm_skip(const custom_prompt_t *details)
 {
 	fops_init(&line_prompt, &options_prompt_overwrite);
 	return 's';
@@ -339,7 +335,7 @@ TEST(rename_on_put)
 	fops_init(&line_prompt_rec, &options_prompt_rename_rec);
 	(void)fops_put(&lwin, -1, 'a', 0);
 	/* Continue the operation. */
-	rename_cb("b");
+	rename_cb("b", /*arg=*/NULL);
 
 	restore_cwd(saved_cwd);
 	saved_cwd = save_cwd();
@@ -654,5 +650,73 @@ TEST(cursor_is_moved_even_if_no_file_was_processed)
 	assert_success(unlink(SANDBOX_PATH "/a"));
 }
 
+TEST(show_merge_all_option_if_paths_include_dir)
+{
+	char path[PATH_MAX + 1];
+
+	create_file(SANDBOX_PATH "/a");
+	create_dir(SANDBOX_PATH "/dir");
+	create_file(SANDBOX_PATH "/dir/a");
+	create_file(SANDBOX_PATH "/dir/b");
+	create_dir(SANDBOX_PATH "/dir/sub");
+
+	fops_init(&line_prompt, &options_prompt_abort);
+
+	make_abs_path(path, sizeof(path), SANDBOX_PATH, "/dir/a", saved_cwd);
+	assert_success(regs_append('a', path));
+	make_abs_path(path, sizeof(path), SANDBOX_PATH, "/dir/b", saved_cwd);
+	assert_success(regs_append('a', path));
+
+	options_count = 0;
+	(void)fops_put(&lwin, -1, 'a', 0);
+	assert_int_equal(8, options_count);
+
+	restore_cwd(saved_cwd);
+	saved_cwd = save_cwd();
+
+	make_abs_path(path, sizeof(path), SANDBOX_PATH, "/dir/sub", saved_cwd);
+	assert_success(regs_append('a', path));
+
+	options_count = 0;
+	(void)fops_put(&lwin, -1, 'a', 0);
+	assert_int_equal(9, options_count);
+
+	restore_cwd(saved_cwd);
+	saved_cwd = save_cwd();
+
+	assert_success(unlink(SANDBOX_PATH "/dir/a"));
+	assert_success(unlink(SANDBOX_PATH "/dir/b"));
+	assert_success(rmdir(SANDBOX_PATH "/dir/sub"));
+	assert_success(rmdir(SANDBOX_PATH "/dir"));
+	assert_success(unlink(SANDBOX_PATH "/a"));
+	/* This one doesn't always get copied. */
+	(void)unlink(SANDBOX_PATH "/b");
+}
+
+TEST(no_merge_options_on_putting_links)
+{
+	char path[PATH_MAX + 1];
+
+	create_dir(SANDBOX_PATH "/sub");
+	create_dir(SANDBOX_PATH "/dir");
+	create_dir(SANDBOX_PATH "/dir/sub");
+
+	fops_init(&line_prompt, &options_prompt_abort);
+
+	make_abs_path(path, sizeof(path), SANDBOX_PATH, "/dir/sub", saved_cwd);
+	assert_success(regs_append('a', path));
+
+	options_count = 0;
+	(void)fops_put_links(&lwin, 'a', 0);
+	assert_int_equal(7, options_count);
+
+	restore_cwd(saved_cwd);
+	saved_cwd = save_cwd();
+
+	assert_success(rmdir(SANDBOX_PATH "/dir/sub"));
+	assert_success(rmdir(SANDBOX_PATH "/dir"));
+	assert_success(rmdir(SANDBOX_PATH "/sub"));
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
