@@ -19,7 +19,7 @@
 #include "../../src/status.h"
 
 static void broken_link_name(const char prompt[], const char filename[],
-		fo_prompt_cb cb, fo_complete_cmd_func complete, int allow_ee);
+		fo_prompt_cb cb, void *cb_arg, fo_complete_cmd_func complete, int allow_ee);
 
 static char *saved_cwd;
 
@@ -191,9 +191,9 @@ TEST(rename_to_broken_symlink_name, IF(not_windows))
 
 static void
 broken_link_name(const char prompt[], const char filename[], fo_prompt_cb cb,
-		fo_complete_cmd_func complete, int allow_ee)
+		void *cb_arg, fo_complete_cmd_func complete, int allow_ee)
 {
-	cb("broken-link");
+	cb("broken-link", cb_arg);
 }
 
 TEST(file_list_can_be_edited_including_long_fnames, IF(not_windows))
