@@ -16,6 +16,7 @@
 #include "../../src/engine/functions.h"
 #include "../../src/engine/parsing.h"
 #include "../../src/engine/variables.h"
+#include "../../src/lua/vlua.h"
 #include "../../src/utils/env.h"
 #include "../../src/utils/fs.h"
 #include "../../src/utils/str.h"
@@ -32,7 +33,7 @@ SETUP()
 	update_string(&cfg.shell_cmd_flag, "-c");
 
 	init_builtin_functions();
-	init_parser(NULL);
+	vle_parser_init(NULL);
 	init_variables();
 
 	view_setup(&lwin);
@@ -72,12 +73,49 @@ TEST(executable_false_for_dir)
 	ASSERT_INT_OK("executable('.')", 0);
 }
 
+TEST(filereadable)
+{
+	ASSERT_INT_OK("filereadable('" TEST_DATA_PATH "/read/two-lines')", 1);
+	ASSERT_INT_OK("filereadable('" TEST_DATA_PATH "/read')", 0);
+	ASSERT_INT_OK("filereadable('no-such-path')", 0);
+}
+
 TEST(expand_expands_environment_variables)
 {
 	let_variables("$OPEN_ME = 'Found something interesting?'");
 	ASSERT_OK("expand('$OPEN_ME')", "Found something interesting?");
 }
 
+TEST(expand_allows_escaping_dollar)
+{
+	let_variables("$NO_EXPAND = 'value'");
+	ASSERT_OK("expand('\\$NO_EXPAND')", "$NO_EXPAND");
+}
+
+TEST(expand_escapes_paths)
+{
+	curr_view = &lwin;
+	other_view = &rwin;
+
+	init_view_list(&lwin);
+	replace_string(&lwin.dir_entry[0].name, "f i l e");
+	strcpy(lwin.curr_dir, "/dir");
+
+	ASSERT_OK("expand('%c')", "f\\ i\\ l\\ e");
+}
+
+TEST(expand_does_not_need_double_escaping)
+{
+	curr_view = &lwin;
+	other_view = &rwin;
+
+	init_view_list(&lwin);
+	replace_string(&lwin.dir_entry[0].name, "file");
+	strcpy(lwin.curr_dir, "/dir");
+
+	ASSERT_OK("expand('%c:p:gs!/!\\\\!')", "\\\\dir\\\\file");
+}
+
 TEST(system_catches_stdout)
 {
 	ASSERT_OK("system('echo a')", "a");
@@ -234,7 +272,7 @@ TEST(getpanetype_for_compare_view)
 	other_view = &rwin;
 
 	opt_handlers_setup();
-	compare_one_pane(&lwin, CT_CONTENTS, LT_DUPS, 0);
+	compare_one_pane(&lwin, CT_CONTENTS, LT_DUPS, CF_NONE);
 	opt_handlers_teardown();
 
 	ASSERT_OK("getpanetype()", "compare");
@@ -248,8 +286,9 @@ TEST(chooseopt_options_are_not_set)
 	assert_success(stats_init(&cfg));
 
 	args_parse(&args, ARRAY_LEN(argv) - 1U, argv, "/");
-	args_process(&args, 0);
-	args_process(&args, 1);
+	args_process(&args, AS_GENERAL, /*ipc=*/NULL);
+	args_process(&args, AS_IPC, /*ipc=*/NULL);
+	args_process(&args, AS_OTHER, /*ipc=*/NULL);
 
 	ASSERT_OK("chooseopt('files')", "");
 	ASSERT_OK("chooseopt('dir')", "");
@@ -274,8 +313,9 @@ TEST(chooseopt_options_are_set)
 	assert_success(stats_init(&cfg));
 
 	args_parse(&args, ARRAY_LEN(argv) - 1U, argv, "/");
-	args_process(&args, 0);
-	args_process(&args, 1);
+	args_process(&args, AS_GENERAL, /*ipc=*/NULL);
+	args_process(&args, AS_IPC, /*ipc=*/NULL);
+	args_process(&args, AS_OTHER, /*ipc=*/NULL);
 
 	ASSERT_OK("chooseopt('files')", "/files-file");
 	ASSERT_OK("chooseopt('dir')", "/dir-file");
@@ -375,6 +415,22 @@ TEST(has)
 	ASSERT_OK("has('windows')", "0");
 }
 
+TEST(has_handler)
+{
+	curr_stats.vlua = vlua_init();
+
+	assert_success(vlua_run_string(curr_stats.vlua,
+				"function handler(info) end"));
+	assert_success(vlua_run_string(curr_stats.vlua,
+				"vifm.addhandler{ name = 'handler', handler = handler }"));
+
+	ASSERT_OK("has('#vifmtest#nohandler')", "0");
+	ASSERT_OK("has('#vifmtest#handler')", "1");
+
+	vlua_finish(curr_stats.vlua);
+	curr_stats.vlua = NULL;
+}
+
 TEST(extcached)
 {
 	make_abs_path(lwin.curr_dir, sizeof(lwin.curr_dir), SANDBOX_PATH, "", NULL);
@@ -432,5 +488,10 @@ TEST(extcached)
 	set_extcached_monitor_type(FMT_CHANGED);
 }
 
+TEST(input_wrong_arg)
+{
+	ASSERT_FAIL("input('prompt', 'input', 'bla')", PE_INVALID_EXPRESSION);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
