@@ -30,7 +30,7 @@ SETUP()
 	curr_view = &lwin;
 	other_view = &rwin;
 
-	init_commands();
+	cmds_init();
 
 	cfg.slow_fs_list = strdup("");
 	cfg.chase_links = 1;
