@@ -9,7 +9,7 @@
 
 SETUP()
 {
-	init_commands();
+	cmds_init();
 }
 
 TEARDOWN()
