@@ -55,7 +55,7 @@ TEARDOWN()
 TEST(empty_root_directories_abort_single_comparison)
 {
 	strcpy(lwin.curr_dir, SANDBOX_PATH);
-	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, 0);
+	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, CF_NONE);
 
 	assert_false(flist_custom_active(&lwin));
 }
@@ -68,7 +68,7 @@ TEST(empty_root_directories_abort_dual_comparison)
 	strcpy(lwin.curr_dir, SANDBOX_PATH "/a");
 	strcpy(rwin.curr_dir, SANDBOX_PATH "/b");
 
-	compare_two_panes(CT_CONTENTS, LT_ALL, 0, 0);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_SHOW);
 	assert_false(flist_custom_active(&lwin));
 
 	assert_success(rmdir(SANDBOX_PATH "/a"));
@@ -83,7 +83,7 @@ TEST(empty_unique_cv_are_created)
 	strcpy(lwin.curr_dir, SANDBOX_PATH "/a");
 	strcpy(rwin.curr_dir, SANDBOX_PATH "/b");
 
-	compare_two_panes(CT_CONTENTS, LT_UNIQUE, 0, 0);
+	compare_two_panes(CT_CONTENTS, LT_UNIQUE, CF_SHOW);
 	assert_true(flist_custom_active(&lwin));
 	assert_true(flist_custom_active(&rwin));
 
@@ -100,7 +100,7 @@ TEST(empty_unique_cv_are_created)
 TEST(listing_wrong_path_does_nothing)
 {
 	strcpy(lwin.curr_dir, SANDBOX_PATH "/does-not-exist");
-	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, 0);
+	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, CF_NONE);
 
 	assert_false(flist_custom_active(&lwin));
 }
@@ -108,7 +108,7 @@ TEST(listing_wrong_path_does_nothing)
 TEST(files_are_found_recursively)
 {
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/tree");
-	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, 0);
+	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, CF_NONE);
 
 	assert_int_equal(7, lwin.list_rows);
 }
@@ -121,7 +121,7 @@ TEST(compare_skips_dir_symlinks, IF(not_windows))
 #endif
 
 	strcpy(lwin.curr_dir, SANDBOX_PATH);
-	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, 0);
+	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, CF_NONE);
 	assert_false(flist_custom_active(&lwin));
 
 	assert_success(remove(SANDBOX_PATH "/link"));
@@ -130,13 +130,16 @@ TEST(compare_skips_dir_symlinks, IF(not_windows))
 TEST(not_available_files_are_ignored, IF(regular_unix_user))
 {
 	copy_file(TEST_DATA_PATH "/read/utf8-bom", SANDBOX_PATH "/utf8-bom");
+	copy_file(TEST_DATA_PATH "/read/utf8-bom", SANDBOX_PATH "/utf8-bom-2");
 	assert_success(chmod(SANDBOX_PATH "/utf8-bom", 0000));
+	assert_success(chmod(SANDBOX_PATH "/utf8-bom-2", 0000));
 
 	strcpy(lwin.curr_dir, SANDBOX_PATH);
-	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, 0);
+	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, CF_SKIP_EMPTY);
 	assert_false(flist_custom_active(&lwin));
 
 	assert_success(remove(SANDBOX_PATH "/utf8-bom"));
+	assert_success(remove(SANDBOX_PATH "/utf8-bom-2"));
 }
 
 TEST(two_panes_are_left_in_sync)
@@ -144,7 +147,7 @@ TEST(two_panes_are_left_in_sync)
 	strcpy(lwin.curr_dir, SANDBOX_PATH);
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/a");
 
-	compare_two_panes(CT_CONTENTS, LT_ALL, 0, 0);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_SHOW);
 	assert_true(flist_custom_active(&lwin));
 	assert_true(flist_custom_active(&rwin));
 
@@ -166,7 +169,7 @@ TEST(exclude_works_with_entries_or_their_groups)
 
 	strcpy(lwin.curr_dir, SANDBOX_PATH);
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/a");
-	compare_two_panes(CT_CONTENTS, LT_ALL, 0, 0);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_SHOW);
 	check_compare_invariants(5);
 
 	/* Does nothing on separator. */
@@ -217,12 +220,12 @@ TEST(local_filter_is_not_set)
 {
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/compare/a");
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/b");
-	compare_two_panes(CT_NAME, LT_ALL, 0, 0);
+	compare_two_panes(CT_NAME, LT_ALL, CF_SHOW);
 
-	exec_command("f", &lwin, CIT_FILTER_PATTERN);
+	cmds_dispatch1("f", &lwin, CIT_FILTER_PATTERN);
 	assert_true(filter_is_empty(&lwin.local_filter.filter));
 
-	modcline_enter(CLS_FILTER, lwin.local_filter.filter.raw, NULL);
+	modcline_enter(CLS_FILTER, lwin.local_filter.filter.raw);
 	assert_true(vle_mode_is(NORMAL_MODE));
 }
 
@@ -239,7 +242,7 @@ TEST(removed_files_disappear_in_both_views_on_reload)
 
 	strcpy(lwin.curr_dir, SANDBOX_PATH);
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/a");
-	compare_two_panes(CT_CONTENTS, LT_ALL, 0, 0);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_SHOW);
 	check_compare_invariants(5);
 
 	assert_success(remove(SANDBOX_PATH "/same-content-different-name-1"));
@@ -268,7 +271,7 @@ TEST(comparison_views_are_closed_when_no_files_are_left)
 
 	strcpy(lwin.curr_dir, SANDBOX_PATH "/a");
 	strcpy(rwin.curr_dir, SANDBOX_PATH "/b");
-	compare_two_panes(CT_CONTENTS, LT_ALL, 0, 0);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_SHOW);
 	check_compare_invariants(1);
 
 	assert_success(remove(SANDBOX_PATH "/a/same-content-different-name-1"));
@@ -291,13 +294,13 @@ TEST(comparison_views_are_closed_when_no_files_are_left)
 TEST(sorting_is_not_changed)
 {
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/compare/a");
-	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, 0);
+	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, CF_NONE);
 
 	change_sort_type(&lwin, SK_BY_SIZE, 0);
 	assert_int_equal(SK_NONE, lwin.sort[0]);
 
-	init_commands();
-	assert_success(exec_commands("set sort=ext", &lwin, CIT_COMMAND));
+	cmds_init();
+	assert_success(cmds_dispatch1("set sort=ext", &lwin, CIT_COMMAND));
 	assert_int_equal(SK_NONE, lwin.sort[0]);
 	vle_cmds_reset();
 }
@@ -306,7 +309,7 @@ TEST(cursor_moves_in_both_views_synchronously)
 {
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/compare/a");
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/b");
-	compare_two_panes(CT_NAME, LT_ALL, 0, 0);
+	compare_two_panes(CT_NAME, LT_ALL, CF_SHOW);
 
 	assert_int_equal(0, lwin.list_pos);
 	assert_int_equal(lwin.list_pos, rwin.list_pos);
@@ -324,8 +327,8 @@ TEST(two_independent_compare_views_are_not_bound)
 {
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/compare/a");
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/b");
-	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, 0);
-	compare_one_pane(&rwin, CT_CONTENTS, LT_ALL, 0);
+	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, CF_NONE);
+	compare_one_pane(&rwin, CT_CONTENTS, LT_ALL, CF_NONE);
 
 	assert_int_equal(0, lwin.list_pos);
 	assert_int_equal(lwin.list_pos, rwin.list_pos);
@@ -343,12 +346,12 @@ TEST(diff_is_closed_by_single_compare)
 {
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/compare/a");
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/b");
-	compare_two_panes(CT_NAME, LT_ALL, 0, 0);
+	compare_two_panes(CT_NAME, LT_ALL, CF_SHOW);
 
 	assert_int_equal(CV_DIFF, lwin.custom.type);
 	assert_int_equal(CV_DIFF, rwin.custom.type);
 
-	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, 0);
+	compare_one_pane(&lwin, CT_CONTENTS, LT_ALL, CF_NONE);
 	assert_int_equal(CV_COMPARE, lwin.custom.type);
 	assert_int_equal(CV_REGULAR, rwin.custom.type);
 }
@@ -359,7 +362,7 @@ TEST(filtering_fake_entry_does_nothing)
 	other_view = &lwin;
 	strcpy(lwin.curr_dir, SANDBOX_PATH);
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/b");
-	compare_two_panes(CT_CONTENTS, LT_ALL, 1, 0);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_GROUP_PATHS | CF_SHOW);
 
 	assert_int_equal(4, lwin.list_rows);
 	assert_int_equal(4, rwin.list_rows);
@@ -381,7 +384,7 @@ TEST(filtering_updates_two_bound_views)
 	other_view = &lwin;
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/compare/a");
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/b");
-	compare_two_panes(CT_CONTENTS, LT_ALL, 0, 0);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_SHOW);
 
 	/* Check that single file is excluded. */
 
@@ -427,7 +430,7 @@ TEST(two_pane_dups_renumbering)
 	other_view = &lwin;
 	strcpy(lwin.curr_dir, SANDBOX_PATH);
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/read");
-	compare_two_panes(CT_CONTENTS, LT_DUPS, 1, 0);
+	compare_two_panes(CT_CONTENTS, LT_DUPS, CF_SHOW);
 
 	check_compare_invariants(2);
 
@@ -446,7 +449,7 @@ TEST(removing_all_files_of_same_id_and_fake_entry_on_the_other_side)
 
 	strcpy(lwin.curr_dir, SANDBOX_PATH);
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/read");
-	compare_two_panes(CT_CONTENTS, LT_ALL, 1, 0);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_SHOW);
 
 	check_compare_invariants(7);
 
@@ -471,9 +474,9 @@ TEST(compare_considers_dot_filter)
 	rwin.hide_dot = 1;
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/tree");
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/tree/dir5");
-	compare_two_panes(CT_CONTENTS, LT_ALL, 1, 0);
-	assert_int_equal(5, lwin.list_rows);
-	assert_int_equal(5, rwin.list_rows);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_GROUP_PATHS | CF_SHOW);
+	assert_int_equal(6, lwin.list_rows);
+	assert_int_equal(6, rwin.list_rows);
 }
 
 TEST(compare_considers_name_filters)
@@ -488,11 +491,11 @@ TEST(compare_considers_name_filters)
 	name_filters_add_active(&lwin);
 	assert_int_equal(2, lwin.list_rows);
 
-	exec_command("different-content", &rwin, CIT_FILTER_PATTERN);
+	cmds_dispatch1("different-content", &rwin, CIT_FILTER_PATTERN);
 	load_dir_list(&rwin, 1);
 	assert_int_equal(1, rwin.list_rows);
 
-	compare_two_panes(CT_CONTENTS, LT_ALL, /*group_paths=*/0, /*skip_empty=*/0);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_SHOW);
 
 	check_compare_invariants(3);
 
@@ -509,7 +512,7 @@ TEST(empty_files_are_skipped_if_requested)
 {
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/compare/a");
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/b");
-	compare_two_panes(CT_CONTENTS, LT_ALL, 0, 1);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_SKIP_EMPTY | CF_SHOW);
 	check_compare_invariants(4);
 }
 
@@ -517,7 +520,8 @@ TEST(custom_views_are_compared)
 {
 	char path[PATH_MAX + 1];
 
-	strcpy(lwin.curr_dir, "no-such-path");
+	make_abs_path(lwin.curr_dir, sizeof(lwin.curr_dir), TEST_DATA_PATH,
+			"/compare/a", saved_cwd);
 	flist_custom_start(&lwin, "test");
 	make_abs_path(path, sizeof(path), TEST_DATA_PATH,
 			"compare/a/same-content-different-name-1", saved_cwd);
@@ -532,7 +536,7 @@ TEST(custom_views_are_compared)
 
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare/b");
 
-	compare_two_panes(CT_NAME, LT_ALL, 1, 0);
+	compare_two_panes(CT_NAME, LT_ALL, CF_GROUP_PATHS | CF_SHOW);
 
 	check_compare_invariants(4);
 
@@ -561,7 +565,7 @@ TEST(directories_are_not_added_from_custom_views)
 
 	strcpy(rwin.curr_dir, SANDBOX_PATH);
 
-	compare_two_panes(CT_NAME, LT_ALL, 1, 0);
+	compare_two_panes(CT_NAME, LT_ALL, CF_GROUP_PATHS | CF_SHOW);
 
 	check_compare_invariants(1);
 
@@ -574,10 +578,53 @@ TEST(the_same_directories_are_not_compared)
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/compare");
 	strcpy(rwin.curr_dir, TEST_DATA_PATH "/compare");
 
-	compare_two_panes(CT_CONTENTS, LT_ALL, 0, 0);
+	compare_two_panes(CT_CONTENTS, LT_ALL, CF_SHOW);
 	assert_false(flist_custom_active(&lwin));
 	assert_false(flist_custom_active(&rwin));
 }
 
+/* Results should be the same regardless of which pane is active. */
+TEST(two_panes_unique_is_symmetric)
+{
+	create_dir(SANDBOX_PATH "/a");
+	create_dir(SANDBOX_PATH "/b");
+	create_file(SANDBOX_PATH "/a/filea");
+	create_file(SANDBOX_PATH "/a/fileb");
+	create_file(SANDBOX_PATH "/b/filea");
+
+	make_abs_path(lwin.curr_dir, sizeof(lwin.curr_dir), SANDBOX_PATH, "a",
+			saved_cwd);
+	make_abs_path(rwin.curr_dir, sizeof(rwin.curr_dir), SANDBOX_PATH, "b",
+			saved_cwd);
+
+	curr_view = &lwin;
+	other_view = &rwin;
+	compare_two_panes(CT_NAME, LT_UNIQUE, CF_SHOW);
+	assert_int_equal(1, lwin.list_rows);
+	assert_int_equal(1, rwin.list_rows);
+	assert_string_equal("fileb", lwin.dir_entry[0].name);
+	assert_string_equal("..", rwin.dir_entry[0].name);
+
+	rn_leave(&lwin, 1);
+	rn_leave(&rwin, 1);
+
+	curr_view = &rwin;
+	other_view = &lwin;
+	compare_two_panes(CT_NAME, LT_UNIQUE, CF_SHOW);
+	assert_int_equal(1, lwin.list_rows);
+	assert_int_equal(1, rwin.list_rows);
+	assert_string_equal("fileb", lwin.dir_entry[0].name);
+	assert_string_equal("..", rwin.dir_entry[0].name);
+
+	restore_cwd(saved_cwd);
+	saved_cwd = save_cwd();
+
+	remove_file(SANDBOX_PATH "/a/filea");
+	remove_file(SANDBOX_PATH "/a/fileb");
+	remove_file(SANDBOX_PATH "/b/filea");
+	remove_dir(SANDBOX_PATH "/a");
+	remove_dir(SANDBOX_PATH "/b");
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
