@@ -4,6 +4,7 @@
 
 #include "../../src/cmd_completion.h"
 #include "../../src/filetype.h"
+#include "../../src/status.h"
 
 TEST(system_shell_exists)
 {
@@ -16,6 +17,15 @@ TEST(system_shell_exists)
 	assert_true(exists);
 }
 
+TEST(start_always_exists_in_cmd_like_shells)
+{
+	curr_stats.shell_type = ST_CMD;
+	assert_true(external_command_exists("start"));
+
+	curr_stats.shell_type = ST_YORI;
+	assert_true(external_command_exists("start"));
+}
+
 TEST(exe_path_with_backslashes, IF(windows))
 {
 	ft_init(&external_command_exists);
