@@ -109,7 +109,7 @@ TEST(preview_is_closed_on_request)
 TEST(macros_are_expanded_for_viewer)
 {
 	strcpy(curr_view->curr_dir, "echo");
-	char *expanded = qv_expand_viewer("%d 1");
+	char *expanded = qv_expand_viewer(curr_view, "%d 1", /*macros=*/NULL);
 	assert_string_equal("echo 1", expanded);
 	free(expanded);
 }
@@ -204,7 +204,7 @@ TEST(no_switch_into_view_mode_of_hidden_pane)
 	view_setup(&rwin);
 	make_abs_path(rwin.curr_dir, sizeof(rwin.curr_dir), TEST_DATA_PATH, "", cwd);
 	populate_dir_list(&rwin, 1);
-	init_modes();
+	modes_init();
 
 	curr_stats.preview.on = 1;
 	curr_stats.number_of_windows = 1;
